package com.starbuds.server.api.app.config;

import com.starbuds.server.game.sdk.GameClient;
import com.starbuds.server.game.sdk.GameClientManage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

/**
 * 游戏配置
 *
 * @author aoe
 * @date 2020/9/8
 */
@Configuration
@Slf4j
public class GameConfig {

    private GameClient client;

    /**
     * @param host         主机地址
     * @param appId        直播平台id
     * @param apiSecretKey API秘钥
     * @param timout       HTTP超时时间（毫秒）
     */
    public GameConfig(@Value("${game.host}") String host,
                      @Value("${game.appId}") String appId,
                      @Value("${game.apiSecretKey}") String apiSecretKey,
                      @Value("${game.timout}") Integer timout) {
        log.info("游戏配置 host={} appId={} apiSecretKey={} timout={}", host, appId, apiSecretKey, timout);
        GameClientManage clientManage = new GameClientManage(host, appId, apiSecretKey, timout);
        client = clientManage.getClient();
    }

    /**
     * 游戏客户端
     *
     * @return
     */
    public GameClient client() {
        return client;
    }
}
