package com.starbuds.server.api.app.config;

import com.starbuds.server.game.sdkdev.GameDevClient;
import com.starbuds.server.game.sdkdev.GameDevClientManage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

/**
 * 游戏Dev配置
 *
 * @author aoe
 * @date 2020/9/8
 */
@Configuration
@Slf4j
public class GameDevConfig {

    private GameDevClient client;

    /**
     * @param host         主机地址
     * @param appId        直播平台id
     * @param apiSecretKey API秘钥
     * @param timout       HTTP超时时间（毫秒）
     */
    public GameDevConfig(@Value("${gamedev.host}") String host,
                         @Value("${gamedev.appId}") String appId,
                         @Value("${gamedev.apiSecretKey}") String apiSecretKey,
                         @Value("${gamedev.timout}") Integer timout) {
        log.info("游戏Dev配置 host={} appId={} apiSecretKey={} timout={}", host, appId, apiSecretKey, timout);
        GameDevClientManage clientManage = new GameDevClientManage(host, appId, apiSecretKey, timout);
        client = clientManage.getClient();
    }

    /**
     * 游戏客户端
     *
     * @return
     */
    public GameDevClient client() {
        return client;
    }
}
