package com.starbuds.server.api.app.config;

import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * 服务更新配置
 *
 * @author Lay
 * @date 2021/3/12
 */
@Component
@Slf4j
public class UpgradeConfig {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    private static final String KEY_UPDATE = "isUpdate";
    private static final String UPDATE = "update";

    /**
     * 是否更新
     */
    public boolean isUpgrade() {

        String result = redisService.get(KEY_UPDATE);
        //数据库获取配置
        if (StringUtils.isEmpty(result)) {
            String isUpdate = releaseConfigService.getReleaseConfig(KEY_UPDATE);
            //数据库没有配置：未在更新
            if (isUpdate == null) {
                return false;
            }

            //数据库配置更新中
            if (isUpdate.equals(UPDATE)) {
                redisService.set(KEY_UPDATE, UPDATE, 2, TimeUnit.SECONDS);
                return true;
            }
        }

        //redis中缓存配置服务更新中
        if (StringUtils.isNotEmpty(result) && result.equals("update")) {
            return true;
        }

        return false;
    }
}
