package com.starbuds.server.api.app.manage;

import com.starbuds.server.api.app.pojo.official.form.ForbiddenForm;
import com.starbuds.server.api.app.util.ManageHttpTools;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 管理后台服务
 * 2.5.0版本开始为了管理员在APP上使用部分管理后台功能，增加了本类
 *
 * @author aoe
 * @date 2022/1/10
 */
@Service
public class ManageService {

    @Resource
    private ManageHttpTools manageHttpTools;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    /**
     * 封禁(禁言)
     *
     * @param form
     * @return
     */
    public Result forbidden(ForbiddenForm form, String adminPhone) throws IOException {
        String action = "api-manage/v1/user/forbidden";
        Map<String, String> params = new HashMap<>();
        params.put("userId", form.getUserId() + "");
        params.put("way", form.getWay() + "");
        params.put("durationType", form.getDurationType() + "");
        params.put("remark", form.getRemark());
        return manageHttpTools.post(action, params, adminPhone);
    }

    /**
     * 重置昵称
     *
     * @param userId
     * @param adminPhone
     * @return
     * @throws IOException
     */
    public Result resetUserName(Long userId, String adminPhone) throws IOException {
        String action = "api-manage/v1/user/setUserName";
        Map<String, String> params = new HashMap<>();
        params.put("userId", userId + "");
        String randomUserName = userService.getRandomUserName();
        params.put("userName", randomUserName);
        return manageHttpTools.post(action, params, adminPhone);
    }

    /**
     * 重置用户签名
     *
     * @param userId
     * @param adminPhone
     * @return
     * @throws IOException
     */
    public Result resetUserSign(Long userId, String adminPhone) throws IOException {
        String action = "api-manage/v1/user/editUserSign";
        Map<String, String> params = new HashMap<>();
        params.put("userId", userId + "");
        String userSign = releaseConfigService.getCachReleaseConfig("resetUserSign");
        params.put("userSign", userSign);
        return manageHttpTools.post(action, params, adminPhone);
    }

    /**
     * 重置用户签名
     *
     * @param userId
     * @param adminPhone
     * @return
     * @throws IOException
     */
    public Result resetUserAvatar(Long userId, String adminPhone) throws IOException {
        String action = "api-manage/v1/user/editUserAvatar";
        Map<String, String> params = new HashMap<>();
        params.put("userId", userId + "");
        String userAvatar = releaseConfigService.getCachReleaseConfig("resetUserAvatar");
        params.put("userAvatar", userAvatar);
        return manageHttpTools.post(action, params, adminPhone);
    }

    /**
     * 删除照片墙
     *
     * @param feedPhotoWallId
     * @param userId
     * @param adminPhone
     * @return
     * @throws IOException
     */
    public Result deletePhotoWall(Long feedPhotoWallId, Long userId, String adminPhone) throws IOException {
        String action = "api-manage/v1/feed/deletePhotoWall";
        Map<String, String> params = new HashMap<>();
        params.put("feedPhotoWallId", feedPhotoWallId + "");
        params.put("userId", userId + "");
        return manageHttpTools.post(action, params, adminPhone);
    }

    /**
     * 删除动态
     *
     * @param feedId
     * @param auditMsg
     * @param adminPhone
     * @return
     * @throws IOException
     */
    public Result deleteFeed(Long feedId, String auditMsg, String adminPhone) throws IOException {
        String action = "api-manage/v1/feed/delFeed";
        Map<String, String> params = new HashMap<>();
        params.put("feedId", feedId + "");
        params.put("auditMsg", auditMsg);
        return manageHttpTools.post(action, params, adminPhone);
    }

}
