package com.starbuds.server.api.app.provider;

import com.starbuds.server.api.app.pojo.gift.vo.GiftRewardVO;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 打赏礼物
 *
 * @author aoe
 * @date 2021/11/17
 */
@Component
@Slf4j
public class RewardGiftProvider {

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    /**
     * (动态)打赏礼物列表
     *
     * @return
     */
    public List<GiftRewardVO> getRewardGifts() {
        List<GiftDO> gifts = giftService.getRewardGifts();
        if (CollectionUtils.isEmpty(gifts)) {
            return Collections.emptyList();
        }

        Map<Integer, String> walletTypeNameMap = walletService.getWalletTypeNameMap();
        return gifts.stream()
                .filter(Objects::nonNull)
                .filter(gift -> gift.getGiftPrice() != null)
                .filter(gift -> gift.getGiftPriceType() != null)
                .map(gift -> {
                    GiftRewardVO vo = new GiftRewardVO();
                    BeanUtils.copyProperties(gift, vo);
                    vo.setGiftPriceTypeName(walletTypeNameMap.get(WalletTypeEnum.toEnum(gift.getGiftPriceType())));
                    return vo;
                })
                .collect(Collectors.toList());
    }
}
