package com.starbuds.server.api.app.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.user.AdminService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * 管理后台Http请求工具
 *
 * @author aoe
 */
@Slf4j
@Component
public class ManageHttpTools {

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    private AdminService adminService;

    public Result post(String action, Map<String, String> params, String adminPhone) throws IOException {
        if (StringUtils.isBlank(action)) {
            return Result.ofBadRequest("无效的地址");
        }
        if (StringUtils.isBlank(adminPhone)) {
            return Result.ofBadRequest("无效的管理员账号");
        }
        String adminPkg = adminService.getAdminPkg(adminPhone);
        if (StringUtils.isBlank(adminPkg)) {
            return Result.ofBadRequest("无效的管理员");
        }
        String rsp = postByPkg(action, params, adminPkg);
        return JSON.parseObject(rsp, Result.class);
    }

    private String postByPkg(String action, Map<String, String> params, String pkg) throws IOException {
        Map<String, String> headers = new HashMap<>();
        headers.put("pkg", pkg);
        headers.put("Accept", "application/json");
        headers.put("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");

        String url = getManageHost() + action;
        if (isDev()) {
            log.info("官方账号操作管理后台 url={} params={}, headers={}", url, JSON.toJSONString(params, true),
                    JSON.toJSONString(headers, true));
        }
        String res = OkHttpTools.post(url, params, headers);
        JSONObject resJson = JSON.parseObject(res);
        log.info("\n\n 官方账号POST {} \n RESULT \n\n {} \n", url, JSON.toJSONString(resJson, true));
        return res;
    }

    /**
     * 获得管理后台主机地址
     *
     * @return
     */
    private String getManageHost() {
        return releaseConfigService.getCachReleaseConfig("manageHost");
    }
}
