package com.starbuds.server.api.app.util;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import org.apache.commons.lang3.StringUtils;

/**
 * 拼音工具类
 *
 * @author Lay
 * @date 2020/5/15
 */
public class PinyinUtil {

    private static HanyuPinyinOutputFormat getDefaultFormat() {
        HanyuPinyinOutputFormat format = new HanyuPinyinOutputFormat();
        format.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        format.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        format.setVCharType(HanyuPinyinVCharType.WITH_V);
        return format;
    }

    public static String convert(String words) {
        return convert(words, getDefaultFormat(), "", true);
    }

    public static String convert(String words, boolean retain) {
        return convert(words, getDefaultFormat(), "", retain);
    }

    public static String convert(String words, String separate, boolean retain) {
        return convert(words, getDefaultFormat(), separate, retain);
    }

    public static String convert(String words, HanyuPinyinOutputFormat format, boolean retain) {
        return convert(words, format, "", retain);
    }

    public static String convert(String words, HanyuPinyinOutputFormat format, String separate, boolean retain) {
        try {
            return PinyinHelper.toHanYuPinyinString(words, format, separate, retain);
        } catch (Exception e) {
            return words;
        }
    }

    public static String firstCharUpperCase(String words) {
        if (StringUtils.isNotBlank(words)) {
            words = String.valueOf(words.charAt(0)).toUpperCase() + words.substring(1);
        }
        return words;
    }
}
