package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

import javax.ws.rs.FormParam;

/**
 * 活动接口
 *
 * @author Lay
 * @date 2020/2/20
 */
public interface ActivityFacade {

    /**
     * 获取直播间活动
     *
     * @return
     */
    Result getLiveActivities();

    /**
     * 获取语聊房活动
     *
     * @return
     */
    Result getRtcActivities(Long roomId);

    /**
     * 获取抽奖活动
     *
     * @param lotteryId
     * @return
     */
    Result getLottery(String lotteryId);

    /**
     * 抽奖
     *
     * @param lotteryId
     * @param times
     * @return
     */
    Result lottery(String lotteryId, int times);


    /**
     * 获取用户抽奖活动次数
     *
     * @param lotteryId
     * @return
     */
    Result getLotteryUserNum(@FormParam("lotteryId") String lotteryId);


    /**
     * 通过活动ID获取抽奖列表
     *
     * @param activityId
     * @return
     */
    Result getLotteryByProp(Long activityId);

    /**
     * 获取抽奖活动中奖记录
     *
     * @param lotteryId
     * @return
     */
    Result getLotteryNewlyRewards(String lotteryId);

    /**
     * 获取活动中奖纪录
     *
     * @param activityId
     * @return
     */
    Result getActivityNewlyRewards(Long activityId);

    /**
     * 获取抽奖记录
     *
     * @param lotteryId
     * @param pageIndex
     * @return
     */
    Result getLotteryHis(String lotteryId, int pageIndex);

    /**
     * 获取活动抽奖纪录
     *
     * @param activityId
     * @param pageIndex
     * @return
     */
    Result getActivityLotteryHis(Long activityId, int pageIndex);

    /**
     * 获取中奖记录
     *
     * @param hisId
     * @return
     */
    Result getLotteryHisRewards(Long hisId);

    /**
     * 获取抽奖中奖记录
     *
     * @param lotteryId
     * @param pageIndex
     * @return
     */
    Result getLotteryRewardsList(String lotteryId, int pageIndex);

    /**
     * 获取抽奖活动排行榜
     *
     * @param lotteryId
     * @param rankType
     * @return
     */
    Result getLotteryRankList(String lotteryId, int rankType);

    /**
     * 获取抽奖活动中奖排行榜
     *
     * @param lotteryId
     * @param rankType
     * @return
     */
    Result getLotteryRewardRankList(String lotteryId, int rankType);

    /**
     * 获取活动中奖纪录
     *
     * @param activityId
     * @param pageIndex
     * @return
     */
    Result getActivityRewardList(Long activityId, int pageIndex);

    /**
     * 获取充值活动详情
     *
     * @param activityId
     * @return
     */
    Result getRecharge(Long activityId);

    /**
     * 获取奖品清单
     *
     * @param lotteryId
     * @return
     */
    Result getRewardItem(String lotteryId);

    /**
     * 获取活动排名
     *
     * @param activityId
     * @return
     */
    Result getRankTopN(Long activityId);

    /**
     * 获取活动详情
     *
     * @param activityId
     * @return
     */
    Result getInfo(Long activityId);

    /**
     * 获取礼物活动主播排行
     *
     * @param activityId
     * @return
     */
    Result getGiftAnchorRank(Long activityId);

    /**
     * 获取礼物活动用户排行
     *
     * @param activityId
     * @return
     */
    Result getGiftUserRank(Long activityId);

    /**
     * 获取活动达标情况
     *
     * @param collectId
     */
    Result getCollectStatus(String collectId);

    /**
     * 领取奖励
     *
     * @param collectId
     * @return
     */
    Result receiveReward(String collectId);

    /**
     * 获取等级奖励
     *
     * @param scoreType
     * @return
     */
    Result getLevelReward(Integer scoreType);

    /**
     * 参与头条抽奖活动
     *
     * @param configId
     * @param periodId
     * @param content
     * @return
     */
    Result goHeadlineLotteryActivity(long configId, int periodId, String content);

    /**
     * 获取头条活动
     *
     * @param configId
     * @param periodId
     * @return
     */
    Result getHeadlineLotteryActivity(Long configId, Integer periodId);

    /**
     * 获取头条活动中奖历史
     *
     * @return
     */
    Result getHeadlineLotteryWinnerHis(Long activityId);

    /**
     * 获取抢龙活动详情
     *
     * @param activityId
     * @return
     */
    Result getGrabActivity(long activityId);

    /**
     * 参加抢龙活动
     *
     * @param periodId
     * @return
     */
    Result joinGrabActivity(long periodId);

    /**
     * 获取我参与的抢龙记录
     *
     * @param activityId
     * @param pageIndex
     * @return
     */
    Result getGrabPeriodHis(long activityId, int pageIndex);

    /**
     * 获取多礼物活动的礼物列表
     *
     * @param activityId
     * @return
     */
    Result getGiftsByGiftManyActivity(long activityId);
}
