package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/23 14:53
 * @Description 主播api
 */
public interface AnchorFacade {

    /**
     * 发起主播认证申请
     *
     * @param idCardHoldImg
     * @param idCardFaceImg
     * @param idCardNationImg
     * @return
     */
    Result addApply(String idCardHoldImg, String idCardFaceImg, String idCardNationImg);

    /**
     * 获取最近一次认证记录
     *
     * @return
     */
    Result getLastApplyRecord();

    /**
     * 更新主播信息  直播分类、直播标题、颜值封面
     *
     * @param categoryId
     * @param liveTitle
     * @param liveCover
     * @return
     */
    Result editAnchorInfo(Long categoryId,
                          String liveTitle,
                          String liveCover);

    /**
     * 设置粉丝团名称
     *
     * @param fansName
     * @return
     */
    Result setFansName(String fansName);

    /**
     * 添加房管
     *
     * @param userId
     * @return
     */
    Result addMaster(Long userId);

    /**
     * 取消房管
     *
     * @param userId
     * @return
     */
    Result removeMaster(Long userId);

    /**
     * 获取房管列表
     *
     * @return
     */
    Result getMasters();

    /**
     * 禁言用户
     *
     * @param anchorUserId
     * @param targetUserId
     * @param remarks
     * @param optionId
     * @return
     */
    Result forbiddenWords(Long anchorUserId, Long targetUserId, String remarks, Long optionId);

    /**
     * 解除禁言
     *
     * @param anchorUserId
     * @param targetUserId
     * @param remarks
     * @return
     */
    Result enableWords(Long anchorUserId, Long targetUserId, String remarks);

    /**
     * 获取禁言用户列表
     *
     * @return
     */
    Result getForbiddenUsers();

    /**
     * 申请签约公会
     *
     * @param agentId  公会id
     * @param applyMsg 申请时备注
     * @return result对象
     */
    Result applyAgent(Long agentId, String applyMsg);


    /**
     * 申请解约公会
     *
     * @param agentId  公会id
     * @param applyMsg 申请时备注
     * @return result对象
     */
    Result cancelAgent(Long agentId, String applyMsg);


    /**
     * 获取公会列表
     *
     * @param content  搜索内容
     * @param pageNum  当前页码
     * @param pageSize 分页大小
     * @return result对象
     */
    Result getAgentPageList(String content, int pageNum, int pageSize);


    /**
     * 获取公会详情
     *
     * @param agentId 公会id
     * @return result对象
     */
    Result getAgentDetail(Long agentId);

    /**
     * 获取用户最近一次申请信息
     *
     * @return result对象
     */
    Result getLastApplyInfo();

    /**
     * 获取信息
     *
     * @return result对象
     */
    Result getInfo(Long userId);

    /**
     * 获取粉丝团成员列表
     *
     * @param pageIndex
     * @return
     */
    Result getFansList(int pageIndex);

    /**
     * 拉黑
     *
     * @param userId
     * @param anchorUserId
     * @param remarks
     * @return
     */
    Result addBlack(Long userId, Long anchorUserId, String remarks);

    /**
     * 解除拉黑
     *
     * @param userId
     * @return
     */
    Result removeBlack(Long userId);

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @return
     */
    Result getBlackList(int pageIndex);

    /**
     * 封禁主播
     *
     * @param anchorUserId
     * @param duration
     * @param remarks
     * @param reportId
     * @return
     */
    Result disableAnchor(Long anchorUserId, Long duration, String remarks, Long reportId);

    /**
     * 获取pk主播列表
     *
     * @param type
     * @return
     */
    Result getAnchorPkList(int type, int pageIndex);

    /**
     * 获取主播列表
     *
     * @param content
     * @param pageIndex
     * @return
     */
    Result getAnchorList(String content, int pageIndex);

    /**
     * 获取房间
     *
     * @param userId
     * @return
     */
    Result getRoom(Long userId);
}
