package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * 聊天接口
 *
 * @author Lay
 * @date 2020/3/22
 */
public interface ChatFacade {
    /**
     * 发起视频单聊请求
     *
     * @param targetUserId
     * @param matchId
     * @return
     */
    Result sendSingleChatRequest(Long targetUserId, String matchId);

    /**
     * 决定视频单聊请求
     *
     * @param hisId
     * @param decision
     * @return
     */
    Result decideSingleChatRequest(String hisId, int decision);

    /**
     * 取消视频单聊请求
     *
     * @param hisId
     * @return
     */
    Result cancelSingleChatRequest(String hisId);

    /**
     * 中止单聊
     *
     * @param hisId
     * @return
     */
    Result stopSingleChat(String hisId);

    /**
     * 单聊评价
     *
     * @param hisId
     * @param scoreValue
     * @param tagIds
     * @return
     */
    Result evaluateSingleChat(String hisId, Integer scoreValue, String tagIds);

    /**
     * 获取单聊推荐列表
     *
     * @return
     */
    Result getSingleChatSuggestList();

    /**
     * 获取单聊主播列表
     *
     * @param pageIndex
     * @param userSex
     * @param categoryId
     * @return
     */
    Result getSingleChatList(int pageIndex, Integer userSex, Long categoryId);

    /**
     * 获取关注的单聊主播列表
     *
     * @return
     */
    Result getSingleChatFollowList();

    /**
     * 查询单聊信息详情
     *
     * @param hisId
     * @return
     */
    Result getSingleChatHisInfo(String hisId);

    /**
     * 随机获取单聊用户
     *
     * @param userSex
     * @param categoryId
     * @return
     */
    Result getSingleChatRandom(Integer userSex, Long categoryId);

    /**
     * 获取附近单聊用户列表
     *
     * @param pageIndex
     * @param userSex
     * @param userAgeMin
     * @param userAgeMax
     * @param categoryId
     * @param areaId
     * @param isNew
     * @return
     */
    Result getSingleChatNearbyList(int pageIndex, Integer userSex, Integer userAgeMin, Integer userAgeMax, Long categoryId, Integer areaId, int isNew);

    /**
     * 获取单聊主播详情
     *
     * @param userId
     * @return
     */
    Result getSingleChatAnchorInfo(Long userId);

    /**
     * 获取单聊匹配记录
     *
     * @return
     */
    Result getSingleChatMatchedHis();

    /**
     * 刷新单聊提醒
     *
     * @param hisId
     * @return
     */
    Result refreshSingleChatRemind(String hisId);

    /**
     * 获取vip搜索
     *
     * @param pageIndex            当前页码
     * @param userSex              性别
     * @param userAgeMin           最小年龄
     * @param userAgeMax           最大年龄
     * @param singleChatStatus     单聊状态
     * @param areaId               区域id
     * @param distance             距离
     * @param userConstellation    星座
     * @param userMarkingFriends   交友目的
     * @param userMarriage         婚姻状况
     * @param userHobby            兴趣爱好
     * @param userTags             个性标签
     * @param singleChatCategoryId 单聊分类id
     * @return
     */
    Result getVipSingleChatList(int pageIndex,
                                Integer userSex,
                                Integer userAgeMin,
                                Integer userAgeMax,
                                Integer singleChatStatus,
                                Integer areaId,
                                Double distance,
                                Integer userConstellation,
                                String userMarkingFriends,
                                String userMarriage,
                                String userHobby,
                                String userTags,
                                Long singleChatCategoryId);

    /**
     * 发送头条
     *
     * @param msgContent
     * @return
     */
    Result sendHeadline(String msgContent);

    /**
     * 获取头条
     *
     * @return
     */
    Result getHeadline();

    /**
     * 获取头条配置
     *
     * @return
     */
    Result getHeadlineConfig();

    /**
     * 发起语音速配
     *
     * @param userSex
     * @return
     */
    Result sendVoiceMatch(Integer userSex);

    /**
     * 决定语音速配  同意、拒绝
     *
     * @param hisId
     * @param decision
     * @param round
     * @return
     */
    Result decideVoiceMatch(Long hisId, Integer decision, int round);

    /**
     * 暂停语音速配
     *
     * @param hisId
     * @return
     */
    Result cancelVoiceChat(Long hisId);

    /**
     * 结束语音速配聊天
     *
     * @param hisId
     * @return
     */
    Result stopVoiceChat(Long hisId);

    /**
     * 获取语音速配配置
     *
     * @return
     */
    Result getVoiceChatConfig();

    /**
     * 获取语音速配详情
     *
     * @return
     */
    Result getVoiceChatDetail(Long hisId);

    /**
     * 获取语音速配token
     *
     * @param hisId
     * @return
     */
    Result getVoiceChatToken(Long hisId);

    /**
     * 获取语音速配列表
     *
     * @param pageIndex
     * @param hisStatus
     * @return
     */
    Result getVoiceChatList(Integer pageIndex, Integer hisStatus);

    /**
     * 获取匹配成功记录
     *
     * @return
     */
    Result getMatchSuccessList();

    /**
     * 获取符合条件用户
     *
     * @param userSex
     * @return
     */
    Result getMatchUserList(Integer userSex);

    /**
     * 拉取公聊大厅消息记录
     *
     * @param groupId
     * @param pageSize
     * @param lastMsgId
     * @return
     */
    Result getPublicChatRoomMsgHis(String groupId, int pageSize, Integer lastMsgId);
}
