package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/14 16:51
 * @Description 家族接口
 */
public interface FamilyFacade {

    /**
     * 获取家族列表
     *
     * @param content   搜索内容
     * @param pageIndex 当前页码
     * @return result对象
     */
    Result getFamilyList(String content, int pageIndex);

    /**
     * 获取家族详情
     *
     * @param familyId
     * @return
     */
    Result getFamilyDetail(Long familyId);

    /**
     * 添加技能认证
     *
     * @param skillId
     * @param skillTagId
     * @param skillIntro
     * @param images
     * @param voice
     * @param familyId
     * @param duration
     * @return
     */
    Result addSkillAudit(Long skillId, Long skillTagId, String skillIntro, String images, String voice, Long familyId, Long duration);

    /**
     * 提交家族签约/解约流程
     *
     * @param familyId
     * @param applyType
     * @param applyMsg
     * @return
     */
    Result addFamilyAudit(Long familyId, Integer applyType, String applyMsg);

    /**
     * 获取已经认证的技能
     *
     * @param userId
     * @return
     */
    Result getSkillsAuth(Long userId);

    /**
     * 获取已经认证的技能 不包括语音速配
     *
     * @param userId
     * @return
     */
    Result getSkillsAuthIgnoreVoiceMatch(Long userId);

    /**
     * 获取已提交的技能列表
     *
     * @return
     */
    Result getSkillsSubmit();

    /**
     * 技能设置
     *
     * @param skillId
     * @param orderPrice
     * @param orderTime
     * @param orderStatus
     * @return
     */
    Result setSkill(Long skillId, Long orderPrice, String orderTime, Integer orderStatus);

    /**
     * 获取技能设置
     *
     * @param skillId
     * @return
     */
    Result getSkill(Long skillId);

    /**
     * 下单
     *
     * @param receiveUserId
     * @param skillId
     * @param quantity
     * @param remark
     * @param serviceTime
     * @return
     */
    Result addOrder(Long receiveUserId, Long skillId, Integer quantity, String remark, Long serviceTime);

    /**
     * 支付订单
     *
     * @param orderId
     * @return
     */
    Result payOrder(Long orderId);

    /**
     * 取消支付
     *
     * @param orderId
     * @return
     */
    Result cancelPay(Long orderId);

    /**
     * 获取订单详情
     *
     * @param orderId
     * @return
     */
    Result getOrderDetail(Long orderId);

    /**
     * 获取接单记录
     *
     * @param pageIndex
     * @param orderStatus
     * @return
     */
    Result getOrderReceiveList(int pageIndex, Integer orderStatus);

    /**
     * 获取点单记录
     *
     * @param pageIndex
     * @param orderStatus
     * @param payStatus
     * @return
     */
    Result getOrderPayList(int pageIndex, Integer orderStatus, Integer payStatus);

    /**
     * 接收订单
     *
     * @param orderId
     * @return
     */
    Result receiveOrder(Long orderId);

    /**
     * 拒绝订单
     *
     * @param orderId
     * @return
     */
    Result rejectOrder(Long orderId);

    /**
     * 开始服务
     *
     * @param orderId
     * @return
     */
    Result startService(Long orderId);

    /**
     * 确认完成
     *
     * @return
     */
    Result confirmService(Long orderId);

    /**
     * 获取技能用户列表
     *
     * @param pageIndex
     * @param skillId
     * @return
     */
    Result getSkillUserList(int pageIndex, Long skillId);

    /**
     * 获取推荐技能用户列表
     *
     * @param pageIndex
     * @return
     */
    Result getRecommendSkillUserList(int pageIndex);

    /**
     * 获取陪玩头条
     *
     * @return
     */
    Result getPlayTop();

    /**
     * 获取用户技能详情
     *
     * @param skillId
     * @param userId
     * @return
     */
    Result getSkillUserDetail(Long skillId, Long userId);

    /**
     * 获取我的家族以及签约状态
     *
     * @return
     */
    Result getFamily();

    /**
     * 强制解约申请
     *
     * @param applyMsg
     * @return
     */
    Result forceLeave(String applyMsg);

}
