package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.api.app.pojo.AddGameDevOrderForm;
import com.starbuds.server.common.pojo.api.Result;

/**
 * 游戏Dev接口
 *
 * @author aoe
 * @date 2020/9/23
 */
public interface GameDevFacade {

    /**
     * 初始化用户
     *
     * @return
     */
    Result initUser();

    /**
     * 获取游戏列表
     *
     * @return
     */
    Result getGameList();

    /**
     * 获取游戏
     *
     * @param gameId 游戏id
     * @return
     */
    Result getGame(String gameId);

    /**
     * 获取玩家余额
     *
     * @return
     */
    Result getBalance();

    /**
     * 获取指定游戏期号信息
     *
     * @param gameId
     * @return
     */
    Result getGamePlay(String gameId);

    /**
     * 获取开奖历史分页数据
     *
     * @param gameId
     * @param pageIndex 当前页码 从1开始
     * @param pageSize  分页大小
     * @return
     */
    Result getWinNosPageList(String gameId, Integer pageIndex, Integer pageSize);

    /**
     * 开奖
     *
     * @return
     */
    Result getNewWinNo();

    /**
     * 获取指定游戏玩法
     *
     * @param gameId
     * @return
     */
    Result getGamePlayInfo(String gameId);

    /**
     * 进入下一期
     *
     * @param gameId
     * @return
     */
    Result getNextPeriod(String gameId);

    /**
     * 添加游戏币订单(划转资产，转入/转出)
     *
     * @param operate 操作（10:转入;20:转出）
     * @param coin    游戏币数量
     * @return true:成功
     */
    Result addGameCoinOrder(Integer operate, Long coin);

    /**
     * 游戏Dev下单
     *
     * @param form
     * @return
     */
    Result addOrder(AddGameDevOrderForm form);

    /**
     * 获取订单分页列表
     *
     * @param gameId
     * @param status    状态 0全部 1未开奖 2中奖
     * @param pageIndex 当前页码 从1开始
     * @param pageSize  分页大小
     * @return
     */
    Result getOrderPageList(String gameId, Integer status, Integer pageIndex, Integer pageSize);

    /**
     * 游戏订单详情
     *
     * @param orderNo 订单号
     * @return
     */
    Result getOrderDetail(String orderNo);

    /**
     * 订单统计数据（自主游戏、第三方游戏总和）
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     */
    Result getOrderStatistic(Long startTime, Long endTime);

}
