package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * 游戏接口
 *
 * @author Lay
 * @date 2020/9/4
 */
public interface GameFacade {

    /**
     * 获取游戏房间列表
     *
     * @return
     */
    Result getRoomList();

    /**
     * 获取游戏列表
     *
     * @param roomCode   游戏分组id（原：房间号）
     * @param deviceType 设备类型 pc,mobile
     * @return
     */
    Result getGameList(String roomCode, String deviceType);

    /**
     * 获取热门游戏列表
     *
     * @param deviceType 设备类型 pc,mobile
     * @return
     */
    Result getHotGameList(String deviceType);

    /**
     * 进入游戏
     *
     * @param gameCode 游戏代码
     * @return
     */
    Result enterGame(String gameCode);

    /**
     * 添加游戏币订单
     *
     * @param platform 平台代码
     * @param operate  操作（10:转入;20:转出）
     * @param coin     游戏币数量
     * @return true:成功
     */
    Result addGameCoinOrder(String platform, Integer operate, Long coin);

    /**
     * 获得游戏币订单状态
     *
     * @param orderId 订单状态
     * @return
     */
    Result getOrderStatus(Long orderId);

    /**
     * 拉取所有游戏
     *
     * @return
     */
    Result pullAllGames();

    /**
     * 获取游戏大厅链接
     *
     * @return
     */
    Result getHallUrl();

    /**
     * 游戏分享
     *
     * @param params
     * @param gameName
     * @param gameIcon
     * @param targetType
     * @param targetId
     * @return
     */
    Result shareGame(String params, String gameName, String gameIcon, int targetType, Long targetId);
}
