package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.api.app.pojo.gift.SendGiftForm;
import com.starbuds.server.api.app.pojo.gift.SendRtcGiftForm;
import com.starbuds.server.common.pojo.api.Result;

import javax.validation.constraints.NotNull;

/**
 * 礼物接口
 *
 * @author Lay
 * @date 2019/12/31
 */
public interface GiftFacade {
    /**
     * 获取礼物
     *
     * @param giftType
     * @param giftCategory
     * @param giftPriceType
     * @param displayType
     * @return
     */
    Result getGifts(Integer giftType, Integer giftCategory, Integer giftPriceType, int displayType);

    /**
     * 获取礼物
     *
     * @param giftId
     * @return
     */
    Result getGift(Long giftId);

    /**
     * 赠送礼物
     *
     * @param userId
     * @param giftId
     * @param quantity
     * @param isFromBag
     * @param sceneType
     * @param channelName
     * @return
     */
    Result sendGift(Long userId, Long giftId, @NotNull Integer quantity, boolean isFromBag, int sceneType, String channelName);

    /**
     * 赠送 RTC 礼物
     *
     * @param roomId
     * @param userIds
     * @param giftId
     * @param quantity
     * @param isFromBag
     * @return
     */
    Result sendRtcGift(Long roomId, String userIds, Long giftId, @NotNull Integer quantity, boolean isFromBag);

    /**
     * 语音房麦下送礼物
     *
     * @param form
     * @return
     */
    Result sendRtcUnderMaiGift(SendRtcGiftForm form);

    /**
     * 私聊送礼物
     *
     * @param form
     * @return
     */
    Result sendPrivateChatGift(SendGiftForm form);

    /**
     * 赠送语音速配礼物
     *
     * @param hisId
     * @param userId
     * @param giftId
     * @param quantity
     * @param isFromBag
     * @return
     */
    Result sendVoiceGift(Long hisId, Long userId, Long giftId, @NotNull Integer quantity, boolean isFromBag);

    /**
     * 获取周星礼物
     *
     * @return
     */
    Result getWeekGifts();

    /**
     * 获取周星礼物  上期
     *
     * @return
     */
    Result getWeekGiftsPrevious();

    /**
     * 获取真爱榜礼物
     *
     * @return
     */
    Result getTrueLoveGifts();
}
