package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

import javax.validation.constraints.Min;

/**
 * 直播接口
 *
 * @author Lay
 * @date 2019/12/23
 */
public interface LiveFacade {
    /**
     * 开始直播
     *
     * @param liveTitle
     * @param liveCover
     * @param categoryId
     * @param areaId
     * @param productIds
     * @param gameId     游戏id
     * @return
     */
    Result startLive(String liveTitle, String liveCover, Long categoryId, int areaId, String productIds,
                     String gameId);

    /**
     * 下播接口
     *
     * @param liveHisId
     * @return
     */
    Result stopLive(Long liveHisId);

    /**
     * 继续直播
     *
     * @param liveHisId
     * @return
     */
    Result resumeLive(Long liveHisId);

    /**
     * 获取置顶推荐直播
     *
     * @return
     */
    Result getTopSuggest();

    /**
     * 获取热门列表
     *
     * @param lastId
     * @return
     */
    Result getHotList(int lastId);

    /**
     * 获取类目直播列表
     *
     * @param categoryId
     * @return
     */
    Result getListByCategory(Long categoryId, @Min(1) int pageIndex);

    /**
     * 获取关注直播列表
     *
     * @param pageIndex
     * @return
     */
    Result getFollowList(int pageIndex);

    /**
     * 获取附近直播列表
     *
     * @param pageIndex
     * @param areaId
     * @return
     */
    Result getNearByList(int pageIndex, Integer areaId);

    /**
     * 获取直播详情
     *
     * @param userId
     * @return
     */
    Result getLiveInfo(Long userId);

    /**
     * 获取直播间详情（游客）
     *
     * @param userId
     * @return
     */
    Result getLiveInfoLite(Long userId);

    /**
     * 获取日榜数据
     *
     * @param userId
     * @return
     */
    Result getRankDay(Long userId);

    /**
     * 获取周榜数据
     *
     * @param userId
     * @return
     */
    Result getRankWeek(Long userId);

    /**
     * 获取月榜数据
     *
     * @param userId
     * @return
     */
    Result getRankMonth(Long userId);

    /**
     * 获取总榜数据
     *
     * @param userId
     * @return
     */
    Result getRankTotal(Long userId);

    /**
     * 获取贵族
     *
     * @param userId
     * @return
     */
    Result getNobles(Long userId);

    /**
     * 获取粉丝团
     *
     * @param userId
     * @return
     */
    Result getFans(Long userId);

    /**
     * 获取直播记录
     *
     * @param pageIndex
     * @param yearMonth
     * @return
     */
    Result getLiveHisList(int pageIndex, Integer yearMonth);

    /**
     * 分享直播间
     *
     * @param userId
     * @return
     */
    Result shareLive(Long userId);

    /**
     * 获取在线用户
     *
     * @param userId
     * @return
     */
    Result getUsers(Long userId);

    /**
     * 设置展示商品
     *
     * @param productId
     * @return
     */
    Result setShowProduct(Long productId);
}
