package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;

/**
 * RTC 接口
 *
 * @author Lay
 * @date 2020/7/6
 */
public interface RtcFacade {
    /**
     * 创建房间
     *
     * @param roomType
     * @param categoryId
     * @param roomName
     * @param roomCover
     * @param roomHorizontalCover
     * @return
     */
    Result createRoom(Integer roomType, Long categoryId, String roomName, String roomCover, String roomHorizontalCover);

    /**
     * 进入语音房（客户端2.0.0以上版本开始调用）
     *
     * @param roomId
     * @return
     */
    Result enterRoom(Long roomId);

    /**
     * 获取我的房间列表
     *
     * @return
     */
    Result getMyRooms();

    /**
     * 获取房间详情
     *
     * @param roomId
     * @param pass
     * @return
     */
    Result getRoomInfo(Long roomId, String pass);

    /**
     * 刷新房间token
     *
     * @param roomID
     * @return
     */
    Result getRefreshToken(Long roomID);

    /**
     * 设置房间锁密码
     *
     * @param roomId
     * @param pass
     * @return
     */
    Result setRoomLockPass(Long roomId, String pass);

    /**
     * 编辑房间
     *
     * @param roomId
     * @param roomCover
     * @param roomHorizontalCover
     * @param roomName
     * @param roomWelcome
     * @param roomRule
     * @param categoryId
     * @return
     */
    Result editRoom(Long roomId, String roomCover, String roomHorizontalCover, String roomName, String roomWelcome, String roomRule, Long categoryId);

    /**
     * 获取房管列表
     *
     * @param roomId
     * @param roomRole
     * @return
     */
    Result getRoomMangers(Long roomId, Integer roomRole);

    /**
     * 添加房间主持人
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result addRoomHost(Long roomId, Long userId);

    /**
     * 移除房间主持人
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result removeRoomHost(Long roomId, Long userId);

    /**
     * 添加房管
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result addRoomMaster(Long roomId, Long userId);

    /**
     * 移除房管
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result removeRoomMaster(Long roomId, Long userId);

    /**
     * 申请上麦
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result sitSeat(Long roomId, Integer seatNo);

    /**
     * 下麦
     *
     * @param roomId
     * @return
     */
    Result leaveSeat(Long roomId);

    /**
     * 离开麦序队列
     *
     * @param roomId
     * @return
     */
    Result leaveQueue(Long roomId);

    /**
     * 获取麦序用户列表
     *
     * @param roomId
     * @return
     */
    Result getQueueUsers(Long roomId);

    /**
     * 获取麦序用户数量
     *
     * @param roomId
     * @return
     */
    Result getQueueUsersCount(Long roomId);

    /**
     * 获取麦位信息
     *
     * @param roomId
     * @return
     */
    Result getSeats(Long roomId);

    /**
     * 从麦序移除用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result removeQueueUser(Long roomId, Long userId);

    /**
     * 抱用户上麦
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result setUserToSeat(Long roomId, Long userId, Integer seatNo);

    /**
     * 移除麦位用户
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result removeSeatUser(Long roomId, int seatNo);

    /**
     * 快速上麦
     *
     * @param roomId
     * @return
     */
    Result setUserToSeatInQueue(Long roomId);

    /**
     * 锁定麦位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result lockSeat(Long roomId, Integer seatNo);

    /**
     * 解锁麦位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result unlockSeat(Long roomId, Integer seatNo);

    /**
     * 麦位禁麦
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result setMicDisabled(Long roomId, int seatNo);

    /**
     * 麦位开麦
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result setMicEnabled(Long roomId, int seatNo);

    /**
     * 随机获取相亲房
     *
     * @return
     */
    Result getRandomLoveRoom();

    /**
     * 随机获取拍卖房
     *
     * @return
     */
    Result getRandomAuctionRoom();

    /**
     * 获取房间列表
     *
     * @param pageIndex
     * @param categoryId
     * @return
     */
    Result getRoomList(int pageIndex, Long categoryId, Integer businessType);


    /**
     * 获取首页房间
     *
     * @return
     */
    Result getMainPageRoomList();

    /**
     * 获取交友房间列表
     *
     * @param pageIndex
     * @return
     */
    Result getMakeFriendRoomList(int pageIndex);

    /**
     * 收藏房间
     *
     * @param roomId
     * @return
     */
    Result addRoomFav(Long roomId);

    /**
     * 取消收藏房间
     *
     * @param roomId
     * @return
     */
    Result removeRoomFav(Long roomId);

    /**
     * 获取收藏的房间列表
     *
     * @param pageIndex
     * @return
     */
    Result getFavRoomList(int pageIndex);

    /**
     * 设置上麦模式
     *
     * @param roomId
     * @param micFree
     * @return
     */
    Result setMicFree(Long roomId, int micFree);

    /**
     * 设置心动值开关
     *
     * @param roomId
     * @param scoreEnabled
     * @return
     */
    Result setScoreEnabled(Long roomId, int scoreEnabled);

    /**
     * 设置公屏开关
     *
     * @param roomId
     * @param chatEnabled
     * @return
     */
    Result setChatEnabled(Long roomId, int chatEnabled);

    /**
     * 设置大冒险开关
     *
     * @param roomId
     * @param bigAdventureEnabled
     * @return
     */
    Result setBigAdventureEnabled(Long roomId, int bigAdventureEnabled);

    /**
     * 设置麦克风静音状态
     *
     * @param roomId
     * @param micSilenced
     * @return
     */
    Result setMicSilenced(Long roomId, int micSilenced);

    /**
     * 设置房间背景
     *
     * @param roomId
     * @param backgroundId
     * @return
     */
    Result setRoomBackground(Long roomId, Integer backgroundId);

    /**
     * 获取星钻日榜
     *
     * @param roomId
     * @return
     */
    Result getDiamondRankDay(Long roomId);

    /**
     * 获取星钻周榜
     *
     * @param roomId
     * @return
     */
    Result getDiamondRankWeek(Long roomId);

    /**
     * 获取星钻月榜
     *
     * @param roomId
     * @return
     */
    Result getDiamondRankMonth(Long roomId);

    /**
     * 获取星尘日榜
     *
     * @param roomId
     * @return
     */
    Result getDustRankDay(Long roomId);

    /**
     * 获取星尘周榜
     *
     * @param roomId
     * @return
     */
    Result getDustRankWeek(Long roomId);

    /**
     * 获取星尘月榜
     *
     * @param roomId
     * @return
     */
    Result getDustRankMonth(Long roomId);

    /**
     * 获取星钻往期日榜
     *
     * @param roomId
     * @return
     */
    Result getDiamondRankDayPrevious(Long roomId);

    /**
     * 获取星钻往期周榜
     *
     * @param roomId
     * @return
     */
    Result getDiamondRankWeekPrevious(Long roomId);

    /**
     * 获取星钻往期月榜
     *
     * @param roomId
     * @return
     */
    Result getDiamondRankMonthPrevious(Long roomId);

    /**
     * 获取星尘往期日榜
     *
     * @param roomId
     * @return
     */
    Result getDustRankDayPrevious(Long roomId);

    /**
     * 获取星尘往期周榜
     *
     * @param roomId
     * @return
     */
    Result getDustRankWeekPrevious(Long roomId);

    /**
     * 获取星尘往期月榜
     *
     * @param roomId
     * @return
     */
    Result getDustRankMonthPrevious(Long roomId);

    /**
     * 设置禁言
     *
     * @param roomId
     * @param userId
     * @param duration
     * @param remarks
     * @return
     */
    Result setForbidden(Long roomId, Long userId, Long duration, String remarks);

    /**
     * 移除禁言
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result removeForbidden(Long roomId, Long userId);

    /**
     * 设置黑名单
     *
     * @param roomId
     * @param userId
     * @param duration
     * @param blackRemarks
     * @return
     */
    Result setBlack(Long roomId, Long userId, Long duration, String blackRemarks);

    /**
     * 移除黑名单
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result removeBlack(Long roomId, Long userId);

    /**
     * 获取禁言列表
     *
     * @param pageIndex
     * @param roomId
     * @return
     */
    Result getForbiddenList(int pageIndex, Long roomId);

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @param roomId
     * @param userNo
     * @return
     */
    Result getBlackList(int pageIndex, Long roomId, Long userNo);

    /**
     * 获取管理数量
     *
     * @param roomId
     * @return
     */
    Result getMasterNum(Long roomId);

    /**
     * 获取在线用户
     *
     * @param roomId
     * @return
     */
    Result getOnlineUser(Long roomId);

    /**
     * 获取在线用户-贵族
     *
     * @param roomId
     * @return
     */
    Result getOnlineUserNoble(Long roomId);

    /**
     * 清空心动值
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result clearScore(Long roomId, Long userId);

    /**
     * 设置老板位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result setBossSeat(Long roomId, Integer seatNo);

    /**
     * 移除老板位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    Result removeBossSeat(Long roomId, Integer seatNo);

    /**
     * 设置倒计时
     *
     * @param roomId
     * @param seatNo
     * @param seconds
     * @return
     */
    Result setCountDown(Long roomId, Integer seatNo, Integer seconds);

    /**
     * 获取心动值列表
     *
     * @param roomId
     * @return
     */
    Result getSeatScoreList(Long roomId);

    /**
     * 获取用户心动值列表
     *
     * @param roomId
     * @return
     */
    Result getUserScoreList(Long roomId);

    /**
     * 获取授权语聊配置
     *
     * @return
     */
    Result getAuthRtcConfig();

    /**
     * 获取授权信息
     *
     * @return
     */
    Result getRtcAuthInfo(Long roomId);

    /**
     * 相亲模式下一阶段
     *
     * @param roomId
     * @return
     */
    Result nextLoveStage(Long roomId);

    /**
     * 相亲模式阶段重置
     *
     * @param roomId
     * @return
     */
    Result resetLoveStage(Long roomId);

    /**
     * 心动选人
     *
     * @param roomId
     * @param targetSeatNo
     * @return
     */
    Result chooseLove(Long roomId, int targetSeatNo);

    /**
     * 获取热门房间列表
     *
     * @return
     */
    Result getHotRoomList(Integer businessType);

    /**
     * 发送表情
     *
     * @param roomId
     * @param expId
     * @return
     */
    Result sendFacialExpression(Long roomId, Integer expId);

    /**
     * 获取表情列表
     *
     * @param roomId
     * @return
     */
    Result getFacialExpressionList(Long roomId);

    /**
     * 获取房间日流水
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @return
     */
    Result getRtcRoomDayStream(Long roomId, Long startTime, Long endTime);

    /**
     * 获取房间小时流水
     *
     * @param roomId
     * @param date
     * @return
     */
    Result getRtcRoomHourStream(Long roomId, Long date);

    /**
     * 获取房间周流水
     *
     * @param roomId
     * @return
     */
    Result getRtcRoomWeekStream(Long roomId);

    /**
     * 获取房间月流水
     *
     * @param roomId
     * @return
     */
    Result getRtcRoomMonthStream(Long roomId);

    /**
     * 获取心动值助攻用户列表
     *
     * @param roomId
     * @param userId
     * @return
     */
    Result getRtcScoreAssisterList(Long roomId, Long userId);

    /**
     * 召集粉丝
     * 1、在语音房麦位上，给所有粉丝一键发私信，召集粉丝来语音房；
     * 2、每次至少间隔1小时，每天三次；
     * 3、私信内容：我正在xxx（语音房名称）房间，快来和我一起玩耍吧！！！
     * 4、点击私信链接，可以直接跳转到语音房。
     *
     * @param roomId
     * @return
     */
    Result conveneFans(Long roomId);

    /**
     * 获取房间昨日钻石MVP
     *
     * @param roomId
     * @return
     */
    Result getYesterdayDiamondRankMVP(Long roomId);

    /**
     * 设置转盘配置
     *
     * @param roomId
     * @param turntableConfig
     * @return
     */
    Result setTurntableConfig(Long roomId, String turntableConfig);

    /**
     * 获取转盘配置
     *
     * @param roomId
     * @return
     */
    Result getTurntableConfig(Long roomId);

    /**
     * 转盘抽奖
     *
     * @param roomId
     * @return
     */
    Result turntableLottery(Long roomId);

    /**
     * 开启房间内 PK
     *
     * @param roomId
     * @param topic
     * @param duration
     * @return
     */
    Result startBattleRoomIn(Long roomId, String topic, long duration);

    /**
     * 提前结束 PK
     *
     * @param roomId
     * @return
     */
    Result finishBattleRoomIn(Long roomId);

    /**
     * 获取 PK 记录
     *
     * @param roomId
     * @param pageIndex
     * @return
     */
    Result getBattleRoomInHis(Long roomId, int pageIndex);

    /**
     * 获取 跨房PK 房间是否收听状态
     *
     * @return
     */
    Result setBattleRoomCrossListenStatus(Long roomId, Integer enableListen);

    /**
     * 开启咵房 PK
     *
     * @param roomId
     * @param targetRoomId
     * @param topic
     * @param duration
     * @return
     */
    Result startBattleRoomCross(Long roomId, Long targetRoomId, String topic, long duration);

    /**
     * 获取 跨房PK 房间状态
     *
     * @return
     */
    Result getBattleRoomCrossStatus(Long roomId);

    /**
     * 获取跨房pk列表
     *
     * @return
     */
    Result getBattleRoomCrossList(Integer pageIndex, Integer pageSize, String keyword, Long roomId);

    /**
     * 取消跨房 PK
     *
     * @param roomId
     * @return
     */
    Result cancelBattleRoomCross(Long roomId);

    /**
     * 决定跨房 PK
     *
     * @param roomId
     * @param battleId
     * @param decision
     * @return
     */
    Result decideBattleRoomCross(Long roomId, Long battleId, int decision);

    /**
     * 提前结束 PK
     *
     * @param roomId
     * @return
     */
    Result finishBattleRoomCross(Long roomId);

    /**
     * 获取 PK 记录
     *
     * @param roomId
     * @param pageIndex
     * @return
     */
    Result getBattleRoomCrossHis(Long roomId, int pageIndex);

    /**
     * 设置跨房PK随机匹配开关
     *
     * @param roomId
     * @param enabled
     * @return
     */
    Result setBattleCrossMatchEnabled(Long roomId, int enabled);

    /**
     * 开播
     *
     * @param roomId
     * @return
     */
    Result startLive(Long roomId);

    /**
     * 关播
     */
    Result stopLive(Long roomId);


    /**
     * 获取用户房间历史
     *
     * @return
     */
    public Result getRtcRoomHisList();

    /**
     * 获取用户房间历史分页
     *
     * @return
     */
    Result getRtcRoomHisPageList(Integer pageIndex, Integer pageSize);

    /**
     * 房间底部菜单入口
     *
     * @return
     */
    Result getBottomMenuEntry();
}
