package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * 拍卖房接口
 *
 * @author aoe
 * @date 2021/7/27
 */
public interface RtcRoomAuctionFacade {

    /**
     * 获取拍卖房起拍设置的相关数据
     *
     * @return
     */
    Result getStartAuctionSettingData();

    /**
     * 设置拍卖信息(设置成功后竞拍开始)
     *
     * @param roomId        拍卖房id
     * @param watchConfigId 拍卖物品（守护）id
     * @param watchDuration 守护时长
     * @param startGiftId   起拍礼物id
     * @return
     */
    Result setAuction(Long roomId, Long watchConfigId, Integer watchDuration, Long startGiftId);

    /**
     * 结束竞拍
     *
     * @param roomId
     * @return
     */
    Result endAuction(Long roomId);

    /**
     * 申请上麦-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    Result sitSeatInAuction(Long roomId, Integer type);

    /**
     * 获取麦序用户列表-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    Result getQueueUsersInAuction(Long roomId, Integer type);


    /**
     * 麦序移除-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @param userId 用户ID为空, 移除所有
     * @return
     */
    Result removeQueueUserInAuction(Long roomId, Integer type, Long userId);


    /**
     * 抱用户上麦-拍卖房
     *
     * @param roomId
     * @param userId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    Result setUserToSeatInAuction(Long roomId, Long userId, Integer type);


    /**
     * 离开麦序队列-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    Result leaveQueueInAuction(Long roomId, Integer type);

    /**
     * 快速上麦-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    Result setUserToSeatInQueueInAuction(Long roomId, Integer type);

}
