package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/20 17:41
 * @Description 社交接口
 */
public interface SocialFacade {

    /**
     * 关注用户
     *
     * @param targetId 用户id
     * @return result对象
     */
    Result followUser(Long targetId);

    /**
     * 取消关注用户
     *
     * @param targetId 用户id
     * @return result对象
     */
    Result unFollowUser(Long targetId);

    /**
     * 添加好友请求
     *
     * @param targetId 用户id
     * @param reqMsg   添加好友时的验证消息
     * @return result对象
     */
    Result addFriendReq(Long targetId, String reqMsg);

    /**
     * 同意好友请求
     *
     * @param reqId  好友请求id
     * @param optMsg 处理意见
     * @return result对象
     */
    Result agreeFriendReq(Long reqId, String optMsg);

    /**
     * 拒绝好友申请
     *
     * @param reqId  好友请求id
     * @param optMsg 处理意见
     * @return result对象
     */
    Result denyFriendReq(Long reqId, String optMsg);

    /**
     * 删除好友
     *
     * @param targetId 用户id
     * @return result对象
     */
    Result delFriend(Long targetId);

    /**
     * 获取用户关注列表
     *
     * @param pageIndex 当前页码
     * @param userId
     * @return result对象
     */
    Result getFollowList(int pageIndex, Long userId);

    /**
     * 获取用户粉丝列表
     *
     * @param pageIndex 当前页码
     * @param userId
     * @return result对象
     */
    Result getFansList(int pageIndex, Long userId);

    /**
     * 获取用户好友列表
     *
     * @param pageIndex 当前页码
     * @param userId
     * @return result对象
     */
    Result getFriendList(int pageIndex, Long userId);

    /**
     * 获取好友申请列表
     *
     * @param pageIndex 当前页码
     * @return result对象
     */
    Result getFriendReqList(int pageIndex);

    /**
     * 获取访客列表
     *
     * @param pageIndex
     * @return
     */
    Result getVisitorList(int pageIndex);

    /**
     * 获取足迹列表
     *
     * @param pageIndex
     * @return
     */
    Result getIntervieweeList(int pageIndex);

    /**
     * 最近看过的用户列表
     *
     * @return
     */
    Result getRecentlySeenUsers();

    /**
     * 获取访客列表
     *
     * @param pageIndex
     * @return
     */
    Result getVisitorListV2(Integer pageIndex, Integer pageSize);

    /**
     * 获取足迹列表
     *
     * @param pageIndex
     * @return
     */
    Result getIntervieweeListV2(Integer pageIndex, Integer pageSize);

    /**
     * 谁看过我总数
     *
     * @return
     */
    Result getIntervieweeTotal();

    /**
     * 未读访客消息数
     *
     * @return
     */
    Result getIntervieweeUnReadMsgCount();

    /**
     * 添加黑名单
     *
     * @param userId
     * @return
     */
    Result addBlack(Long userId);

    /**
     * 删除黑名单
     *
     * @param userId
     * @return
     */
    Result delBlack(Long userId);

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @param lastSeq
     * @return
     */
    Result getBlackList(int pageIndex, Integer lastSeq);

    /**
     * 获取附近的人
     *
     * @param
     * @return
     */
    Result getPeopleNearby();

    /**
     * 获取附近的人分页数据
     *
     * @param pageIndex  当前页码
     * @param userSex    用户性别 1：男 2：女
     * @param userAgeMin 最小年龄
     * @param userAgeMax 最大年龄
     * @return
     */
    Result getPeopleNearbyPageList(int pageIndex, Integer userSex, Integer userAgeMin, Integer userAgeMax);

    /**
     * 获得搜索默认关键词
     *
     * @return 搜索默认关键词
     */
    Result getSearchDefaultKeywords();

    /**
     * 猜你喜欢列表
     *
     * @return
     */
    Result getMayLike();
}
