package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.api.app.pojo.DataFeedbackReq;
import com.starbuds.server.api.app.pojo.QuestionAnswerReq;
import com.starbuds.server.api.app.pojo.UserReq;
import com.starbuds.server.api.app.pojo.UserSwitchReq;
import com.starbuds.server.common.pojo.api.Result;

import javax.ws.rs.QueryParam;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/18 17:30
 * @Description 用户接口
 */
public interface UserFacade {

    /**
     * 修改个人资料
     *
     * @param userName   用户姓名
     * @param userSex    用户性别
     * @param userAvatar 用户头像
     * @param userAge    用户年龄
     * @param countryId  国家
     * @param areaId     区域id
     * @param userSign   个性签名
     * @param userCover  颜值封面
     * @return result对象
     */
    Result editProfile(String userName,
                       Integer userSex,
                       String userAvatar,
                       Integer userAge,
                       Integer countryId,
                       Integer areaId,
                       String userSign,
                       String userCover,
                       String userTags,
                       String userHobby,
                       String userMarriage,
                       String userMarkingFriends,
                       String userHeight,
                       String userWeight,
                       String userIncome,
                       Long userBornTime,
                       Integer userConstellation);


    /**
     * 编辑用户资料
     *
     * @param userReq
     * @return
     */
    Result editUserNew(UserReq userReq);

    /**
     * 修改昵称
     *
     * @param userName
     * @return
     */
    Result setName(String userName);

    /**
     * 获取用户资料
     *
     * @param userId 用户id
     * @return result对象
     */
    Result getProfile(Long userId);

    /**
     * 获取用户资料
     *
     * @param userId 用户id
     * @return result对象
     */
    Result getProfileNew(Long userId);


    /**
     * 检查用户是否已经出发首充
     *
     * @param userId
     * @return
     */
    Result getUserFirstPayCheck(@QueryParam("userId") Long userId);

    /**
     * 获取用户资料分数
     *
     * @param userId
     * @return
     */
    Result getProfileScore(Long userId);

    /**
     * 举报用户
     *
     * @param userId
     * @param roomId
     * @param reasonId
     * @param remarks
     * @param images
     * @return
     */
    Result report(Long userId, Long roomId, Long reasonId, String remarks, String images);

    /**
     * 设置开关
     *
     * @param switchReq
     * @return
     */
    Result setSwitch(UserSwitchReq switchReq);

    /**
     * 获取用户开关
     *
     * @return
     */
    Result getUserSwitch();

    /**
     * 获取用户开关
     *
     * @param userId
     * @return
     */
    Result getSwitchList(Long userId);

    /**
     * 提交意见
     *
     * @param feedbackReq
     * @return
     */
    Result submitSuggest(DataFeedbackReq feedbackReq);

    /**
     * 获取意见反馈详情
     *
     * @param feedId
     * @return
     */
    Result getSuggestDetail(Long feedId);

    /**
     * 获取我的意见反馈列表
     *
     * @param feedType
     * @param handleStatus
     * @param pageNum
     * @param pageSize
     * @return
     */
    Result getSuggestPageList(Integer feedType, Integer handleStatus, int pageNum, int pageSize);

    /**
     * 获取积分等级详情
     *
     * @param scoreType
     * @return
     */
    Result getScoreLevelInfo(int scoreType);

    /**
     * 获取绑定账户
     *
     * @return
     */
    Result getBingAccount();

    /**
     * 绑定oauth
     *
     * @param oauthType
     * @param code
     * @return
     */
    Result bindOauth(Integer oauthType, String code);

    /**
     * 绑定手机号
     *
     * @param userPhone
     * @param code
     * @param countryId
     * @return
     */
    Result bindPhone(String userPhone, String code, Integer countryId);

    /**
     * 绑定支付宝
     *
     * @param code
     * @param alipayAccount
     * @return
     */
    Result bindAlipay(String code, String alipayAccount);

    /**
     * 获取支付宝授权绑定参数
     *
     * @return
     */
    Result getAlipayBindParam();

    /**
     * 获取绑定的支付宝账户
     *
     * @return
     */
    Result getAlipay();

    /**
     * 获取邀请码二维码
     *
     * @param userId
     * @return
     */
    Result getInviteQrCode(Long userId);

    /**
     * 获取手机号
     *
     * @return
     */
    Result getPhone();

    /**
     * 申请注销
     *
     * @return
     */
    Result applyLogout();

    /**
     * 取消注销
     *
     * @return
     */
    Result cancelLogout();

    /**
     * 获取用户
     *
     * @param params
     * @return
     */
    Result getUserByUserNoOrNiceNo(Long params);

    /**
     * 获取邀请用户列表
     *
     * @param pageIndex
     * @param hasRecharged
     * @return
     */
    Result getInviteList(int pageIndex, Integer hasRecharged);

    /**
     * code 换取 openId
     *
     * @param code
     * @return
     */
    Result getOpenId(String code);

    /**
     * 获取礼物墙
     *
     * @param userId
     * @return
     */
    Result getGiftWallList(Long userId);

    /**
     * 获取周礼物列表
     *
     * @param userId
     * @return
     */
    Result getWeekGiftWallList(Long userId);

    /**
     * 获取礼物墙统计
     *
     * @param userId
     * @return
     */
    Result getGiftWallStat(Long userId);

    /**
     * 获取等级
     *
     * @param scoreType
     * @return
     */
    Result getLevelAndScore(Integer scoreType);

    /**
     * 获取用户贵族
     *
     * @return
     */
    Result getUserNobleLevel();

    /**
     * 获取用户模块
     *
     * @return
     */
    Result getUserModules();

    /**
     * 设置自问自答
     *
     * @param answerReq
     * @return
     */
    Result setQuestionAnswer(QuestionAnswerReq answerReq);

    /**
     * 删除自问自答
     *
     * @param questionId
     * @return
     */
    Result delQuestionAnswer(Long questionId);

    /**
     * 获取用户简单信息
     *
     * @param userIds
     * @return
     */
    Result getSimpleInfo(String userIds);

    /**
     * 获取用户IM信息
     *
     * @param userId
     * @return
     */
    Result getImInfo(String userId);
}
