package com.starbuds.server.api.app.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * 钱包接口
 *
 * @author Lay
 * @date 2019/12/23
 */
public interface WalletFacade {
    /**
     * 查询用户钱包
     *
     * @return
     */
    Result getUserWallet();

    /**
     * 查询用户虚拟币钱包
     *
     * @return
     */
    Result getUserVirtualWallet();

    /**
     * 获取用户金币明细
     *
     * @param lastId
     * @param yearMonth
     * @param isIncome
     * @param walletType
     * @return
     */
    Result getUserCoinHisList(Long lastId, Integer yearMonth, Integer isIncome, Integer walletType);

    /**
     * 获取用户金币统计
     *
     * @param yearMonth
     * @param isIncome
     * @return
     */
    Result getUserCoinHisSum(Integer yearMonth, Integer isIncome, Integer walletType);

    /**
     * 申请提现
     *
     * @param amount
     * @return
     */
    Result applyWithdraw(Long amount);

    /**
     * 申请提现(星尘)
     * 使用 申请提现(粉钻) 代替，只能提取粉钻
     *
     * @param amount
     * @param cardId
     * @return
     */
    @Deprecated
    Result applyWithdrawByStarDust(Long amount, String cardId);

    /**
     * 申请提现(粉钻)
     *
     * @param amount
     * @param cardId 银行id
     * @return
     */
    Result applyWithdrawByPinkDiamond(Long amount, String cardId);

    /**
     * 获取提现记录列表
     *
     * @param pageIndex
     * @param yearMonth
     * @param applyStatus
     * @param walletType
     * @return
     */
    Result getWithdrawHisList(int pageIndex, Integer yearMonth, Integer applyStatus, Integer walletType);

    /**
     * 获取时间钱包
     *
     * @return
     */
    Result getUserChatTimeWallet();

    /**
     * 用户是否实名认证
     *
     * @return
     */
    Result getAuthInfo();

    /**
     * 添加身份证
     *
     * @param faceImg
     * @param nationImg
     * @return
     */
    Result addId(String faceImg, String nationImg);

    /**
     * 银⾏卡信息查询接⼝
     *
     * @param cardNo
     * @param bankName
     * @return
     */
    Result queryCard(String cardNo, String bankName);

    /**
     * 银⾏卡四要素请求鉴权（下发短信验证码）
     *
     * @param cardNo
     * @param mobile
     * @return
     */
    Result verifyCard(String cardNo, String mobile);

    /**
     * 银⾏卡四要素确认鉴权（上传短信验证码）
     *
     * @param cardNo
     * @param mobile
     * @param captcha
     * @param ref
     * @return
     */
    Result verifyConfirm(String cardNo, String mobile, String captcha, String ref);

    /**
     * 获取银行卡列表
     *
     * @return
     */
    Result getBankcardList();

    /**
     * 解绑卡号
     *
     * @param cardId
     * @return
     */
    Result unBindCard(String cardId);

    /**
     * 转星钻
     *
     * @param walletType
     * @param amount
     * @return
     */
    Result transDiamond(int walletType, Long amount);

    /**
     * 兑换钻石（粉钻-不可提现、粉钻-可提现 兑换 星钻）
     *
     * @param total 兑换总额
     * @return
     */
    Result transDustPinkToDiamond(Long total);

    /**
     * 转游戏币
     *
     * @param walletType
     * @param amount
     * @return
     */
    Result transGameCoin(int walletType, Long amount);

    /**
     * 查询转换记录
     *
     * @param lastId
     * @param fromWalletType
     * @param destWalletType
     * @param yearMonth
     * @return
     */
    Result getTransferHisList(Long lastId, int fromWalletType, int destWalletType, Integer yearMonth);

    /**
     * 获取游戏余额
     *
     * @param platform
     * @return
     */
    Result getGameBalance(String platform);

    /**
     * 获取游戏余额-所有平台
     *
     * @return
     */
    Result getGameBalanceAll();

    /**
     * 一键回收所有游戏币
     *
     * @return
     */
    Result recyclingAllGameCoin();

    /**
     * 游戏币兑换星钻
     *
     * @param coin
     * @return
     */
    Result transGameCoinToDiamond(Long coin);

    /**
     * 获取游戏余额-Dev接口平台
     *
     * @return
     */
    Result getGameDevBalance();

    /**
     * 获粉钻提现金额选项
     *
     * @return
     */
    Result getWithdrawPinkList();
}
