package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.activity.ActivityInfluentialRankInfo;
import com.starbuds.server.api.app.v1.facade.ActivityAnniversary;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.activity.MedalPersonRankVO;
import com.starbuds.server.common.pojo.activity.bo.AnnualGalaPeakFamilyRankBO;
import com.starbuds.server.common.pojo.activity.vo.*;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.common.LevelScoreVO;
import com.starbuds.server.common.pojo.db.ActivityAnnualAwardConfigDO;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.db.DataLevelScoreDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.message.im.activity.ActivityMedalLevelUpLevelMsg;
import com.starbuds.server.common.pojo.message.im.activity.AnnualGalaFinishMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.awt.*;
import java.math.BigDecimal;
import java.util.*;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 活动周年庆接口实现
 *
 * @author lujiao
 * @date 2021/12/7 14:39
 */
@Path("/v1/activity")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ActivityAnniversaryImpl extends BaseFacade implements ActivityAnniversary {

    @Reference(version = "1.0.0")
    private ActivityAnniversaryService activityService;

    @Reference(version = "1.0.0")
    private ActivityService actService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private DataService dataService;

    @Reference(version = "1.0.0")
    private ScoreService scoreService;

    @Reference(version = "1.0.0")
    private FamilyService familyService;

    @Reference(version = "1.0.0")
    ImService imService;

    /**
     * 魅力花房排行
     *
     * @param activityId
     * @return
     */
    @Override
    @POST
    @Path("getFlowerRoomRank")
    public Result getFlowerRoomRank(@FormParam("activityId") Long activityId, @FormParam("subActivityId") Long subActivityId, @FormParam("needDanger") Integer needDanger) {

        List<ActivityAnniversaryRankVO> res = activityService.getFlowerRoomRankList(activityId, subActivityId, needDanger);
        return Result.ofList(res);
    }

    /**
     * 获取活动信息
     *
     * @param activityId
     * @return
     */
    @Override
    @POST
    @Path("getActivityInfo")
    public Result getActivityInfo(@FormParam("activityId") Long activityId) {
        ActivityDO activityDO = actService.getActivity(activityId);
        if (activityDO != null) {
            Map<String, String> res = new HashMap<>();
            res.put("beginTime", String.valueOf(activityDO.getActivityBeginTime()));
            res.put("endTime", String.valueOf(activityDO.getActivityEndTime()));
            res.put("activityName", activityDO.getActivityName());
            return Result.ofSuccess(res);
        }
        return Result.ofBadRequest("未找到该活动");
    }

    /**
     * 魅力花房活动入口信息
     *
     * @param activityId
     * @return
     */
    @Override
    @POST
    @Path("getFlowerRoomRankInfo")
    public Result getFlowerRoomRankInfo(@FormParam("activityId") Long activityId, @FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();
        ActivityJoinRankVO joinInfo = activityService.getFlowerRoomRank(userId, activityId, roomId);
        return Result.ofSuccess(joinInfo);
    }

    /**
     * 我的魅力花房排行信息
     *
     * @param activityId
     * @param roomId
     * @return
     */
    @Override
    @POST
    @Path("getMyselfFlowerRoomRank")
    public Result getMyselfFlowerRoomRank(@FormParam("activityId") Long activityId, @FormParam("roomId") Long roomId) {
        ActivityFlowRoomRankInfoVO rankInfo = activityService.getMyselfFlowerRoomRank(activityId, roomId);
        return Result.ofSuccess(rankInfo);
    }


    /**
     * 风云人物/魅力人物入口信息
     *
     * @param activityId
     * @return
     */
    @Override
    @POST
    @Path("getInfluentialRankInfo")
    public Result getInfluentialRankInfo(@FormParam("activityId") Long activityId, @FormParam("rankType") Integer rankType) {
        Long userId = this.getToken().getUserId();
        ActivityJoinRankVO joinInfo = activityService.getInfluentialRankInfo(userId, activityId, rankType);
        return Result.ofSuccess(joinInfo);
    }

    /**
     * 风云人物/魅力人物排行榜
     *
     * @param activityId
     * @param rankType
     * @return
     */
    @Override
    @POST
    @Path("getInfluentialRank")
    public Result getInfluentialRank(@FormParam("activityId") Long activityId, @FormParam("rankType") Integer rankType) {
        List<ActivityAnniversaryRankVO> res = activityService.getInfluentialRankList(activityId, rankType);
        return Result.ofSuccess(res);
    }

    /**
     * 影响力排行榜
     *
     * @param activityId
     * @return
     */
    @Override
    @POST
    @Path("getMyInfluentialRank")
    public Result getMyInfluentialRank(@FormParam("activityId") Long activityId, @FormParam("rankType") Integer rankType) {
        ActivityInfluentialRankInfo info = new ActivityInfluentialRankInfo();
        Long userId = this.getToken().getUserId();
        UserDO userDO = userService.getUserById(userId);
        info.setUserName(userDO.getUserName());
        info.setRank(activityService.getMyInfluentialRank(activityId, userId, rankType));
        info.setScore(activityService.getUserScore(activityId, userId, rankType));
        return Result.ofSuccess(info);
    }

    /**
     * 巅峰家族排行榜
     *
     * @param activityId
     * @return
     */
    @Override
    @Path("getPeakFamilyRank")
    @GET
    public Result getPeakFamilyRank(@QueryParam("activityId") Long activityId) {

        if (activityId == null) {
            return Result.ofOperationFailed("activityId不能为空");
        }

        List<ActivityAnniversaryRankVO> list = new ArrayList<>();

        List<AnnualGalaPeakFamilyRankBO> familyRankList = activityService.getPeakFamilyRankList(activityId);

        if (familyRankList == null) {
            return Result.ofSuccess();
        }

        List<AnnualGalaPeakFamilyRankBO> familySortedRankList = familyRankList.stream().sorted(Comparator.comparing(AnnualGalaPeakFamilyRankBO::getScore).reversed()).collect(Collectors.toList());

        for (int i = 0; i < familySortedRankList.size(); i++) {

            AnnualGalaPeakFamilyRankBO familyRankBO = familySortedRankList.get(i);
            ActivityAnniversaryRankVO vo = new ActivityAnniversaryRankVO();
            vo.setRank(i + 1);
            vo.setName(familyRankBO.getFamilyName());
            vo.setScore(familyRankBO.getScore());
            vo.setId(String.valueOf(familyRankBO.getFamilyNo()));
            vo.setImgUrl(familyRankBO.getFamilyIcon());
            list.add(vo);
        }
        return Result.ofSuccess(list);
    }

    /**
     * 巅峰家族我的贡献信息
     *
     * @param activityId
     * @param userId
     * @return
     */
    @Override
    @Path("getPeakFamilyPersonScore")
    @GET
    public Result getPeakFamilyPersonScore(@QueryParam("activityId") Long activityId, @QueryParam("userId") Long userId) {

        if (userId == null) {
            userId = this.getToken().getUserId();
        }

        Long familyId = familyService.getFamilyId(userId);
        if (familyId == null) {
            Map<String, Object> res = new HashMap<>();
            res.put("myScore", 0);
            res.put("familyScore", 0);
            res.put("familyName", "暂无家族");
            return Result.ofSuccess(res);
        }

        Map<String, Object> res = new HashMap<>();
        res.put("myScore", activityService.getPeakFamilyPersonScore(activityId, familyId, userId));
        res.put("familyScore", activityService.getPeakFamilyScore(activityId, familyId));
        res.put("familyName", familyService.getFamilyName(familyId));
        return Result.ofSuccess(res);
    }

    /**
     * 娱乐勋章-成长值
     *
     * @param userId
     * @return
     */
    @Override
    @Path("getRecreationMedalPersonInfo")
    @GET
    public Result getRecreationMedalPersonInfo(@QueryParam("userId") Long userId) {

        if (userId == null) {
            userId = this.getToken().getUserId();
        }

        Long score = Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.GIFT_SEND_21)).orElse(0L);

        List<MedalPersonRankVO> rankList = scoreService.getActivitiesMedalPersonRankList(ScoreTypeEnum.GIFT_SEND_21);

        MedalPersonRankVO myRank = new MedalPersonRankVO();

        if (CollectionUtil.isNotEmpty(rankList)) {
            Long finalUserId = userId;
            rankList.forEach(val -> {
                if (val.getUserId().equals(finalUserId)) {
                    BeanUtils.copyProperties(val, myRank);
                }
            });
        }

        Map<String, Object> res = new HashMap<>();
        res.put("userName", userService.getUserName(userId));
        res.put("rank", myRank.getRankSort() == null ? null : myRank.getRankSort());
        res.put("medalIconUrl", myRank.getLevelImg() == null ? null : myRank.getLevelImg());
        res.put("myScore", BigDecimal.valueOf(score).divide(BigDecimal.valueOf(100)).longValue());
        return Result.ofSuccess(res);
    }

    /**
     * 偶像勋章-成长值
     *
     * @param userId
     * @return
     */
    @Override
    @Path("getIdolMedalPersonInfo")
    @GET
    public Result getIdolMedalPersonInfo(@QueryParam("userId") Long userId) {

        if (userId == null) {
            userId = this.getToken().getUserId();
        }

        Long score = Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.GIFT_RECEIVE_21)).orElse(0L);

        List<MedalPersonRankVO> rankList = scoreService.getActivitiesMedalPersonRankList(ScoreTypeEnum.GIFT_RECEIVE_21);

        MedalPersonRankVO myRank = new MedalPersonRankVO();

        if (CollectionUtil.isNotEmpty(rankList)) {
            Long finalUserId = userId;
            rankList.forEach(val -> {
                if (val.getUserId().equals(finalUserId)) {
                    BeanUtils.copyProperties(val, myRank);
                }
            });
        }

        Map<String, Object> res = new HashMap<>();
        res.put("userName", userService.getUserName(userId));
        res.put("rank", myRank.getRankSort() == null ? null : myRank.getRankSort());
        res.put("medalIconUrl", myRank.getLevelImg() == null ? null : myRank.getLevelImg());
        res.put("myScore", BigDecimal.valueOf(score).divide(BigDecimal.valueOf(100)).longValue());
        return Result.ofSuccess(res);
    }

    /**
     * 娱乐勋章排行榜
     *
     * @return
     */
    @Override
    @Path("getRecreationMedalPersonRankList")
    @GET
    @PermitGuest
    public Result getRecreationMedalPersonRankList() {

        return Result.ofList(scoreService.getActivitiesMedalPersonRankList(ScoreTypeEnum.GIFT_SEND_21));
    }

    /**
     * 偶像勋章排行榜
     *
     * @return
     */
    @Override
    @Path("getIdolMedalPersonRankList")
    @GET
    @PermitGuest
    public Result getIdolMedalPersonRankList() {

        return Result.ofList(scoreService.getActivitiesMedalPersonRankList(ScoreTypeEnum.GIFT_RECEIVE_21));
    }
}
