package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.config.UpgradeConfig;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.activity.ActivityLotteryPropConfigVO;
import com.starbuds.server.api.app.v1.facade.ActivityFacade;
import com.starbuds.server.common.constant.activity.ActivityLotteryRankTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.data.BroadcastTypeEnum;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.LotteryPayTypeEnum;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.activity.*;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.ActivityCollectDO;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.db.ActivityLotteryDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.activity.GrabActivityService;
import com.starbuds.server.common.service.activity.HeadlineLotteryActivityService;
import com.starbuds.server.common.service.activity.LotteryActivityService;
import com.starbuds.server.common.service.data.BroadcastHisService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 活动接口实现
 *
 * @author Lay
 * @date 2020/2/20
 */
@Path("/v1/activity")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ActivityFacadeImpl extends BaseFacade implements ActivityFacade {

    @Resource
    UpgradeConfig upgradeConfig;

    @Reference(version = "1.0.0")
    ActivityService activityService;

    @Reference(version = "1.0.0")
    LotteryActivityService lotteryActivityService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Reference(version = "1.0.0")
    HeadlineLotteryActivityService headlineLotteryActivityService;

    @Reference(version = "1.0.0")
    WordFilterService wordFilterService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GrabActivityService grabActivityService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    BroadcastHisService broadcastHisService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;


    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 获取直播间活动
     *
     * @return
     */
    @GET
    @Path("getLiveActivities")
    @Override
    public Result getLiveActivities() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getLiveList(this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取语聊房活动
     *
     * @return
     */
    @GET
    @Path("getRtcActivities")
    @Override
    public Result getRtcActivities(@QueryParam("roomId") Long roomId) {
        MutableResult mResult = new MutableResult();
        if (roomId == null) {
            mResult.put("list", activityService.getRtcList(this.getToken().getUserId()));
        } else {
            mResult.put("list", activityService.getRtcList(this.getToken().getUserId(), roomId));
        }
        return mResult.build();
    }

    /**
     * 获取抽奖活动
     *
     * @param lotteryId
     * @return
     */
    @GET
    @Path("getLottery")
    @PermitGuest
    @Override
    public Result getLottery(@QueryParam("lotteryId") String lotteryId) {
        // 读取活动配置
        ActivityLotteryDO activityLotteryDO = lotteryActivityService.getLottery(lotteryId);
        if (activityLotteryDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖活动");
        }

        // 奖项处理
        List<LotteryReward> lotteryRewards = JSON.parseArray(activityLotteryDO.getLotteryRewards(), LotteryReward.class);
        List<LotteryReward> poolRewards = JSON.parseArray(activityLotteryDO.getLotteryPoolRewards(), LotteryReward.class);

        // 价值 奖项id=>单价
        Map<Long, Long> valueMap = new HashMap<>();
        if (!CollectionUtil.isEmpty(lotteryRewards)) {
            for (LotteryReward lotteryReward : lotteryRewards) {
                valueMap.put(lotteryReward.getRewardId(), lotteryReward.getRewardValue() / Optional.ofNullable(lotteryReward.getQuantity()).orElse(1));
            }
        }

        if (poolRewards != null) {
            if (lotteryRewards == null) {
                lotteryRewards = new ArrayList<>();
            }
            lotteryRewards.addAll(poolRewards);
        }

        List<LotteryRewardItemVO> rewardItems = new ArrayList<>();
        if (lotteryRewards != null) {
            Set<Long> giftIds = new HashSet<>();
            Set<Long> propsIds = new HashSet<>();
            for (LotteryReward lotteryReward : lotteryRewards) {
                LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(lotteryReward.getRewardType());
                if (lotteryRewardType == null) {
                    continue;
                }
                switch (lotteryRewardType) {
                    case GIFT:
                        giftIds.add(lotteryReward.getRewardId());
                        break;
                    case PROPS:
                        propsIds.add(lotteryReward.getRewardId());
                        break;
                    default:
                        break;
                }
            }
            if (CollectionUtil.isNotEmpty(giftIds)) {
                List<GiftItemVO> giftItems = giftService.getGiftItemsByIds(new ArrayList<>(giftIds));
                if (CollectionUtil.isNotEmpty(giftItems)) {
                    for (GiftItemVO giftItem : giftItems) {
                        LotteryRewardItemVO rewardItem = new LotteryRewardItemVO();
                        rewardItem.setRewardId(giftItem.getGiftId());
                        rewardItem.setRewardType(LotteryRewardTypeEnum.GIFT.getValue());
                        rewardItem.setRewardName(giftItem.getGiftName());
                        rewardItem.setRewardIcon(giftItem.getGiftIcon());
                        rewardItem.setRewardValue(!CollectionUtil.isEmpty(valueMap) ? valueMap.get(rewardItem.getRewardId()) : null);
                        rewardItems.add(rewardItem);
                    }
                }
            }
            if (CollectionUtil.isNotEmpty(propsIds)) {
                List<PropsVO> propsList = propsService.getPropsByIds(new ArrayList<>(propsIds));
                if (CollectionUtil.isNotEmpty(propsList)) {
                    for (PropsVO props : propsList) {
                        LotteryRewardItemVO rewardItem = new LotteryRewardItemVO();
                        rewardItem.setRewardId(props.getPropsId());
                        rewardItem.setRewardType(LotteryRewardTypeEnum.PROPS.getValue());
                        rewardItem.setRewardName(props.getPropsName());
                        rewardItem.setRewardIcon(props.getPropsIcon());
                        rewardItem.setRewardValue(!CollectionUtil.isEmpty(valueMap) ? valueMap.get(rewardItem.getRewardId()) : null);
                        rewardItems.add(rewardItem);
                    }
                }
            }
        }

        MutableResult mResult = new MutableResult();
        mResult.put("activityId", activityLotteryDO.getActivityId());
        mResult.put("lotteryId", activityLotteryDO.getLotteryId());
        mResult.put("lotteryName", activityLotteryDO.getLotteryName());
        mResult.put("lotteryPrice", activityLotteryDO.getLotteryPrice());
        mResult.put("lotteryPriceType", activityLotteryDO.getLotteryPriceType());
        mResult.put("lotteryPriceTypeName", WalletTypeEnum.toEnum(activityLotteryDO.getLotteryPriceType()).getName());
        mResult.put("quantities", Optional.ofNullable(activityLotteryDO.getLotteryQuantityConfig()).orElse("1"));
        mResult.put("rewardItems", rewardItems);
        return mResult.build();
    }

    /**
     * 抽奖
     *
     * @param lotteryId
     * @param times
     * @return
     */
    @POST
    @Path("lottery")
    @Override
    public Result lottery(@FormParam("lotteryId") String lotteryId,
                          @FormParam("times") int times) {

        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        if (times < 1) {
            times = 1;
        }

        // 读取抽奖配置
        ActivityLotteryDO activityLotteryDO = lotteryActivityService.getLottery(lotteryId);
        if (activityLotteryDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖活动");
        }


        // 读取活动配置
        ActivityDO activityDO = activityService.getActivity(activityLotteryDO.getActivityId());
        if (activityDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该活动");
        }

        // 判断活动是否过期
        long curTime = System.currentTimeMillis();
        Long beginTime = activityDO.getActivityBeginTime();
        if (beginTime != null) {
            if (beginTime > curTime) {
                return new Result(GlobalApiError.OperationFailed, "来早啦，活动还未开始");
            }
        }
        Long endTime = activityDO.getActivityEndTime();
        if (endTime != null) {
            if (curTime > endTime) {
                return new Result(GlobalApiError.OperationFailed, "来晚啦，活动已经结束");
            }
        }

        // 判断数量合法性
        String quantityConfig = Optional.ofNullable(activityLotteryDO.getLotteryQuantityConfig()).orElse("1");
        List<String> quantities = Arrays.asList(quantityConfig.split(","));
        if (!quantities.contains(String.valueOf(times))) {
            return new Result(GlobalApiError.OperationFailed, "抽奖次数无效");
        }

        // 判断余额是否足够
        Long userId = this.getToken().getUserId();

        // 等级限制判断
        int wealthLevelLimit = Optional.ofNullable(activityDO.getWealthLevelLimit()).orElse(0);
        if (wealthLevelLimit > 0) {
            int userWealthLevel = userService.getWealthLevelByUserId(userId);
            if (userWealthLevel < wealthLevelLimit) {
                return new Result(GlobalApiError.OperationFailed, "您的财富等级过低无法参与活动，快去提升等级吧");
            }
        }

        if (activityLotteryDO.getLotteryPayType() != null && activityLotteryDO.getLotteryPayType() == LotteryPayTypeEnum.PROP.getValue()) {
            Integer needNum = activityLotteryDO.getLotteryPropNum() * times;
            Integer leftItemNum = bagService.getItemQuantity(userId, BagItemTypeEnum.PROPS, activityLotteryDO.getLotteryPropId());
            if (leftItemNum < needNum) {
                return Result.ofBadRequest("道具碎片不足");
            }
        } else {
            if (activityLotteryDO.getLotteryPrice() <= 0L || activityLotteryDO.getLotteryPrice() < 1L) {
                return new Result(GlobalApiError.OperationFailed, "活动无效");
            }
            Long amount = activityLotteryDO.getLotteryPrice() * times;
            Long balance = walletService.getBalance(userId, WalletTypeEnum.toEnum(activityLotteryDO.getLotteryPriceType()));
            if (balance < amount) {
                MutableResult mResult = new MutableResult();
                mResult.put("balance", balance - amount);
                return mResult.build();
            }
        }

        // 抽奖
        ActivityLotteryResult result = lotteryActivityService.lottery(lotteryId, userId, times, this.getHeader().getChannelId());
        if (result == null) {
            return new Result(GlobalApiError.OperationFailed);
        }
        result.setWinRewards(wrapReward(result.getWinRewards()));
        if (CollectionUtil.isNotEmpty(result.getWinRewards())) {
            Long roomId = rtcUserService.getUserStayRoomId(userId);
            for (LotteryHisRewardVO winReward : result.getWinRewards()) {
                broadcastHisService.insert(userId, 0L, BroadcastTypeEnum.LOTTERY, winReward.getRewardValue()
                        , winReward.getRewardIcon(), winReward.getQuantity(), roomId, 0L, activityLotteryDO.getLotteryName());
            }
        }
        return new Result(result);
    }

    /**
     * 获取用户抽奖活动次数
     *
     * @param lotteryId
     * @return
     */
    @POST
    @Path("getLotteryUserNum")
    @Override
    public Result getLotteryUserNum(@FormParam("lotteryId") String lotteryId) {

        // 读取抽奖配置
        ActivityLotteryDO activityLotteryDO = lotteryActivityService.getLottery(lotteryId);
        if (activityLotteryDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖活动");
        }

        Long userId = this.getToken().getUserId();
        //查看用户背包是否消耗物
        Integer ItemNum = bagService.getItemQuantity(userId, BagItemTypeEnum.PROPS, activityLotteryDO.getLotteryPropId());

        MutableResult mResult = new MutableResult();
        mResult.put("num", ItemNum);
        return mResult.build();
    }

    /**
     * 通过活动ID获取抽奖列表
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getLotteryByProp")
    @Override
    public Result getLotteryByProp(@QueryParam("activityId") Long activityId) {
        Long userId = this.getToken().getUserId();
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null || activityDO.getActivityId() == 0L) {
            return Result.ofBadRequest("未找到该活动");
        }
        List<ActivityLotteryDO> lotteryList = lotteryActivityService.getLotteryByActivityId(activityId);
        if (CollectionUtil.isNotEmpty(lotteryList)) {
            List<Long> itemSet = new ArrayList<>(lotteryList.size());
            lotteryList.forEach(i -> {
                if (i.getLotteryPropId() != null && i.getLotteryPropId() > 0) {
                    itemSet.add(i.getLotteryPropId());
                }
            });
            Map<Long, Integer> userOwnMap = bagService.getGiftOwnNumMap(itemSet, userId);
            List<ActivityLotteryPropConfigVO> res = new ArrayList<>();
            lotteryList.forEach(i -> {
                ActivityLotteryPropConfigVO tmp = new ActivityLotteryPropConfigVO();
                BeanUtils.copyProperties(i, tmp);
                tmp.setLotteryPropOwnNum(Optional.ofNullable(userOwnMap.get(i.getLotteryPropId())).orElse(0));
                res.add(tmp);
            });
            return Result.ofSuccess(res);
        }

        return new Result();
    }

    /**
     * 获取抽奖活动中奖记录
     *
     * @param lotteryId
     * @return
     */
    @GET
    @Path("getLotteryNewlyRewards")
    @PermitGuest
    @Override
    public Result getLotteryNewlyRewards(@QueryParam("lotteryId") String lotteryId) {
        MutableResult mResult = new MutableResult();
        List<LotteryHisRewardVO> list = lotteryActivityService.getLotteryHisRewards(lotteryId, GlobalConfig.PAGE_SIZE);
        mResult.put("list", wrapReward(list));
        return mResult.build();
    }

    /**
     * 获取活动中奖纪录
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getActivityNewlyRewards")
    @Override
    public Result getActivityNewlyRewards(@QueryParam("activityId") Long activityId) {
        MutableResult mResult = new MutableResult();
        List<LotteryHisRewardVO> list = lotteryActivityService.getLotteryHisRewards(activityId, GlobalConfig.PAGE_SIZE);
        mResult.put("list", wrapReward(list));
        return mResult.build();
    }

    /**
     * 获取抽奖记录
     *
     * @param lotteryId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getLotteryHis")
    @Override
    public Result getLotteryHis(@QueryParam("lotteryId") String lotteryId,
                                @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(lotteryActivityService.getLotteryHisList(pageIndex, GlobalConfig.PAGE_SIZE, lotteryId, this.getToken().getUserId()));
    }

    /**
     * 获取活动抽奖纪录
     *
     * @param activityId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getActivityLotteryHis")
    @Override
    public Result getActivityLotteryHis(@QueryParam("activityId") Long activityId,
                                        @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        return new Result(lotteryActivityService.getLotteryHisList(pageIndex, GlobalConfig.PAGE_SIZE, activityId, this.getToken().getUserId()));
    }

    /**
     * 获取中奖记录
     *
     * @param hisId
     * @return
     */
    @GET
    @Path("getLotteryHisRewards")
    @Override
    public Result getLotteryHisRewards(@QueryParam("hisId") Long hisId) {
        Long userId = this.getToken().getUserId();

        // 判断抽奖记录是否存在
        if (!lotteryActivityService.isLotteryHisExist(hisId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖记录");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", wrapReward(lotteryActivityService.getLotteryHisRewardsListByHisId(hisId, userId)));
        return mResult.build();
    }

    /**
     * 获取抽奖中奖记录
     *
     * @param lotteryId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getLotteryRewardsList")
    @Override
    public Result getLotteryRewardsList(@QueryParam("lotteryId") String lotteryId,
                                        @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        PageList<LotteryHisRewardVO> pageList = lotteryActivityService.getLotteryHisRewardsList(pageIndex, GlobalConfig.PAGE_SIZE, lotteryId, this.getToken().getUserId());
        pageList.setList(wrapReward(pageList.getList()));
        return new Result(pageList);
    }

    /**
     * 获取抽奖活动排行榜
     *
     * @param lotteryId
     * @param rankType
     * @return
     */
    @GET
    @Path("getLotteryRankList")
    @Override
    public Result getLotteryRankList(@QueryParam("lotteryId") String lotteryId, @QueryParam("rankType") int rankType) {
        ActivityLotteryRankTypeEnum rankTypeEnum = ActivityLotteryRankTypeEnum.toEnum(rankType);
        if (rankTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "类型错误");
        }

        // 判断活动是否存在
        if (!lotteryActivityService.isLotteryExist(lotteryId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖活动");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", lotteryActivityService.getLotteryRank(lotteryId, rankTypeEnum, 20));
        return mResult.build();
    }

    /**
     * 获取抽奖活动中奖排行榜
     *
     * @param lotteryId
     * @param rankType
     * @return
     */
    @GET
    @Path("getLotteryRewardRankList")
    @Override
    public Result getLotteryRewardRankList(@QueryParam("lotteryId") String lotteryId, @QueryParam("rankType") int rankType) {
        ActivityLotteryRankTypeEnum rankTypeEnum = ActivityLotteryRankTypeEnum.toEnum(rankType);
        if (rankTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "类型错误");
        }

        // 判断活动是否存在
        if (!lotteryActivityService.isLotteryExist(lotteryId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖活动");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", lotteryActivityService.getLotteryRewardRank(lotteryId, rankTypeEnum, 20));
        return mResult.build();
    }

    /**
     * 获取活动中奖纪录
     *
     * @param activityId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getActivityRewardList")
    @Override
    public Result getActivityRewardList(@QueryParam("activityId") Long activityId,
                                        @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        PageList<LotteryHisRewardVO> pageList = lotteryActivityService.getLotteryHisRewardsList(pageIndex, GlobalConfig.PAGE_SIZE, activityId, this.getToken().getUserId());
        pageList.setList(wrapReward(pageList.getList()));
        return new Result(pageList);
    }

    /**
     * 获取充值活动详情
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getRecharge")
    @Override
    public Result getRecharge(@QueryParam("activityId") Long activityId) {
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null || ActivityTypeEnum.RECHARGE.getValue() != activityDO.getActivityType()) {
            return new Result(GlobalApiError.OperationFailed, "未找到该充值活动");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("activityId", activityId);
        mResult.put("activityName", activityDO.getActivityName());
        mResult.put("unionList", activityService.getActivityUnion(activityId));
        return mResult.build();
    }

    /**
     * 获取奖品清单
     *
     * @param lotteryId
     * @return
     */
    @GET
    @Path("getRewardItem")
    @Override
    public Result getRewardItem(@QueryParam("lotteryId") String lotteryId) {

        if (!lotteryActivityService.isLotteryExist(lotteryId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该抽奖活动");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", lotteryActivityService.getRewardItem(lotteryId));
        return mResult.build();
    }

    /**
     * 获取活动排名
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getRankTopN")
    @Override
    public Result getRankTopN(@QueryParam("activityId") Long activityId) {
        if (!activityService.isActivityExist(activityId)) {
            return new Result(GlobalApiError.ServerError, "活动不存在");
        }
        return new Result(activityService.getRankTopN(activityId, 10));
    }

    /**
     * 获取活动详情
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo(@QueryParam("activityId") Long activityId) {
        return new Result(activityService.getInfo(activityId));
    }

    private List<LotteryHisRewardVO> wrapReward(List<LotteryHisRewardVO> rewards) {
        if (CollectionUtil.isEmpty(rewards)) {
            return Collections.emptyList();
        }

        Set<Long> giftIdSet = new HashSet<>();
        Set<Long> propsIdSet = new HashSet<>();
        for (LotteryHisRewardVO reward : rewards) {
            LotteryRewardTypeEnum rewardType = LotteryRewardTypeEnum.toEnum(reward.getRewardType());
            if (LotteryRewardTypeEnum.GIFT == rewardType) {
                giftIdSet.add(reward.getRewardId());
            } else if (LotteryRewardTypeEnum.PROPS == rewardType) {
                propsIdSet.add(reward.getRewardId());
            }
        }

        Map<Long, GiftItemVO> giftItemMap = null;
        if (CollectionUtil.isNotEmpty(giftIdSet)) {
            List<GiftItemVO> giftItems = giftService.getGiftItemsByIds(new ArrayList<>(giftIdSet));
            if (CollectionUtil.isNotEmpty(giftItems)) {
                giftItemMap = giftItems.stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));
            }
        }

        Map<Long, PropsVO> propsMap = null;
        if (CollectionUtil.isNotEmpty(propsIdSet)) {
            List<PropsVO> propsList = propsService.getPropsByIds(new ArrayList<>(propsIdSet));
            if (CollectionUtil.isNotEmpty(propsList)) {
                propsMap = propsList.stream().collect(Collectors.toMap(PropsVO::getPropsId, Function.identity()));
            }
        }

        for (LotteryHisRewardVO reward : rewards) {
            LotteryRewardTypeEnum rewardType = LotteryRewardTypeEnum.toEnum(reward.getRewardType());
            if (LotteryRewardTypeEnum.GIFT == rewardType && giftItemMap != null) {
                GiftItemVO giftItem = giftItemMap.get(reward.getRewardId());
                if (giftItem != null) {
                    reward.setRewardName(giftItem.getGiftName());
                    reward.setRewardIcon(giftItem.getGiftIcon());
                }
            } else if (LotteryRewardTypeEnum.PROPS == rewardType && propsMap != null) {
                PropsVO propsVO = propsMap.get(reward.getRewardId());
                if (propsVO != null) {
                    reward.setRewardName(propsVO.getPropsName());
                    reward.setRewardIcon(propsVO.getPropsIcon());
                }
            }
        }

        return rewards;
    }

    /**
     * 获取礼物活动主播排行
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getGiftAnchorRank")
    @PermitGuest
    @Override
    public Result getGiftAnchorRank(@QueryParam("activityId") Long activityId) {
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null || ActivityTypeEnum.GIFT_NUM.getValue() != activityDO.getActivityType()) {
            return new Result(GlobalApiError.OperationFailed, "未找到该活动");
        }
        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getGiftAnchorRankList(activityId, 20));
        return mResult.build();
    }

    /**
     * 获取礼物活动用户排行
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getGiftUserRank")
    @PermitGuest
    @Override
    public Result getGiftUserRank(@QueryParam("activityId") Long activityId) {
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null || ActivityTypeEnum.GIFT_NUM.getValue() != activityDO.getActivityType()) {
            return new Result(GlobalApiError.OperationFailed, "未找到该活动");
        }
        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getGiftUserRankList(activityId, 20));
        mResult.put("myRank", activityService.getMyRank(activityId, this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取活动达标情况
     *
     * @param collectId
     */
    @GET
    @Path("getCollectStatus")
    @Override
    public Result getCollectStatus(@QueryParam("collectId") String collectId) {
        if (!activityService.isCollectExist(collectId)) {
            return new Result(GlobalApiError.ServerError, "收集活动不存在");
        }

        Long userId = this.getToken().getUserId();

        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getCollectStatus(userId, collectId));
        mResult.put("isReceive", activityService.isReceive(userId, collectId));
        mResult.put("isMatch", activityService.isMatch(userId, collectId));

        return mResult.build();
    }

    /**
     * 领取奖励
     *
     * @param collectId
     * @return
     */
    @POST
    @Path("receiveReward")
    @Override
    public Result receiveReward(@FormParam("collectId") String collectId) {

        ActivityCollectDO collectDO = activityService.getCollect(collectId);
        if (collectDO == null) {
            return new Result(GlobalApiError.ServerError, "收集活动不存在");
        }
        ActivityDO activityDO = activityService.getActivity(collectDO.getActivityId());
        if (activityDO == null) {
            return new Result(GlobalApiError.ServerError, "未找到该活动或已过期");
        }

        long curTime = System.currentTimeMillis();
        Long userId = this.getToken().getUserId();

        if (activityService.isReceive(userId, collectId)) {
            return new Result(GlobalApiError.ServerError, "您已领取过啦，快去背包看看");
        }
        if (curTime < activityDO.getActivityBeginTime()) {
            return new Result(GlobalApiError.ServerError, "活动尚未开始，请晚点再来");
        }
        if (curTime > activityDO.getActivityEndTime()) {
            return new Result(GlobalApiError.ServerError, "活动结束了，下次早点来哟");
        }
        if (!activityService.isMatch(userId, collectId)) {
            return new Result(GlobalApiError.ServerError, "您当前不符合领取条件");
        }

        if (activityService.receiveReward(userId, collectId, activityDO.getActivityBeginTime(), activityDO.getActivityEndTime())) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError, "活动太火爆了，请稍后重试");
    }

    /**
     * 获取等级奖励
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getLevelReward")
    @Override
    public Result getLevelReward(@QueryParam("scoreType") Integer scoreType) {
        if (scoreType == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.ServerError, "请确认类型是否正确");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getLevelReward(scoreType));
        return mResult.build();
    }

    /**
     * 参与头条抽奖活动
     *
     * @param configId
     * @param periodId
     * @param content
     * @return
     */
    @POST
    @Path("goHeadlineLotteryActivity")
    @Override
    public Result goHeadlineLotteryActivity(@FormParam("configId") long configId,
                                            @FormParam("periodId") int periodId,
                                            @FormParam("content") String content) {
        if (!headlineLotteryActivityService.isPeriodGoing(configId, periodId)) {
            return new Result(GlobalApiError.OperationFailed, "活动未开始或已结束");
        }

        // 内容过滤
        if (wordFilterService.isMatchedPhrases(WordFilterTypeEnum.LIVE_TEXT, content)) {
            return new Result(GlobalApiError.OperationFailed, "您发布的内容包含法律法规禁止的敏感词汇，请修改后重试");
        }

        Long balance = headlineLotteryActivityService.participate(configId, periodId, this.getToken().getUserId(), content, this.getHeader().getChannelId());
        if (balance != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("balance", balance);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取头条活动
     *
     * @param configId
     * @param periodId
     * @return
     */
    @GET
    @Path("getHeadlineLotteryActivity")
    @PermitGuest
    @Override
    public Result getHeadlineLotteryActivity(@QueryParam("configId") Long configId, @QueryParam("periodId") Integer periodId) {
        boolean findCurrent = false;
        if (configId != null && periodId != null) {
            // 判断活动是否存在
            if (!headlineLotteryActivityService.isPeriodExists(configId, periodId)) {
                return new Result(GlobalApiError.OperationFailed, "未找到该期活动");
            }
        } else {
            findCurrent = true;
        }

        ActivityHeadlinePeriodVO period;
        if (findCurrent) {
            period = headlineLotteryActivityService.getGoingPeriod();
        } else {
            period = headlineLotteryActivityService.getPeriod(configId, periodId);
        }

        return new Result(period);
    }

    /**
     * 获取头条活动中奖历史
     *
     * @return
     */
    @GET
    @Path("getHeadlineLotteryWinnerHis")
    @Override
    public Result getHeadlineLotteryWinnerHis(@QueryParam("activityId") Long activityId) {
        if (!activityService.isActivityExist(activityId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该活动");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", headlineLotteryActivityService.getWinnerHisList(activityId));
        return mResult.build();
    }

    /**
     * 获取抢龙活动详情
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getGrabActivity")
    @Override
    public Result getGrabActivity(@QueryParam("activityId") long activityId) {
        if (!activityService.isActivityExist(activityId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该活动");
        }

        ActivityGrabPeriodVO period = grabActivityService.getGoingPeriod(activityId);

        long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        mResult.put("period", period);
        mResult.put("rtmToken", imService.getRongCloudToken(this.getToken().getUserId()));
        mResult.put("isPlayer", grabActivityService.isPlayer(period.getPeriodId(), userId));
        mResult.put("userId", userId);
        return mResult.build();
    }

    /**
     * 参加抢龙活动
     *
     * @param periodId
     * @return
     */
    @POST
    @Path("joinGrabActivity")
    @Override
    public Result joinGrabActivity(@FormParam("periodId") long periodId) {

        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        long userId = this.getToken().getUserId();
        if (!grabActivityService.isPeriodExists(periodId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该活动");
        }

        // 锁定用户
        String lockKey = RedisRtcKeys.getActivitySeatLock(userId);
        if (!redisService.setIfAbsent(lockKey, String.valueOf(userId), TimeConstants.MIN_MILLS, TimeUnit.MILLISECONDS)) {
            return new Result(GlobalApiError.OperationFailed, "参与失败，请稍后再试");
        }
        //加入活动
        Long balance = grabActivityService.join(periodId, userId);
        // 移除锁
        redisService.delKey(lockKey);

        if (balance == null) {
            return new Result(GlobalApiError.OperationFailed, "参与失败，请检查余额");
        }

        if (balance == -1) {
            //没有空位
            return new Result(GlobalApiError.OperationFailed, "参与失败，请稍后再试");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", balance);
        return mResult.build();
    }

    /**
     * 获取我参与的抢龙记录
     *
     * @param activityId
     * @param pageIndex
     * @return
     */
    @GET
    @Path("getGrabPeriodHis")
    @Override
    public Result getGrabPeriodHis(@QueryParam("activityId") long activityId,
                                   @QueryParam("pageIndex") int pageIndex) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        long userId = this.getToken().getUserId();
        return new Result(grabActivityService.getUserPeriods(activityId, userId, pageIndex, GlobalConfig.PAGE_SIZE));
    }

    /**
     * 获取多礼物活动的礼物列表
     *
     * @param activityId
     * @return
     */
    @GET
    @Path("getGiftsByGiftManyActivity")
    @PermitGuest
    @Override
    public Result getGiftsByGiftManyActivity(@QueryParam("activityId") long activityId) {
        ActivityDO activity = activityService.getActivity(activityId);
        if (activity == null) {
            return new Result(GlobalApiError.BadRequest, "无效的id");
        }

        String giftIdsStr = activity.getGiftIds();
        if (StringUtils.isNotBlank(giftIdsStr)) {
            List<Long> giftIds = Arrays.stream(StringUtils.split(giftIdsStr, ","))
                    .map(NumberUtils::toLong)
                    .filter(giftId -> giftId > 0)
                    .collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(giftIds)) {
                return new Result(giftService.getGiftItemsByIds(giftIds));
            }
        }

        return new Result();
    }
}
