package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.pojo.activity.ActivityGiftsUserVO;
import com.starbuds.server.api.app.v1.facade.ActivityGiftsFacade;
import com.starbuds.server.common.constant.activity.ActivityGiftsRankCategoryEnum;
import com.starbuds.server.common.constant.activity.ActivityGiftsRankTypeEnum;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.activity.*;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * 活动礼物接口实现
 *
 * @author laoliu
 * @date 2021/08/09
 */
@Path("/v1/activity/gifts/")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ActivityGiftsFacadeImpl extends BaseFacade implements ActivityGiftsFacade {


    @Reference(version = "1.0.0")
    ActivityGiftsRankService activityGiftsRankService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ActivityService activityService;

    /**
     * 获取关系亲密度排行榜
     *
     * @return
     */
    @GET
    @Path("getRankTopList")
    @Override
    public Result getRankTopList(@QueryParam("activityId") Long activityId,
                                 @QueryParam("type") Integer type) {

        if (activityId == null) {
            return new Result(GlobalApiError.BadRequest, "请传入活动id");
        }

        if (type == null) {
            return new Result(GlobalApiError.BadRequest, "请传入类型,今日昨天总");
        }

        if (ActivityGiftsRankTypeEnum.toEnum(type) == null) {
            return new Result(GlobalApiError.BadRequest, "type不支持");
        }

        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();

        if (isDev()) {
            log.info("[getRankTopList]参数: activityId: {}, type: {}, userId: {}", activityId, type, userId);
        }

        // 获取用户信息
        ActivityGiftsUserVO userVO = new ActivityGiftsUserVO();
        UserBasicInfo userBasicInfo = userService.getBasicInfo(userId);

        userVO.setUserId(userId);
        userVO.setUserName(userBasicInfo.getUserName());
        userVO.setUserAvatar(userBasicInfo.getUserAvatar());
        userVO.setTotalValue(activityGiftsRankService.getTotalValueByUser(activityId, userId));


        mResult.put("list", activityGiftsRankService.getRankTopList(activityId, type));
        mResult.put("user", userVO);
        mResult.put("isFinish", isActivityFinish(activityId));
        return mResult.build();
    }

    /**
     * 获取个人亲密度排行榜
     *
     * @return
     */
    @GET
    @Path("getRankListByUser")
    @Override
    public Result getRankListByUser(@QueryParam("activityId") Long activityId,
                                    @QueryParam("type") Integer type,
                                    @QueryParam("category") Integer category,
                                    @QueryParam("pageIndex") Integer pageIndex,
                                    @QueryParam("pageSize") Integer pageSize) {

        if (activityId == null) {
            return new Result(GlobalApiError.BadRequest, "请传入活动id");
        }

        if (type == null) {
            return new Result(GlobalApiError.BadRequest, "请传入类型,今日昨天总");
        }

        if (category == null) {
            return new Result(GlobalApiError.BadRequest, "请传入分类,心动我的");
        }

        if (pageIndex == null) {
            return new Result(GlobalApiError.BadRequest, "请传入分页");
        }

        if (pageSize == null) {
            return new Result(GlobalApiError.BadRequest, "请传入分页");
        }

        if (ActivityGiftsRankCategoryEnum.toEnum(category) == null) {
            return new Result(GlobalApiError.BadRequest, "category不支持");
        }

        if (ActivityGiftsRankTypeEnum.toEnum(type) == null) {
            return new Result(GlobalApiError.BadRequest, "type不支持");
        }

        Long userId = this.getToken().getUserId();

        if (isDev()) {
            log.info("[getRankTopList]参数: activityId: {}, type: {}, category: {}, pageIndex: {}, pageSize: {}", activityId, type, category, pageIndex, pageSize);
        }


        MutableResult mResult = new MutableResult();
        mResult.put("list", activityGiftsRankService.getRankListByUser(activityId, userId, type, category, pageIndex, pageSize));
        mResult.put("isFinish", isActivityFinish(activityId));
        return mResult.build();
    }

    private boolean isActivityFinish(Long activityId) {
        ActivityDO activityDO = activityService.getActivity(activityId);
        boolean isFinish = true;
        Long currentTime = System.currentTimeMillis();
        if (activityDO != null) {
            if (currentTime >= activityDO.getActivityBeginTime() && currentTime < activityDO.getActivityEndTime()) {
                isFinish = false;
            }
        }
        return isFinish;
    }
}
