package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.v1.facade.BagFacade;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.i18n.I18nService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

/**
 * 背包接口实现
 *
 * @author Lay
 * @date 2020/1/12
 */
@Path("/v1/bag")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class BagFacadeImpl extends BaseFacade implements BagFacade {

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    private I18nService i18nService;

    /**
     * 获取背包礼物列表
     *
     * @return
     */
    @GET
    @Path("getGiftList")
    @Override
    public Result getGiftList() {
        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        mResult.put("list", bagService.getGiftList(userId));
        return mResult.build();
    }

    /**
     * 获取背包道具列表
     *
     * @param propsType
     * @return
     */
    @GET
    @Path("getPropsList")
    @Override
    public Result getPropsList(@QueryParam("propsType") Integer propsType) {
        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        mResult.put("list", bagService.getPropsList(userId, propsType));
        return mResult.build();
    }

    /**
     * 获取背包坐骑列表
     *
     * @return
     */
    @GET
    @Path("getMountsList")
    @Override
    public Result getMountsList(@QueryParam("userId") Long userId) {
        userId = userId == null ? this.getToken().getUserId() : userId;
        MutableResult mResult = new MutableResult();
        mResult.put("list", bagService.getMountsList(userId));
        return mResult.build();
    }

    /**
     * 获取背包勋章列表
     *
     * @return
     */
    @GET
    @Path("getMedalList")
    @Override
    public Result getMedalList() {
        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        mResult.put("list", bagService.getMedalList(userId));
        return mResult.build();
    }

    /**
     * 获取背包头饰列表
     *
     * @return
     */
    @GET
    @Path("getHeadwearList")
    @Override
    public Result getHeadwearList(@QueryParam("userId") Long userId) {
        userId = userId == null ? this.getToken().getUserId() : userId;
        MutableResult mResult = new MutableResult();
        mResult.put("list", bagService.getHeadwearList(userId));
        return mResult.build();
    }

    /**
     * 获取背包聊天气泡列表
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getChatBubbleList")
    @Override
    public Result getChatBubbleList(@QueryParam("userId") Long userId) {
        userId = userId == null ? this.getToken().getUserId() : userId;
        MutableResult mResult = new MutableResult();
        mResult.put("list", bagService.getChatBubbleList(userId));
        return mResult.build();
    }

    /**
     * 获取道具数量
     *
     * @param propsType
     * @return
     */
    @GET
    @Path("getPropsQuantity")
    @Override
    public Result getPropsQuantity(@QueryParam("propsType") int propsType) {
        PropsTypeEnum propsTypeEnum = PropsTypeEnum.toEnum(propsType);
        MutableResult mResult = new MutableResult();
        mResult.put("quantity", bagService.getPropsQuantityByType(this.getToken().getUserId(), propsTypeEnum));
        return mResult.build();
    }
}
