package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.starbuds.server.api.app.constant.ApiError;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.ChatFacade;
import com.starbuds.server.common.constant.cache.RedisChatKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.constant.finance.ChatTimeWalletType;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.ChatStatusEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.media.VoiceChatStopReasonEnum;
import com.starbuds.server.common.constant.user.SingleChatStatus;
import com.starbuds.server.common.constant.user.VoiceChatStatus;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.pojo.db.SingleChatHisDO;
import com.starbuds.server.common.pojo.db.SingleChatMatchHisDO;
import com.starbuds.server.common.pojo.db.VoiceChatHisDO;
import com.starbuds.server.common.pojo.media.*;
import com.starbuds.server.common.pojo.user.AnchorSingleChatInfoVO;
import com.starbuds.server.common.pojo.user.SingleChatUserVO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.common.service.finance.VipService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.ChatService;
import com.starbuds.server.common.service.media.SingleChatService;
import com.starbuds.server.common.service.media.VoiceChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 聊天接口实现
 *
 * @author Lay
 * @date 2020/3/22
 */
@Path("/v1/chat")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChatFacadeImpl extends BaseFacade implements ChatFacade {

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    SingleChatService singleChatService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    VipService vipService;

    @Reference(version = "1.0.0")
    ChatService chatService;

    @Reference(version = "1.0.0")
    VoiceChatService voiceChatService;

    @Reference(version = "1.0.0")
    WordFilterService wordFilterService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 发起视频单聊请求
     *
     * @param targetUserId
     * @param matchId
     * @return
     */
    @POST
    @Path("sendSingleChatRequest")
    @Override
    public Result sendSingleChatRequest(@FormParam("targetUserId") Long targetUserId, @FormParam("matchId") String matchId) {
        Long userId = this.getToken().getUserId();
        if (userId.equals(targetUserId)) {
            return new Result(GlobalApiError.OperationFailed, "你好调皮~ 不能逗自己玩哦！");
        }

        if (StringUtils.isNotBlank(matchId)) {
            SingleChatMatchHisDO singleChatMatchHis = singleChatService.getSingleChatMatchHis(matchId);
            if (singleChatMatchHis == null) {
                return new Result(GlobalApiError.OperationFailed, "未找到匹配记录");
            }
            // 判断是否有效
            targetUserId = singleChatMatchHis.getUserId();
            if (targetUserId != null) {
                String hisId = singleChatService.sendSingleChatRequestByAnchor(SingleChatRequestDTO.builder()
                        .userId(userId)
                        .targetUserId(targetUserId)
                        .matchId(matchId)
                        .channelId(this.getHeader().getChannelId()).build());
                if (StringUtils.isNotBlank(hisId)) {
                    MutableResult mResult = new MutableResult();
                    mResult.put("hisId", hisId);
                    return mResult.build();
                }
            }
        } else {
            // 判断目标用户是否开通视频单聊权限
            AnchorSingleChatInfoVO singleChatInfo = anchorService.getSingleChatInfo(targetUserId);
            if (singleChatInfo == null) {
                return new Result(GlobalApiError.OperationFailed, "该用户未开通视频单聊权限");
            }

            // 判断钱包是否足够支持1分钟
            Long balance = walletService.getBalance(userId, singleChatInfo.getSingleChatPriceType());
            Integer minutes = walletService.getChatTimeWalletMinutes(userId, ChatTimeWalletType.SINGLE_CHAT);
            if (minutes < 1 && balance < singleChatInfo.getSingleChatPrice()) {
                MutableResult mResult = new MutableResult();
                mResult.put("balance", balance - singleChatInfo.getSingleChatPrice());
                mResult.put("singleChatStatus", singleChatInfo.getSingleChatStatus());
                return mResult.build();
            }

            // 判断目标用户是否直播中
            if (LiveStatusEnum.Living == anchorService.getLiveStatus(targetUserId)) {
                return new Result(ApiError.SINGLE_CHAT_BUSY);
            }

            // 判断是否在线
            if (!userService.isOnline(targetUserId)) {
                return new Result(ApiError.SINGLE_CHAT_BUSY);
            }

            // 判断目标用户是否空闲中
            if (SingleChatStatus.IDLE != Optional.ofNullable(singleChatInfo.getSingleChatStatus()).orElse(SingleChatStatus.BUSY)) {
                return new Result(ApiError.SINGLE_CHAT_BUSY);
            }

            // 发送视频聊天请求
            String hisId = singleChatService.sendSingleChatRequest(SingleChatRequestDTO.builder().userId(userId).targetUserId(targetUserId).channelId(this.getHeader().getChannelId()).build());
            if (StringUtils.isNotBlank(hisId)) {
                MutableResult mResult = new MutableResult();
                mResult.put("hisId", hisId);
                mResult.put("balance", balance);
                return mResult.build();
            }
        }

        return new Result(GlobalApiError.OperationFailed, "通话失败，等会再试试吧！");
    }

    /**
     * 决定视频单聊请求
     *
     * @param hisId
     * @param decision
     * @return
     */
    @POST
    @Path("decideSingleChatRequest")
    @Override
    public Result decideSingleChatRequest(@FormParam("hisId") String hisId,
                                          @FormParam("decision") int decision) {
        Long userId = this.getToken().getUserId();

        SingleChatHisDO singleChatHis = singleChatService.getSingleChatHis(hisId);
        if (singleChatHis == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
        }

        boolean isMatch = Optional.ofNullable(singleChatHis.getHisIsMatch()).map(t -> BoolIntVal.True == t).orElse(false);
        if (isMatch) {
            if (!userId.equals(singleChatHis.getUserId())) {
                return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
            }

            // 判断是否领取过时间了
            if (!singleChatService.hasGetFreeSingleChatMinutes(userId)) {
                singleChatService.getFreeSingleChatMinutes(FreeSingleChatMinutesDTO.builder().userId(userId).channelId(this.getHeader().getChannelId()).minutes(1).chatHisId(hisId).build());
            }

            // 判断钱包和余额
            if (walletService.getChatTimeWalletMinutes(userId, ChatTimeWalletType.SINGLE_CHAT) < 1 && walletService.getBalance(userId, singleChatHis.getHisPriceType()) < singleChatHis.getHisPrice()) {
                return new Result(GlobalApiError.OperationFailed, "余额不足，请先充值哦！");
            }

            if (!userService.isOnline(singleChatHis.getTargetUserId())) {
                return new Result(GlobalApiError.OperationFailed, "对方已经下线了");
            }
        } else {
            if (!userId.equals(singleChatHis.getTargetUserId())) {
                return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
            }

            if (!userService.isOnline(singleChatHis.getUserId())) {
                return new Result(GlobalApiError.OperationFailed, "对方已经下线了");
            }
        }

        // 获取状态
        ChatStatusEnum chatStatus = ChatStatusEnum.toEnum(singleChatHis.getHisStatus());
        if (ChatStatusEnum.Canceled == chatStatus) {
            if (BoolIntVal.True == decision) {
                return new Result(GlobalApiError.OperationFailed, "通话已取消");
            }
            return new Result();
        }

        if (ChatStatusEnum.Ringing != chatStatus) {
            return new Result(GlobalApiError.OperationFailed, "通话状态已改变");
        }

        // 决定接受与否
        String token = singleChatService.decideSingleChatRequest(hisId, decision, getHeader().getAppVersion());
        if (BoolIntVal.True == decision) {
            if (StringUtils.isNotBlank(token)) {
                MutableResult mResult = new MutableResult();
                mResult.put("channelName", hisId);
                mResult.put("token", token);
                return mResult.build();
            }
            return new Result(GlobalApiError.OperationFailed, "令牌生成失败");
        } else {
            return new Result();
        }
    }

    /**
     * 取消视频单聊请求
     *
     * @param hisId
     * @return
     */
    @POST
    @Path("cancelSingleChatRequest")
    @Override
    public Result cancelSingleChatRequest(@FormParam("hisId") String hisId) {
        Long userId = this.getToken().getUserId();

        // 获取视频单聊请求状态
        SingleChatHisDO singleChatHis = singleChatService.getSingleChatHis(hisId);
        if (singleChatHis == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
        }

        boolean isMatch = Optional.ofNullable(singleChatHis.getHisIsMatch()).map(t -> BoolIntVal.True == t).orElse(false);
        if (isMatch) {
            if (!userId.equals(singleChatHis.getTargetUserId())) {
                return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
            }
        } else {
            if (!userId.equals(singleChatHis.getUserId())) {
                return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
            }
        }

        // 获取状态
        ChatStatusEnum chatStatus = ChatStatusEnum.toEnum(singleChatHis.getHisStatus());
        if (chatStatus == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
        }

        // 如果是已取消的直接返回 ok
        if (ChatStatusEnum.Canceled == chatStatus) {
            return new Result();
        }

        if (ChatStatusEnum.Ringing != chatStatus) {
            return new Result(GlobalApiError.OperationFailed, "通话状态已改变");
        }

        if (singleChatService.cancelSingleChatRequest(hisId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 中止单聊
     *
     * @param hisId
     * @return
     */
    @POST
    @Path("stopSingleChat")
    @Override
    public Result stopSingleChat(@FormParam("hisId") String hisId) {
        Long userId = this.getToken().getUserId();

        // 获取视频单聊请求状态
        SingleChatHisDO singleChatHis = singleChatService.getSingleChatHis(hisId);
        if (singleChatHis == null || (!singleChatHis.getUserId().equals(userId) && !singleChatHis.getTargetUserId().equals(userId))) {
            return new Result(GlobalApiError.OperationFailed, "未找到通话记录");
        }
        ChatStatusEnum chatStatus = ChatStatusEnum.toEnum(singleChatHis.getHisStatus());
        if (ChatStatusEnum.Closed == chatStatus) {
            MutableResult mResult = new MutableResult();
            mResult.put("hisMinutes", singleChatHis.getHisMinutes());
            mResult.put("hisAmount", singleChatHis.getHisPrice() * singleChatHis.getHisMinutes());
            mResult.put("hisAmountName", WalletTypeEnum.toEnum(singleChatHis.getHisPriceType()).getName());
            return mResult.build();
        }

        if (ChatStatusEnum.Calling != chatStatus) {
            return new Result(GlobalApiError.OperationFailed, "状态无效");
        }

        singleChatHis = singleChatService.stopSingleChat(hisId, userId, getHeader().getAppVersion());
        if (singleChatHis != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("hisMinutes", singleChatHis.getHisMinutes());
            mResult.put("hisAmount", singleChatHis.getHisPrice() * singleChatHis.getHisCoinMinutes());
            mResult.put("hisAmountName", WalletTypeEnum.toEnum(singleChatHis.getHisPriceType()).getName());
            mResult.put("hisCoinMinutes", singleChatHis.getHisCoinMinutes());
            mResult.put("hisTimeMinutes", singleChatHis.getHisTimeMinutes());
            return mResult.build();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 单聊评价
     *
     * @param hisId
     * @param scoreValue
     * @param tagIds
     * @return
     */
    @POST
    @Path("evaluateSingleChat")
    @Override
    public Result evaluateSingleChat(@FormParam("hisId") String hisId,
                                     @FormParam("scoreValue") Integer scoreValue,
                                     @FormParam("tagIds") String tagIds) {
        long userId = this.getToken().getUserId();

        // 获取视频单聊请求状态
        SingleChatHisDO singleChatHis = singleChatService.getSingleChatHis(hisId);
        if (singleChatHis == null || (!singleChatHis.getUserId().equals(userId) && !singleChatHis.getTargetUserId().equals(userId))) {
            return new Result(GlobalApiError.OperationFailed, "未找到请求记录");
        }
        ChatStatusEnum chatStatus = ChatStatusEnum.toEnum(singleChatHis.getHisStatus());
        if (ChatStatusEnum.Closed != chatStatus) {
            return new Result(GlobalApiError.OperationFailed, "该通话暂时无法评价");
        }

        // 判断是否已评价
        if (singleChatService.isSingleChatEvaluateExists(hisId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "您已经评价过啦!");
        }

        // 判断积分是否有效
        if (scoreValue < 0 || scoreValue > 5) {
            return new Result(GlobalApiError.OperationFailed, "评分无效");
        }

        // 判断标签
        if (StringUtils.isNotBlank(tagIds)) {
            List<Long> tagIdList = Arrays.stream(tagIds.split(",")).map(Convert::toLong).collect(Collectors.toList());
            if (!dataService.isSingleChatTagIdsExists(tagIdList)) {
                return new Result(GlobalApiError.OperationFailed, "评价中含有无效标签");
            }
        }

        boolean isAnchor = userId == singleChatHis.getTargetUserId();
        long targetUserId = singleChatHis.getTargetUserId();
        if (isAnchor) {
            targetUserId = userId;
            userId = singleChatHis.getTargetUserId();
        }
        if (singleChatService.addSingleChatEvaluate(hisId, userId, targetUserId, scoreValue, tagIds, isAnchor)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取单聊推荐列表
     *
     * @return
     */
    @GET
    @Path("getSingleChatSuggestList")
    @Override
    public Result getSingleChatSuggestList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", singleChatService.getSingleChatSuggestList(6, this.getToken().getUserId(), SingleChatStatus.IDLE, BoolIntVal.True, this.getHeader().getChannelId()));
        return mResult.build();
    }

    /**
     * 获取单聊主播列表
     *
     * @param pageIndex
     * @param userSex
     * @param categoryId
     * @return
     */
    @GET
    @Path("getSingleChatList")
    @Override
    public Result getSingleChatList(@QueryParam("pageIndex") int pageIndex,
                                    @QueryParam("userSex") Integer userSex,
                                    @QueryParam("categoryId") Long categoryId) {
        return new Result(singleChatService.getSingleChatList(pageIndex, GlobalConfig.PAGE_SIZE, userSex, categoryId, null, null, this.getHeader().getChannelId()));
    }

    /**
     * 获取关注的单聊主播列表
     *
     * @return
     */
    @GET
    @Path("getSingleChatFollowList")
    @Override
    public Result getSingleChatFollowList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", socialRelationService.getSingleChatFollowList(this.getToken().getUserId(), this.getHeader().getChannelId()));
        return mResult.build();
    }

    /**
     * 查询单聊信息详情
     *
     * @param hisId
     * @return
     */
    @GET
    @Path("getSingleChatHisInfo")
    @Override
    public Result getSingleChatHisInfo(@QueryParam("hisId") String hisId) {
        Long userId = this.getToken().getUserId();
        SingleChatHisDO singleChatHis = singleChatService.getSingleChatHis(hisId);
        if (singleChatHis == null || (!singleChatHis.getUserId().equals(userId) && !singleChatHis.getTargetUserId().equals(userId))) {
            return new Result(GlobalApiError.OperationFailed, "未找到记录");
        }

        return new Result(singleChatHis);
    }

    /**
     * 随机获取单聊用户
     *
     * @param userSex
     * @param categoryId
     * @return
     */
    @GET
    @Path("getSingleChatRandom")
    @Override
    public Result getSingleChatRandom(@QueryParam("userSex") Integer userSex,
                                      @QueryParam("categoryId") Long categoryId) {
        Long userId = this.getToken().getUserId();
        return new Result(singleChatService.getSingleChatRandomList(3, userId, userSex, categoryId, SingleChatStatus.IDLE, BoolIntVal.True, this.getHeader().getChannelId()));
    }

    /**
     * 获取附近单聊用户列表
     *
     * @param pageIndex
     * @param userSex
     * @param userAgeMin
     * @param userAgeMax
     * @param categoryId
     * @param areaId
     * @param isNew
     * @return
     */
    @GET
    @Path("getSingleChatNearbyList")
    @Override
    public Result getSingleChatNearbyList(@QueryParam("pageIndex") int pageIndex,
                                          @QueryParam("userSex") Integer userSex,
                                          @QueryParam("userAgeMin") Integer userAgeMin,
                                          @QueryParam("userAgeMax") Integer userAgeMax,
                                          @QueryParam("categoryId") Long categoryId,
                                          @QueryParam("areaId") Integer areaId,
                                          @QueryParam("isNew") int isNew) {
        Double latitude = Convert.toDouble(this.getHeader().getLatitude());
        Double longitude = Convert.toDouble(this.getHeader().getLongitude());
        Double distance = 1000000d;

        if (latitude == null || longitude == null) {
            IpLocation ipLocation = areaService.getIpLocation(this.getIpAddress());
            if (ipLocation != null) {
                latitude = Optional.ofNullable(ipLocation.getCenterLatitude()).orElse(ipLocation.getLbLatitude());
                longitude = Optional.ofNullable(ipLocation.getCenterLongitude()).orElse(ipLocation.getLbLongitude());
            }
            if (latitude == null && longitude == null) {
                return new Result(GlobalApiError.OperationFailed, "未能获取您的位置，请先开启定位后重试！");
            }
        }
        return new Result(singleChatService.getSingleChatNearbyList(pageIndex, GlobalConfig.PAGE_SIZE, longitude, latitude, distance, userSex, categoryId, null, this.getHeader().getChannelId(), 0, userAgeMax, areaId, isNew));
    }

    /**
     * 获取单聊主播详情
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getSingleChatAnchorInfo")
    @Override
    public Result getSingleChatAnchorInfo(@QueryParam("userId") Long userId) {
        SingleChatUserVO singleChatUser = singleChatService.getSingleChatUser(userId);
        if (singleChatUser == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到主播");
        }
        if (BoolIntVal.False == singleChatUser.getIsOnline() || LiveStatusEnum.Living.getValue() == singleChatUser.getLiveStatus()) {
            singleChatUser.setSingleChatStatus(SingleChatStatus.BUSY);
        }
        return new Result(singleChatUser);
    }

    /**
     * 获取单聊匹配记录
     *
     * @return
     */
    @GET
    @Path("getSingleChatMatchedHis")
    @Override
    public Result getSingleChatMatchedHis() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", singleChatService.getSingleChatMatchedHisList(GlobalConfig.PAGE_SIZE));
        return mResult.build();
    }

    /**
     * 刷新单聊提醒
     *
     * @param hisId
     * @return
     */
    @POST
    @Path("refreshSingleChatRemind")
    @Override
    public Result refreshSingleChatRemind(@FormParam("hisId") String hisId) {

        ChatStatusEnum chatStatus = singleChatService.getSingleChatStatus(hisId, null, null);
        if (chatStatus == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到通话记录");
        }

        if (ChatStatusEnum.Calling != chatStatus) {
            return new Result(GlobalApiError.OperationFailed, "已结束的通话");
        }

        Long minutesRemain = singleChatService.refreshSingleChatRemind(hisId);
        if (minutesRemain == null) {
            return new Result(GlobalApiError.OperationFailed);
        }

        MutableResult mResult = new MutableResult();
        mResult.put("minutesRemain", minutesRemain);
        return mResult.build();
    }

    /**
     * 获取vip搜索
     *
     * @param pageIndex            当前页码
     * @param userSex              性别
     * @param userAgeMin           最小年龄
     * @param userAgeMax           最大年龄
     * @param singleChatStatus     单聊状态
     * @param areaId               区域id
     * @param distance             距离
     * @param userConstellation    星座
     * @param userMarkingFriends   交友目的
     * @param userMarriage         婚姻状况
     * @param userHobby            兴趣爱好
     * @param userTags             个性标签
     * @param singleChatCategoryId 单聊分类id
     * @return
     */
    @GET
    @Path("getVipSingleChatList")
    @Override
    public Result getVipSingleChatList(@QueryParam("pageIndex") int pageIndex,
                                       @QueryParam("userSex") Integer userSex,
                                       @QueryParam("userAgeMin") Integer userAgeMin,
                                       @QueryParam("userAgeMax") Integer userAgeMax,
                                       @QueryParam("singleChatStatus") Integer singleChatStatus,
                                       @QueryParam("areaId") Integer areaId,
                                       @QueryParam("distance") Double distance,
                                       @QueryParam("userConstellation") Integer userConstellation,
                                       @QueryParam("userMarkingFriends") String userMarkingFriends,
                                       @QueryParam("userMarriage") String userMarriage,
                                       @QueryParam("userHobby") String userHobby,
                                       @QueryParam("userTags") String userTags,
                                       @QueryParam("singleChatCategoryId") Long singleChatCategoryId) {
        if (!vipService.isVip(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "请先开通会员哦~");
        }

        Double latitude = Convert.toDouble(this.getHeader().getLatitude());
        Double longitude = Convert.toDouble(this.getHeader().getLongitude());

        if (distance != null && (latitude == null || longitude == null)) {
            IpLocation ipLocation = areaService.getIpLocation(this.getIpAddress());
            if (ipLocation != null) {
                latitude = Optional.ofNullable(ipLocation.getCenterLatitude()).orElse(ipLocation.getLbLatitude());
                longitude = Optional.ofNullable(ipLocation.getCenterLongitude()).orElse(ipLocation.getLbLongitude());
            }
            if (latitude == null && longitude == null) {
                return new Result(GlobalApiError.OperationFailed, "未能获取您的位置，请先开启定位后重试！");
            }
        }

        VipSearchDTO searchDTO = new VipSearchDTO();
        searchDTO.setUserSex(userSex);
        searchDTO.setUserAgeMin(userAgeMin);
        searchDTO.setUserAgeMax(userAgeMax);
        searchDTO.setSingleChatStatus(singleChatStatus);
        searchDTO.setAreaId(areaId);
        searchDTO.setDistance(distance);
        searchDTO.setUserMarkingFriends(userMarkingFriends);
        searchDTO.setUserConstellation(userConstellation);
        searchDTO.setUserMarriage(userMarriage);
        searchDTO.setUserHobby(userHobby);
        searchDTO.setUserTags(userTags);
        searchDTO.setSingleChatCategoryId(singleChatCategoryId);
        searchDTO.setPageIndex(pageIndex);
        searchDTO.setPageSize(GlobalConfig.PAGE_SIZE);
        searchDTO.setUserLatitude(latitude);
        searchDTO.setUserLongitude(longitude);

        return new Result(singleChatService.getVipSingleChatList(searchDTO));
    }

    /**
     * 发送头条
     *
     * @param msgContent
     * @return
     */
    @POST
    @Path("sendHeadline")
    @Override
    public Result sendHeadline(@FormParam("msgContent") String msgContent) {

        if (StringUtils.isBlank(msgContent)) {
            return new Result(GlobalApiError.ServerError, "头条不能为空");
        }

        Long userId = this.getToken().getUserId();
        // 判断内容
        if (StringUtils.isNotBlank(msgContent) && wordFilterService.isMatchedPhrases(WordFilterTypeEnum.LIVE_TEXT, msgContent)) {
            return new Result(GlobalApiError.ServerError, "头条中包含敏感词汇，请重新输入");
        }

        // 判断金额是否足够
        long balance = walletService.getBalance(userId, WalletTypeEnum.STAR_DIAMOND) - 1000L;
        MutableResult mResult = new MutableResult();
        if (balance >= 0) {
            if (chatService.sendHeadline(userId, msgContent)) {
                mResult.put("balance", balance);
                return mResult.build();
            } else {
                return new Result(GlobalApiError.ServerError,"余额不足，发送头条需要100钻石");
            }
        }
        mResult.put("balance", balance);
        return mResult.build();

    }

    /**
     * 获取头条
     *
     * @return
     */
    @GET
    @Path("getHeadline")
    @PermitGuest
    @Override
    public Result getHeadline() {
        return new Result(chatService.getHeadline());
    }

    /**
     * 获取头条配置
     *
     * @return
     */
    @GET
    @Path("getHeadlineConfig")
    @PermitGuest
    @Override
    public Result getHeadlineConfig() {
        MutableResult mResult = new MutableResult();
        mResult.put("groupId", "BCR2020");
        mResult.put("price", 1000L);
        mResult.put("walletType", WalletTypeEnum.STAR_DIAMOND.getValue());
        return mResult.build();
    }

    /**
     * 发起语音速配
     *
     * @param userSex
     * @return
     */
    @POST
    @Path("sendVoiceMatch")
    @Override
    public Result sendVoiceMatch(@FormParam("userSex") Integer userSex) {
        Long userId = this.getToken().getUserId();
        Long channelId = this.getHeader().getChannelId();

//        // 判断资料评分是否达到75分
//        Integer profileScore = userService.getProfileScore(userId);
//        if (profileScore < 75){
//            return new Result(GlobalApiError.OperationFailed, String.format("资料完成度超过75%s才能参与匹配哦~\n你当前资料完成度为%d%s", '%', profileScore, '%'));
//        }

        if (!voiceChatService.isUserIdle(userId)) {
            if (voiceChatService.getHisIdByUserId(userId) != null) {
                userService.updateVoiceChatStatus(userId, VoiceChatStatus.IDLE);
                return new Result(GlobalApiError.OperationFailed, "你不是空闲状态");
            }
        }

        VoiceChatRequest chatRequest = new VoiceChatRequest();
        chatRequest.setUserId(userId);
        chatRequest.setUserSex(userSex);
        chatRequest.setChannelId(channelId);

        VoiceChatResponse chatResponse = voiceChatService.sendVoiceMatch(chatRequest);

        if (chatResponse.isSuccess()) {
            MutableResult mResult = new MutableResult();
            mResult.put("hisId", chatResponse.getHisId());
            return mResult.build();
        } else {
            return new Result(GlobalApiError.OperationFailed, chatResponse.getMsg());
        }
    }

    /**
     * 决定语音速配  同意、拒绝
     *
     * @param hisId
     * @param decision
     * @param round
     * @return
     */
    @POST
    @Path("decideVoiceMatch")
    @Override
    public Result decideVoiceMatch(@FormParam("hisId") Long hisId,
                                   @FormParam("decision") Integer decision,
                                   @FormParam("round") int round) {
        if (hisId == null || decision == null || round < 1) {
            return new Result(GlobalApiError.OperationFailed, "缺少必要参数");
        }
        VoiceChatHisDO hisDO = voiceChatService.getChatHis(hisId);
        if (hisDO == null) {
            return new Result(GlobalApiError.OperationFailed, "记录不存在");
        }

        Long userId = this.getToken().getUserId();

        int userRound = voiceChatService.getUserRound(hisId, userId);

        if (userRound < 0) {
            return new Result(GlobalApiError.OperationFailed, "系统未匹配到你");
        }
        log.info("userRound {}", userRound);
        if (userRound != round) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限");
        }

        // 当前轮次
        int curRound = voiceChatService.getCurRound(hisId);
        if (curRound > round) {
            return new Result(GlobalApiError.OperationFailed, "速配已过期");
        }

        ChatStatusEnum statusEnum = ChatStatusEnum.toEnum(hisDO.getHisStatus());
        if (statusEnum == ChatStatusEnum.Closed || statusEnum == ChatStatusEnum.Canceled || statusEnum == ChatStatusEnum.Rejected) {
            return new Result(GlobalApiError.OperationFailed, "速配" + statusEnum.getDesc());
        }

        if (statusEnum == ChatStatusEnum.Calling) {
            return new Result(GlobalApiError.OperationFailed, "速配已被抢");
        }

        ChatResponse chatResponse = null;

        // 同意
        if (decision == BoolIntVal.True) {
            chatResponse = voiceChatService.startChat(hisId, userId, getHeader().getAppVersion());
        } else {
            // 拒绝
            chatResponse = voiceChatService.rejectChat(hisId, this.getToken().getUserId(), round);
        }
        if (chatResponse.isSuccess()) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed, chatResponse.getMsg());
    }

    /**
     * 暂停语音速配
     *
     * @param hisId
     * @return
     */
    @POST
    @Path("cancelVoiceChat")
    @Override
    public Result cancelVoiceChat(@FormParam("hisId") Long hisId) {
        VoiceChatHisDO hisDO = voiceChatService.getChatHis(hisId);
        if (hisDO == null) {
            return new Result(GlobalApiError.OperationFailed, "记录不存在");
        }

        Long userId = this.getToken().getUserId();
        if (!userId.equals(hisDO.getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "只有发起人才能操作");
        }

        ChatStatusEnum statusEnum = ChatStatusEnum.toEnum(hisDO.getHisStatus());
        if (statusEnum != ChatStatusEnum.Ringing) {
            return new Result(GlobalApiError.OperationFailed, "当前速配" + statusEnum.getDesc());
        }

        String chatHandleKey = RedisChatKeys.getVoiceChatHandleLock(hisId);
        String uuid = IdUtil.simpleUUID();
        // 需抢占资源
        ChatResponse chatResponse = null;
        try {
            boolean lock = redisService.setIfAbsent(chatHandleKey, uuid, 10, TimeUnit.SECONDS);
            chatResponse = null;
            if (lock) {
                chatResponse = voiceChatService.cancelVoiceChat(hisId, 0);
            } else {
                return new Result(GlobalApiError.OperationFailed, "太火了,速配正被抢");
            }
        } catch (Exception e) {
            log.error("出现错误", e);
        } finally {
            if (uuid.equals(redisService.get(chatHandleKey))) {
                redisService.delKey(chatHandleKey);
            }
        }
        if (chatResponse != null && chatResponse.isSuccess()) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed, chatResponse != null ? chatResponse.getMsg() : GlobalApiError.OperationFailed.getMsg());
    }

    /**
     * 结束语音速配聊天
     *
     * @param hisId
     * @return
     */
    @POST
    @Path("stopVoiceChat")
    @Override
    public Result stopVoiceChat(@FormParam("hisId") Long hisId) {
        VoiceChatHisDO hisDO = voiceChatService.getChatHis(hisId);
        if (hisDO == null) {
            return new Result(GlobalApiError.OperationFailed, "记录不存在");
        }

        Long curUserId = this.getToken().getUserId();
        Long userId = hisDO.getUserId();
        Long targetUserId = hisDO.getTargetUserId();

        if (!curUserId.equals(userId) && !curUserId.equals(targetUserId)) {
            return new Result(GlobalApiError.OperationFailed, "你没有权限，只有会话者才能操作");
        }

        ChatStatusEnum statusEnum = ChatStatusEnum.toEnum(hisDO.getHisStatus());
        if (statusEnum != ChatStatusEnum.Ringing && statusEnum != ChatStatusEnum.Calling) {
            return new Result(GlobalApiError.OperationFailed, "当前速配" + statusEnum.getDesc());
        }
        VoiceChatStopReasonEnum reasonEnum = curUserId.equals(userId) ? VoiceChatStopReasonEnum.UserClose : VoiceChatStopReasonEnum.TargetUserClose;

        long curTime = System.currentTimeMillis();

        ChatResponse chatResponse = voiceChatService.stopChat(hisId, reasonEnum, curTime, 0, getHeader().getAppVersion());
        if (chatResponse.isSuccess()) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed, chatResponse.getMsg());
    }

    /**
     * 获取语音速配配置
     *
     * @return
     */
    @GET
    @Path("getVoiceChatConfig")
    @PermitGuest
    @Override
    public Result getVoiceChatConfig() {
        long curTime = System.currentTimeMillis();
        int hasFreeTime = 0;
        if (getToken() != null) {
            Long userId = this.getToken().getUserId();
            hasFreeTime = voiceChatService.hasFreeTime(userId, curTime) ? 60 : 0;
        }

        MutableResult mResult = new MutableResult();
        mResult.put("freeTime", hasFreeTime);
        mResult.put("voiceChatPrice", 100L);
        mResult.put("voiceChatPriceName", WalletTypeEnum.STAR_DIAMOND.getName());
        return mResult.build();
    }

    /**
     * 获取语音速配详情
     *
     * @return
     */
    @GET
    @Path("getVoiceChatDetail")
    @Override
    public Result getVoiceChatDetail(@QueryParam("hisId") Long hisId) {
        VoiceChatHisDO hisDO = voiceChatService.getChatHis(hisId);
        if (hisDO == null) {
            return new Result(GlobalApiError.OperationFailed, "记录不存在");
        }

        Long curUserId = this.getToken().getUserId();

        Long userId = hisDO.getUserId();
        Long targetUserId = hisDO.getTargetUserId();
        UserCardVO userCard = userService.getCard(userId);
        UserCardVO targetUserCard = userService.getCard(targetUserId);

        MutableResult mResult = new MutableResult();
        mResult.put("skillName", "语音速配");
        mResult.put("userId", userId);
        mResult.put("userName", Optional.ofNullable(userCard).map(UserCardVO::getUserName).orElse(""));
        mResult.put("userAvatar", Optional.ofNullable(userCard).map(UserCardVO::getUserAvatar).orElse(""));
        mResult.put("targetUserId", targetUserId);
        mResult.put("targetUserName", Optional.ofNullable(targetUserCard).map(UserCardVO::getUserName).orElse(""));
        mResult.put("targetUserAvatar", Optional.ofNullable(targetUserCard).map(UserCardVO::getUserAvatar).orElse(""));
        mResult.put("hisStartTime", hisDO.getHisStartTime());
        mResult.put("hisStopTime", hisDO.getHisStopTime());
        mResult.put("stopReason", hisDO.getStopReason());
        mResult.put("stopReasonName", hisDO.getStopReason() == null ? "" : VoiceChatStopReasonEnum.toEnum(hisDO.getStopReason()).getDesc());
        mResult.put("hisStatus", hisDO.getHisStatus());
        mResult.put("hisCoinMinutes", hisDO.getHisCoinMinutes());
        mResult.put("hisTimeMinutes", hisDO.getHisTimeMinutes());
        mResult.put("hisCreateTime", hisDO.getHisCreateTime());
        mResult.put("matchStartTime", hisDO.getMatchStartTime());
        mResult.put("matchEndTime", hisDO.getMatchEndTime());
        mResult.put("hisPrice", hisDO.getHisPrice());
        mResult.put("hisPriceType", hisDO.getHisPriceType());
        mResult.put("hisPriceTypeName", WalletTypeEnum.toEnum(hisDO.getHisPriceType()).getName());
        mResult.put("consumeAmount", hisDO.getHisCoinMinutes() == null ? 0L : hisDO.getHisCoinMinutes() * hisDO.getHisPrice());
        mResult.put("followStatus", socialRelationService.isFollower(curUserId, userId));
        mResult.put("targetFollowStatus", socialRelationService.isFollower(curUserId, targetUserId));
        mResult.put("profile", userService.getChatUserProfile(userId, String.valueOf(hisId)));

        return mResult.build();
    }

    /**
     * 获取语音速配token
     *
     * @param hisId
     * @return
     */
    @GET
    @Path("getVoiceChatToken")
    @Override
    public Result getVoiceChatToken(@QueryParam("hisId") Long hisId) {
        VoiceChatHisDO hisDO = voiceChatService.getChatHis(hisId);
        if (hisDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到记录");
        }
        Long curUserId = this.getToken().getUserId();
        Long userId = hisDO.getUserId();
        Long targetUserId = hisDO.getTargetUserId();

        String token = "";
        // 用户
        if (curUserId.equals(userId)) {
            token = voiceChatService.getVoiceChatToken(Convert.toStr(hisId), curUserId, false, getHeader().getAppVersion());
        } else if (curUserId.equals(targetUserId)) {
            // 目标用户
            token = voiceChatService.getVoiceChatToken(Convert.toStr(hisId), curUserId, true, getHeader().getAppVersion());
        } else {
            return new Result(GlobalApiError.OperationFailed, "您没有权限");
        }
        if (StringUtils.isNotEmpty(token)) {
            MutableResult mResult = new MutableResult();
            mResult.put("token", token);
            return mResult.build();
        }
        return new Result(GlobalApiError.OperationFailed, "获取token失败");
    }

    /**
     * 获取语音速配列表
     *
     * @param pageIndex
     * @param hisStatus
     * @return
     */
    @GET
    @Path("getVoiceChatList")
    @Override
    public Result getVoiceChatList(@QueryParam("pageIndex") Integer pageIndex, @QueryParam("hisStatus") Integer hisStatus) {
        PageList<VoiceChatDTO> list = voiceChatService.getVoiceChatList(pageIndex, GlobalConfig.PAGE_SIZE, hisStatus, this.getToken().getUserId());
        return new Result(list);
    }

    /**
     * 获取匹配成功记录
     *
     * @return
     */
    @GET
    @Path("getMatchSuccessList")
    @PermitGuest
    @Override
    public Result getMatchSuccessList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", voiceChatService.getMatchSuccessList());
        return mResult.build();
    }

    /**
     * 获取符合条件用户
     *
     * @param userSex
     * @return
     */
    @GET
    @Path("getMatchUserList")
    @PermitGuest
    @Override
    public Result getMatchUserList(@QueryParam("userSex") Integer userSex) {
        MutableResult mResult = new MutableResult();
//        mResult.put("list", voiceChatService.getMatchUserList(userSex, getToken() == null ? 0 : getToken().getUserId()));
        mResult.put("list", voiceChatService.getMatchPeipiList(userSex));
        return mResult.build();
    }
    /**
     *
     * 拉取公聊大厅消息记录
     *
     * @param groupId
     * @param pageSize
     * @param lastMsgId
     * @return
     */
    @GET
    @Path("getPublicChatRoomMsgHis")
    @Override
    public Result getPublicChatRoomMsgHis(@QueryParam("groupId") String groupId,
                                          @QueryParam("pageSize") int pageSize,
                                          @QueryParam("lastMsgId") Integer lastMsgId) {
        if (pageSize <= 0 || pageSize > 50) {
            pageSize = 50;
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", imService.getPublicChatRoomMsgHisList(groupId, pageSize, lastMsgId));
        return mResult.build();
    }
}
