package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.rtc.RtcRoomBackgroundVO;
import com.starbuds.server.api.app.v1.facade.ConfigFacade;
import com.starbuds.server.common.constant.biz.BannerPositionEnum;
import com.starbuds.server.common.constant.data.PosterImageTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.PosterImageVO;
import com.starbuds.server.common.pojo.db.DeviceDO;
import com.starbuds.server.common.pojo.db.PosterImageDO;
import com.starbuds.server.common.pojo.db.UserFirstRechargeRemindDO;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.i18n.I18nService;
import com.starbuds.server.common.service.rtc.RtcBattleService;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.common.service.user.DeviceService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 系统接口实现
 *
 * @author Lay
 * @date 2019/12/17
 */
@Path("/v1/config")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ConfigFacadeImpl extends BaseFacade implements ConfigFacade {

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    MediaFileService mediaFileService;

    @Reference(version = "1.0.0")
    PayService payService;

    @Reference(version = "1.0.0")
    DeviceService deviceService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    I18nService i18nService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    RtcBattleService rtcBattleService;

    @Reference(version = "1.0.0")
    ChatBubbleService chatBubbleService;

    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    EntranceService entranceService;

    @Value("${h5.rtcRankUrl}")
    private String rtcRankUrl;

    @Value("${h5.rtcRankUrlHome}")
    private String rtcRankUrlHome;

    @Value("${h5.giftRankUrl}")
    private String giftRankUrl;

    @Value("${h5.rtcHlRankUrl}")
    private String rtcHlRankUrl;

    @Value("${h5.giftWallUrl}")
    private String giftWallUrl;

    @Value("${h5.activityRuleUrl}")
    private String activityRuleUrl;

    @Value("${h5.rtcRoomStreamUrl}")
    private String rtcRoomStreamUrl;

    @Value("${h5.transDiamondUrl}")
    private String transDiamondUrl;

    @Value("${h5.productDetailsUrl}")
    private String productDetailsUrl;

    @Value("${h5.productOrdersUrl}")
    private String productOrdersUrl;

    @Value("${h5.shippingAddressUrl}")
    private String shippingAddressUrl;

    @Value("${music.url}")
    private String musicUrl;

    /**
     * 获取初始化配置
     *
     * @return
     */
    @GET
    @Path("getInitConfig")
    @PermitGuest
    @Override
    public Result getInitConfig() {
        DeviceDO deviceDO = new DeviceDO();
        deviceDO.setChannelId(this.getHeader().getChannelId());
        deviceDO.setClientType(this.getHeader().getClientType().getValue());
        deviceDO.setDeviceId(this.getHeader().getDeviceId());
        deviceDO.setIdfa(this.getHeaderString("idfa"));
        deviceDO.setImeiMd5(this.getHeaderString("imei"));
        deviceService.addDevice(deviceDO);
        MutableResult mResult = new MutableResult();
        mResult.put("channelConfig", channelService.getChannelInitConfig(this.getHeader().getChannelId(), this.getHeader().getClientType().getValue(), this.getHeader().getAppVersion(), this.getHeader().getPackageName()));
        mResult.put("priceConfig", payService.getEnabledPriceConfig());
        mResult.put("forbiddenWordOptions", dataService.getForbiddenWordOptions());
        mResult.put("liveRates", dataService.getLiveRates());
        mResult.put("rtcRankUrl", rtcRankUrl);
        mResult.put("rtcRankUrlHome", rtcRankUrlHome);
        mResult.put("giftRankUrl", giftRankUrl);
        mResult.put("rtcHlRankUrl", rtcHlRankUrl);
        mResult.put("activityRuleUrl", activityRuleUrl);
        mResult.put("giftWallUrl", giftWallUrl);
        mResult.put("rtcRoomStreamUrl", rtcRoomStreamUrl);
        mResult.put("transDiamondUrl", transDiamondUrl);
        mResult.put("musicUrl", musicUrl);
        mResult.put("productDetailsUrl", productDetailsUrl);
        mResult.put("productOrdersUrl", productOrdersUrl);
        mResult.put("shippingAddressUrl", shippingAddressUrl);
        mResult.put("watchRankUrl", releaseConfigService.getCachReleaseConfig("watchRankUrl"));
        mResult = (MutableResult) i18nService.doI18n(mResult, lang(), 6);
        return mResult.build();
    }

    /**
     * 获取动效
     *
     * @return
     */
    @GET
    @Path("getAnimations")
    @PermitGuest
    @Override
    public Result getAnimations() {
        MutableResult mResult = new MutableResult();
        String appVersion = this.getHeader().getAppVersion();
        int ver = appVersion != null ? Integer.parseInt(StringUtils.replace(appVersion, ".", "")) : 100;
        mResult.put("list", mediaFileService.getAnimations(ver));
        return mResult.build();
    }

    /**
     * 获取举报原因
     *
     * @return
     */
    @GET
    @Path("getReportReasons")
    @PermitGuest
    @Override
    public Result getReportReasons() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(dataService.getReportReasonList(), lang()));
        return mResult.build();
    }

    /**
     * 获取礼物标签
     *
     * @return
     */
    @GET
    @Path("getGiftTags")
    @PermitGuest
    @Override
    public Result getGiftTags() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", mediaFileService.getGiftTags());
        return mResult.build();
    }

    /**
     * 获取 Banner
     *
     * @param bannerPosition
     * @param categoryId
     * @return
     */
    @GET
    @Path("getBanners")
    @PermitGuest
    @Override
    public Result getBanners(@QueryParam("bannerPosition") Integer bannerPosition,
                             @QueryParam("categoryId") Long categoryId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", dataService.getBanners(bannerPosition, categoryId, this.getHeader().getChannelId()));
        return mResult.build();
    }

    /**
     * 获取金币类型名称
     *
     * @return
     */
    @GET
    @Path("getCoinTypeNameMap")
    @PermitGuest
    @Override
    public Result getCoinTypeNameMap() {
        return new Result(walletService.getWalletTypeNameMap());
    }

    /**
     * 获取单聊评价标签
     *
     * @param tagType
     * @return
     */
    @GET
    @Path("getSingleChatEvaluateTags")
    @PermitGuest
    @Override
    public Result getSingleChatEvaluateTags(@QueryParam("tagType") Integer tagType) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(dataService.getSingleChatEvaluateTags(tagType), lang()));
        return mResult.build();
    }

    /**
     * 获取用户标签
     *
     * @param tagType
     * @return
     */
    @GET
    @Path("getUserTags")
    @PermitGuest
    @Override
    public Result getUserTags(@QueryParam("tagType") Integer tagType) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(dataService.getUserTags(tagType), lang()));
        return mResult.build();
    }

    /**
     * 获取兴趣标签
     *
     * @param tagType
     * @return
     */
    @GET
    @Path("getHobbyTags")
    @PermitGuest
    @Override
    public Result getHobbyTags(@QueryParam("tagType") Integer tagType) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(dataService.getHobbyTags(tagType), lang()));
        return mResult.build();
    }

    /**
     * 获取海报图
     *
     * @return
     */
    @GET
    @Path("getPosterImages")
    @Override
    public Result getPosterImages() {

        Long userId = this.getToken().getUserId();
        Long userNo = userService.getUserNo(userId);
        String inviteCode = Long.toHexString(userNo);
        String qs = String.format("?ci=%d&i=%s", this.getHeader().getChannelId(), inviteCode);

        List<PosterImageDO> posterImages = dataService.getPosterImages(this.getHeader().getChannelId(), PosterImageTypeEnum.INVITE_FRIENDS.getCode());
        List<PosterImageVO> list = new ArrayList<>();
        for (PosterImageDO posterImage : posterImages) {
            PosterImageVO image = new PosterImageVO();
            BeanUtils.copyProperties(posterImage, image);
            image.setPosterLink(image.getPosterLink() + qs);
            image.setInviteCode(inviteCode);
            list.add(image);
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", list);
        return mResult.build();
    }

    /**
     * 获取开屏广告
     *
     * @return
     */
    @GET
    @Path("getOpenScreenBanners")
    @PermitGuest
    @Override
    public Result getOpenScreenBanners() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", dataService.getOpenScreenBanners(BannerPositionEnum.OPEN_SCREEN.getCode(), null, this.getHeader().getChannelId()));
        return mResult.build();
    }

    /**
     * 获取语聊房壁纸列表
     *
     * @return
     */
    @GET
    @Path("getRtcRoomBackgrounds")
    @PermitGuest
    @Override
    public Result getRtcRoomBackgrounds(@QueryParam("roomId") Long roomId) {
        roomId = Optional.ofNullable(roomId).orElse(0L);

        List<RtcRoomBackgroundVO> searchBackgroundList = new ArrayList<>();
        dataService.getRtcRoomBackgrounds(roomId).forEach(m -> {
            RtcRoomBackgroundVO backgroundVO = new RtcRoomBackgroundVO();
            BeanUtils.copyProperties(m, backgroundVO);
            searchBackgroundList.add(backgroundVO);
        });

        MutableResult mResult = new MutableResult();
        mResult.put("list", searchBackgroundList);
        return mResult.build();
    }

    /**
     * 获取技能列表
     *
     * @return
     */
    @GET
    @Path("getSkills")
    @PermitGuest
    @Override
    public Result getSkills() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", skillService.getSkills());
        return mResult.build();
    }

    /**
     * 获取房间内 PK 配置
     *
     * @return
     */
    @GET
    @Path("getBattleRoomInConfig")
    @Override
    public Result getBattleRoomInConfig() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcBattleService.getRoomInConfigList());
        return mResult.build();
    }

    /**
     * 获取聊天气泡框列表
     *
     * @return
     */
    @GET
    @Path("getBubbleList")
    @PermitGuest
    @Override
    public Result getBubbleList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", chatBubbleService.getBubbleList());
        return mResult.build();
    }

    /**
     * 获取勋章列表
     *
     * @return
     */
    @GET
    @Path("getMedalList")
    @PermitGuest
    @Override
    public Result getMedalList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", medalService.getMedalList());
        return mResult.build();
    }

    /**
     * 获取活动勋章列表
     *
     * @return
     */
    @GET
    @Path("getActivitiesMedalList")
    @PermitGuest
    @Override
    public Result getActivitiesMedalList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", dataService.getActivitiesMedalList());
        return mResult.build();
    }

    /**
     * 获取入口
     *
     * @return
     */
    @GET
    @Path("getEntrances")
    @PermitGuest
    @Override
    public Result getEntrances() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", entranceService.getEntrances());
        return mResult.build();
    }

    /**
     * 获取挂件
     *
     * @return
     */
    @GET
    @Path("getConfigWidget")
    @PermitGuest
    @Override
    public Result getConfigWidget() {
        String widgetConfig = releaseConfigService.getReleaseConfig("widget");
        if (StringUtils.isBlank(widgetConfig)) {
            return new Result();
        }

        long currentTimestamp = System.currentTimeMillis();
        JSONObject widgetJSON = JSON.parseObject(widgetConfig);
        boolean display = currentTimestamp > widgetJSON.getLongValue("st") && currentTimestamp < widgetJSON.getLongValue("et");

        MutableResult mResult = new MutableResult();
        mResult.put("img", widgetJSON.getString("img"));
        mResult.put("path", widgetJSON.getString("path"));
        mResult.put("display", display);

        return mResult.build();
    }


    /**
     * 获取世界消息列表
     *
     * @return
     */
    @Override
    @GET
    @Path("getRoomMsgTypes")
    public Result getRoomMsgTypes() {
        return Result.ofList(dataService.getRoomTypeList());
    }

    /**
     * 获取首充入口
     *
     * @return
     */
    @GET
    @Path("getFirstRechargeEntry")
    @Override
    public Result getFirstRechargeEntry() {
        long userId = this.getToken().getUserId();
        long currentTimestamp = System.currentTimeMillis();
        boolean hasRecharged = userService.hasRecharged(userId);
        if (hasRecharged) {
            MutableResult result = new MutableResult();
            result.put("systemTimestamp", currentTimestamp);
            result.put("endTimestamp", currentTimestamp);
            result.put("display", false);
            return result.build();
        }

        //新用户不弹***(产品并没有明确什么是新用户，暂不加入逻辑)

        String appVersion = this.getHeader().getAppVersion();
        Long ver = StringUtils.isNotBlank(appVersion) ? Long.parseLong(StringUtils.replace(appVersion, ".", "")) : 100L;

        UserFirstRechargeRemindDO userFirstRechargeRemindDO = userService.getUserFirstRechargeRemind(userId, ver);
        if (userFirstRechargeRemindDO == null) {
            userFirstRechargeRemindDO = new UserFirstRechargeRemindDO();
            userFirstRechargeRemindDO.setUserId(userId);
            userFirstRechargeRemindDO.setVersion(ver);
            userFirstRechargeRemindDO.setStartTimestamp(currentTimestamp);
            userFirstRechargeRemindDO.setEndTimestamp(currentTimestamp + 24 * 60 * 60 * 1000);
            userService.addUserFirstRechargeRemind(userFirstRechargeRemindDO);
        }

        boolean display = currentTimestamp > userFirstRechargeRemindDO.getStartTimestamp() && currentTimestamp < userFirstRechargeRemindDO.getEndTimestamp();
        MutableResult result = new MutableResult();
        result.put("systemTimestamp", currentTimestamp);
        result.put("endTimestamp", userFirstRechargeRemindDO.getEndTimestamp());
        result.put("display", display);
        return result.build();
    }

    /**
     * 获取首充礼包档次数据
     *
     * @return
     */
    @GET
    @Path("getFirstRechargePackageGrade")
    @Override
    public Result getFirstRechargePackageGrade() {
        String firstRechargeGradeValue = releaseConfigService.getCachReleaseConfig("first_recharge_grade");
        if (StringUtils.isBlank(firstRechargeGradeValue)) {
            return new Result(new ArrayList<>());
        }
        return new Result(JSONArray.parse(firstRechargeGradeValue));
    }
}
