package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.v1.facade.FeedPhotoWallFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.feed.FeedStatusEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.FeedPhotoWallDO;
import com.starbuds.server.common.pojo.feed.FeedPhotoWallVO;
import com.starbuds.server.common.service.feed.FeedPohtoWallService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.List;

/**
 * 照片墙
 *
 * @author aoe
 * @date 2021/1/29
 */
@Path("/v1/feed")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FeedPhotoWallFacadeImpl extends BaseFacade implements FeedPhotoWallFacade {

    @Reference(version = "1.0.0")
    private FeedPohtoWallService feedPohtoWallService;

    /**
     * 创建照片墙
     *
     * @param url
     * @return
     */
    @POST
    @Path("createPhotoWall")
    @Override
    public Result createPhotoWall(@FormParam("url") String url) {
        if (StringUtils.isBlank(url)) {
            return new Result(GlobalApiError.BadRequest, "url不能为空");
        }
        Long userId = this.getToken().getUserId();
        if (feedPohtoWallService.getPhotoWallCount(userId) > 16) {
            return new Result(GlobalApiError.BadRequest, "照片墙最多上传16张");
        }
        FeedPhotoWallDO fpw = new FeedPhotoWallDO();
        fpw.setUserId(userId);
        fpw.setUrl(url);
        fpw.setSort(0);
        fpw.setIsDeleted(BoolIntVal.False);

        FeedPhotoWallVO vo = feedPohtoWallService.createPhotoWall(fpw, this.getHeader().getChannelId());
        if (vo == null) {
            return new Result(GlobalApiError.OperationFailed, "创建失败");
        }
        return new Result(vo);
    }

    /**
     * 获得照片墙列表
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getPhotoWallList")
    @Override
    public Result getPhotoWallList(@QueryParam("userId") Long userId) {
        Integer status = userId == null ? null : FeedStatusEnum.PASSED.value();
        userId = userId == null ? this.getToken().getUserId() : userId;
        List<FeedPhotoWallVO> result = feedPohtoWallService.getPhotoWallList(userId, status);
        return new Result(result);
    }

    /**
     * 删除照片墙
     *
     * @param id
     * @return
     */
    @POST
    @Path("deletePhotoWall")
    @Override
    public Result deletePhotoWall(@FormParam("id") Long id) {
        if (id == null || id < 1) {
            return new Result(GlobalApiError.BadRequest, "无效的id");
        }
        return new Result(feedPohtoWallService.deletePhotoWall(id, this.getToken().getUserId()));
    }

}
