package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.pojo.activity.*;
import com.starbuds.server.api.app.v1.facade.FestivalActivityFacade;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.activity.*;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.MedalVO;
import com.starbuds.server.common.pojo.data.MountsVO;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.service.activity.ActivityFestivalService;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.data.MedalService;
import com.starbuds.server.common.service.data.MountsService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.common.utils.LogUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 节日活动
 *
 * @author lujiao
 * @date 2021/9/2 14:55
 */
@Path("/v1/activity")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FestivalActivityFacadeImpl extends BaseFacade implements FestivalActivityFacade {

    @Reference(version = "1.0.0")
    private ActivityFestivalService activityFestivalService;

    @Reference(version = "1.0.0")
    private ActivityService activityService;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private BagService bagService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Reference
    private MountsService mountsService;

    @Reference(version = "1.0.0")
    private UserService userService;

    /**
     * 获取中秋详情
     *
     * @param activityId
     * @return
     */
    @Override
    @Path("getMiddleAutumnFestivalDetail")
    @GET
    public Result getMiddleAutumnFestivalDetail(@QueryParam("activityId") Long activityId) {
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null) {
            return Result.ofBadRequest("未找到该活动");
        }
        Long userId = this.getToken().getUserId();
        MiddleAutumnFestivalVo middleAutumnFestivalVo = new MiddleAutumnFestivalVo();
        middleAutumnFestivalVo.setStartTime(activityDO.getActivityBeginTime());
        middleAutumnFestivalVo.setEndTime(activityDO.getActivityEndTime());
        buildGiftData(activityDO, middleAutumnFestivalVo, userId);
        fullGiftData(middleAutumnFestivalVo);
        return Result.ofSuccess(middleAutumnFestivalVo);
    }


    /**
     * 填充活动里礼物数据
     *
     * @param middleAutumnFestivalVo
     * @return
     */
    private MiddleAutumnFestivalVo fullGiftData(MiddleAutumnFestivalVo middleAutumnFestivalVo) {
        Map<Long, GiftDO> giftDOMap = giftService.getGiftMapByGiftIds(middleAutumnFestivalVo.getGiftIds());
        if (CollectionUtil.isNotEmpty(middleAutumnFestivalVo.getExchangeVoList())) {
            middleAutumnFestivalVo.getExchangeVoList().forEach(i -> {
                GiftDO gift = giftDOMap.get(i.getGiftId());
                if (gift != null) {
                    i.setImgUrl(gift.getGiftIcon());
                    i.setGiftName(gift.getGiftName());
                }

                if (CollectionUtil.isNotEmpty(i.getFragmentVoList())) {
                    i.getFragmentVoList().forEach(frag -> {
                        GiftDO fragItem = giftDOMap.get(frag.getGiftId());
                        if (fragItem != null) {
                            frag.setGiftName(fragItem.getGiftName());
                            frag.setImgUrl(fragItem.getGiftIcon());
                        }
                    });
                    boolean canExchange = true;
                    for (ActivityGiftFragmentVo fragmentVo : i.getFragmentVoList()) {
                        if (fragmentVo.getOwnNum() < fragmentVo.getCostNum()) {
                            canExchange = false;
                        }
                    }
                    i.setCanExchange(canExchange ? BoolIntVal.True : BoolIntVal.False);
                }
            });
        }
        return middleAutumnFestivalVo;
    }

    /**
     * 构建抽奖兑换数据
     *
     * @param activityDO
     * @param middleAutumnFestivalVo
     * @return
     */
    private MiddleAutumnFestivalVo buildGiftData(ActivityDO activityDO, MiddleAutumnFestivalVo middleAutumnFestivalVo, Long userId) {
        if (activityDO != null && StringUtils.isNotBlank(activityDO.getAssociatedActivities())) {
            List<AssociatedActivityItem> associatedActivityItems = JSON.parseArray(activityDO.getAssociatedActivities(), AssociatedActivityItem.class);
            if (CollectionUtil.isNotEmpty(associatedActivityItems)) {
                List<Long> collectActivityIds = new ArrayList<>();
                associatedActivityItems.forEach(i -> {
                    if (i.getItemType() == ActivityTypeEnum.COLLECT.getValue()) {
                        collectActivityIds.add(i.getItemId());
                    }
                });
                List<ActivityCollectDO> activityCollectDOList = activityFestivalService.getActivityCollectList(collectActivityIds);
                middleAutumnFestivalVo.setAssociatedActivities(activityDO.getAssociatedActivities());
                middleAutumnFestivalVo.setFragments(activityDO.getFragments());
                middleAutumnFestivalVo.setExchangeVoList(buildExchangeGiftList(activityCollectDOList, middleAutumnFestivalVo, userId, activityDO));
            }
        }
        return middleAutumnFestivalVo;
    }

    /**
     * 收集兑换列表
     *
     * @param activityCollectDOList
     * @return
     */
    private List<ActivityExchangeVo> buildExchangeGiftList(List<ActivityCollectDO> activityCollectDOList, MiddleAutumnFestivalVo middleAutumnFestivalVo, Long userId, ActivityDO activityDO) {
        List<ActivityExchangeVo> res = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(activityCollectDOList)) {
            activityCollectDOList.forEach(i -> {
                ActivityExchangeVo tmp = new ActivityExchangeVo();
                tmp.setActivityId(i.getActivityId());
                tmp.setCollectId(i.getCollectId());
                if (StringUtils.isNotBlank(i.getCollectGifts()) && StringUtils.isNotBlank(i.getRewards())) {
                    List<GiftItem> giftItemList = JSON.parseArray(i.getCollectGifts(), GiftItem.class);
                    List<RewardItem> rewardItemList = JSON.parseArray(i.getRewards(), RewardItem.class);
                    List<ActivityGiftFragmentVo> fragmentVoList = new ArrayList<>();
                    List<ActivityGiftFragmentVo> targetList = new ArrayList<>();
                    List<Long> wallGiftIds = new ArrayList<>();
                    giftItemList.forEach(giftItem -> {
                        ActivityGiftFragmentVo fragmentVo = new ActivityGiftFragmentVo();
                        fragmentVo.setCostNum(giftItem.getQuantity());
                        fragmentVo.setGiftId(giftItem.getGiftId());
                        wallGiftIds.add(giftItem.getGiftId());
                        middleAutumnFestivalVo.getGiftIds().add(giftItem.getGiftId());
                        fragmentVoList.add(fragmentVo);
                    });
                    Map<Long, Integer> wallGiftNumMap = activityFestivalService.getWallGiftNumMap(wallGiftIds, userId, activityDO, i.getCollectId());
                    fragmentVoList.forEach(giftItem -> {
                        Integer num = wallGiftNumMap.get(giftItem.getGiftId());
                        if (num == null) {
                            num = 0;
                        }
                        giftItem.setOwnNum(num);
                    });
                    rewardItemList.forEach(rewardItem -> {
                        rewardItem.getBagItems().forEach(bagItem -> {
                            ActivityGiftFragmentVo reward = new ActivityGiftFragmentVo();
                            reward.setCostNum(bagItem.getQuantity());
                            reward.setGiftId(bagItem.getItemId());

                            BagItemTypeEnum bagItemTypeEnum = BagItemTypeEnum.toEnum(bagItem.getItemType());
                            switch (bagItemTypeEnum) {
                                case GIFT:
                                    GiftItemVO gift = giftService.getGift(bagItem.getItemId());
                                    if (gift != null) {
                                        reward.setGiftName(gift.getGiftName());
                                        reward.setImgUrl(gift.getGiftIcon());
                                    }
                                    break;
                                case PROPS:
                                    PropsVO props = propsService.getProps(bagItem.getItemId());
                                    if (props != null) {
                                        reward.setGiftName(props.getPropsName());
                                        reward.setImgUrl(props.getPropsIcon());
                                    }
                                    break;
                                case MEDAL:
                                    MedalVO medal = medalService.getMedal(bagItem.getItemId());
                                    if (medal != null) {
                                        reward.setGiftName(medal.getMedalName());
                                        reward.setImgUrl(medal.getMedalImage());
                                    }
                                    break;
                                case MOUNTS:
                                    MountsVO mounts = mountsService.getMounts(bagItem.getItemId());
                                    if (mounts != null) {
                                        reward.setGiftName(mounts.getMountsName());
                                        reward.setImgUrl(mounts.getMountsPreview());
                                    }
                                    break;
                                default:
                                    break;
                            }
                            targetList.add(reward);
                        });
                    });
                    tmp.setFragmentVoList(fragmentVoList);
                    tmp.setTargetList(targetList);
                    res.add(tmp);
                }
            });
        }
        return res;
    }

    /**
     * 获取碎片拥有数量
     *
     * @param ids
     * @return
     */
    @POST
    @Override
    @Path("getGiftOwnNum")
    public Result getGiftOwnNum(@FormParam("ids") String ids) {
        List<Long> giftIds = Arrays.stream(ids.split(",")).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        Long userId = this.getToken().getUserId();
        Map<Long, Integer> ownMap = bagService.getGiftOwnNumMap(giftIds, userId);
        Map<Long, String> bagItemMap = activityFestivalService.getGiftBagMap(giftIds, userId);
        List<GiftOwnVo> res = new ArrayList<>(giftIds.size());
        for (Long giftId : giftIds) {
            GiftOwnVo tmp = new GiftOwnVo();
            tmp.setGiftId(giftId);
            Integer ownNUm = ownMap.get(giftId);
            String bagItemId = bagItemMap.get(giftId);
            tmp.setOwnNum(ownNUm == null ? 0 : ownNUm);
            tmp.setBagItemId(bagItemId == null ? "" : bagItemId);
            res.add(tmp);
        }
        return Result.ofSuccess(res);
    }

    /**
     * 兑换碎片
     *
     * @param activityId
     * @param bagItemId
     * @return
     */
    @Override
    @Path("composeByFragment")
    @POST
    public Result composeByFragment(@FormParam("activityId") Long activityId, @FormParam("bagItemId") String bagItemId) {
        // 首先检查是否活动开启
        ActivityDO activityDO = activityService.getActivity(activityId);
        Long userId = this.getToken().getUserId();
        Result judge = judgeActivity(activityDO, userId);
        if (!judge.isSuccess()) {
            return judge;
        }
        if (bagService.addGiftByFragment(bagItemId, userId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed, "礼物合成失败");
    }

    /**
     * 兑换奖励
     *
     * @param activityId 主活动
     * @param collectId  收集活动
     * @return
     */
    @Override
    @Path("receiveActivityReward")
    @POST
    public Result receiveActivityReward(@FormParam("activityId") Long activityId, @FormParam("collectId") String collectId) {
        // 首先检查是否活动开启
        ActivityDO activityDO = activityService.getActivity(activityId);
        Long userId = this.getToken().getUserId();
        Result judge = judgeActivity(activityDO, userId);
        if (!judge.isSuccess()) {
            return judge;
        }
        return activityFestivalService.receiveActivityReward(collectId, userId, activityDO);
    }

    /**
     * 活动判断
     *
     * @param activityDO
     * @return
     */
    private Result judgeActivity(ActivityDO activityDO, Long userId) {
        Long now = System.currentTimeMillis();
        if (activityDO == null) {
            return Result.ofBadRequest("未找到该活动");
        }
        if (activityDO.getActivityBeginTime() > now) {
            return Result.ofBadRequest("活动尚未开始");
        }
        if (activityDO.getActivityEndTime() < now) {
            return Result.ofBadRequest("活动已结束");
        }
        if (!redisService.tryLock(RedisActivityKeys.getActivityLock(activityDO.getActivityId(), userId), "", 1, TimeUnit.SECONDS)) {
            return Result.ofBadRequest("访问过快, 请稍后重试");
        }
        return Result.ofSuccess();
    }

    /**
     * 获取十一活动排行
     *
     * @param lotteryActivityId
     * @param activityId
     * @return
     */
    @Override
    @GET
    @Path("getOctoberRank")
    public Result getOctoberRank(@QueryParam("lotteryActivityId") Long lotteryActivityId, @QueryParam("activityId") Long activityId, @QueryParam("rankType") Integer rankType) {
        List<ActivityOctoberRankVo> res = new ArrayList<>(10);
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null) {
            return Result.ofBadRequest("未找到该活动");
        }
        if (rankType == null) {
            rankType = 0;
        }
        List<ActivityStatRankDO> rankList;
        if (rankType == 0) {
            rankList = activityFestivalService.getOctoberRank(lotteryActivityId, activityDO.getActivityBeginTime(), activityDO.getActivityEndTime());
        } else {
            rankList = activityFestivalService.getOctoberRank(lotteryActivityId, DateUtil.getDayBegin(), DateUtil.getDayEnd());
        }
        if (CollectionUtil.isNotEmpty(rankList)) {
            List<Long> userIds = new ArrayList<>(10);
            rankList.forEach(i -> userIds.add(i.getUserId()));
            Map<Long, UserDO> userMap = userService.getUserMapByIds(userIds);
            Long last = 0L;
            for (ActivityStatRankDO hisDO : rankList) {
                ActivityOctoberRankVo vo = new ActivityOctoberRankVo();
                UserDO userDO = userMap.get(hisDO.getUserId());
                if (userDO != null) {
                    vo.setAvatar(userDO.getUserAvatar());
                    vo.setUserId(hisDO.getUserId());
                    vo.setTravelDistance(Math.abs(last - hisDO.getHisCostAmount()));
                    last = hisDO.getHisCostAmount();
                    vo.setUserName(userDO.getUserName());
                    vo.setIsOnline(userDO.getIsOnline() == null ? BoolIntVal.False : BoolIntVal.True);
                    res.add(vo);
                }
            }
        }
        return Result.ofSuccess(res);
    }

    /**
     * 获取十一活动详情
     *
     * @param activityId
     * @param lotteryActivityId
     * @return
     */
    @Override
    @GET
    @Path("getOctoberDetail")
    public Result getOctoberDetail(@QueryParam("lotteryActivityId") Long lotteryActivityId, @QueryParam("activityId") Long activityId, @QueryParam("rankType") Integer rankType) {
        ActivityOctoberDetailVo detailVo = new ActivityOctoberDetailVo();
        Long userId = this.getToken().getUserId();
        ActivityDO activityDO = activityService.getActivity(activityId);
        if (activityDO == null) {
            return Result.ofBadRequest("未找到该活动");
        }
        ActivityStatRankDO myRank;
        if (rankType == null) {
            rankType = 0;
        }
        if (rankType == 0) {
            myRank = activityFestivalService.getMySelfRank(lotteryActivityId, userId, activityDO.getActivityBeginTime(), activityDO.getActivityEndTime());
        } else {
            myRank = activityFestivalService.getMySelfRank(lotteryActivityId, userId, DateUtil.getDayBegin(), DateUtil.getDayEnd());
        }
        detailVo.setBeginTime(activityDO.getActivityBeginTime());
        detailVo.setEndTime(activityDO.getActivityEndTime());
        detailVo.setMyTravelDistance(myRank != null ? myRank.getHisCostAmount() : 0);
        return Result.ofSuccess(detailVo);
    }
}
