package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.config.GameConfig;
import com.starbuds.server.api.app.v1.facade.GameFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.GameCoinOrderApiPlatformEnum;
import com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GameShareTargetTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.GameInfoDO;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.service.finance.GameService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GameGroupService;
import com.starbuds.server.common.service.media.GameInfoService;
import com.starbuds.server.common.service.media.ThirdGameService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.game.sdk.request.ForwardGameRequest;
import com.starbuds.server.game.sdk.request.QueryGameListRequest;
import com.starbuds.server.game.sdk.request.QueryRoomListRequest;
import com.starbuds.server.game.sdk.request.TransferRequest;
import com.starbuds.server.game.sdk.response.ForwardGameResponse;
import com.starbuds.server.game.sdk.response.QueryGameListResponse;
import com.starbuds.server.game.sdk.response.QueryRoomListResponse;
import com.starbuds.server.game.sdk.response.TransferResponse;
import com.starbuds.server.game.sdk.response.data.Game;
import com.starbuds.server.game.sdk.response.data.Room;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum.IN;

/**
 * 游戏接口实现
 *
 * @author Lay
 * @date 2020/9/4
 */
@Path("/v1/game")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class GameFacadeImpl extends BaseFacade implements GameFacade {

    @Resource
    private GameConfig gameConfig;

    @Reference(version = "1.0.0")
    private GameService gameService;

    @Value("${game.notifyUrl}")
    private String notifyUrl;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    @Reference(version = "1.0.0")
    private GameGroupService gameGroupService;

    @Reference(version = "1.0.0")
    private GameInfoService gameInfoService;

    @Reference(version = "1.0.0")
    ThirdGameService thirdGameService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    /**
     * 获取游戏房间列表
     *
     * @return
     */
    @GET
    @Path("getRoomList")
    @Override
    public Result getRoomList() {
        /*// TODO 暂时保留
        QueryRoomListRequest request = new QueryRoomListRequest();
        QueryRoomListResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0) {
            List<Room> data = res.getData();
            Result result = new Result(data);
            result.setMsg(res.getMsg());
            return result;
        }*/

        return new Result(gameGroupService.getAllGroups());
    }

    /**
     * 获取游戏列表
     *
     * @param roomCode   游戏分组id（原：房间号）
     * @param deviceType 设备类型 pc,mobile
     * @return
     */
    @GET
    @Path("getGameList")
    @Override
    public Result getGameList(@QueryParam("roomCode") String roomCode, @QueryParam("deviceType") String deviceType) {
        /*QueryGameListRequest request = new QueryGameListRequest(roomCode, deviceType);
        QueryGameListResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0) {
            Result result = new Result(res.getData());
            result.setMsg(res.getMsg());
            return result;
        }*/
        if (roomCode == null) {
            return new Result(GlobalApiError.BadRequest, "roomCode 必须传");
        }
        if (!NumberUtil.isLong(roomCode)) {
            return new Result(GlobalApiError.BadRequest, "roomCode 类型错误");
        }

        return new Result(changeToGameVOList(gameInfoService.getListByGroupId(Long.parseLong(roomCode))));
    }

    /**
     * 获取热门游戏列表
     *
     * @param deviceType 设备类型 pc,mobile
     * @return
     */
    @GET
    @Path("getHotGameList")
    @Override
    public Result getHotGameList(@QueryParam("deviceType") String deviceType) {
        /*if (StringUtils.isBlank(deviceType)) {
            return new Result(GlobalApiError.BadRequest, "deviceType不能为空");
        }
        QueryHotGameListRequest request = new QueryHotGameListRequest(deviceType);
        QueryHotGameListResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0) {
            Result result = new Result(res.getData());
            result.setMsg(res.getMsg());
            return result;
        }
        return new Result(GlobalApiError.OperationFailed, res.getMsg());*/
        return new Result(changeToGameVOList(gameInfoService.getHotGames()));
    }

    /**
     * 进入游戏
     *
     * @param gameCode 游戏代码
     * @return
     */
    @POST
    @Path("enterGame")
    @Override
    public Result enterGame(@FormParam("gameCode") String gameCode) {
        TokenBO token = getToken();
        String userId = token.getUserId() + "";
        ForwardGameRequest request = new ForwardGameRequest(userId, userId, gameCode, getIpAddress());
        ForwardGameResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0) {
            Result result = new Result(res.getData());
            result.setMsg(res.getMsg());
            return result;
        }
        return new Result(GlobalApiError.OperationFailed, res.getMsg());
    }

    /**
     * 添加游戏币订单
     *
     * @param platform 平台代码
     * @param operate  操作（10:转入;20:转出）
     * @param coin     游戏币数量(单位：转入：游戏币；转出：人民币)
     * @return true:成功
     */
    @POST
    @Path("addGameCoinOrder")
    @Override
    public Result addGameCoinOrder(@FormParam("platform") String platform,
                                   @FormParam("operate") Integer operate,
                                   @FormParam("coin") Long coin) {
        if (StringUtils.isBlank(platform)) {
            return new Result(GlobalApiError.BadRequest, "平台代码不能为空");
        }
        if (coin == null) {
            return new Result(GlobalApiError.BadRequest, "游戏币数量不能为空");
        }
        if (coin < 1) {
            return new Result(GlobalApiError.BadRequest, "游戏币数量必须大于0");
        }
        GameCoinOrderOperateEnum operateEnum = GameCoinOrderOperateEnum.toEnum(operate);
        if (operateEnum == null) {
            return new Result(GlobalApiError.BadRequest, "无效的操作代码");
        }

        // 统一换算成游戏币
        Long userId = getToken().getUserId();
        if (IN == operateEnum) {
            Long balance = walletService.getBalance(userId, WalletTypeEnum.GAME_COIN);
            // 转入在途总金额
            Long inOnWayTotal = gameService.getInOnWayTotal(userId);
            if (coin > balance - inOnWayTotal) {
                return new Result(GlobalApiError.BadRequest, "余额不足");
            }
        }

        // 保存订单
        Long orderId = gameService.addGameCoinOrder(userId, platform, operateEnum,
                GameCoinOrderApiPlatformEnum.API_SAAS, coin);
        if (orderId == null) {
            return new Result(GlobalApiError.OperationFailed);
        }

        // 转出时，调用第三方接口游戏币转RMB
        String userIdStr = userId + "";
        TransferRequest request = new TransferRequest(platform, userIdStr, userIdStr,
                operateEnum.getTranType(), coin, orderId + "", notifyUrl);
        TransferResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0 && res.getData() != null && res.getData().getResult()) {
            Result result = new Result(orderId);
            return result;
        }
        gameService.updateGameCoinOrder(orderId, false, JSON.toJSONString(res.getData()));
        return new Result(GlobalApiError.OperationFailed, res.getMsg());
    }

    /**
     * 获得游戏币订单状态
     *
     * @param orderId 订单状态
     * @return
     */
    @GET
    @Path("getOrderStatus")
    @Override
    public Result getOrderStatus(@QueryParam("orderId") Long orderId) {
        log.info("获得游戏币订单状态orderId={}", orderId);
        if (!gameService.exist(orderId)) {
            return new Result(GlobalApiError.BadRequest, "订单id无效");
        }
        Integer status = gameService.getGameCoinOrderStatus(orderId);
        log.info("获得游戏币订单状态status={}", status);
        return new Result(status);
    }

    /**
     * 拉取所有游戏
     *
     * @return
     */
    @POST
    @Path("pullAllGames")
    @Override
    public Result pullAllGames() {
        QueryRoomListRequest request = new QueryRoomListRequest();
        QueryRoomListResponse res = gameConfig.client().execute(request);
        if (res.getCode() != 0) {
            return new Result(GlobalApiError.OperationFailed, "拉取游戏失败，获取房间列表失败");
        }
        List<Room> rooms = res.getData();
        if (rooms == null || rooms.isEmpty()) {
            return new Result(GlobalApiError.OperationFailed, "拉取游戏失败，房间列表为空");
        }

        // 所有游戏列表
        List<GameInfoDO> list = new ArrayList<>();
        // "pc", "mobile" 目前不拉取PC游戏
        String[] devices = new String[]{"mobile"};
        Long createTime = System.currentTimeMillis();
        // 通过SDK获取所有游戏
        for (Room room : rooms) {
            for (String device : devices) {
                List<Game> games = getGameListFromSDK(room.getRoomCode(), device);
                if (CollectionUtils.isEmpty(games)) {
                    continue;
                }
                for (Game game : games) {
                    if (game == null) {
                        continue;
                    }
                    GameInfoDO po = new GameInfoDO();
                    po.setName(game.getGameName());
                    po.setIsHorizontal(Integer.parseInt(game.getIsHorizontal()));
                    po.setIcon(game.getIcon());
                    po.setPlatform(game.getPlatform());
                    po.setGameCode(game.getGameCode());
                    po.setDeviceType(game.getDeviceType());
                    po.setCreateTime(createTime);
                    po.setUpdateTime(createTime);
                    // 设置默认分组
                    po.setGameGroupId(0L);
                    po.setIsHot(BoolIntVal.False);
                    po.setSort(1);
                    po.setStatus(CommonStatus.Normal);
                    list.add(po);
                }
            }
        }

        if (list.isEmpty()) {
            return new Result(GlobalApiError.OperationFailed, "拉取游戏失败，SDK没有获得游戏");
        }
        if (gameInfoService.createBatch(list)) {
            return new Result("拉取游戏成功，总共 " + list.size());
        }

        return new Result(GlobalApiError.OperationFailed, "拉取游戏失败，插入数据库失败，共 " + list.size());
    }

    /**
     * 获取游戏大厅链接
     *
     * @return
     */
    @GET
    @Path("getHallUrl")
    @Override
    public Result getHallUrl() {
        Long userId = this.getToken().getUserId();
        String url = thirdGameService.getGameHallUrl(userId);
        if (StringUtils.isBlank(url)) {
            return new Result(GlobalApiError.OperationFailed, "加载游戏大厅失败");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("url", url);
        return mResult.build();
    }

    /**
     * 游戏分享
     *
     * @param params
     * @param gameName
     * @param gameIcon
     * @param targetType
     * @param targetId
     * @return
     */
    @POST
    @Path("shareGame")
    @Override
    public Result shareGame(@FormParam("params") String params,
                            @FormParam("gameName") String gameName,
                            @FormParam("gameIcon") String gameIcon,
                            @FormParam("targetType") int targetType,
                            @FormParam("targetId") Long targetId) {
        Long userId = this.getToken().getUserId();

        // 判断类型
        GameShareTargetTypeEnum gameShareTargetType = GameShareTargetTypeEnum.toEnum(targetType);
        if (gameShareTargetType == null) {
            return new Result(GlobalApiError.OperationFailed, "暂不支持分享到该目标类型");
        }

        if (GameShareTargetTypeEnum.Fans == gameShareTargetType) {
            // 判断是否粉丝关系
            if (!socialRelationService.isFollower(targetId, userId)) {
                return new Result(GlobalApiError.OperationFailed, "该用户不是您的粉丝，无法分享");
            }
        }

        // 判断限制
        if (thirdGameService.isShareLimited(userId, targetType, targetId)) {
            return new Result(GlobalApiError.OperationFailed, "分享太频繁了，请稍后再试");
        }

        if (thirdGameService.share(userId, params, gameName, gameIcon, targetType, targetId)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed, "分享失败，请稍后再试");
    }

    /**
     * 通过SDK获取游戏列表
     *
     * @param roomCode   房间号
     * @param deviceType 设备类型 pc,mobile
     * @return
     */
    private List<Game> getGameListFromSDK(String roomCode, String deviceType) {
        QueryGameListRequest request = new QueryGameListRequest(roomCode, deviceType);
        QueryGameListResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0 && res.getData() != null) {
            return res.getData();
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 转换游戏列表
     *
     * @param poList
     * @return
     */
    private List<Game> changeToGameVOList(List<GameInfoDO> poList) {
        if (CollectionUtils.isEmpty(poList)) {
            return Collections.EMPTY_LIST;
        }
        List<Game> list = new ArrayList<>(poList.size());
        for (GameInfoDO po : poList) {
            Game g = new Game();
            BeanUtils.copyProperties(po, g);
            g.setGameName(po.getName());
            g.setIsHorizontal(po.getIsHorizontal() + "");
            list.add(g);
        }
        return list;
    }
}
