package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.config.UpgradeConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.gift.SendGiftForm;
import com.starbuds.server.api.app.pojo.gift.SendRtcGiftForm;
import com.starbuds.server.api.app.provider.SendGiftProvider;
import com.starbuds.server.api.app.v1.facade.GiftFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.media.ChatStatusEnum;
import com.starbuds.server.common.constant.media.GiftSceneTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.VoiceChatHisDO;
import com.starbuds.server.common.pojo.finance.gift.SendGiftOrder;
import com.starbuds.server.common.pojo.finance.gift.SendGiftPrivateChatBO;
import com.starbuds.server.common.pojo.finance.gift.SendGiftRtcUnderMaiBO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.media.GiftItemsGetQuery;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.i18n.I18nService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.media.SingleChatService;
import com.starbuds.server.common.service.media.VoiceChatService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import javax.ws.rs.*;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * 礼物接口实现
 *
 * @author Lay
 * @date 2019/12/31
 */
@Path("/v1/gift")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class GiftFacadeImpl extends BaseFacade implements GiftFacade {

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    PayService payService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    SingleChatService singleChatService;

    @Reference(version = "1.0.0")
    private I18nService i18nService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    VoiceChatService voiceChatService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    ActivityService activityService;

    @Resource
    UpgradeConfig upgradeConfig;

    @Resource
    private SendGiftProvider sendGiftProvider;

    /**
     * 获取礼物
     *
     * @param giftType
     * @param giftCategory
     * @param giftPriceType
     * @param displayType
     * @return
     */
    @GET
    @Path("getGifts")
    @PermitGuest
    @Override
    public Result getGifts(@QueryParam("giftType") Integer giftType,
                           @QueryParam("giftCategory") Integer giftCategory,
                           @QueryParam("giftPriceType") Integer giftPriceType,
                           @QueryParam("displayType") int displayType) {
        MutableResult mResult = new MutableResult();
        Long userId = 0L;
        if (this.getToken() != null) {
            userId = this.getToken().getUserId();
        }
        mResult.put("list", i18nService.doI18n(giftService.getGiftItems(GiftItemsGetQuery.builder().giftType(giftType).giftCategory(giftCategory).giftPriceType(giftPriceType).displayType(displayType).userId(userId).build()), lang()));
        return mResult.build();
    }

    /**
     * 获取礼物
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGift")
    @PermitGuest
    @Override
    public Result getGift(@QueryParam("giftId") Long giftId) {
        return new Result(i18nService.doI18n(giftService.getGift(giftId), lang()));
    }

    /**
     * 赠送礼物
     *
     * @param userId
     * @param giftId
     * @param quantity
     * @param isFromBag
     * @param sceneType
     * @param channelName
     * @return
     */
    @POST
    @Path("sendGift")
    @Override
    public Result sendGift(@FormParam("userId") Long userId,
                           @FormParam("giftId") Long giftId,
                           @FormParam("quantity") Integer quantity,
                           @FormParam("isFromBag") boolean isFromBag,
                           @FormParam("sceneType") int sceneType,
                           @FormParam("channelName") String channelName) {

        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        // 判断受赠人是否主播
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "目前只支持给主播赠送礼物");
        }

        // 判断礼物数量
        if (quantity < 1) {
            return new Result(GlobalApiError.OperationFailed, "赠送数量必须是大于0");
        }

        if (quantity > 9999) {
            return new Result(GlobalApiError.OperationFailed, "单次最多赠送9999个哦");
        }

        // 我的 id
        Long myUserId = this.getToken().getUserId();
        GiftSceneTypeEnum giftSceneType = GiftSceneTypeEnum.toEnum(sceneType);

        if (GiftSceneTypeEnum.SingleChat == giftSceneType) {
            ChatStatusEnum chatStatus = singleChatService.getSingleChatStatus(channelName, myUserId, userId);
            if (ChatStatusEnum.Calling != chatStatus) {
                return new Result(GlobalApiError.OperationFailed, "没在通话中无法赠送哦！");
            }
        }

        if (isFromBag) {
            // 判断背包中是否有该礼物
            if (bagService.getItemQuantity(myUserId, BagItemTypeEnum.GIFT, giftId) == 0) {
                return new Result(GlobalApiError.OperationFailed, "数量不足");
            }

            // 消耗礼物
            Integer quantityRemain = bagService.sendGift(userId, myUserId, giftId, quantity, this.getHeader().getChannelId(), giftSceneType, channelName);
            if (quantityRemain != null) {
                MutableResult mResult = new MutableResult();
                mResult.put("quantity", quantityRemain);
                return mResult.build();
            }
        } else {
            // 判断礼物是否有效
            GiftItemVO giftItem = giftService.getGift(giftId);
            if (giftItem == null) {
                return new Result(GlobalApiError.OperationFailed, "无效的礼物或礼物已过期");
            }

            // 礼物限制判断
            String conditionCheckResult = giftService.checkGiftConditions(giftId, userId, myUserId);
            if (StringUtils.isNotBlank(conditionCheckResult)) {
                return new Result(GlobalApiError.OperationFailed, conditionCheckResult);
            }

            // 支付礼物
            Long balance = payService.payForGift(userId, myUserId, giftId, quantity, this.getHeader().getChannelId(), giftSceneType, channelName);
            if (balance != null) {
                MutableResult mResult = new MutableResult();
                mResult.put("balance", balance);
                mResult.put("walletType", giftItem.getGiftPriceType());
                return mResult.build();
            } else {
                return sendGiftProvider.needRecharge(giftItem.getGiftPriceType());
            }
        }

        return new Result(GlobalApiError.OperationFailed, "礼物赠送失败");
    }

    /**
     * 赠送 RTC 礼物
     *
     * @param roomId
     * @param userIds
     * @param giftId
     * @param quantity
     * @param isFromBag
     * @return
     */
    @POST
    @Path("sendRtcGift")
    @Override
    public Result sendRtcGift(@FormParam("roomId") Long roomId,
                              @FormParam("userIds") String userIds,
                              @FormParam("giftId") Long giftId,
                              @FormParam("quantity") Integer quantity,
                              @FormParam("isFromBag") boolean isFromBag) {
        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        // 判断礼物数量
        if (quantity < 1) {
            return new Result(GlobalApiError.OperationFailed, "赠送数量必须是大于0");
        }

        if (quantity > 9999) {
            return new Result(GlobalApiError.OperationFailed, "单次最多赠送9999个哦");
        }

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 赠送用户处理
        if (StringUtils.isBlank(userIds)) {
            return new Result(GlobalApiError.OperationFailed, "请选择要赠送的用户");
        }

        Set<Long> userIdSet = Arrays.stream(userIds.split(",")).map(Convert::toLong).collect(Collectors.toSet());
        if (CollectionUtil.isEmpty(userIdSet)) {
            return new Result(GlobalApiError.OperationFailed, "请选择要赠送的用户");
        }

        Long userId = this.getToken().getUserId();
        if (userIdSet.contains(userId)) {
            return new Result(GlobalApiError.OperationFailed, "不能给自己赠送礼物哦");
        }

        // 取得用户麦位 userId:seatNo
        Map<Long, Integer> userSeats = rtcSeatService.getUserSeats(roomId, new ArrayList<>(userIdSet));
        if (userSeats.size() != userIdSet.size()) {
            return new Result(GlobalApiError.OperationFailed, "用户麦位已发生变化");
        }

        GiftItemVO giftItem = giftService.getGift(giftId);

        // 判断礼物是否有效
        if (giftItem == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的礼物或礼物已过期");
        }

        if (!isFromBag) {
            // 判断礼物是否列表可见
            if (Optional.ofNullable(giftItem.getGiftListVisible()).orElse(BoolIntVal.False) == BoolIntVal.False) {
                return new Result(GlobalApiError.OperationFailed, "该礼物已下架，您只能从背包赠送");
            }
        }

        // 判断礼物类型
        GiftTypeEnum giftTypeEnum = GiftTypeEnum.toEnum(giftItem.getGiftType());
        if (GiftTypeEnum.COMMON != giftTypeEnum && GiftTypeEnum.LOTTERY != giftTypeEnum) {
            return new Result(GlobalApiError.OperationFailed, "暂不支持该礼物类型");
        }

        SendGiftOrder order;
        MutableResult mResult = new MutableResult();
        if (isFromBag) {
            // 从背包赠送

            // 判断背包中是否有该礼物
            if (bagService.getItemQuantity(userId, BagItemTypeEnum.GIFT, giftId) < userSeats.size() * quantity) {
                return new Result(GlobalApiError.OperationFailed, "数量不足");
            }

            // 消耗礼物
            order = bagService.sendRtcGift(roomId, userSeats, userId, giftId, quantity, this.getHeader().getChannelId(), getHeader().getAppVersion());
            if (order != null) {
                mResult.put("quantity", order.getQuantity());
            }
        } else {
            // 消耗星钻赠送

            // 礼物限制判断
            String conditionCheckResult = giftService.checkGiftConditions(giftId, null, userId);
            if (StringUtils.isNotBlank(conditionCheckResult)) {
                return new Result(GlobalApiError.OperationFailed, conditionCheckResult);
            }

            // 支付礼物
            order = payService.payForRtcGift(roomId, userSeats, userId, giftId, quantity, this.getHeader().getChannelId());
            if (order != null) {
                mResult.put("balance", order.getBalance());
                mResult.put("walletType", giftItem.getGiftPriceType());
            } else {
                return sendGiftProvider.needRecharge(giftItem.getGiftPriceType());
            }
        }

        if (order != null) {
            if (isDev()) {
                mResult.put("orderId", order.getOrderId());
            }
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed, "礼物赠送失败");
    }

    /**
     * 语音房麦下送礼物
     *
     * @param form
     * @return
     */
    @POST
    @Path("sendRtcUnderMaiGift")
    @Override
    public Result sendRtcUnderMaiGift(@BeanParam SendRtcGiftForm form) {
        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        SendGiftRtcUnderMaiBO sendGift = new SendGiftRtcUnderMaiBO();
        sendGift.setRoomId(form.getRoomId());
        sendGift.setAppVer(getHeader().getAppVersion());
        log.info("语音房麦下送礼物 userId={} form={}", getToken().getUserId(), JSON.toJSONString(form));
        return sendGiftProvider.sendGift2Result(form, sendGift, () -> {
            // 判断房间是否存在
            if (!rtcRoomService.isRoomExists(form.getRoomId())) {
                return new Result(GlobalApiError.OperationFailed, "未找到该房间");
            }
            return null;
        }, getToken().getUserId(), getHeader().getChannelId());
    }

    /**
     * 私聊送礼物
     *
     * @param form
     * @return
     */
    @POST
    @Path("sendPrivateChatGift")
    @Override
    public Result sendPrivateChatGift(@BeanParam SendGiftForm form) {
        if (upgradeConfig.isUpgrade()) {
            return new Result(GlobalApiError.OperationFailed, "服务升级中，本功能暂时关闭，请稍后再试");
        }

        SendGiftPrivateChatBO sendGift = new SendGiftPrivateChatBO();
        sendGift.setAppVer(getHeader().getAppVersion());
        log.info("私聊送礼物 {}", JSON.toJSONString(form));
        return sendGiftProvider.sendGift2Result(form, sendGift, () -> null, getToken().getUserId(), getHeader().getChannelId());
    }

    /**
     * 赠送语音速配礼物
     *
     * @param hisId
     * @param userId
     * @param giftId
     * @param quantity
     * @param isFromBag
     * @return
     */
    @POST
    @Path("sendVoiceGift")
    @Override
    public Result sendVoiceGift(@FormParam("hisId") Long hisId, @FormParam("userId") Long userId, @FormParam("giftId") Long giftId, @FormParam("quantity") @NotNull Integer quantity, @FormParam("isFromBag") boolean isFromBag) {

        // 判断礼物数量
        if (quantity < 1) {
            return new Result(GlobalApiError.OperationFailed, "赠送数量必须是大于0");
        }

        if (quantity > 9999) {
            return new Result(GlobalApiError.OperationFailed, "单次最多赠送9999个哦");
        }

        // 判断回话是否存在
        VoiceChatHisDO hisDO = voiceChatService.getChatHis(hisId);
        if (hisDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该记录");
        }

        Long curUserId = this.getToken().getUserId();

        List<Long> userIds = new ArrayList<>();
        userIds.add(hisDO.getUserId());
        userIds.add(hisDO.getTargetUserId());

        // 送礼物判断
        if (!userIds.contains(curUserId)) {
            return new Result(GlobalApiError.OperationFailed, "只有会话者才可送礼物");
        }

        // 收礼物判断
        if (!userIds.contains(userId)) {
            return new Result(GlobalApiError.OperationFailed, "只有会话者才可送礼物");
        }
        if (curUserId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "不能给自己送礼物");
        }

        ChatStatusEnum statusEnum = ChatStatusEnum.toEnum(hisDO.getHisStatus());
        if (ChatStatusEnum.Calling != statusEnum) {
            return new Result(GlobalApiError.OperationFailed, "没在通话中无法赠送哦！");
        }

        GiftItemVO giftItem = giftService.getGift(giftId);

        // 判断礼物是否有效
        if (giftItem == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的礼物或礼物已过期");
        }

        // 判断礼物类型
        GiftTypeEnum giftTypeEnum = GiftTypeEnum.toEnum(giftItem.getGiftType());
        if (GiftTypeEnum.COMMON != giftTypeEnum && GiftTypeEnum.LOTTERY != giftTypeEnum) {
            return new Result(GlobalApiError.OperationFailed, "暂不支持该礼物类型");
        }

        if (isFromBag) {
            // 从背包赠送

            // 判断背包中是否有该礼物
            if (bagService.getItemQuantity(userId, BagItemTypeEnum.GIFT, giftId) < quantity) {
                return new Result(GlobalApiError.OperationFailed, "数量不足");
            }

            // 消耗礼物
            Integer quantityRemain = bagService.sendVoiceGift(hisId, userId, curUserId, giftId, quantity, this.getHeader().getChannelId());
            if (quantityRemain != null) {
                MutableResult mResult = new MutableResult();
                mResult.put("quantity", quantityRemain);
                return mResult.build();
            }
        } else {
            // 消耗星钻赠送

            // 支付礼物
            Long balance = payService.payForVoiceGift(hisId, userId, curUserId, giftId, quantity, this.getHeader().getChannelId());
            if (balance != null) {
                MutableResult mResult = new MutableResult();
                mResult.put("balance", balance);
                mResult.put("walletType", giftItem.getGiftPriceType());
                return mResult.build();
            } else {
                return sendGiftProvider.needRecharge(giftItem.getGiftPriceType());
            }
        }

        return new Result(GlobalApiError.OperationFailed, "礼物赠送失败");
    }

    /**
     * 获取周星礼物
     *
     * @return
     */
    @GET
    @Path("getWeekGifts")
    @Override
    public Result getWeekGifts() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getWeekGifts());
        return mResult.build();
    }

    /**
     * 获取周星礼物  上期
     *
     * @return
     */
    @GET
    @Path("getWeekGiftsPrevious")
    @Override
    public Result getWeekGiftsPrevious() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", activityService.getWeekGiftsPrevious());
        return mResult.build();
    }

    /**
     * 获取真爱榜礼物
     *
     * @return
     */
    @GET
    @Path("getTrueLoveGifts")
    @Override
    public Result getTrueLoveGifts() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftService.getTrueLoveGifts());
        return mResult.build();
    }
}
