package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.google.common.collect.ImmutableMap;
import com.starbuds.server.api.app.pojo.activity.LotteryGameResultDivideCakeVO;
import com.starbuds.server.api.app.pojo.activity.LotteryGameResultRedBagVO;
import com.starbuds.server.api.app.pojo.activity.LotteryGameVO;
import com.starbuds.server.api.app.v1.facade.LotteryGameFacade;
import com.starbuds.server.common.constant.activity.LotteryGameCostTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.activity.bo.DivideCakeWinInfo;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameResultDivideCake;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameResultRedBag;
import com.starbuds.server.common.pojo.activity.vo.LotteryGameSummary;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.pojo.finance.params.SaveLotteryGameQualificationParam;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.activity.LotteryGameSnapshotService;
import com.starbuds.server.common.service.activity.LotteryGameUserQualificationHisService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.SaveLotteryGameQualificationService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.ws.rs.*;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * 抽奖游戏接口
 *
 * @author aoe
 * @date 2021/11/22
 */
@Path("/v1/lotteryGame")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class LotteryGameFacadeImpl extends BaseFacade implements LotteryGameFacade {

    @Reference(version = "1.0.0")
    LotteryGameUserQualificationHisService lotteryGameUserQualificationHisService;

    @Reference(version = "1.0.0")
    LotteryGameSnapshotService lotteryGameSnapshotService;

    @Reference(version = "1.0.0")
    private LotteryGameService lotteryGameService;

    @Reference(version = "1.0.0")
    private SaveLotteryGameQualificationService saveLotteryGameQualificationService;

    @Reference(version = "1.0.0")
    private BagService bagService;

    /**
     * 查询抽奖游戏开奖结果-红包
     *
     * @param snapId
     * @return
     */
    @GET
    @Path("getResultByRedBag")
    @Override
    public Result getResultByRedBag(@QueryParam("snapId") Long snapId) {
        if (snapId == null) {
            return Result.ofBadRequest("snapId 无效");
        }

        LotteryGameResultRedBag winResult = lotteryGameService.getResultByRedBag(getToken().getUserId(), snapId);
        List<LotteryGameResultRedBagVO> voList = winResult.getWinInfos().stream()
                .map(bo -> {
                    LotteryGameResultRedBagVO vo = new LotteryGameResultRedBagVO();
                    BeanUtils.copyProperties(bo, vo);
                    return vo;
                }).collect(Collectors.toList());

        MutableResult mResult = new MutableResult();
        mResult.put("list", voList);
        mResult.put("winCount", winResult.getWinCount());
        return mResult.build();
    }

    /**
     * 查询抽奖游戏开奖结果-瓜分蛋糕
     *
     * @param snapId
     * @return
     */
    @GET
    @Path("getResultByDivideCake")
    @Override
    public Result getResultByDivideCake(@QueryParam("snapId") Long snapId) {
        if (snapId == null) {
            return Result.ofBadRequest("snapId 无效");
        }
        LotteryGameResultDivideCake winResult = lotteryGameService.getResultByDivideCake(getToken().getUserId(), snapId);
        List<DivideCakeWinInfo> winInfos = winResult.getWinInfos();
        List<LotteryGameResultDivideCakeVO> voList = winInfos.stream()
                .map(win -> {
                    LotteryGameResultDivideCakeVO vo = new LotteryGameResultDivideCakeVO();
                    BeanUtils.copyProperties(win, vo);
                    return vo;
                })
                .collect(Collectors.toList());
        return Result.ofList(voList);
    }

    /**
     * 获取抽奖游戏详情
     *
     * @param snapId
     * @return
     */
    @GET
    @Path("getLotteryGameDetail")
    @Override
    public Result getLotteryGameDetail(@QueryParam("snapId") Long snapId) {
        if (snapId == null) {
            return Result.ofBadRequest("snapId 无效");
        }

        LotteryGameVO vo = new LotteryGameVO();
        LotteryGameSummary summary = lotteryGameSnapshotService.getSummaryByCache(snapId);

        if (summary == null) {
            return Result.ofOperationFailed("活动不存在");
        }

        BeanUtils.copyProperties(summary, vo);
        long now = System.currentTimeMillis();
        if (vo.getStartTime() >= now) {
            vo.setRemainTime(BigDecimal.valueOf(vo.getStartTime()).subtract(BigDecimal.valueOf(now)).divide(BigDecimal.valueOf(1000)).longValue());
        } else {
            vo.setRemainTime(0L);
        }

        return Result.ofSuccess(vo);
    }


    /**
     * 参与抽奖游戏-红包雨
     *
     * @param snapId
     * @return
     */
    @POST
    @Path("joinLotteryGame")
    @Override
    public Result joinLotteryGame(@FormParam("snapId") Long snapId,
                                  @FormParam("roomId") Long roomId) {
        return joinGame(snapId, roomId, 0, LotteryGamePlayTypeEnum.RED_BAG);
    }

    /**
     * 参与抽奖游戏-瓜分蛋糕
     *
     * @param snapId
     * @param roomId
     * @param quantity 消耗道具数量
     * @return
     */
    @POST
    @Path("joinDivideCake")
    @Override
    public Result joinDivideCake(@FormParam("snapId") Long snapId,
                                 @FormParam("roomId") Long roomId,
                                 @FormParam("quantity") Integer quantity) {
        if (quantity == null) {
            return Result.ofBadRequest("请选择道具数量");
        }
        return joinGame(snapId, roomId, quantity, LotteryGamePlayTypeEnum.DIVIDE_CAKE);
    }

    /**
     * 获取正在进行中的抽奖活动
     *
     * @return
     */
    @POST
    @Path("getLotteryGameList")
    @Override
    public Result getLotteryGameList() {
        List<LotteryGameSnapshotDO> snapshotDOList = lotteryGameService.getInProgressingSnaps();
        List<LotteryGameVO> resList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(snapshotDOList)) {
            for (LotteryGameSnapshotDO snapshotDO : snapshotDOList) {
                LotteryGameSummary summary = lotteryGameSnapshotService.getSummaryByCache(snapshotDO.getSnapId());
                if (summary != null) {
                    LotteryGameVO tmp = new LotteryGameVO();
                    BeanUtils.copyProperties(summary, tmp);
                    tmp.setSnapId(snapshotDO.getSnapId());
                    tmp.setEndTime(snapshotDO.getMaxEndTime());
                    resList.add(tmp);
                }
            }
        }
        List<LotteryGameSnapshotDO> waitList = lotteryGameService.getWaitLotteryGameList();
        List<LotteryGameVO> waitResList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(waitList)) {
            // 开始填充数据
            for (LotteryGameSnapshotDO snapshotDO : waitList) {
                LotteryGameSummary summary = lotteryGameSnapshotService.getSummary(snapshotDO.getSnapId());
                if (summary != null) {
                    LotteryGameVO tmp = new LotteryGameVO();
                    BeanUtils.copyProperties(summary, tmp);
                    tmp.setSnapId(snapshotDO.getSnapId());
                    tmp.setEndTime(snapshotDO.getMaxEndTime());
                    waitResList.add(tmp);
                }
            }
        }

        Map<String, Object> res = new HashMap<>();
        res.put("resList", resList);
        res.put("waitList", waitResList);
        return Result.ofSuccess(res);
    }

    /**
     * 获取背包中参与活动必备道具的数量
     *
     * @param snapId
     * @return
     */
    @GET
    @Path("getPropsCountByBag")
    @Override
    public Result getPropsCountByBag(@QueryParam("snapId") Long snapId) {
        if (snapId == null) {
            return Result.ofBadRequest("snapId 必传");
        }
        String keyCount = "count";
        LotteryGameSummary summary = lotteryGameSnapshotService.getSummaryByCache(snapId);
        if (summary == null) {
            return Result.ofSuccess(ImmutableMap.of(keyCount, 0));
        }
        Long itemId = summary.getCostItem();
        if (itemId == null) {
            return Result.ofSuccess(ImmutableMap.of(keyCount, 0));
        }
        Integer quantity = bagService.getItemQuantity(this.getToken().getUserId(), BagItemTypeEnum.PROPS, itemId);
        return Result.ofSuccess(ImmutableMap.of(keyCount, Optional.ofNullable(quantity).orElse(0)));
    }

    private SaveLotteryGameQualificationParam getSaveLotteryGameQualificationParam(Long userId, Long roomId,
                                                                                   LotteryGameSummary summary,
                                                                                   Integer costQuantity,
                                                                                   double clickRate) {
        return SaveLotteryGameQualificationParam.builder()
                .costItemId(summary.getCostItem())
                .costQuantity(costQuantity)
                .snapId(summary.getSnapId())
                .userId(userId)
                .costType(LotteryGameCostTypeEnum.toEnum(summary.getCostType()))
                .lotteryGamePlayType(LotteryGamePlayTypeEnum.toEnum(summary.getPlayType()))
                .roomId(roomId)
                .clickRate(clickRate)
                .build();
    }

    /**
     * 参与抽奖游戏
     *
     * @param snapId
     * @param roomId
     * @param costQuantity 消耗道具数量
     * @return
     */
    private Result joinGame(Long snapId, Long roomId, Integer costQuantity, LotteryGamePlayTypeEnum playTypeEnum) {
        if (snapId == null) {
            return Result.ofBadRequest("snapId 无效");
        }
        if (roomId == null) {
            return Result.ofBadRequest("roomId 无效");
        }
        if (!lotteryGameService.inProgressing(snapId)) {
            return Result.ofOperationFailed("不在时间活动范围内");
        }
        if (lotteryGameService.isRateLimit(snapId)) {
            return effectiveEmpty();
        }
        LotteryGameSummary summary = lotteryGameSnapshotService.getSummaryByCache(snapId);
        if (summary == null) {
            return Result.ofBadRequest("无效的活动");
        }
        if (playTypeEnum != LotteryGamePlayTypeEnum.toEnum(summary.getPlayType())) {
            return Result.ofBadRequest("活动类型不匹配");
        }

        if (costQuantity > 0) {
            try {
                return joinGameCostSomething(roomId, costQuantity, summary);
            } catch (Exception e) {
                // 频繁点击时，因背包更新不及时会报错，返回空字符串给用户
                log.error("频繁点击时参与瓜分蛋糕失败");
            }
            return effectiveEmpty();
        }
        try {
            return joinGameCostNothing(roomId, summary);
        } catch (Exception e) {
            log.error("频繁点击时参与红包雨失败");
        }
        return effectiveEmpty();
    }

    private Result effective(int clickCount) {
        return Result.ofSuccess(ImmutableMap.of("isSuccess", clickCount));
    }

    private Result effectiveEmpty() {
        return Result.ofSuccess(ImmutableMap.of("isSuccess", ""));
    }

    private Result joinGameCostNothing(Long roomId, LotteryGameSummary summary) {
        Long snapId = summary.getSnapId();
        Long userId = this.getToken().getUserId();
        if (lotteryGameService.existBySnapIdUserId(snapId, userId)) {
            // 已经保存过参与记录，按概率返回
            double clickRate = Optional.ofNullable(summary.getClickRate()).orElse(BigDecimal.ZERO).doubleValue();
            return lotteryGameService.effectiveClick(clickRate) ? effective(1) : effective(0);
        }

        if (lotteryGameService.getRewardCount(snapId) < 1) {
            // 库存不足时，尝试结束活动
//            lotteryGameService.markUpEnd(snapId);
            return effective(0);
        }

        List<LotteryGameUserQualificationHisDO> saveQualification = saveLotteryGameQualificationService
                .costItemAndSaveQualification(getSaveLotteryGameQualificationParam(userId, roomId, summary, 0, 1));
        if (!CollectionUtils.isEmpty(saveQualification)) {
            // 保存中奖记录
            lotteryGameService.lotteryAndSaveWinHis(snapId, LotteryGamePlayTypeEnum.toEnum(summary.getPlayType()),
                    userId, 1);
            return effective(1);
        }
        return effective(0);
    }

    private Result joinGameCostSomething(Long roomId, Integer costQuantity, LotteryGameSummary summary) {
        Long userId = this.getToken().getUserId();
        Integer costTotalQuantity = 0;
        // 验证道具数量
        if (LotteryGameCostTypeEnum.toEnum(summary.getCostType()) == LotteryGameCostTypeEnum.PROPS) {
            costTotalQuantity = costQuantity * summary.getCostValue();
            Integer quantity = bagService.getItemQuantity(this.getToken().getUserId(), BagItemTypeEnum.PROPS, summary.getCostItem());
            if (quantity == null || quantity < costTotalQuantity) {
                return Result.ofBadRequest("道具数量不足");
            }
        }

        Long snapId = summary.getSnapId();
        if (lotteryGameService.getRewardCount(snapId) < 1) {
            // 库存不足时，尝试结束活动
            if (isDev()) {
                log.info("没获得抽奖资格-库存不足 snapId={} userId={}", snapId, getToken().getUserId());
            }
//            lotteryGameService.markUpEnd(snapId);
            return effective(0);
        }

        double clickRate = Optional.ofNullable(summary.getClickRate()).orElse(BigDecimal.ZERO).doubleValue();
        List<LotteryGameUserQualificationHisDO> saveQualifications = saveLotteryGameQualificationService
                .costItemAndSaveQualification(getSaveLotteryGameQualificationParam(userId, roomId, summary, costTotalQuantity, clickRate));

        if (CollectionUtils.isEmpty(saveQualifications)) {
            if (isDev()) {
                log.info("没获得抽奖资格-saveQualification失败 {}", snapId);
            }
            return effective(0);
        }
        int effectiveCount = (int) saveQualifications.stream()
                .filter(Objects::nonNull)
                .map(his -> his.getIsEffective())
                .filter(effective -> BoolIntVal.True == effective)
                .count();

        if (effectiveCount > 0) {
            // 保存中奖记录
            lotteryGameService.lotteryAndSaveWinHis(snapId, LotteryGamePlayTypeEnum.toEnum(summary.getPlayType()),
                    userId, effectiveCount);
        }

        return effective(costQuantity);
    }
}
