package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.v1.facade.MissionFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.activity.DailySignInService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;

/**
 * 任务接口实现
 *
 * @author Lay
 * @date 2020/9/23
 */
@Path("/v1/mission")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class MissionFacadeImpl extends BaseFacade implements MissionFacade {

    @Reference(version = "1.0.0")
    DailySignInService dailySignInService;

    /**
     * 签到
     *
     * @return
     */
    @POST
    @Path("signIn")
    @Override
    public Result signIn() {
        Long userId = this.getToken().getUserId();

        // 判断今日是否已签到
        if (dailySignInService.isSigned(userId)) {
            return new Result(GlobalApiError.OperationFailed, "您今天已经签到过了哦");
        }

        if (dailySignInService.signIn(userId, this.getHeader().getChannelId())) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取本周签到状态
     *
     * @return
     */
    @GET
    @Path("getWeekSignInHis")
    @Override
    public Result getWeekSignInHis() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", dailySignInService.getWeekSignInItems(this.getToken().getUserId()));
        return mResult.build();
    }
}
