package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.app.v1.facade.PropsFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.NiceNoPoolDO;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.user.NiceNoVO;
import com.starbuds.server.common.pojo.user.UserNobleVO;
import com.starbuds.server.common.service.data.ChatBubbleService;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.MedalService;
import com.starbuds.server.common.service.data.MountsService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.user.NiceNoService;
import com.starbuds.server.common.service.user.NobleService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;

/**
 * 道具接口实现
 *
 * @author Lay
 * @date 2020/3/2
 */
@Path("/v1/props")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class PropsFacadeImpl extends BaseFacade implements PropsFacade {

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    NiceNoService niceNoService;

    @Reference(version = "1.0.0")
    MountsService mountsService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    ChatBubbleService chatBubbleService;

    /**
     * 兑换靓号
     *
     * @param bagId
     * @param niceNo
     * @return
     */
    @POST
    @Path("exchangeNiceNo")
    @Override
    public Result exchangeNiceNo(@FormParam("bagId") String bagId, @FormParam("niceNo") Long niceNo) {
        Long userId = this.getToken().getUserId();

        // 取出卡片
        BagItem<PropsVO> bagItem = bagService.getProps(bagId, userId);
        if (bagItem == null || bagItem.getQuantity() < 1) {
            return new Result(GlobalApiError.OperationFailed, "靓号卡无效或已过期");
        }

        // 取出靓号
        NiceNoPoolDO niceNoItem = niceNoService.getNiceNo(niceNo);
        if (niceNoItem == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该靓号");
        }

        // 靓号级别匹配
        if (!niceNoItem.getNiceLevel().equals(bagItem.getItem().getPropsExtInt())) {
            return new Result(GlobalApiError.OperationFailed, "该靓号无法使用此兑换卡");
        }

        // 靓号占用
        if (BoolIntVal.True == niceNoItem.getBindStatus()) {
            return new Result(GlobalApiError.OperationFailed, "手慢啦，靓号已经被抢走了");
        }

        // 尝试绑定靓号
        if (niceNoService.bindNiceNoByBagId(bagId, userId, niceNo, bagItem.getItem().getPropsExtLong(), String.format("使用%s兑换", bagItem.getItem().getPropsName()))) {
            return new Result();
        } else {
            return new Result(GlobalApiError.OperationFailed, "兑换失败，请稍后再试");
        }
    }

    /**
     * 续期靓号
     *
     * @param bagId
     * @return
     */
    @POST
    @Path("renewNiceNo")
    @Override
    public Result renewNiceNo(@FormParam("bagId") String bagId) {
        Long userId = this.getToken().getUserId();

        NiceNoVO niceNo = niceNoService.getUserNiceNo(userId);
        if (niceNo == null) {
            return new Result(GlobalApiError.OperationFailed, "您当前未绑定靓号");
        }

        // 判断永久靓号
        if (niceNo.getBindExpireTime() < 0) {
            return new Result(GlobalApiError.OperationFailed, "永久靓号无需续期");
        }

        // 取出卡片
        BagItem<PropsVO> bagItem = bagService.getProps(bagId, userId);
        if (bagItem == null || bagItem.getQuantity() < 1) {
            return new Result(GlobalApiError.OperationFailed, "靓号卡无效或已过期");
        }

        // 靓号级别匹配
        if (!niceNo.getNiceLevel().equals(bagItem.getItem().getPropsExtInt())) {
            return new Result(GlobalApiError.OperationFailed, "您的靓号无法使用此卡续期");
        }

        // 尝试续期靓号
        if (niceNoService.renewNiceNoByBagId(bagId, userId, niceNo.getNiceNo(), bagItem.getItem().getPropsExtLong(), String.format("使用%s续期", bagItem.getItem().getPropsName()))) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 使用道具
     *
     * @param bagId
     * @return
     */
    @POST
    @Path("useProps")
    @Override
    public Result useProps(@FormParam("bagId") String bagId) {
        Long userId = this.getToken().getUserId();

        // 取出卡片
        BagItem<PropsVO> bagItem = bagService.getProps(bagId, userId);
        if (bagItem == null || bagItem.getQuantity() < 1) {
            return new Result(GlobalApiError.OperationFailed, "道具无效或已过期");
        }

        PropsTypeEnum propsType = PropsTypeEnum.toEnum(bagItem.getItem().getPropsType());
        if (PropsTypeEnum.MOUNTS_CARD == propsType) {
            Long mountsId = Convert.toLong(bagItem.getItem().getPropsExtString());
            // 判断是否已有坐骑
            if (bagService.getItemQuantity(userId, BagItemTypeEnum.MOUNTS, mountsId) > 0) {
                return new Result(GlobalApiError.OperationFailed, "您的背包中已有该坐骑");
            }

            // 判断坐骑是否存在
            if (!mountsService.isExists(mountsId)) {
                return new Result(GlobalApiError.OperationFailed, "该坐骑已下架");
            }

            // 兑换坐骑
            if (bagService.addMountsByBagId(bagId, userId)) {
                return new Result();
            }

            return new Result(GlobalApiError.OperationFailed, "坐骑兑换失败");
        } else if (PropsTypeEnum.HEADWEAR_CARD == propsType) {
            Long headwearId = Convert.toLong(bagItem.getItem().getPropsExtString());
            // 判断是否已有头饰
            if (bagService.getItemQuantity(userId, BagItemTypeEnum.HEADWEAR, headwearId) > 0) {
                return new Result(GlobalApiError.OperationFailed, "您的背包中已有该头饰");
            }

            // 判断头饰是否存在
            if (!headwearService.isExists(headwearId)) {
                return new Result(GlobalApiError.OperationFailed, "该头饰已下架");
            }

            // 兑换头饰
            if (bagService.addHeadwearByBagId(bagId, userId)) {
                return new Result();
            }

            return new Result(GlobalApiError.OperationFailed, "头饰兑换失败");
        } else if (PropsTypeEnum.NOBLE_CARD == propsType) {
            // 判断当前是否已是贵族
            UserNobleVO userNoble = nobleService.getUserNoble(userId);
            if (userNoble != null) {
                return new Result(GlobalApiError.OperationFailed, "您当前已是贵族");
            }

            // 兑换贵族
            if (nobleService.newNobleByBagId(userId, bagId, this.getHeader().getChannelId())) {
                return new Result();
            }

            return new Result(GlobalApiError.OperationFailed, "贵族兑换失败");
        } else if (PropsTypeEnum.CHAT_TIME_CARD == propsType) {
            // 兑换时长
            if (walletService.addChatTimeByProps(userId, bagId)) {
                return new Result();
            }

            return new Result(GlobalApiError.OperationFailed, "时长兑换失败");
        } else if (PropsTypeEnum.GIFT_FRAGMENT == propsType) {
            // 碎片兑换礼物 判断碎片是否足够
            if (bagItem.getQuantity() < bagItem.getItem().getPropsExtInt()) {
                return new Result(GlobalApiError.OperationFailed, "碎片数量不足");
            }

            // 兑换
            if (bagService.addGiftByFragment(bagId, userId)) {
                return new Result();
            }
            return new Result(GlobalApiError.OperationFailed, "礼物合成失败");
        } else if (PropsTypeEnum.CHAT_BUBBLE_CARD == propsType) {
            Long bubbleId = Convert.toLong(bagItem.getItem().getPropsExtString());
            // 兑换聊天气泡框
            if (bagService.getItemQuantity(userId, BagItemTypeEnum.CHAT_BUBBLE, bubbleId) > 0) {
                return new Result(GlobalApiError.OperationFailed, "您的背包中已有该聊天气泡");
            }

            // 判断聊天气泡是否存在
            if (!chatBubbleService.isExists(bubbleId)) {
                return new Result(GlobalApiError.OperationFailed, "该聊天气泡已下架");
            }

            // 兑换聊天气泡
            if (bagService.addChatBubbleByBagId(bagId, userId)) {
                return new Result();
            }

            return new Result(GlobalApiError.OperationFailed, "聊天气泡兑换失败");
        } else if (PropsTypeEnum.MEDAL_CARD == propsType) {
            Long medalId = NumberUtils.toLong(bagItem.getItem().getPropsExtString(), -1);
            // 判断是否已有勋章卡
            if (bagService.getItemQuantity(userId, BagItemTypeEnum.MEDAL, medalId) > 0) {
                return new Result(GlobalApiError.OperationFailed, "您的背包中已有该勋章");
            }

            // 判断勋章是否存在
            if (!medalService.isExists(medalId)) {
                return new Result(GlobalApiError.OperationFailed, "该勋章已下架");
            }

            // 兑换勋章
            if (bagService.addMedalByBagId(bagId, userId)) {
                return new Result();
            }

            return new Result(GlobalApiError.OperationFailed, "勋章兑换失败");
        }

        return new Result(GlobalApiError.OperationFailed, "未找到该道具类型");
    }

    /**
     * 获取靓号列表
     *
     * @param niceLevel
     * @return
     */
    @GET
    @Path("getNiceNoList")
    @Override
    public Result getNiceNoList(@QueryParam("niceLevel") int niceLevel) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", niceNoService.getNiceNoListRandom(30, niceLevel, BoolIntVal.False));
        return mResult.build();
    }
}
