package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.PushFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.PushDeviceDO;
import com.starbuds.server.common.service.message.PushService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;

/**
 * 推送管理
 *
 * @author jiangxiaowei
 * @date 2020/6/17 10:01
 */
@Path("/v1/push")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class PushFacadeImpl extends BaseFacade implements PushFacade {

    @Reference(version = "1.0.0")
    PushService pushService;

    /**
     * 注册设备
     *
     * @param deviceId
     * @return
     */
    @POST
    @Path("registerDevice")
    @PermitGuest
    @Override
    public Result registerDevice(@FormParam("deviceId") String deviceId) {

        if (StringUtils.isEmpty(deviceId)) {
            return new Result(GlobalApiError.ServerError, "设备id为空");
        }

        // 移除之前注册的设备
        pushService.removeDevice(deviceId);

        PushDeviceDO deviceDO = new PushDeviceDO();
        deviceDO.setUserId(getToken() == null ? 0 : getToken().getUserId());
        deviceDO.setClientType(this.getHeader().getClientType().getValue());
        deviceDO.setDeviceId(deviceId);
        deviceDO.setPackageName(this.getHeader().getPackageName());
        deviceDO.setUpdateTime(System.currentTimeMillis());

        if (pushService.registerDevice(deviceDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

}
