package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.RankFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.user.HotLoveLeaderboardDayTypeEnum;
import com.starbuds.server.common.constant.user.ScoreRankTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.rtc.RtcRankItem;
import com.starbuds.server.common.pojo.rtc.RtcRankService;
import com.starbuds.server.common.service.activity.GiftRankService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.ScoreRankService;
import com.starbuds.server.common.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import java.text.DateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Optional;

/**
 * 排行接口实现
 *
 * @author Lay
 * @date 2020/3/1
 */
@Path("/v1/rank")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RankFacadeImpl extends BaseFacade implements RankFacade {

    @Reference(version = "1.0.0")
    ScoreRankService scoreRankService;

    @Reference(version = "1.0.0")
    GiftRankService giftRankService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    RtcRankService rtcRankService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    /**
     * 获取积分小时榜
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getScoreHourly")
    @PermitGuest
    @Override
    public Result getScoreHourly(@QueryParam("scoreType") int scoreType) {
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的积分类型");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", scoreRankService.getRankItems(ScoreRankTypeEnum.HOURLY, scoreTypeEnum, 0, 49, this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取积分日榜
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getScoreDaily")
    @PermitGuest
    @Override
    public Result getScoreDaily(@QueryParam("scoreType") int scoreType) {
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的积分类型");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", scoreRankService.getRankItems(ScoreRankTypeEnum.DAILY, scoreTypeEnum, 0, 49, this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取积分周榜
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getScoreWeekly")
    @PermitGuest
    @Override
    public Result getScoreWeekly(@QueryParam("scoreType") int scoreType) {
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的积分类型");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", scoreRankService.getRankItems(ScoreRankTypeEnum.WEEKLY, scoreTypeEnum, 0, 49, this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取积分月榜
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getScoreMonthly")
    @PermitGuest
    @Override
    public Result getScoreMonthly(@QueryParam("scoreType") int scoreType) {
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的积分类型");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("list", scoreRankService.getRankItems(ScoreRankTypeEnum.MONTHLY, scoreTypeEnum, 0, 49, this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取热爱榜数据
     *
     * @param type -1:昨天 1:今天
     * @return
     */
    @GET
    @Path("getHotLoveLeaderboardList")
    @Override
    public Result getHotLoveLeaderboardList(@QueryParam("type") int type, @QueryParam("userId") Long userId) {
        HotLoveLeaderboardDayTypeEnum typeEnum = HotLoveLeaderboardDayTypeEnum.toEnum(type);
        if (typeEnum == null) {
            return new Result(GlobalApiError.BadRequest, "无效的类型");
        }
        MutableResult mResult = new MutableResult();
        userId = userId == null ? getToken().getUserId() : userId;
        mResult.put("list", scoreRankService.getHotLoveLeaderboardList(typeEnum, userId));
        return mResult.build();
    }


    /**
     * 获取周星榜日榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftWeekStarRankDay")
    @Override
    public Result getGiftWeekStarRankDay(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftWeekStarRankDay(giftId, 0, 19, false));
        // 我的排名信息
        mResult.put("myRankItem", giftRankService.getGiftWeekStarRankItemDay(giftId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取周星榜周榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftWeekStarRankWeek")
    @Override
    public Result getGiftWeekStarRankWeek(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftWeekStarRankWeek(giftId, 0, 19, false));
        // 我的排名信息
        mResult.put("myRankItem", giftRankService.getGiftWeekStarRankItemWeek(giftId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取周星榜月榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftWeekStarRankMonth")
    @Override
    public Result getGiftWeekStarRankMonth(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftWeekStarRankMonth(giftId, 0, 19, false));
        // 我的排名信息
        mResult.put("myRankItem", giftRankService.getGiftWeekStarRankItemMonth(giftId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取壕星榜日榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftRichStarRankDay")
    @Override
    public Result getGiftRichStarRankDay(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftRichStarRankDay(giftId, 0, 19, false));
        // 我的排名信息
        mResult.put("myRankItem", giftRankService.getGiftRichStarRankItemDay(giftId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取壕星榜周榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftRichStarRankWeek")
    @Override
    public Result getGiftRichStarRankWeek(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftRichStarRankWeek(giftId, 0, 19, false));
        // 我的排名信息
        mResult.put("myRankItem", giftRankService.getGiftRichStarRankItemWeek(giftId, this.getToken().getUserId(), false));
        return mResult.build();
    }

    /**
     * 获取壕星榜月榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftRichStarRankMonth")
    @Override
    public Result getGiftRichStarRankMonth(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftRichStarRankMonth(giftId, 0, 19, false));
        // 我的排名信息
        mResult.put("myRankItem", giftRankService.getGiftRichStarRankItemMonth(giftId, this.getToken().getUserId(), false));
        return mResult.build();
    }


    /**
     * 获取周星榜日榜 上期
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftWeekStarRankDayPrevious")
    @Override
    public Result getGiftWeekStarRankDayPrevious(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftWeekStarRankDay(giftId, 0, 0, true));
        return mResult.build();
    }

    /**
     * 获取周星榜周榜 上期
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftWeekStarRankWeekPrevious")
    @Override
    public Result getGiftWeekStarRankWeekPrevious(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftWeekStarRankWeek(giftId, 0, 0, true));
        return mResult.build();
    }

    /**
     * 获取周星榜月榜 上期
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftWeekStarRankMonthPrevious")
    @Override
    public Result getGiftWeekStarRankMonthPrevious(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftWeekStarRankMonth(giftId, 0, 0, true));
        return mResult.build();
    }

    /**
     * 获取壕星榜日榜  上期
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftRichStarRankDayPrevious")
    @Override
    public Result getGiftRichStarRankDayPrevious(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftRichStarRankDay(giftId, 0, 0, true));
        return mResult.build();
    }

    /**
     * 获取壕星榜周榜  上期
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftRichStarRankWeekPrevious")
    @Override
    public Result getGiftRichStarRankWeekPrevious(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftRichStarRankWeek(giftId, 0, 0, true));
        return mResult.build();
    }

    /**
     * 获取壕星榜月榜  上期
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getGiftRichStarRankMonthPrevious")
    @Override
    public Result getGiftRichStarRankMonthPrevious(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getGiftRichStarRankMonth(giftId, 0, 0, true));
        return mResult.build();
    }

    /**
     * 获取礼物真爱榜
     *
     * @param giftId
     * @return
     */
    @GET
    @Path("getTrueLoveGiftRank")
    @Override
    public Result getTrueLoveGiftRank(@QueryParam("giftId") Long giftId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", giftRankService.getTrueLoveGiftRank(giftId, 30));
        return mResult.build();
    }

    /**
     * 获取欧气榜
     *
     * @return
     */
    @GET
    @Path("getGiftAirRank")
    @Override
    public Result getGiftAirRank() {
        MutableResult mResult = new MutableResult();
        long startTime = LocalDateTime.now().withHour(0).withMinute(0).withSecond(0).withNano(0).atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
        long endTime = LocalDateTime.now().withHour(23).withMinute(59).withSecond(59).withNano(999999999).atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
        mResult.put("list", giftRankService.getGiftAirRank(20, startTime, endTime));
        mResult.put("topItem", giftRankService.getGiftAirTop(startTime, endTime));
        return mResult.build();
    }

    /**
     * 获取语音房小时榜
     *
     * @return
     */
    @GET
    @Path("getRtcHourRank")
    @Override
    public Result getRtcHourRank() {
        MutableResult mResult = new MutableResult();
        mResult.put("hourRank", releaseConfigService.getCachReleaseConfig("hour_rank_url"));
        mResult.put("list", rtcRankService.getRtcHourRank(Convert.toLong(LocalDateTime.now().minusHours(1).format(DateTimeFormatter.ofPattern("yyyyMMddHH")))));
        return mResult.build();
    }

    /**
     * 条件查询N条语聊房小时榜
     *
     * @param momentType 是否是当前时刻 1：当前时刻 0：过去小时榜
     * @return
     */
    @GET
    @Path("getConditionRtcHourRank")
    @Override
    public Result getConditionRtcHourRank(@QueryParam("momentType") Integer momentType) {
        momentType = Optional.ofNullable(momentType).orElse(0);

        DateTimeFormatter timeFormatter = DateTimeFormatter.ofPattern("yyyyMMddHH");
        LocalDateTime searchDateTime = momentType != 0 ? LocalDateTime.now() : LocalDateTime.now().minusHours(1);
        long statTime = Convert.toLong(searchDateTime.format(timeFormatter));
        List<RtcRankItem> list = rtcRankService.getRtcHourRankPageList(statTime, 1, 20).getList();

        for (int idx = list.size() - 1; idx >= 0; idx--) {
            //当前元素
            RtcRankItem rankItem = list.get(idx);
            if (idx == 0) {
                rankItem.setDiffValue(0L);
                break;
            }
            //上一名元素
            RtcRankItem preRankItem = list.get(idx - 1);
            rankItem.setDiffValue(preRankItem.getScoreValue() - rankItem.getScoreValue());
        }

        MutableResult mResult = new MutableResult();
        mResult.put("momentType", momentType);
        mResult.put("timestamp", System.currentTimeMillis());
        mResult.put("cycle", searchDateTime.format(DateTimeFormatter.ofPattern("MM月dd日 HH时")));
        mResult.put("list", list);

        return mResult.build();
    }

    /**
     * 获取pk擂台榜
     *
     * @return
     */
    @GET
    @Path("getRtcBattleRank")
    @Override
    public Result getRtcBattleRank() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", rtcRankService.getRtcBattleRank(DateUtil.getWeekStartTimeStamp(), DateUtil.getWeekEndTimeStamp()));
        return mResult.build();
    }
}
