package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.pojo.rtc.AppRtcRoomAuctionVO;
import com.starbuds.server.api.app.util.RtcUtil;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcRoomRole;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.ShareParam;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.db.RtcRoomLiveHisDO;
import com.starbuds.server.common.pojo.rtc.RtcRankService;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomAuctionVO;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.*;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.NoblePrivilegeService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.skywalking.apm.toolkit.trace.TraceContext;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.QueryParam;
import java.util.Optional;
import java.util.stream.Collectors;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * @author laoliu
 * @date 2021/11/1 18:37
 * @description 语聊房处理基类
 */
@Component
@Slf4j
public class RtcBaseFacade extends BaseFacade {

    @Value("${domain}")
    protected String domain;

    @Value("${appName}")
    protected String appName;

    @Reference(version = "1.0.0")
    protected RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    protected FamilyService familyService;

    @Reference(version = "1.0.0")
    protected DataService dataService;

    @Reference(version = "1.0.0")
    protected UserService userService;

    @Reference(version = "1.0.0")
    protected RtcQueueService rtcQueueService;

    @Reference(version = "1.0.0")
    protected RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    protected RtcRankService rtcRankService;

    @Reference(version = "1.0.0")
    protected RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    protected RtcLoveService rtcLoveService;

    @Reference(version = "1.0.0")
    protected RedisService redisService;

    @Reference(version = "1.0.0")
    protected NoblePrivilegeService noblePrivilegeService;

    @Reference(version = "1.0.0")
    protected ImService imService;

    @Reference(version = "1.0.0")
    protected RtcBattleService rtcBattleService;

    @Resource
    protected RtcUtil rtcUtil;

    @Reference(version = "1.0.0")
    protected RtcRoomAuctionService rtcRoomAuctionService;

    @Reference(version = "1.0.0")
    protected RtcRoomLiveHisService rtcRoomLiveHisService;

    @Reference(version = "1.0.0")
    protected LotteryGameService lotteryGameService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;


    public Result getRoomInfoBase(Long roomId, String pass) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "未找到该房间");
        }

        Long userId = this.getToken().getUserId();

        // 判断是否小黑屋
        if (rtcRoomService.isBlackListUser(roomId, userId)) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "您已被拒绝进入该房间");
        }

        RtcRoom rtcRoom = rtcRoomService.getRoom(roomId);
        //是否休息
        boolean isRest = !rtcRoomService.isRoomOnline(roomId);
        //个播房 且 休息中
        if ((RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoom.getBusinessType().intValue()) && isRest) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "休息中");
        }
        //判断语音房是否有权限进入
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if ((roomRole == RtcRoomRole.AUDIENCE || roomRole == RtcRoomRole.HOST) && isRest) {
            return buildFailResult(false, GlobalApiError.OperationFailed, "休息中");
        }

        if (roomRole == RtcRoomRole.AUDIENCE) {
            // 判断房间是否上锁
            String lockPass = rtcRoomService.getRoomLockPass(roomId);
            if (StringUtils.isNotBlank(lockPass)) {
                // 上锁了需要密码
                if (StringUtils.isBlank(pass)) {
                    MutableResult mResult = new MutableResult();
                    mResult.put("needPass", true);
                    mResult.put("goAuth", "0");
                    return mResult.build();
                } else if (!pass.equals(lockPass)) {
                    return buildFailResult(false, GlobalApiError.OperationFailed, "房间密码错误");
                }
            }
        }

        MutableResult mResult = new MutableResult();
        // 拍卖房关闭自由上麦
        if (rtcRoom != null && RoomTypeEnum.AUCTION == RoomTypeEnum.toEnum(rtcRoom.getRoomType())) {
            rtcRoom.setRoomMicFree(CommonStatus.Disabled);
            rtcRoom.setBackgroundUrl("http://media.qn.panygo.com/rtc/background/1627802066221.png");
        }
        mResult.put("timestamp", System.currentTimeMillis());
        mResult.put("needPass", false);
        mResult.put("room", rtcRoom);

        mResult.put("isLive", isRest == false);
        mResult.put("role", rtcRoomService.getRoomRole(roomId, userId));
        mResult.put("seats", rtcSeatService.getSeats(roomId));
        mResult.put("authInfo", rtcRoomService.getAgoraAuthInfo(roomId, userId, getHeader().getAppVersion()));
        mResult.put("isFav", rtcRoomService.isRoomFavorite(roomId, userId));
        mResult.put("profile", rtcUserService.getUserProfile(userId, roomId));
        mResult.put("battleRoomIn", rtcBattleService.getCurrentBattleRoomIn(roomId));
        mResult.put("battleEnabled", rtcBattleService.isRoomBattleEnabled(roomId));
        mResult.put("hourRankEnabled", rtcRoomService.getRtcHourRankStatus());
        mResult.put("battleRoomCross", rtcBattleService.getCurrentBattleRoomCross(roomId));
        if (userId.equals(rtcRoom.getUserId())) {
            if (!userService.isAuthExist(userId)) {
                return buildFailResult(true, GlobalApiError.OperationFailed, "您尚未实名认证");
            } else {
                mResult.put("goAuth", "0");
            }
        } else {
            mResult.put("goAuth", "0");
        }

        // 红包雨活动参数
        mResult.put("lotteryGameIds", lotteryGameService.getInProgressingSnaps().stream().map(LotteryGameSnapshotDO::getSnapId).collect(Collectors.toList()));

        // 分享参数
        String title = String.format("来%s，一起玩起来！", appName);
        String content = String.format("「%s」房间很好玩，推荐给你，快来和我一块玩吧。", rtcRoom.getRoomName());
        ShareParam shareParam = new ShareParam();
        shareParam.setShareUrl(String.format("http://iv.m.%s/?ci=%d&i=%s", domain, this.getHeader().getChannelId(), Long.toHexString(userService.getUserNo(userId))));
        shareParam.setShareTitle(title);
        shareParam.setShareSubTitle(content);
        shareParam.setSharePic(rtcRoom.getRoomCover());
        mResult.put("shareParam", shareParam);

        // 相亲房
        if (RoomTypeEnum.LOVE.getValue() == rtcRoom.getRoomType()) {
            mResult.put("stage", rtcLoveService.getStage(roomId));
        }
        // 拍卖房
        if (RoomTypeEnum.AUCTION.getValue() - rtcRoom.getRoomType() == 0) {
            // 拍卖中
            if (rtcRoomAuctionService.isBidding(roomId)) {
                // 拍卖信息
                String auctionStr = redisService.get(RedisRtcKeys.getRtcAuctionRoomInfo(roomId));
                if (JSON.isValidObject(auctionStr)) {
                    RtcRoomAuctionVO auction = JSON.parseObject(auctionStr, RtcRoomAuctionVO.class);
                    if (auction != null) {
                        AppRtcRoomAuctionVO auctionAppVO = new AppRtcRoomAuctionVO();
                        BeanUtils.copyProperties(auction, auctionAppVO);
                        mResult.put("auction", auctionAppVO);
                    }
                }
            }
        }

        return mResult.build();
    }


    private Result buildFailResult(boolean needGoAuth, GlobalApiError errCode, String msg) {
        MutableResult failBuild = new MutableResult();
        if (needGoAuth) {
            failBuild.put("goAuth", "1");
        } else {
            failBuild.put("goAuth", "0");
        }
        Result failResult = failBuild.build();
        failResult.setSuccess(false);
        failResult.setCode(errCode.getCode());
        failResult.setMsg(msg);
        return failResult;
    }
}
