package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.app.pojo.rtc.AppRtcRoomAuctionVO;
import com.starbuds.server.api.app.pojo.rtc.singleRtc.SingleRtcRoomAppVO;
import com.starbuds.server.api.app.pojo.rtc.singleRtc.SingleRtcRoomPassEnum;
import com.starbuds.server.api.app.pojo.rtc.singleRtc.SingleRtcRoomStatVO;
import com.starbuds.server.api.app.pojo.rtc.singleRtc.SingleRtcRoomVO;
import com.starbuds.server.api.app.util.RtcUtil;
import com.starbuds.server.api.app.v1.facade.RtcRoomSingleLiveFacade;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.DataGreenTypeEnum;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.*;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.ShareParam;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.rtc.RtcLiveStatusMsg;
import com.starbuds.server.common.pojo.message.im.rtc.SingleRtcLiveStatusMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcUserJoinMsg;
import com.starbuds.server.common.pojo.rtc.*;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomAuctionVO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.pojo.util.PageQO;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomCoverAuditService;
import com.starbuds.server.common.service.rtc.RtcRoomLiveHisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * RTC 房间个播接口实现
 *
 * @author laoliu
 * @date 2021/10/27
 */
@Path("/v1/rtc/single/")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RtcRoomSingleLiveFacadeImpl extends BaseFacade implements RtcRoomSingleLiveFacade {

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    WordFilterService wordFilterService;

    @Reference(version = "1.0.0", timeout = 10000, retries = 0)
    private DataGreenService dataGreenService;

    @Reference(version = "1.0.0")
    private DataGreenUserService dataGreenUserService;

    @Reference(version = "1.0.0")
    private RtcRoomLiveHisService rtcRoomLiveHisService;

    @Reference(version = "1.0.0")
    private RtcRoomCoverAuditService rtcRoomCoverAuditService;

    @Reference(version = "1.0.0")
    private ImService imService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Resource
    private RtcUtil rtcUtil;

    /**
     * 获取个播前置信息
     */
    @GET
    @Path("getLive")
    @Override
    public Result getSingleLive() {
        // 参数封面地址，是否是新图片，直播标题，房间标签(男神，女神)
        Long userId = this.getToken().getUserId();
        Long channelId = this.getHeader().getChannelId();

        // 校验
        SingleRtcRoomPassEnum isPassEnum = checkSingleRtcRoomPermit(userId);
        if (!isPassEnum.equals(SingleRtcRoomPassEnum.PASS)) {
            SingleRtcRoomAppVO appVO = new SingleRtcRoomAppVO();
            appVO.setPassType(isPassEnum.getValue());
            return Result.ofSuccess(appVO);
        }

        // 判断该用户在该家族下是否有直播间
        Long familyId = familyService.getFamilyId(userId);

        RtcRoomDO singleRtcRoom = rtcRoomService.getSingleRtcRoom(userId, familyId);
        if (singleRtcRoom == null) {
            // 无直播间，则将默认的头像，家族返回回去
            SingleRtcRoomAppVO appVO = new SingleRtcRoomAppVO();
            appVO.setPassType(SingleRtcRoomPassEnum.PASS.getValue());
            SingleRtcRoomVO roomVO = new SingleRtcRoomVO();
            AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(userId, channelId, RtcBusinessTypeEnum.RTC_SINGLE.getValue());
            roomVO.setRoomCover(userService.getUserAvatar(userId));
            roomVO.setFamilyName(familyService.getFamilyName(familyId));
            roomVO.setRoomCoverAuditStatus(AuditStatusEnum.SUCCEED.getValue());
            roomVO.setFamilyId(familyId);
            roomVO.setRtcRoomCategories(authRtcConfig.getRtcRoomCategories());
            appVO.setSingleRtcRoom(roomVO);
            return Result.ofSuccess(appVO);
        }

        // 有直播间，直接设置为2：关播状态
//        if (singleRtcRoom.getStatus().equals(BoolIntVal.False)) {
//            RtcRoomEditDTO dto = new RtcRoomEditDTO();
//            dto.setStatus(RtcRoomStatusEnum.CLOSE.getValue());
//            dto.setRoomId(singleRtcRoom.getRoomId());
//            rtcRoomService.editRoom(dto);
//        }

        SingleRtcRoomAppVO appVO = new SingleRtcRoomAppVO();
        appVO.setPassType(SingleRtcRoomPassEnum.PASS.getValue());
        SingleRtcRoomVO roomVO = new SingleRtcRoomVO();
        AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(userId, channelId, RtcBusinessTypeEnum.RTC_SINGLE.getValue());
        BeanUtils.copyProperties(singleRtcRoom, roomVO);
        roomVO.setFamilyName(familyService.getFamilyName(familyId));

        // 将正在审核中的图片展示出来
        RtcRoomCoverAuditDO record = rtcRoomCoverAuditService.getLastestAuditingRecord(singleRtcRoom.getRoomId());
        if (record == null) {
            roomVO.setRoomCover(singleRtcRoom.getRoomCover());
            roomVO.setRoomCoverAuditStatus(AuditStatusEnum.SUCCEED.getValue());
        } else {
            roomVO.setRoomCover(record.getRoomCover());
            GreenAuditStatusEnum auditStatusEnum = GreenAuditStatusEnum.toEnum(record.getStatus());
            if (GreenAuditStatusEnum.AUDITING.equals(auditStatusEnum)) {
                roomVO.setRoomCoverAuditStatus(AuditStatusEnum.AUDITING.getValue());
            } else if (GreenAuditStatusEnum.PASSED_ADMIN.equals(auditStatusEnum) || GreenAuditStatusEnum.PASSED_ROBOT.equals(auditStatusEnum)) {
                roomVO.setRoomCoverAuditStatus(AuditStatusEnum.SUCCEED.getValue());
            } else {
                roomVO.setRoomCoverAuditStatus(AuditStatusEnum.FAILED.getValue());
            }
        }
        roomVO.setRtcRoomCategories(authRtcConfig.getRtcRoomCategories());
        roomVO.setRtcRoomTypes(authRtcConfig.getRtcRoomTypes());
        appVO.setSingleRtcRoom(roomVO);
        return Result.ofSuccess(appVO);
    }

    /**
     * 开始直播
     */
    @POST
    @Path("startLive")
    @Override
    public Result startLive(@FormParam("roomCover") String roomCover,
                            @FormParam("roomHorizontalCover") String roomHorizontalCover,
                            @FormParam("roomName") String roomName,
                            @FormParam("categoryId") Long categoryId,
                            @FormParam("roomType") Integer roomType) {

        Long userId = this.getToken().getUserId();
        Long channelId = this.getHeader().getChannelId();

        Long familyId = familyService.getFamilyId(userId);
        RtcRoomDO singleRtcRoom = rtcRoomService.getSingleRtcRoom(userId, familyId);

        if (isDev()) {
            log.info("[startLive]userId:{}, channelId: {}, familyId:{},singleRtcRoom:{}", userId, channelId, familyId, JSON.toJSONString(singleRtcRoom));
        }

        RtcRoomEditDTO dto = new RtcRoomEditDTO();

        if (singleRtcRoom == null) {

            // 创建房间

            AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(userId, channelId, RtcBusinessTypeEnum.RTC_SINGLE.getValue());

            if (isDev()) {
                log.info("[startLive]authRtcConfig:{}", JSON.toJSONString(authRtcConfig));
            }

            // 判断房间类型
            if (roomType == null) {
                if (CollectionUtil.isNotEmpty(authRtcConfig.getRtcRoomTypes())) {
                    roomType = authRtcConfig.getRtcRoomTypes().get(0).getRoomType();
                } else {
                    return Result.ofBadRequest("请联系管理员添加个播分类");
                }
            }


            if (RoomTypeEnum.toEnum(roomType) == null) {
                return Result.ofBadRequest("不支持该房间类型");
            }

            // 判断类目是否存在
            if (!dataService.isRtcRoomCategoryExists(categoryId)) {
                return Result.ofBadRequest("无法设置该标签");
            }

            // 判断房间名称
            if (StringUtils.isBlank(roomName)) {
                return Result.ofBadRequest("请填写房间名称");
            }

            // 授权语聊配置
            if (authRtcConfig == null) {
                return Result.ofBadRequest("该功能逐步开放中，敬请期待");
            }

            // 用户开通房间数
            // 个播先不限制房间数
//            int roomCount = rtcRoomService.getSingleRtcRoomCountByUserId(userId);
//            if (roomCount >= authRtcConfig.getRoomNum()) {
//                return Result.ofBadRequest("您的房间数量超上限");
//            }

            // 房间分类
            String authRoomCategory = authRtcConfig.getRoomCategory();
            if (!authRoomCategory.contains(Convert.toStr(categoryId))) {
                return Result.ofBadRequest("您未获得该类目授权");
            }

            // 房间类型
            String authRoomType = authRtcConfig.getRoomType();
            if (!authRoomType.contains(Convert.toStr(roomType))) {
                return Result.ofBadRequest("您未获得该类型授权");
            }


            /**
             * 开通房间
             */
            UserCardVO userCardVO = userService.getNameAndAvatarById(userId);

            // 审核房间标题
            if (!roomName.equals(userCardVO.getUserName())) {
                String auditDesc = checkSingleRtcRoomName(roomName);
                if (auditDesc != null) {
                    return Result.ofOperationFailed(auditDesc);
                }
            }

            RtcRoomAddDTO rtcRoomAddDTO = new RtcRoomAddDTO();
            rtcRoomAddDTO.setUserId(userId);
            rtcRoomAddDTO.setCategoryId(categoryId);
            rtcRoomAddDTO.setRoomCover(userCardVO.getUserAvatar());
            rtcRoomAddDTO.setRoomHorizontalCover(userCardVO.getUserAvatar());
            rtcRoomAddDTO.setRoomName(roomName);
            rtcRoomAddDTO.setRoomType(roomType);
            rtcRoomAddDTO.setChannelId(channelId);
            rtcRoomAddDTO.setBusinessType(RtcBusinessTypeEnum.RTC_SINGLE.getValue());
            rtcRoomAddDTO.setFamilyId(familyService.getFamilyId(userId));
            Long roomId = rtcRoomService.addRoom(rtcRoomAddDTO);
            if (roomId == null) {
                return Result.ofOperationFailed("房间创建失败");
            }

            // 开始审核封面与标题
            if (roomCover != null) {
                if (!roomCover.equals(userCardVO.getUserAvatar())) {
                    if (createAuditRecord(userId, roomId, roomCover, familyId)) {
                        dto.setRoomCover(roomCover);
                    }
                }
            }

            // 这里hack产品需求，做一波骚操作，以后要改
//            singleRtcRoom = rtcRoomService.getSingleRtcRoom(userId, familyId);
            return Result.ofOperationFailed("房间审核中, 请联系家族长");
            // 骚操作结束
        } else {

            // 判断房间是否禁用
            if (singleRtcRoom.getStatus().equals(CommonStatus.Disabled)) {

                // 判断房间是否真的被禁用
                if (rtcRoomLiveHisService.existByRoomId(singleRtcRoom.getRoomId())) {
                    return Result.ofOperationFailed("房间已被封禁");
                }

                return Result.ofOperationFailed("房间审核中, 请联系家族长");
            }

            // 判断房间信息是否发生改变
            // 判断封面
            if (roomCover != null) {
                if (!roomCover.equals(singleRtcRoom.getRoomCover())) {
                    if (createAuditRecord(userId, singleRtcRoom.getRoomId(), roomCover, familyId)) {
                        dto.setRoomCover(roomCover);
                    }
                }
            }

            // 判断标题
            if (!roomName.equals(singleRtcRoom.getRoomName())) {
                String auditDesc = checkSingleRtcRoomName(roomName);
                if (auditDesc != null) {
                    return Result.ofOperationFailed(auditDesc);
                }
            }
        }

        // 修改房间信息
        dto.setRoomId(singleRtcRoom.getRoomId());
        dto.setRoomName(roomName);
        dto.setCategoryId(categoryId);
        dto.setStatus(RtcRoomStatusEnum.OPEN.getValue());
        rtcRoomService.editRoom(dto);

        // 将主播上主持麦
        rtcUtil.sitSeat(userId, singleRtcRoom.getRoomId(), 0, getHeader().getAppVersion());

        // 将主播手动加入房间，因为融云此时没有消息推送说用户加入房间
        redisService.hashPut(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId), String.valueOf(singleRtcRoom.getRoomId()));

        if (isDev()) {
            log.info("[startLive]主播上主持麦");
        }

        // 给房间内所有人发送主播返回房间消息
        rtcRoomService.sendAnchorBackRoomMsg(userId, singleRtcRoom.getRoomId());

        if (isDev()) {
            log.info("[startLive]给房间内所有人发送主播返回房间消息");
        }

        // 查询最近一次还在场的个播场次
        RtcRoomLiveHisDO his = rtcRoomLiveHisService.getCurrentRound(singleRtcRoom.getRoomId());
        Long singleRtcHisId = 0L;
        if (his == null) {
            // 创建新的场次
            RtcRoomLiveHisDO hisDO = new RtcRoomLiveHisDO();
            hisDO.setRoomId(singleRtcRoom.getRoomId());
            hisDO.setFamilyId(familyId);
            hisDO.setRoomName(singleRtcRoom.getRoomName());
            hisDO.setUserId(userId);
            hisDO.setUserNo(userService.getUserNo(userId).toString());
            hisDO.setUserName(userService.getUserName(userId));
            hisDO.setLiveBeginTime(System.currentTimeMillis());
            RtcRoomLiveHisDO hisRecord = rtcRoomLiveHisService.create(hisDO);

            singleRtcHisId = hisRecord.getHisId();
        } else {
            // 说明还有进行中的场次，可能是崩溃又重新进来了
            singleRtcHisId = his.getHisId();
        }

        if (isDev()) {
            log.info("[startLive]查询最近一次还在场的个播场次");
        }

        // 发送开播通知
        SingleRtcLiveStatusMsg statusMsg = new SingleRtcLiveStatusMsg();
        statusMsg.setStatus(BoolIntVal.True);
        statusMsg.setRoomId(singleRtcRoom.getRoomId());
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, singleRtcRoom.getRoomId(), statusMsg);

        // 发送开播通知 -- 这个是通用的开关播信息；上面的个播可以几个版本后移除
        RtcLiveStatusMsg normalStatusMsg = new RtcLiveStatusMsg();
        normalStatusMsg.setStatus(BoolIntVal.True);
        normalStatusMsg.setRoomId(singleRtcRoom.getRoomId());
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, singleRtcRoom.getRoomId(), normalStatusMsg);

        MutableResult mResult = new MutableResult();
        mResult.put("roomId", singleRtcRoom.getRoomId());
        mResult.put("singleRtcHisId", singleRtcHisId);

        return mResult.build();
    }

    /**
     * 本场直播收益统计
     * singleRtcHisId 场次id
     */
    @GET
    @Path("getIncome")
    @Override
    public Result getLiveIncome(@QueryParam("roomId") Long roomId) {
        RtcRoomLiveHisDO his = rtcRoomLiveHisService.getLastRound(roomId);

        if (his == null) {
            return Result.ofOperationFailed("未获取到最新一场直播");
        }

        SingleRtcRoomStatVO statVO = new SingleRtcRoomStatVO();

        BeanUtils.copyProperties(his, statVO);

        // 今日总时长，毫秒数
        statVO.setTodayTotalTimeMillis(rtcRoomLiveHisService.getTodayTotalTimeMillis(roomId));
        statVO.setTodayRoundNum(rtcRoomLiveHisService.getTodayRoundNum(roomId));
        return Result.ofSuccess(statVO);
    }

    /**
     * 我的直播记录
     */
    @GET
    @Path("getMyLiveHis")
    @Override
    public Result getMyLiveHis(@BeanParam PageQO pageQO) {

        Long userId = this.getToken().getUserId();

        Long familyId = familyService.getFamilyId(userId);

        RtcRoomDO room = rtcRoomService.getSingleRtcRoom(userId, familyId);

        if (room == null) {
            return Result.ofOperationFailed("还没有开播记录哦");
        }

        PageList<RtcRoomLiveHisDO> his = rtcRoomLiveHisService.getRecentLiveHis(pageQO.getPageIndex(), pageQO.getPageSize(), room.getRoomId());
        return Result.ofSuccess(his);
    }

    /**
     * 关播
     */
    @POST
    @Path("stopLive")
    @Override
    public Result stopLive(@FormParam("roomId") Long roomId) {
        Long userId = this.getToken().getUserId();

        if (roomId == null) {
            return Result.ofBadRequest("roomId缺失");
        }

        RtcRoom room = rtcRoomService.getRoom(roomId);
        if (room == null) {
            return Result.ofOperationFailed("无此房间");
        }

        if (!room.getUserId().equals(userId)) {
            return Result.ofOperationFailed("您没有权限关别人的直播间");
        }

        rtcRoomService.stopSingleRtcLive(roomId, userId, this.getHeader().getAppVersion(), "主播手动关播");

        return Result.ofSuccess();
    }

    /**
     * 个播房间推荐
     * 要个播房间前6个，不足6个的用语聊房补齐
     */
    @GET
    @Path("getSingleRtcRoomSuggestList")
    @Override
    public Result getSingleRtcRoomSuggestList() {
        // 获取热门个播房间
        List<RtcRoomListItem> singleHotRoomList = rtcRoomService.getHotRoomList(6, RtcBusinessTypeEnum.RTC_SINGLE.getValue());
        if (singleHotRoomList == null) {
            singleHotRoomList = new ArrayList<>();
        }

        int length = 6;
        int size = length - singleHotRoomList.size();
        // 不足6个用首页语聊房补齐
        List<RtcRoomListItem> hotRoomList = rtcRoomService.getHotRoomList(size, RtcBusinessTypeEnum.RTC_NORMAL.getValue());
        singleHotRoomList.addAll(hotRoomList);
        return Result.ofList(singleHotRoomList);
    }

    /**
     * 创建封面审核记录
     *
     * @param userId
     * @param roomId
     * @param roomCover
     * @param familyId
     * @return
     */
    private boolean createAuditRecord(Long userId, Long roomId, String roomCover, Long familyId) {
        GreenAuditStatusEnum auditStatusEnum = checkSingleRtcRoomCover(userId, roomCover);
        RtcRoomCoverAuditDO auditDO = new RtcRoomCoverAuditDO();
        auditDO.setRoomId(roomId);
        auditDO.setRoomCover(roomCover);
        auditDO.setUserId(userId);
        auditDO.setStatus(Optional.ofNullable(auditStatusEnum).orElse(GreenAuditStatusEnum.AUDITING).value());
        auditDO.setReason(Optional.ofNullable(auditStatusEnum).orElse(GreenAuditStatusEnum.AUDITING).title());
        auditDO.setFamilyId(familyId);
        auditDO.setFamilyName(familyService.getFamilyName(familyId));
        rtcRoomCoverAuditService.create(auditDO);
        return GreenAuditStatusEnum.PASSED_ROBOT.equals(auditStatusEnum);
    }

    /**
     * 校验用户能否开始直播
     *
     * @param userId
     * @return
     */
    private SingleRtcRoomPassEnum checkSingleRtcRoomPermit(Long userId) {

        // 手机号
        String phone = userService.getPhone(userId);
        if (StringUtils.isBlank(phone)) {
            return SingleRtcRoomPassEnum.NO_PHONE;
        }

        // 身份证
        UserAuthDO user = userService.getUserAuth(userId);
        if (user == null || StringUtils.isBlank(user.getRealName()) || StringUtils.isBlank(user.getIdCardNo())) {
            return SingleRtcRoomPassEnum.NO_REAL;
        }

        // 家族
        Long familyId = familyService.getFamilyId(userId);
        if (familyId == null) {
            return SingleRtcRoomPassEnum.NO_FAMILY;
        }

        return SingleRtcRoomPassEnum.PASS;
    }

    /**
     * 个播房间标题审核
     *
     * @return
     */
    private String checkSingleRtcRoomName(String roomName) {
        if (StringUtils.isBlank(roomName)) {
            return null;
        }

        // 判断是否包含敏感词汇
        Long userId = this.getToken().getUserId();
        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(roomName);
        dataGreenUser.setType(DataGreenTypeEnum.TEXT.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return "保存个播房间标题检测记录失败";
        }

        Long dataId = dataGreenUser.getDataId();
        log.info("个播房间标题审核开始 userId={} roomName={} dataId={}", userId, roomName, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.textAudit(roomName, userId.toString(), dataId, GreenBizTypeEnum.SINGLE_RTC_ROOM_NAME);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("个播房间标题审核接收数据 userId={} roomName={} dataId={} robotAuditStatus={}", userId, roomName, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-用户信息 {} {}", update, upPo);

        if (robotAuditStatus != GreenAuditStatusEnum.PASSED_ROBOT) {
            return "内容涉及敏感词";
        }

        return null;
    }

    /**
     * 个播房间封面审核
     *
     * @return
     */
    private GreenAuditStatusEnum checkSingleRtcRoomCover(Long userId, String roomCover) {
        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(roomCover);
        dataGreenUser.setType(DataGreenTypeEnum.IMAGE.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return null;
        }

        Long dataId = dataGreenUser.getDataId();
        log.info("个播房间封面-审核开始 userId={} url={} dataId={}", userId, roomCover, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.imgAudit(roomCover, userId.toString(), dataId, GreenBizTypeEnum.SINGLE_RTC_ROOM_COVER);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("个播房间封面-审核接收数据 userId={} url={} dataId={} robotAuditStatus={}", userId, roomCover, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-个播房间封面 {} {}", update, upPo);

        return robotAuditStatus;
    }
}
