package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.RtcRoomSuggestFacade;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomSuggestVO;
import com.starbuds.server.common.service.rtc.RtcRoomSuggestService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import java.util.ArrayList;
import java.util.List;

/**
 * RTC 房间推荐
 *
 * @author aoe
 * @date 2021/10/12
 */
@Path("/v1/rtc")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RtcRoomSuggestFacadeImpl extends BaseFacade implements RtcRoomSuggestFacade {

    @Reference(version = "1.0.0")
    private RtcRoomSuggestService rtcRoomSuggestService;

    private static final String DEFAULT_TITLE = "pany语音";

    private static final String DEFAULT_SUB_TITLE = "快乐因为有你";

    /**
     * 首页为你推荐语聊房间列表
     *
     * @return
     */
    @GET
    @Path("getRtcRoomSuggestList")
    @PermitGuest
    @Override
    public Result getRtcRoomSuggestList(@QueryParam("showType") Integer showType) {
        List<RtcRoomSuggestVO> res = rtcRoomSuggestService.getRoomSuggestList(showType);
        if (showType != null && showType == 1 && CollectionUtil.isEmpty(res)) {
            res = new ArrayList<>();
            RtcRoomSuggestVO tmp = new RtcRoomSuggestVO();
            tmp.setTitle(DEFAULT_TITLE);
            tmp.setSubTitle(DEFAULT_SUB_TITLE);
            res.add(tmp);
        }
        return Result.ofList(res);
    }
}
