package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.app.pojo.signIn.UserSignInConfigVO;
import com.starbuds.server.api.app.v1.facade.SignInFacade;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.UserSignInConfigDO;
import com.starbuds.server.common.pojo.db.UserSignInHisDO;
import com.starbuds.server.common.service.rtc.RtcRoomAuctionService;
import com.starbuds.server.common.service.user.UserSignInService;
import com.starbuds.server.common.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 用户签到接口实现
 *
 * @author lujiao
 * @date 2021/8/16 16:01
 */
@Path("/v1/user")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class SignInFacadeImpl extends BaseFacade implements SignInFacade {

    @Reference(version = "1.0.0")
    private UserSignInService userSignInService;

    private static final Integer STEP_END = 7;

    /**
     * 用户签到
     *
     * @return
     */
    @Override
    @GET
    @Path("signIn")
    public Result signIn() {
        Long userId = this.getToken().getUserId();
        Long channelId = this.getHeader().getChannelId();
        return userSignInService.userSignIn(userId, channelId);
    }

    /**
     * 获取签到配置
     *
     * @return
     */
    @Override
    @GET
    @Path("getSignInConfig")
    public Result getSignInConfig() {
        Long userId = this.getToken().getUserId();
        List<UserSignInConfigDO> configDOList = userSignInService.getSignInConfig();
        if (CollectionUtil.isNotEmpty(configDOList)) {
            List<UserSignInConfigVO> configVOList = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(configDOList)) {
                configDOList.forEach(i -> {
                    UserSignInConfigVO vo = new UserSignInConfigVO();
                    BeanUtils.copyProperties(i, vo);
                    configVOList.add(vo);
                });
            }
            UserSignInHisDO lastSignInHis = userSignInService.getLastSignHis(userId);
            Integer index = 0;
            Integer haveSignIn = 0;
            if (lastSignInHis != null && lastSignInHis.getHisId() > 0) {
                index = userSignInService.getStep(lastSignInHis);
                Long dayBegin = DateUtil.getDayBegin();
                if (lastSignInHis.getCreateTime() >= dayBegin) {
                    haveSignIn = 1;
                }
            }

            MutableResult mResult = new MutableResult();
            mResult.put("configList", configVOList);
            mResult.put("index", index);
            mResult.put("haveSignIn", haveSignIn);
            return mResult.build();
        } else {
            MutableResult mResult = new MutableResult();
            mResult.put("configList", new ArrayList<>());
            mResult.put("index", 0);
            mResult.put("haveSignIn", 1);
            return mResult.build();
        }
    }

    /**
     * 新增弹窗记录
     *
     * @param index
     * @return
     */
    @Override
    @POST
    @Path("addSignInToastHis")
    public Result addSignInToastHis(@FormParam("index") Integer index) {
        Long userId = this.getToken().getUserId();
        Long channelId = this.getHeader().getChannelId();
        userSignInService.addSignInToastHis(userId, index, channelId);
        return new Result();
    }
}
