package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.v1.facade.StatFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.statistics.StatInviteService;
import com.starbuds.server.common.service.user.SocialRelationService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

/**
 * 统计接口实现
 *
 * @author Lay
 * @date 2020/2/20
 */
@Path("/v1/stat")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class StatFacadeImpl extends BaseFacade implements StatFacade {


    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    StatInviteService statInviteService;

    /**
     * 获取邀请统计
     *
     * @return
     */
    @GET
    @Path("getInviteStat")
    @Override
    public Result getInviteStat(@QueryParam("agentId") Long agentId) {
        if (agentId == null) {
            return new Result(GlobalApiError.ServerError, "请传入公会id");
        }

        Long userId = this.getToken().getUserId();

        MutableResult mResult = new MutableResult();
        mResult.put("inviteCount", socialRelationService.getInviteCount(userId));
        mResult.put("rewardStar", statInviteService.getTotalReward(agentId, userId));
        return mResult.build();
    }

    /**
     * 获取邀请明细
     *
     * @param pageIndex
     * @param pageSize
     * @param agentId
     * @return
     */
    @GET
    @Path("getInviteDetail")
    @Override
    public Result getInviteDetail(@QueryParam("pageIndex") int pageIndex,
                                  @QueryParam("pageSize") int pageSize,
                                  @QueryParam("agentId") Long agentId) {
        Long userId = this.getToken().getUserId();
        return new Result(statInviteService.getInvitePageList(pageIndex, pageSize, userId, agentId));
    }

}
