package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.finance.WalletService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import java.math.BigDecimal;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;

/**
 * 测试专用
 *
 * @author aoe
 * @date 2020/11/20
 */
@Path("/v1/test")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class TestFacadeImpl extends BaseFacade {

    @Reference(version = "1.0.0")
    private WalletService walletService;

    @Reference(version = "1.0.0")
    PayService payService;

    /**
     * 昨天的星尘转粉钻
     *
     * @param today 今天（格式 2020-07-30）
     * @param ytd   昨天（格式 2020-07-29）
     * @return
     */
    @POST
    @Path("transYtdStarDustToPinkDiamond")
    public Result transYtdStarDustToPinkDiamond(@FormParam("today") String today,
                                                @FormParam("ytd") String ytd) {
        if (!isDev()) {
            return new Result(GlobalApiError.BadRequest);
        }
        Long userId = getToken().getUserId();
        boolean flag = walletService.transYtdStarDustToPinkDiamond(userId, today, ytd);
        return new Result(flag);
    }


    /**
     * 充值成功送叉子消息
     */
    @POST
    @Path("testSendRechargeFinish")
    public void sendRechargeFinish(@FormParam("userId") Long userId, @FormParam("orderPayId") Long orderPayId, @FormParam("amount") BigDecimal amount) {
        if (!isDev()) {
            return;
        }
        payService.testSendRechargeFinish(userId, orderPayId, amount);
    }

    /**
     * 模拟充值回调
     */
    @POST
    @Path("imitatePayCall")
    public String imitateRechargeCallback(@FormParam("orderId") Long orderId,
                                          @FormParam("buyerId") String buyerId,
                                          @FormParam("buyerAccount") String buyerAccount,
                                          @FormParam("sellerId") String sellerId,
                                          @FormParam("sellerAccount") String sellerAccount,
                                          @FormParam("totalFee") BigDecimal totalFee,
                                          @FormParam("transcationId") String transcationId) {
        if (!isDev()) {
            return "环境错误";
        }

        boolean imitateResult = payService.imitateRechargeCallback(orderId, buyerId, buyerAccount, sellerId, sellerAccount, totalFee, transcationId);

        return imitateResult ? "ok" : "fail";
    }
}
