package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.symmetric.AES;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.DataFeedbackReq;
import com.starbuds.server.api.app.pojo.QuestionAnswerReq;
import com.starbuds.server.api.app.pojo.UserReq;
import com.starbuds.server.api.app.pojo.UserSwitchReq;
import com.starbuds.server.api.app.util.PinyinUtil;
import com.starbuds.server.api.app.v1.facade.UserFacade;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.*;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import com.starbuds.server.common.constant.message.sms.SmsActionEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.FriendStatusEnum;
import com.starbuds.server.common.constant.user.LogoutStatusEnum;
import com.starbuds.server.common.constant.user.OAuthTypeEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.ImageGreenAuditResult;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.pojo.data.ReportAddDTO;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.mq.QueueUserFirstRechargeMsg;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.finance.UserVipVO;
import com.starbuds.server.common.service.finance.VipService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.message.SmsService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.*;
import com.starbuds.server.common.utils.LogUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.*;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import static com.starbuds.server.api.app.util.SpringContextUtil.isDev;
import static com.starbuds.server.common.utils.DateUtil.getWeekEnd;
import static com.starbuds.server.common.utils.DateUtil.getWeekStart;

/**
 * 用户接口实现
 *
 * @author jiangxiaowei
 * @date 2019/12/18 17:31
 */
@Path("/v1/user")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class UserFacadeImpl extends BaseFacade implements UserFacade {

    public static final String AES_KEY = "vbrde2efr7w9gcrbdiov3dyf5l3c5r0r";

    public static final long durationTime = 180 * 24 * 3600 * 1000L;

    @Value("${wx.appId}")
    private String appId;

    @Value("${wx.secret}")
    private String secret;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    WordFilterService wordFilterService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    ThirdAuthService thirdAuthService;

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Reference(version = "1.0.0")
    NiceNoService niceNoService;

    @Reference(version = "1.0.0")
    VipService vipService;

    @Reference(version = "1.0.0")
    SecurityService securityService;

    @Reference(version = "1.0.0")
    SmsService smsService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    ThirdPartyService thirdPartyService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    UserGiftWallService userGiftWallService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    ModuleService moduleService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    ChatBubbleService chatBubbleService;

    @Reference(version = "1.0.0")
    PayService payService;


    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Reference(version = "1.0.0")
    private DataGreenUserService dataGreenUserService;

    @Reference(version = "1.0.0", timeout = 10000, retries = 0)
    private DataGreenService dataGreenService;

    /**
     * 修改个人资料
     *
     * @param userName   用户姓名
     * @param userSex    用户性别
     * @param userAvatar 用户头像
     * @param userAge    用户年龄
     * @param countryId  国家
     * @param areaId     区域id
     * @param userSign   个性签名
     * @param userCover  颜值封面
     * @return result对象
     */
    @Path("editProfile")
    @POST
    @Override
    public Result editProfile(@FormParam("userName") String userName,
                              @FormParam("userSex") Integer userSex,
                              @FormParam("userAvatar") String userAvatar,
                              @FormParam("userAge") Integer userAge,
                              @FormParam("countryId") Integer countryId,
                              @FormParam("areaId") Integer areaId,
                              @FormParam("userSign") String userSign,
                              @FormParam("userCover") String userCover,
                              @FormParam("userTags") String userTags,
                              @FormParam("userHobby") String userHobby,
                              @FormParam("userMarriage") String userMarriage,
                              @FormParam("userMarkingFriends") String userMarkingFriends,
                              @FormParam("userHeight") String userHeight,
                              @FormParam("userWeight") String userWeight,
                              @FormParam("userIncome") String userIncome,
                              @FormParam("userBornTime") Long userBornTime,
                              @FormParam("userConstellation") Integer userConstellation) {
        // 签名判断
        String checkSignResult = checkUserSign(userSign);
        if (checkSignResult != null) {
            return new Result(GlobalApiError.BadRequest, checkSignResult);
        }

        // 昵称判断
        if (StringUtils.isNotBlank(userName)) {
            String checkResult = checkUserName(userName);
            if (checkResult != null) {
                return new Result(GlobalApiError.BadRequest, checkResult);
            }
        }

        Long userId = getToken().getUserId();
        // 图片内容审核
        if (StringUtils.isNotBlank(userAvatar)) {
            String errMsg = auditUserAvatar(userId, userAvatar);
            if (errMsg != null) {
                return Result.ofBadRequest(errMsg);
            }
        }

        UserDO userDO = new UserDO();
        userDO.setUserName(userName);
        userDO.setUserSex(userSex);
        userDO.setUserAvatar(userAvatar);
        userDO.setUserAge(userAge);
        userDO.setCountryId(countryId);
        userDO.setAreaId(areaId);
        userDO.setUserSign(userSign);
        userDO.setUserCover(userCover);
        userDO.setUserId(userId);
        userDO.setUserTags(userTags);
        userDO.setUserHobby(userHobby);
        userDO.setUserMarriage(userMarriage);
        userDO.setUserMarkingFriends(userMarkingFriends);
        userDO.setUserHeight(userHeight);
        userDO.setUserWeight(userWeight);
        userDO.setUserIncome(userIncome);
        userDO.setUserBornTime(userBornTime);
        userDO.setUserConstellation(userConstellation);
        userDO.setUserUpdateTime(System.currentTimeMillis());
        boolean isSuccess = userService.editProfile(userDO);
        if (isSuccess) {
            // 星芽用户修改信息通知第三方
            NoticeUserUpdate userUpdate = new NoticeUserUpdate();
            userUpdate.setUserId(userDO.getUserId());
            userUpdate.setUserName(userDO.getUserName());
            userUpdate.setUserAvatar(userDO.getUserAvatar());
            thirdPartyService.sendNoticeUserUpdate(userUpdate);
        }

        if (isDev()) {
            log.info(LogUtil.devLog("编辑用户信息 userId={} areaId={} isSuccess={}"), userId, areaId, isSuccess);
        }

        return new Result();
    }

    /**
     * 编辑用户资料
     *
     * @param req
     * @return
     */
    @POST
    @Path("editUserNew")
    @Override
    public Result editUserNew(@BeanParam UserReq req) {

        Long userId = this.getToken().getUserId();

        String userSign = req.getUserSign();
        String userName = req.getUserName();
        String userAvatar = req.getUserAvatar();

        // 签名判断
        String checkSignResult = checkUserSign(userSign);
        if (checkSignResult != null) {
            return new Result(GlobalApiError.BadRequest, checkSignResult);
        }

        // 昵称判断
        if (StrUtil.isNotEmpty(userName)) {
            String checkResult = checkUserName(userName);
            if (checkResult != null) {
                return new Result(GlobalApiError.BadRequest, checkResult);
            }
        }

        // 图片内容审核
        if (StrUtil.isNotEmpty(userAvatar)) {
            String errMsg = auditUserAvatar(userId, userAvatar);
            if (errMsg != null) {
                return Result.ofOperationFailed(errMsg);
            }
        }

        UserDO userDO = new UserDO();
        BeanUtils.copyProperties(req, userDO);
        userDO.setUserId(userId);
        userDO.setUserUpdateTime(System.currentTimeMillis());
        if (userService.editProfile(userDO)) {
            NoticeUserUpdate userUpdate = null;
            // 星芽用户修改信息通知第三方
            if (StrUtil.isNotEmpty(userDO.getUserName())) {
                userUpdate = new NoticeUserUpdate();
                userUpdate.setUserName(userDO.getUserName());
            }
            if (StrUtil.isNotEmpty(userDO.getUserAvatar())) {
                if (userUpdate == null) {
                    userUpdate = new NoticeUserUpdate();
                }
                userUpdate.setUserAvatar(userDO.getUserAvatar());
            }
            // 只有更新了头像或名称时，才通知
            if (userUpdate != null) {
                userUpdate.setUserId(userId);
                try {
                    thirdPartyService.sendNoticeUserUpdate(userUpdate);
                } catch (Exception e) {
                    log.error("出现错误", e);
                }
            }
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 修改昵称
     *
     * @param userName
     * @return
     */
    @POST
    @Path("setName")
    @Override
    public Result setName(@FormParam("userName") String userName) {
        Long userId = this.getToken().getUserId();
        String checkResult = checkUserName(userName);
        if (checkResult != null) {
            return new Result(GlobalApiError.BadRequest, checkResult);
        }

        // 判断是否合规
//        if (!ReUtil.isMatch("[a-zA-Z0-9\\-\\.【】、\\u4E00-\\u9FA5]{1,15}", userName)) {
//            return new Result(GlobalApiError.OperationFailed, "昵称不符合规范，请重新设置");
//        }

        // 判断改名卡是否足够
//        if (bagService.getPropsQuantityByType(userId, PropsTypeEnum.RENAME_CARD) == 0) {
////            return new Result(GlobalApiError.OperationFailed, "改名卡数量不足");
////        }

        // 判断是否已存在
        boolean isCached = cacheService.hashPutIfAbsent(RedisUserKeys.USER_NAME_LOCK, userName, String.valueOf(userId));
        try {
            if (!isCached) {
                return new Result(GlobalApiError.OperationFailed, "该昵称已被使用");
            }

            // 消耗改名卡
//            Integer remain = bagService.takePropsByType(userId, PropsTypeEnum.RENAME_CARD, 1, "修改昵称", 0L);
//            if (remain != null) {
//                if (remain == -1) {
//                    return new Result(GlobalApiError.OperationFailed, "改名卡数量不足");
//                }
//                // 消耗成功 修改昵称
//                userService.setName(userId, userName);
//
//                return new Result();
//            }
            if (userService.setName(userId, userName)) {
                // 星芽用户修改信息通知第三方
                NoticeUserUpdate userUpdate = new NoticeUserUpdate();
                userUpdate.setUserId(userId);
                userUpdate.setUserName(userName);
                thirdPartyService.sendNoticeUserUpdate(userUpdate);

                return new Result();
            }
        } finally {
            if (isCached) {
                // 释放
                cacheService.hashDel(RedisUserKeys.USER_NAME_LOCK, userName);
            }
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取用户资料
     *
     * @param userId 用户id
     * @return result对象
     */
    @Path("getProfile")
    @GET
    @PermitGuest
    @Override
    public Result getProfile(@QueryParam("userId") Long userId) {
        // 当前用户id
        Long currentUserId = getToken() == null ? 0 : getToken().getUserId();

        if (userId == null) {
            userId = currentUserId;
        } else {
            if (!userService.isUserExist(userId)) {
                return new Result(GlobalApiError.OperationFailed, "用户不存在");
            } else if (securityService.isLogout(userId)) {
                UserProfile profile = new UserProfile();
                profile.setUserName("已注销");
                return new Result(profile);
            } else if (userService.isDisabled(userId)) {
                return new Result(GlobalApiError.OperationFailed, "用户已封禁");
            }
        }

        if (isDev()) {
            log.info("getProfile, userId:{}", userId);
        }

        UserProfile profile = userService.getProfile(userId);

        if (isDev()) {
            log.info("userService.getProfile, userId:{}", JSON.toJSONString(profile));
        }

        // 城市语言处理
        String lang = this.getHeaderString("lang");
        if (I18nTypeEnum.ZH_CN != I18nTypeEnum.toEnum(lang) && profile.getCityInfo() != null) {
            profile.getCityInfo().setAreaCity(PinyinUtil.firstCharUpperCase(PinyinUtil.convert(profile.getCityInfo().getAreaCity())));
            profile.setCountryName(profile.getCountryNameEn());
        }

        // 隐藏关注数
        int hideFollower = userService.getSwitchStatus(userId, SwitchTypeEnum.HIDE_FOLLOWERS);

        if (profile != null) {
            UserNobleVO userNoble = nobleService.getUserNoble(userId);
            UserVipVO userVip = vipService.getUserVip(userId);
            profile.setNobleLevel(userNoble == null ? 0 : userNoble.getNobleLevel());
            profile.setVipLevel(userVip == null ? 0 : userVip.getVipLevel());
            profile.setRtcRoom(rtcRoomService.getUserStayRoom(userId));
            profile.setVisitCount(userService.getVisitorTotalByUser(userId));
            profile.setIntervieweeCount(userService.getIntervieweeTotalByUser(userId));

            profile.setActivitiesMedalIdArr(scoreService.getScoreLevelIds(userId));

            if (userId.equals(currentUserId)) {
                // 自己
                profile.setIsFollower(BoolIntVal.True);
                profile.setIsFriend(BoolIntVal.True);
                profile.setFriendStatus(FriendStatusEnum.ALREADY_ADD.getCode());
                profile.setFansCount(socialRelationService.getFansCount(userId));
                profile.setFollowCount(socialRelationService.getFollowCount(userId));
                profile.setCharmValue(Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.CHARM)).orElse(0L));
                profile.setWealthValue(Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.WEALTH)).orElse(0L));
                profile.setStarValue(Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.STAR)).orElse(0L));
                profile.setIsHideFollow(hideFollower);
                profile.setUserNoble(userNoble);
                profile.setUserNiceNo(niceNoService.getUserNiceNo(userId));
                profile.setUserVip(userVip);

                // 自己的经纬度
                Double myselfLatitude = Convert.toDouble(this.getHeader().getLatitude());
                Double myselfLongitude = Convert.toDouble(this.getHeader().getLongitude());
                Integer areaId = null;
                if (myselfLatitude == null || myselfLongitude == null) {
                    IpLocation ipLocation = areaService.getIpLocation(this.getIpAddress());
                    if (ipLocation != null) {
                        areaId = ipLocation.getAreaId();
                        myselfLatitude = Optional.ofNullable(ipLocation.getCenterLatitude()).orElse(ipLocation.getLbLatitude());
                        myselfLongitude = Optional.ofNullable(ipLocation.getCenterLongitude()).orElse(ipLocation.getLbLongitude());
                    }
                }
                // 更新经纬度
                if (myselfLatitude != null && myselfLongitude != null) {
                    Double finalMyselfLatitude = myselfLatitude;
                    Double finalMyselfLongitude = myselfLongitude;
                    Integer finalAreaId = areaId;
                    CompletableFuture.runAsync(() -> {
                        userService.setPosition(currentUserId, finalMyselfLatitude, finalMyselfLongitude, finalAreaId);
                    });
                }
            } else {
                if (BoolIntVal.True == hideFollower) {
                    profile.setFollowCount(0);
                } else {
                    profile.setFollowCount(socialRelationService.getFollowCount(userId));
                }
                profile.setIsFollower(socialRelationService.isFollower(currentUserId, userId) ? BoolIntVal.True : BoolIntVal.False);
                profile.setIsFriend(socialRelationService.isFriend(currentUserId, userId) ? BoolIntVal.True : BoolIntVal.False);
                profile.setFriendStatus(profile.getIsFriend() == BoolIntVal.True ? FriendStatusEnum.ALREADY_ADD.getCode() : socialRelationService.getFriendStatus(currentUserId, userId));
                profile.setFansCount(socialRelationService.getFansCount(userId));
                profile.setCharmValue(Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.CHARM)).orElse(0L));
                profile.setWealthValue(Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.WEALTH)).orElse(0L));
                profile.setStarValue(Optional.ofNullable(scoreService.getScoreValue(userId, ScoreTypeEnum.STAR)).orElse(0L));
                profile.setIsHideFollow(hideFollower);
                profile.setUserNoble(userNoble);
                profile.setUserNiceNo(niceNoService.getUserNiceNo(userId));
                profile.setUserVip(userVip);

                // 隐藏经纬度手机号
                profile.setUserLatitude(0D);
                profile.setUserLongitude(0D);
                profile.setUserPhone("");

                // 创建访问记录
                userService.addVisitHis(currentUserId, userId);
            }

            // 头饰
            profile.setHeadwear(headwearService.getUserEnabledHeadwear(userId));
            // 聊天气泡
            profile.setChatBubble(chatBubbleService.getUserEnabledBubble(userId));
            // 勋章
            profile.setMedal(medalService.getEnabledMedal(userId));

            // 用户定位城市
            String userLocationCity = profile.getLocationCity();
            if (StringUtils.isNotEmpty(userLocationCity)) {
                String[] split = userLocationCity.split("-");
                // 定位信息只显示城市
                if (split.length > 2) {
                    profile.setLocationCity(split[2]);
                }
            }
        }

        return new Result(profile);
    }

    /**
     * 获取用户资料
     *
     * @param userId 用户id
     * @return result对象
     */
    @GET
    @Path("getProfileNew")
    @PermitGuest
    @Override
    public Result getProfileNew(@QueryParam("userId") Long userId) {

        // 当前用户id
        Long currentUserId = getToken() == null ? 0 : getToken().getUserId();
        if (userId == null) {
            userId = currentUserId;
        } else {
            if (!userService.isUserExist(userId)) {
                return new Result(GlobalApiError.OperationFailed, "用户不存在");
            } else if (securityService.isLogout(userId)) {
                return new Result(GlobalApiError.OperationFailed, "用户已注销");
            }
        }

        MutableResult mResult = new MutableResult();
        List<UserProfileGroup> profileNew = userService.getProfileNew(userId);
        mResult.put("list", profileNew);
        mResult.put("score", CollectionUtil.isNotEmpty(profileNew) ? Convert.toInt(profileNew.stream().mapToDouble(UserProfileGroup::getGroupScore).average().getAsDouble()) : 0);
        return mResult.build();
    }


    /**
     * 检查用户是否已经触发首充
     *
     * @param userId 用户id
     * @return result对象
     */
    @GET
    @Path("getUserFirstPayCheck")
    @PermitGuest
    @Override
    public Result getUserFirstPayCheck(@QueryParam("userId") Long userId) {

        UserDO user = userService.getUserById(userId);
        int count = payService.getUserPayOrderCount(userId);
        MutableResult mResult = new MutableResult();
        //用户支付成功订单数为1并且没有触发过首充事件
        if (count == 1 && user.getIsFirstRechargedClick() == 0) {
            mResult.put("isCheck", 0);
            userService.setUserIsFirstRechargedClick(userId);
        } else {
            mResult.put("isCheck", 1);
        }
        return mResult.build();
    }


    /**
     * 获取用户资料分数
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getProfileScore")
    @PermitGuest
    @Override
    public Result getProfileScore(@QueryParam("userId") Long userId) {
        // 当前用户id
        Long currentUserId = getToken() == null ? 0 : getToken().getUserId();
        if (userId == null) {
            userId = currentUserId;
        } else {
            if (!userService.isUserExist(userId)) {
                return new Result(GlobalApiError.OperationFailed, "用户不存在");
            } else if (securityService.isLogout(userId)) {
                return new Result(GlobalApiError.OperationFailed, "用户已注销");
            }
        }
        return new Result(userService.getProfileScore(userId));
    }

    /**
     * 举报用户
     *
     * @param userId
     * @param roomId
     * @param reasonId
     * @param remarks
     * @param images
     * @return
     */
    @POST
    @Path("report")
    @Override
    public Result report(@FormParam("userId") Long userId,
                         @FormParam("roomId") Long roomId,
                         @FormParam("reasonId") Long reasonId,
                         @FormParam("remarks") String remarks,
                         @FormParam("images") String images) {
        // 判断用户是否存在
        if (userId != null && !userService.isUserExist(userId)) {
            return new Result(GlobalApiError.OperationFailed, "用户不存在");
        }

        // 判断房间是否存在
        if (roomId != null && !rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "房间不存在");
        }

        // 判断举报原因
        if (!dataService.isReportReasonExists(reasonId)) {
            return new Result(GlobalApiError.OperationFailed, "举报原因错误");
        }

        ReportAddDTO reportAddDTO = ReportAddDTO.builder()
                .reportUserId(userId).roomId(roomId).reportReasonId(reasonId)
                .reportRemarks(remarks).reportImages(images)
                .reporterUserId(this.getToken().getUserId())
                .deviceSystemVersion(Optional.ofNullable(this.getHeader().getSystemVersion()).orElse(""))
                .deviceModel(Optional.ofNullable(this.getHeader().getPhoneModel()).orElse(""))
                .appVersion(Optional.ofNullable(this.getHeader().getAppVersion()).orElse(""))
                .build();

        Long reportId = dataService.addReport(reportAddDTO);
        if (reasonId != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("reportId", reportId);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置开关
     *
     * @param switchReq
     * @return
     */
    @POST
    @Path("setSwitch")
    @Override
    public Result setSwitch(@BeanParam UserSwitchReq switchReq) {
        Long userId = this.getToken().getUserId();
        switchReq.setUserId(userId);

        if (SwitchTypeEnum.MOUNTS.getCode() == switchReq.getType()) {
            // 判断是否有该坐骑
            if (bagService.getItemQuantity(switchReq.getUserId(), BagItemTypeEnum.MOUNTS, switchReq.getItemId()) == 0) {
                return new Result(GlobalApiError.OperationFailed, "未在您的背包中找到该坐骑，或者坐骑已过期");
            }
        }

        if (SwitchTypeEnum.HEADWEAR.getCode() == switchReq.getType()) {
            // 判断是否有该头饰
            if (bagService.getItemQuantity(switchReq.getUserId(), BagItemTypeEnum.HEADWEAR, switchReq.getItemId()) == 0) {
                return new Result(GlobalApiError.OperationFailed, "未在您的背包中找到该头饰，或者头饰已过期");
            }
        }

        if (SwitchTypeEnum.CHAT_BUBBLE.getCode() == switchReq.getType()) {
            // 判断是否有该气泡框
            if (bagService.getItemQuantity(switchReq.getUserId(), BagItemTypeEnum.CHAT_BUBBLE, switchReq.getItemId()) == 0) {
                return new Result(GlobalApiError.OperationFailed, "未在您的背包中找到该聊天气泡，或者聊天气泡已过期");
            }
        }

        if (SwitchTypeEnum.MEDAL.getCode() == switchReq.getType()) {
            // 判断是否有该勋章
            if (bagService.getItemQuantity(switchReq.getUserId(), BagItemTypeEnum.MEDAL, switchReq.getItemId()) == 0) {
                return new Result(GlobalApiError.OperationFailed, "未在您的背包中找到该勋章，或者勋章已过期");
            }
        }

//        if (SwitchTypeEnum.HIDE_FOLLOWERS.getCode() == switchReq.getType() && SwitchStatusEnum.OPEN.getCode() == switchReq.getSwitchStatus()) {
//            if (nobleService.getUserNobleLevel(userId) < NobleLevelEnum.BOJUE.getCode()) {
//                return new Result(GlobalApiError.OperationFailed, String.format("%s及以上贵族专享，快去开通贵族吧", NobleLevelEnum.BOJUE.getDesc()));
//            }
//        }

        UserSwitchDO switchDO = new UserSwitchDO();
        BeanUtils.copyProperties(switchReq, switchDO);
        switchDO.setSwitchId(String.format("%d:%d", switchReq.getUserId(), switchReq.getType()));
        if (userService.setSwitch(switchDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取用户开关
     *
     * @return
     */
    @GET
    @Path("getUserSwitch")
    @Override
    public Result getUserSwitch() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", userService.getSwitchList(this.getToken().getUserId()));
        return mResult.build();
    }

    /**
     * 获取用户开关
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getSwitchList")
    @Override
    public Result getSwitchList(@QueryParam("userId") Long userId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", userService.getSwitchList(userId == null ? this.getToken().getUserId() : userId));
        return mResult.build();
    }

    /**
     * 提交意见
     *
     * @param feedbackReq
     * @return
     */
    @POST
    @Path("submitSuggest")
    @Override
    public Result submitSuggest(@BeanParam DataFeedbackReq feedbackReq) {
        DataFeedbackDO feedbackDO = new DataFeedbackDO();
        BeanUtils.copyProperties(feedbackReq, feedbackDO);

        feedbackDO.setUserId(this.getToken().getUserId());
        feedbackDO.setCreateTime(System.currentTimeMillis());
        feedbackDO.setStatus(CommonStatus.Normal);
        feedbackDO.setHandleStatus(HandleStatusEnum.WAIT.getValue());
        feedbackDO.setDeviceModel(Optional.ofNullable(this.getHeader().getPhoneModel()).orElse(""));
        feedbackDO.setDeviceSystemVersion(Optional.ofNullable(this.getHeader().getSystemVersion()).orElse(""));
        feedbackDO.setAppVersion(Optional.ofNullable(this.getHeader().getAppVersion()).orElse(""));
        if (dataService.submitSuggest(feedbackDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取意见反馈详情
     *
     * @param feedId
     * @return
     */
    @GET
    @Path("getSuggestDetail")
    @Override
    public Result getSuggestDetail(@QueryParam("feedId") Long feedId) {
        return new Result(dataService.getSuggestDetail(feedId));
    }

    /**
     * 获取我的意见反馈列表
     *
     * @param feedType
     * @param handleStatus
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GET
    @Path("getSuggestPageList")
    @Override
    public Result getSuggestPageList(@QueryParam("feedType") Integer feedType,
                                     @QueryParam("handleStatus") Integer handleStatus,
                                     @QueryParam("pageNum") int pageNum,
                                     @QueryParam("pageSize") int pageSize) {
        return new Result(dataService.getFeedbackPageList(this.getToken().getUserId(), feedType, handleStatus, pageNum, pageSize));
    }

    /**
     * 获取积分等级详情
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getScoreLevelInfo")
    @Override
    public Result getScoreLevelInfo(@QueryParam("scoreType") int scoreType) {
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "积分类型无效");
        }
        return new Result(scoreService.getScoreLevelInfo(this.getToken().getUserId(), scoreTypeEnum));
    }

    /**
     * 获取绑定账户
     *
     * @return
     */
    @GET
    @Path("getBingAccount")
    @Override
    public Result getBingAccount() {
        return new Result(userService.getBingAccount(this.getToken().getUserId(), this.getHeader().getChannelId()));
    }

    /**
     * 绑定oauth
     *
     * @param oauthType
     * @param code
     * @return
     */
    @POST
    @Path("bindOauth")
    @Override
    public Result bindOauth(@FormParam("oauthType") Integer oauthType, @FormParam("code") String code) {
        Long channelId = Optional.ofNullable(this.getHeader().getChannelId()).orElse(0L);
        OAuthTypeEnum typeEnum = OAuthTypeEnum.getOauthType(oauthType);
        String unionId = null;
        String openId = null;
        // 判断是否已经绑定
        if (securityService.isOauthBind(this.getToken().getUserId(), channelId, oauthType)) {
            return new Result(GlobalApiError.OperationFailed, "您已绑定此应用，不能重复绑定");
        }
        switch (typeEnum) {
            case WEIXIN_APP:
                WxAccessTokenResp wxAccessToken = thirdAuthService.getWxAppAccessToken(channelId, code);
                if (wxAccessToken == null) {
                    return new Result(GlobalApiError.OperationFailed, "未获取到微信accessToken");
                }
                WxAppUserInfoResp infoResp = thirdAuthService.getWxAppUserInfo(channelId, wxAccessToken.getAccessToken());
                unionId = infoResp.getUnionId();
                openId = infoResp.getOpenId();
                break;
            case QQ_APP:
                unionId = thirdAuthService.getQqUnionId(channelId, code);
                openId = thirdAuthService.getQqOpenId(channelId, code);
                break;
            case WEIBO_APP:
                WbUserInfoResp wbUserInfo = thirdAuthService.getWbUserInfo(channelId, code);
                if (wbUserInfo == null) {
                    return new Result(GlobalApiError.OperationFailed, "未获取到微博用户信息");
                }
                unionId = wbUserInfo.getId().toString();
                break;
            case HUAWEI_APP:
                HuaweiAccessTokenResp accessTokenResp = thirdAuthService.getHuaweiAccessToken(channelId, code);
                if (accessTokenResp == null || StringUtils.isBlank(accessTokenResp.getAccessToken())) {
                    return new Result(GlobalApiError.OperationFailed, "未获取到华为accessToken");
                }
                HuaweiTokenInfoResp tokenInfoResp = thirdAuthService.getHuaweiTokenInfo(channelId, accessTokenResp.getAccessToken());
                if (tokenInfoResp == null) {
                    return new Result(GlobalApiError.OperationFailed, "未获取到华为用户信息");
                }
                unionId = tokenInfoResp.getUnionId();
                break;
            case FACEBOOK_APP:
                FacebookUserInfoResp facebookUserInfo = thirdAuthService.getFacebookUserInfo(channelId, code);
                if (facebookUserInfo != null) {
                    unionId = facebookUserInfo.getClientId();
                    openId = facebookUserInfo.getId();
                }
                break;
            case GOOGLE_APP:
                GoogleUserInfoResp googleUserInfo = thirdAuthService.getGoogleUserInfo(channelId, code);
                if (googleUserInfo != null) {
                    unionId = googleUserInfo.getClientId();
                    openId = googleUserInfo.getId();
                }
                break;
            default:
                return new Result(GlobalApiError.OperationFailed, "暂未开通此方式授权，绑定失败");
        }

        // 根据 unionId 查询的用户id
        Long userIdByUnionId = securityService.getUserIdByUnionId(unionId, channelId, oauthType);
        if (userIdByUnionId != null) {
            return new Result(GlobalApiError.OperationFailed, "该应用已绑定其他账号，不能重复绑定");
        }
        // 执行绑定
        if (securityService.saveOauthInfo(this.getToken().getUserId(), channelId, typeEnum.getType(), unionId, openId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 绑定手机号
     *
     * @param userPhone
     * @param code
     * @param countryId
     * @return
     */
    @POST
    @Path("bindPhone")
    @Override
    public Result bindPhone(@FormParam("userPhone") String userPhone,
                            @FormParam("code") String code,
                            @FormParam("countryId") Integer countryId) {

        if (countryId == null) {
            countryId = 1;
        }

        String areaCode = dataService.getAreaCode(countryId);

        if (securityService.isPhoneExists(userPhone, areaCode)) {
            return new Result(GlobalApiError.OperationFailed, "该手机号已绑定");
        }

        String fullPhone = "86".equals(areaCode) ? userPhone : areaCode + userPhone;
        if (smsService.checkSmsCode(SmsActionEnum.BIND, fullPhone, code) || smsService.checkSmsCode(SmsActionEnum.LOGIN, fullPhone, code)) {
            if (securityService.bindPhone(this.getToken().getUserId(), userPhone, areaCode)) {
                return new Result();
            }
            return new Result(GlobalApiError.OperationFailed, "绑定失败");
        } else {
            return new Result(GlobalApiError.OperationFailed, "短信验证码无效或已过期，请重新输入");
        }

    }

    /**
     * 绑定支付宝
     *
     * @param code
     * @param alipayAccount
     * @return
     */
    @POST
    @Path("bindAlipay")
    @Override
    public Result bindAlipay(@FormParam("code") String code,
                             @FormParam("alipayAccount") String alipayAccount) {
        Long userId = this.getToken().getUserId();

        if (thirdAuthService.isAlipayBind(userId)) {
            return new Result(GlobalApiError.OperationFailed, "您已绑定过支付宝账户");
        }

        if (StringUtils.isBlank(alipayAccount)) {
            return new Result(GlobalApiError.OperationFailed, "请填写支付宝账户并仔细核对！");
        }

        if (StringUtils.isNotBlank(code)) {
            if (thirdAuthService.bindAlipay(userId, alipayAccount, code)) {
                return new Result();
            }
        }

        return new Result(GlobalApiError.OperationFailed, "绑定失败");
    }

    /**
     * 获取支付宝授权绑定参数
     *
     * @return
     */
    @GET
    @Path("getAlipayBindParam")
    @Override
    public Result getAlipayBindParam() {
        AlipayBindParam bindParam = thirdAuthService.getAlipayBindParam(this.getToken().getUserId());
        if (bindParam == null) {
            return new Result(GlobalApiError.OperationFailed, "无法获取支付宝授权参数");
        }
        return new Result(bindParam);
    }

    /**
     * 获取绑定的支付宝账户
     *
     * @return
     */
    @GET
    @Path("getAlipay")
    @PermitGuest
    @Override
    public Result getAlipay() {
        if (getToken() == null) {
            return new Result();
        }
        UserAlipayVO userAlipayVO = thirdAuthService.getAlipay(this.getToken().getUserId());
        if (userAlipayVO != null) {
            userAlipayVO.setAlipayAccount(hideAlipayAccount(userAlipayVO.getAlipayAccount()));
            userAlipayVO.setAlipayUserId(null);
            return new Result(userAlipayVO);
        }
        return new Result();
    }

    /**
     * 获取邀请码二维码
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getInviteQrCode")
    @PermitGuest
    @Override
    public Result getInviteQrCode(@QueryParam("userId") Long userId) {
        if (userId == null) {
            userId = this.getToken().getUserId();
        }
        UserDO userDO = userService.selectById(userId);
        String toEncrypt = String.format("%d:%d:%d", userId, userDO.getUserNo(), userDO.getChannelId());

        AES aes = new AES(AES_KEY.getBytes());
        String qrCode = aes.encryptBase64(toEncrypt);
        MutableResult mResult = new MutableResult();
        mResult.put("qrCode", qrCode);
        mResult.put("inviteCode", intToHex(Convert.toInt(userDO.getUserNo())));
        mResult.put("channelId", userDO.getChannelId());
        return mResult.build();
    }

    /**
     * 获取手机号
     *
     * @return
     */
    @GET
    @Path("getPhone")
    @Override
    public Result getPhone() {
        Long userId = this.getToken().getUserId();
        String phone = userService.getPhone(userId);
        String areaCode = userService.getAreaCode(userId);
        Integer countryId = userService.getCountryId(userId);
        MutableResult mResult = new MutableResult();
        mResult.put("phone", Base64.encode(phone));
        mResult.put("areaCode", areaCode);
        mResult.put("countryId", countryId);
        return mResult.build();
    }

    /**
     * 申请注销
     *
     * @return
     */
    @POST
    @Path("applyLogout")
    @Override
    public Result applyLogout() {
        Long userId = this.getToken().getUserId();

        if (securityService.isCoolPeriod(userId)) {
            return new Result(GlobalApiError.ServerError, "已经处于冷静期，请勿重复提交");
        }

        long curTime = System.currentTimeMillis();
        UserLogoutDO logoutDO = new UserLogoutDO();
        logoutDO.setUserId(userId);
        logoutDO.setCreateTime(curTime);
        logoutDO.setStatus(LogoutStatusEnum.WAITING.getCode());
        logoutDO.setLogoutTime(curTime + durationTime);
        logoutDO.setResumeTime(null);
        if (securityService.applyLogout(logoutDO)) {
            // 清理token
            securityService.removeToken(userId);
            // 清理es
            userSearchService.deleteById(userId);
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 取消注销
     *
     * @return
     */
    @POST
    @Path("cancelLogout")
    @Override
    public Result cancelLogout() {

        Long userId = this.getToken().getUserId();

        if (!securityService.isCoolPeriod(userId)) {
            return new Result(GlobalApiError.ServerError, "不在冷静期，无需取消");
        }

        if (securityService.cancelLogout(userId)) {
            UserDO userDO = userService.selectById(userId);
            userSearchService.insertOrUpdate(userDO);
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取用户号
     *
     * @param params
     * @return
     */
    @GET
    @Path("getUserByUserNoOrNiceNo")
    @PermitGuest
    @Override
    public Result getUserByUserNoOrNiceNo(@QueryParam("params") Long params) {

        if (params == null) {
            return new Result(GlobalApiError.ServerError, "请传入星芽号或靓号");
        }

        return new Result(userService.getUserByUserNoOrNiceNo(params));
    }

    /**
     * 获取邀请用户列表
     *
     * @param pageIndex
     * @param hasRecharged
     * @return
     */
    @GET
    @Path("getInviteList")
    @Override
    public Result getInviteList(@QueryParam("pageIndex") int pageIndex,
                                @QueryParam("hasRecharged") Integer hasRecharged) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        Long userId = this.getToken().getUserId();
        return new Result(userService.getInviteUserList(pageIndex, GlobalConfig.PAGE_SIZE, userId, hasRecharged));
    }

    /**
     * code 换取 openId
     *
     * @param code
     * @return
     */
    @GET
    @Path("getOpenId")
    @PermitGuest
    @Override
    public Result getOpenId(@QueryParam("code") String code) {
        String result = HttpUtil.get(String.format("https://api.weixin.qq.com/sns/oauth2/access_token?appid=%s&secret=%s&code=%s&grant_type=authorization_code", appId, secret, code));
        if (StringUtils.isNotBlank(result)) {
            JSONObject json = JSON.parseObject(result);
            if (!json.containsKey("errcode")) {
                MutableResult mResult = new MutableResult();
                mResult.put("openId", json.getString("openid"));
                return mResult.build();
            } else {
                return new Result(GlobalApiError.OperationFailed, json.getString("errmsg"));
            }
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取礼物墙
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getGiftWallList")
    @Override
    public Result getGiftWallList(@QueryParam("userId") Long userId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", userGiftWallService.getGiftWallList(Optional.ofNullable(userId).orElse(this.getToken().getUserId())));
        return mResult.build();
    }

    /**
     * 获取周礼物列表
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getWeekGiftWallList")
    @PermitGuest
    @Override
    public Result getWeekGiftWallList(@QueryParam("userId") Long userId) {
        if (userId == null && getToken() != null) {
            userId = getToken().getUserId();
        }
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的参数");
        }
        MutableResult mResult = new MutableResult();
        if (isDev()) {
            log.info("获取周礼物列表 {} {} {}", userId, getWeekStart(), getWeekEnd());
        }
        mResult.put("list", userGiftWallService.getGiftWallList(userId, getWeekStart(), getWeekEnd()));
        return mResult.build();
    }

    /**
     * 获取礼物墙统计
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getGiftWallStat")
    @PermitGuest
    @Override
    public Result getGiftWallStat(@QueryParam("userId") Long userId) {
        if (userId == null && getToken() != null) {
            userId = getToken().getUserId();
        }
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的参数");
        }
        return new Result(userGiftWallService.getGiftWallStatistics(userId));
    }

    /**
     * 获取等级
     *
     * @param scoreType
     * @return
     */
    @GET
    @Path("getLevelAndScore")
    @Override
    public Result getLevelAndScore(@QueryParam("scoreType") Integer scoreType) {
        if (scoreType == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        ScoreTypeEnum scoreTypeEnum = ScoreTypeEnum.toEnum(scoreType);
        if (scoreTypeEnum == null) {
            return new Result(GlobalApiError.ServerError, "请确认类型是否正确");
        }
        Long userId = this.getToken().getUserId();
        UserCardVO card = userService.getCard(userId);
        MutableResult mResult = new MutableResult();
        mResult.put("userName", Optional.ofNullable(card).map(UserCardVO::getUserName).orElse(""));
        mResult.put("userAvatar", Optional.ofNullable(card).map(UserCardVO::getUserAvatar).orElse(""));
        mResult.put("scoreValue", userService.getScoreValue(userId, scoreType));
        int scoreLevel = 1;
        switch (scoreTypeEnum) {
            case WEALTH:
                scoreLevel = userService.getWealthLevelByUserId(userId);
                break;
            case CHARM:
                scoreLevel = userService.getCharmLevelByUserId(userId);
                break;
            case STAR:
                scoreLevel = userService.getStarLevelByUserId(userId);
                break;
            default:
                break;
        }
        mResult.put("scoreLevel", scoreLevel);
        DataLevelScoreDO nextLevel = dataService.getNextLevelScore(scoreLevel, scoreTypeEnum);
        mResult.put("nextLevel", Optional.ofNullable(nextLevel).map(DataLevelScoreDO::getScoreLevel).orElse(null));
        mResult.put("nextValue", Optional.ofNullable(nextLevel).map(DataLevelScoreDO::getScoreValue).orElse(null));
        return mResult.build();
    }

    /**
     * 获取用户贵族
     *
     * @return
     */
    @GET
    @Path("getUserNobleLevel")
    @Override
    public Result getUserNobleLevel() {
        return new Result(nobleService.getUserNoble(this.getToken().getUserId()));
    }

    /**
     * 获取用户模块
     *
     * @return
     */
    @GET
    @Path("getUserModules")
    @Override
    public Result getUserModules() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", moduleService.getUserModules());
        return mResult.build();
    }

    /**
     * 设置自问自答
     *
     * @param answerReq
     * @return
     */
    @POST
    @Path("setQuestionAnswer")
    @Override
    public Result setQuestionAnswer(@BeanParam QuestionAnswerReq answerReq) {
        if (StringUtils.isEmpty(answerReq.getQuestion())) {
            return new Result(GlobalApiError.OperationFailed, "问题不能为空");
        }

        Long userId = this.getToken().getUserId();
        QuestionAnswerDO answerDO = new QuestionAnswerDO();
        BeanUtils.copyProperties(answerReq, answerDO);
        answerDO.setStatus(CommonStatus.Normal);
        answerDO.setUserId(userId);
        answerDO.setUpdateTime(System.currentTimeMillis());
        // 添加
        if (answerReq.getQuestionId() == null) {
            if (userService.addQuestionAnswer(answerDO)) {
                return new Result();
            }
        } else {
            if (!userService.isQuestionExist(answerReq.getQuestionId(), userId)) {
                return new Result(GlobalApiError.OperationFailed, "问题不存在");
            }
            // 更新
            if (userService.editQuestionAnswer(answerDO)) {
                return new Result();
            }
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 删除自问自答
     *
     * @param questionId
     * @return
     */
    @POST
    @Path("delQuestionAnswer")
    @Override
    public Result delQuestionAnswer(@FormParam("questionId") Long questionId) {
        Long userId = this.getToken().getUserId();
        if (!userService.isQuestionExist(questionId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "问题不存在");
        }
        if (userService.setQuestionStatus(questionId, CommonStatus.Deleted)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取用户简单信息
     *
     * @param userIds
     * @return
     */
    @GET
    @Path("getSimpleInfo")
    @Override
    public Result getSimpleInfo(@QueryParam("userIds") String userIds) {
        if (StrUtil.isEmpty(userIds)) {
            return new Result(GlobalApiError.OperationFailed, "缺少必要参数");
        }
        List<Long> userIdList = Arrays.stream(userIds.split(",")).map(Convert::toLong).collect(Collectors.toList());
        MutableResult mResult = new MutableResult();
        mResult.put("map", userSearchService.getSimpleInfo(userIdList));
        return mResult.build();
    }

    /**
     * 获取用户IM信息
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getImInfo")
    @PermitAll
    @Override
    public Result getImInfo(@QueryParam("userId") String userId) {
        Long uid = Convert.toLong(userId, 0L);
        if (uid == 0) {
            return new Result();
        }

        UserSimpleInfo userSimpleInfo = imService.getUserInfo(uid);
        if (userSimpleInfo == null) {
            // 降级处理
            Map<Long, UserSimpleInfo> userSimpleInfoMap = userSearchService.getSimpleInfo(Collections.singletonList(uid));
            if (userSimpleInfoMap != null) {
                userSimpleInfo = userSimpleInfoMap.get(uid);
            }
        }

        if (userSimpleInfo != null) {
            return new Result(userSimpleInfo);
        }

        return new Result(GlobalApiError.OperationFailed, "资料拉取失败");
    }

    private static String hideAlipayAccount(String str) {
        if (str.indexOf('@') > -1) {
            return str.replaceAll("(\\w?)(\\w+)(\\w)(@\\w+\\.[a-z]+(\\.[a-z]+)?)", "$1****$3$4");
        } else {
            return str.replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
        }
    }

    /**
     * 转16进制
     *
     * @param n
     * @return
     */
    public static String intToHex(int n) {
        StringBuffer s = new StringBuffer();
        String a;
        char[] b = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        while (n != 0) {
            s = s.append(b[n % 16]);
            n = n / 16;
        }
        a = s.reverse().toString();
        if ("".equals(a)) {
            a = "00";
        }
        if (a.length() == 1) {
            a = "0" + a;
        }
        return a;
    }

    /**
     * 检查用户名
     *
     * @param userName
     * @return null:符合规则；错误信息
     */
    private String checkUserName(String userName) {
        if (StringUtils.isBlank(userName)) {
            return "必须填写昵称";
        }
        if (StringUtils.containsWhitespace(userName)) {
            return "昵称中不能有空格";
        }
        if (userName.length() > 10) {
            return "最长不超过10字";
        }

        if (wordFilterService.isMatchedPhrases(WordFilterTypeEnum.NAME, userName)) {
            return "昵称中包含敏感词汇，请重新设置";
        }

        // 判断是否包含敏感词汇
        Long userId = this.getToken().getUserId();
        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(userName);
        dataGreenUser.setType(DataGreenTypeEnum.TEXT.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return "保存昵称检测记录失败";
        }

        Long dataId = dataGreenUser.getDataId();
        log.info("用户昵称审核开始 userId={} userName={} dataId={}", userId, userName, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.textAudit(userName, userId.toString(), dataId, GreenBizTypeEnum.USER_NAME);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("用户昵称审核接收数据 userId={} userName={} dataId={} robotAuditStatus={}", userId, userName, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-用户信息 {} {}", update, upPo);

        if (robotAuditStatus != GreenAuditStatusEnum.PASSED_ROBOT) {
            return "昵称包含敏感词";
        }

        if (userService.isNameExist(userName, userId)) {
            return "该昵称已被使用";
        }
        return null;
    }

    /**
     * 检查用户签名
     *
     * @param userSign
     * @return
     */
    private String checkUserSign(String userSign) {
        if (StringUtils.isBlank(userSign)) {
            return null;
        }

        if (wordFilterService.isMatchedPhrases(WordFilterTypeEnum.SIGN, userSign)) {
            return "签名中包含敏感词汇，请重新设置";
        }

        // 判断是否包含敏感词汇
        Long userId = this.getToken().getUserId();
        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(userSign);
        dataGreenUser.setType(DataGreenTypeEnum.TEXT.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return "保存签名检测记录失败";
        }

        Long dataId = dataGreenUser.getDataId();
        log.info("用户签名审核开始 userId={} userSign={} dataId={}", userId, userSign, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.textAudit(userSign, userId.toString(), dataId, GreenBizTypeEnum.USER_SIGN);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("用户签名审核接收数据 userId={} userSign={} dataId={} robotAuditStatus={}", userId, userSign, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-用户信息 {} {}", update, upPo);

        if (robotAuditStatus != GreenAuditStatusEnum.PASSED_ROBOT) {
            return "签名包含敏感词";
        }

        return null;
    }

    /**
     * 头像审核
     *
     * @param userId
     * @param userAvatar
     * @return
     */
    private String auditUserAvatar(Long userId, String userAvatar) {
        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(userAvatar);
        dataGreenUser.setType(DataGreenTypeEnum.IMAGE.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return "保存头像检测信息失败";
        }

        Long dataId = dataGreenUser.getDataId();
        log.info("修改个人资料-头像审核开始 userId={} url={} dataId={}", userId, userAvatar, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.imgAudit(userAvatar, userId.toString(), dataId, GreenBizTypeEnum.USER_AVATAR);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("修改个人资料-头像审核接收数据 userId={} url={} dataId={} robotAuditStatus={}", userId, userAvatar, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-用户信息 {} {}", update, upPo);

        if (robotAuditStatus != GreenAuditStatusEnum.PASSED_ROBOT) {
            return "头像图片违规";
        }
        return null;
    }
}
