package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.v1.facade.VipFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.PayWayEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.VipItemDO;
import com.starbuds.server.common.pojo.finance.VipOrderDTO;
import com.starbuds.server.common.service.finance.VipService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;

/**
 * 会员接口实现
 *
 * @author Lay
 * @date 2020/4/4
 */
@Path("/v1/vip")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class VipFacadeImpl extends BaseFacade implements VipFacade {

    @Reference(version = "1.0.0")
    VipService vipService;

    /**
     * 获取 vip 项目
     *
     * @return
     */
    @GET
    @Path("getVipItems")
    @Override
    public Result getVipItems() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", vipService.getVipItems());
        return mResult.build();
    }

    /**
     * 创建会员订单
     *
     * @param itemId
     * @param payWay
     * @return
     */
    @POST
    @Path("addVipOrder")
    @Override
    public Result addVipOrder(@FormParam("itemId") Long itemId,
                              @FormParam("payWay") Integer payWay) {
        // 判断项目是否存在
        VipItemDO vipItem = vipService.getVipItem(itemId);
        if (vipItem == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的项目");
        }

        // 支付方式判断
        if (payWay == null || PayWayEnum.toEnum(payWay) == null) {
            return new Result(GlobalApiError.OperationFailed, "不支持该支付方式");
        }

        VipOrderDTO vipOrderDTO = VipOrderDTO.builder()
                .itemId(itemId)
                .userId(this.getToken().getUserId())
                .channelId(this.getHeader().getChannelId())
                .payWay(payWay)
                .clientIp(this.getIpAddress())
                .build();

        Long orderId = vipService.addVipOrder(vipOrderDTO);
        if (orderId != null) {
            MutableResult mResult = new MutableResult();
            mResult.put("orderId", orderId);
            return mResult.build();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取最近的
     *
     * @return
     */
    @GET
    @Path("getLatestVip")
    @PermitGuest
    @Override
    public Result getLatestVip() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", vipService.getLatestVip());
        return mResult.build();
    }
}
