package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.util.NumberUtil;
import com.starbuds.server.api.app.config.GameConfig;
import com.starbuds.server.api.app.config.GameDevConfig;
import com.starbuds.server.api.app.constant.GlobalConfig;
import com.starbuds.server.api.app.pojo.BatQueryBalanceItem;
import com.starbuds.server.api.app.pojo.BatQueryBalanceVo;
import com.starbuds.server.api.app.pojo.GameDevBalanceVo;
import com.starbuds.server.api.app.provider.GameDevProvider;
import com.starbuds.server.api.app.util.MaskUtil;
import com.starbuds.server.api.app.v1.facade.WalletFacade;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.ChatTimeWalletType;
import com.starbuds.server.common.constant.finance.GameCoinOrderApiPlatformEnum;
import com.starbuds.server.common.constant.finance.GameCoinOrderOperateEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.DataBankDO;
import com.starbuds.server.common.pojo.db.UserAuthDO;
import com.starbuds.server.common.pojo.db.UserIdCardDO;
import com.starbuds.server.common.pojo.finance.*;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.finance.CloudAccountService;
import com.starbuds.server.common.service.finance.GameService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.ThirdAuthService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.GameUtil;
import com.starbuds.server.common.utils.UserUtil;
import com.starbuds.server.game.sdk.request.BatQueryBalanceRequest;
import com.starbuds.server.game.sdk.request.QueryBalanceRequest;
import com.starbuds.server.game.sdk.request.TransferRequest;
import com.starbuds.server.game.sdk.response.BatQueryBalanceResponse;
import com.starbuds.server.game.sdk.response.QueryBalanceResponse;
import com.starbuds.server.game.sdk.response.TransferResponse;
import com.starbuds.server.game.sdk.response.data.BatQueryBalanceData;
import com.starbuds.server.game.sdkdev.param.GetBalanceParam;
import com.starbuds.server.game.sdkdev.request.GetBalanceRequest;
import com.starbuds.server.game.sdkdev.response.GetBalanceResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ws.rs.*;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 钱包接口实现
 *
 * @author Lay
 * @date 2019/12/23
 */
@Path("/v1/wallet")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class WalletFacadeImpl extends BaseFacade implements WalletFacade {

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    ThirdAuthService thirdAuthService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    CloudAccountService cloudAccountService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Resource
    private GameConfig gameConfig;

    @Reference(version = "1.0.0")
    private GameService gameService;

    @Value("${game.notifyUrl}")
    private String notifyUrl;

    @Resource
    private GameDevConfig gameDevConfig;

    @Resource
    private GameDevProvider gameDevProvider;

    /**
     * 粉钻提现金额选项
     */
    @Value("${xy.wallet.withdraw.pinkDiamond}")
    private String withdrawPinkOptionsStr;

    private List<Long> withdrawPinkOptions;

    @PostConstruct
    private void init() {
        withdrawPinkOptions = new ArrayList<>();
        for (String value : withdrawPinkOptionsStr.split(",")) {
            if (NumberUtil.isLong(value)) {
                withdrawPinkOptions.add(Long.parseLong(value));
            }
        }
        log.info("粉钻提现金额选项 {}", Arrays.toString(withdrawPinkOptions.toArray()));
    }

    /**
     * 查询用户钱包
     *
     * @return
     */
    @GET
    @Path("getUserWallet")
    @Override
    public Result getUserWallet() {
        Long userId = this.getToken().getUserId();
        return new Result(walletService.getUserWallet(userId));
    }

    /**
     * 查询用户虚拟币钱包
     *
     * @return
     */
    @GET
    @Path("getUserVirtualWallet")
    @Override
    public Result getUserVirtualWallet() {
        return new Result(walletService.getUserVirtualWallet(this.getToken().getUserId()));
    }

    /**
     * 获取用户金币明细
     *
     * @param lastId
     * @param yearMonth
     * @param isIncome
     * @param walletType
     * @return
     */
    @GET
    @Path("getUserCoinHisList")
    @Override
    public Result getUserCoinHisList(@QueryParam("lastId") Long lastId,
                                     @QueryParam("yearMonth") Integer yearMonth,
                                     @QueryParam("isIncome") Integer isIncome,
                                     @QueryParam("walletType") Integer walletType) {
        if (yearMonth == null || isIncome == null) {
            return new Result(GlobalApiError.OperationFailed, "缺少参数");
        }

        Long userId = this.getToken().getUserId();
        return new Result(walletService.getUserWalletHisListByTypeAndMonth(lastId, GlobalConfig.PAGE_SIZE, userId, walletType, yearMonth, isIncome));
    }

    public static void main(String[] args) {
        Long[] array = new Long[]{1L, 10000L, 30000L};
        if (!Arrays.stream(array).anyMatch(value -> value.equals(100001L))) {
            System.out.println("00000000");
        }
    }

    /**
     * 获取用户金币统计
     *
     * @param yearMonth
     * @param isIncome
     * @return
     */
    @GET
    @Path("getUserCoinHisSum")
    @Override
    public Result getUserCoinHisSum(@QueryParam("yearMonth") Integer yearMonth,
                                    @QueryParam("isIncome") Integer isIncome,
                                    @QueryParam("walletType") Integer walletType) {
        if (yearMonth == null || isIncome == null) {
            return new Result(GlobalApiError.OperationFailed, "缺少参数");
        }

        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        mResult.put("yearMonth", yearMonth);
        mResult.put("total", walletService.getUserWalletHisSumByTypeAndMonth(userId, walletType, yearMonth, isIncome));
        return mResult.build();
    }

    /**
     * 申请提现
     *
     * @param amount
     * @return
     */
    @POST
    @Path("applyWithdraw")
    @Override
    public Result applyWithdraw(@FormParam("amount") Long amount) {
        Long userId = this.getToken().getUserId();

        // 判断是否主播
        if (!anchorService.isAnchor(userId)) {
            return new Result(GlobalApiError.OperationFailed, "请先认证成为主播");
        }

        if (amount < 20000) {
            return new Result(GlobalApiError.OperationFailed, "每次提现不能少于2万星币哦");
        }

        // 判断余额是否足够
        if (walletService.getBalance(userId, WalletTypeEnum.STAR_COIN) < amount) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        // 判断是否绑定支付宝
        if (!thirdAuthService.isAlipayBind(userId)) {
            return new Result(GlobalApiError.OperationFailed, "请先绑定支付宝");
        }

        // 判断是否完成实名认证
        if (userService.getUserAuth(userId) == null) {
            return new Result(GlobalApiError.OperationFailed, "请先完成实名认证");
        }

        Long newBalance = walletService.addWithdrawApply(userId, WalletTypeEnum.STAR_COIN, amount);
        if (newBalance == null) {
            return new Result(GlobalApiError.OperationFailed, "提现申请失败");
        }

        if (newBalance < 0) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", newBalance);
        return mResult.build();
    }

    /**
     * 申请提现(星尘)
     * 使用 申请提现(粉钻) 代替，只能提取粉钻
     *
     * @param amount
     * @param cardId
     * @return
     */
    @POST
    @Path("applyWithdrawByStarDust")
    @Override
    public Result applyWithdrawByStarDust(@FormParam("amount") Long amount, @FormParam("cardId") String cardId) {
//        Long curUserId = this.getToken().getUserId();
//
//        // 判断余额是否足够
//        if (walletService.getBalance(curUserId, WalletTypeEnum.STAR_DUST) < amount) {
//            return new Result(GlobalApiError.OperationFailed, "余额不足");
//        }
//
//        // 判断是否完成实名认证
//        if (userService.getUserAuth(curUserId) == null) {
//            return new Result(GlobalApiError.OperationFailed, "请先实名认证");
//        }
//
//        // 判断数量
//        if (amount < 10000) {
//            return new Result(GlobalApiError.OperationFailed, "每次提现不能少于1万星尘哦");
//        }
//
//        if (!walletService.isCardIdBind(cardId)){
//            return new Result(GlobalApiError.ServerError,"该卡未绑定");
//        }
//        Long userId = walletService.getUserIdByCardId(cardId);
//        if (!curUserId.equals(userId)) {
//            return new Result(GlobalApiError.ServerError,"您未绑定此卡号");
//        }
//
//        // 提现的银行卡
//        BankcardVO card = walletService.getCard(cardId);
//        if (card == null){
//            return new Result(GlobalApiError.ServerError,"卡号异常");
//        }
//
//        WithdrawApplyDTO applyDTO = walletService.addWithdrawApplyByStarDust(curUserId, WalletTypeEnum.STAR_DUST, amount, card.getCardNo());
//        if (applyDTO == null) {
//            return new Result(GlobalApiError.OperationFailed, "提现申请失败");
//        }
//
//        if (applyDTO.getNewBalance().compareTo(BigDecimal.ZERO) < 0) {
//            return new Result(GlobalApiError.OperationFailed, "余额不足");
//        }
//
//        MutableResult mResult = new MutableResult();
//        mResult.put("balance", applyDTO.getNewBalance());
//        mResult.put("actualAmount", applyDTO.getActualAmount());
//        mResult.put("taxAmount", applyDTO.getTaxAmount());
//        return mResult.build();
        return applyWithdrawByPinkDiamond(amount, cardId);
    }

    /**
     * 申请提现(粉钻)
     *
     * @param amount
     * @param cardId 银行id
     * @return
     */
    @POST
    @Path("applyWithdrawByPink")
    @Override
    public Result applyWithdrawByPinkDiamond(@FormParam("amount") Long amount, @FormParam("cardId") String cardId) {
        if (amount == null) {
            return new Result(GlobalApiError.BadRequest, "请输入正确的提现金额");
        }
        if (org.apache.commons.lang3.StringUtils.isBlank(cardId)) {
            return new Result(GlobalApiError.BadRequest, "请选择银行卡");
        }
        Long curUserId = this.getToken().getUserId();

        // 判断余额是否足够
        if (walletService.getBalance(curUserId, WalletTypeEnum.PINK_DIAMOND) < amount) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        // 判断是否完成实名认证
        if (userService.getUserAuth(curUserId) == null) {
            return new Result(GlobalApiError.OperationFailed, "请先实名认证");
        }

        // 判断数量
        if (!withdrawPinkOptions.stream().anyMatch(value -> value.equals(amount))) {
            return new Result(GlobalApiError.BadRequest, "请输入正确的提现金额");
        }

        if (!walletService.isCardIdBind(cardId)) {
            return new Result(GlobalApiError.ServerError, "该卡未绑定");
        }
        Long userId = walletService.getUserIdByCardId(cardId);
        if (!curUserId.equals(userId)) {
            return new Result(GlobalApiError.ServerError, "您未绑定此卡号");
        }

        // 提现的银行卡
        BankcardVO card = walletService.getCard(cardId);
        if (card == null) {
            return new Result(GlobalApiError.ServerError, "卡号异常");
        }

        WithdrawApplyDTO applyDTO = walletService.addWithdrawApplyByStarDust(curUserId, WalletTypeEnum.PINK_DIAMOND, amount, card.getCardNo());
        if (applyDTO == null) {
            return new Result(GlobalApiError.OperationFailed, "提现申请失败");
        }

        if (applyDTO.getNewBalance().compareTo(BigDecimal.ZERO) < 0) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", applyDTO.getNewBalance());
        mResult.put("actualAmount", applyDTO.getActualAmount());
        mResult.put("taxAmount", applyDTO.getTaxAmount());
        return mResult.build();
    }

    /**
     * 获取提现记录列表
     *
     * @param pageIndex
     * @param yearMonth
     * @param applyStatus
     * @param walletType
     * @return
     */
    @GET
    @Path("getWithdrawHisList")
    @Override
    public Result getWithdrawHisList(@QueryParam("pageIndex") int pageIndex,
                                     @QueryParam("yearMonth") Integer yearMonth,
                                     @QueryParam("applyStatus") Integer applyStatus,
                                     @QueryParam("walletType") Integer walletType) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        // 钱包类型
        WalletTypeEnum typeEnum = null;
        if (walletType == null) {
            typeEnum = WalletTypeEnum.STAR_COIN;
        } else {
            typeEnum = WalletTypeEnum.toEnum(walletType);
        }
        return new Result(walletService.getWithdrawHisList(pageIndex, GlobalConfig.PAGE_SIZE, this.getToken().getUserId(), typeEnum, applyStatus, yearMonth));
    }

    /**
     * 获取时间钱包
     *
     * @return
     */
    @GET
    @Path("getUserChatTimeWallet")
    @Override
    public Result getUserChatTimeWallet() {
        Long userId = this.getToken().getUserId();
        Long curTime = System.currentTimeMillis();
        Integer minutes = walletService.getChatTimeWalletMinutes(userId, ChatTimeWalletType.SINGLE_CHAT, curTime);
        List<ChatTimeWalletItem> items = walletService.getChatTimeWalletItems(userId, ChatTimeWalletType.SINGLE_CHAT, curTime);
        MutableResult mResult = new MutableResult();
        mResult.put("minutes", minutes);
        mResult.put("list", items);
        return mResult.build();
    }

    /**
     * 用户是否实名认证
     *
     * @return
     */
    @GET
    @Path("getAuthInfo")
    @Override
    public Result getAuthInfo() {
        Long userId = this.getToken().getUserId();
        MutableResult mResult = new MutableResult();
        UserAuthDO userAuth = userService.getUserAuth(userId);
        if (userAuth == null) {
            mResult.put("authStatus", false);
            UserIdCardDO idCard = userService.getIdCard(userId);
            if (idCard == null) {
                mResult.put("ocrStatus", null);
            } else {
                mResult.put("ocrStatus", idCard.getStatus());
                mResult.put("remark", StringUtils.isEmpty(idCard.getRemark()) ? "请上传有效的身份证照片" : idCard.getRemark());
            }
            return mResult.build();
        }
        String realName = userAuth.getRealName();
        String idCardNo = userAuth.getIdCardNo();
        mResult.put("authStatus", true);
        mResult.put("realName", MaskUtil.userNameMask(realName));
        mResult.put("idCardNo", MaskUtil.idCardMask(idCardNo));
        mResult.put("remark", "");

        int userAge = UserUtil.getAgeCertID(idCardNo);
        if (userAge >= 50) {
            mResult.put("remark", "账号存在异常情况，请及时联系客服");
        }

        return mResult.build();
    }

    /**
     * 添加身份证
     *
     * @param faceImg
     * @param nationImg
     * @return
     */
    @POST
    @Path("addId")
    @Override
    public Result addId(@FormParam("faceImg") String faceImg,
                        @FormParam("nationImg") String nationImg) {
        Long userId = this.getToken().getUserId();
        if (userService.isAuthExist(userId)) {
            return new Result(GlobalApiError.ServerError, "实名认证已通过");
        }
        if (StringUtils.isEmpty(faceImg)) {
            return new Result(GlobalApiError.ServerError, "请上传身份证正面照");
        }
        if (StringUtils.isEmpty(nationImg)) {
            return new Result(GlobalApiError.ServerError, "请上传身份证反面照");
        }
        UserIdCardDO idCardDO = new UserIdCardDO();
        idCardDO.setUserId(userId);
        idCardDO.setFaceImg(faceImg);
        idCardDO.setNationImg(nationImg);
        idCardDO.setStatus(CommonStatus.Disabled);
        idCardDO.setUpdateTime(System.currentTimeMillis());
        if (userService.addId(idCardDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 银⾏卡信息查询接⼝
     *
     * @param cardNo
     * @param bankName
     * @return
     */
    @GET
    @Path("queryCard")
    @Override
    public Result queryCard(@QueryParam("cardNo") String cardNo,
                            @QueryParam("bankName") String bankName) {
        Long userId = this.getToken().getUserId();
        if (StringUtils.isEmpty(cardNo)) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        if (cardNo.length() < 10) {
            return new Result(GlobalApiError.ServerError, "卡号格式不正确");
        }
        CardInfo cardInfo = cloudAccountService.queryCard(cardNo, bankName);
        if (cardInfo == null) {
            return new Result(GlobalApiError.ServerError, "暂时无法查询");
        }
        if (cardInfo.getIsSupport()) {
            UserAuthDO userAuth = userService.getUserAuth(userId);
            if (userAuth != null) {
                cardInfo.setCardNoMask(cardNo.substring(cardNo.length() - 4));
                cardInfo.setRealNameMask(MaskUtil.userNameMask(userAuth.getRealName()));
                cardInfo.setIdCardMask(MaskUtil.idCardMask(userAuth.getIdCardNo()));
            }
            DataBankDO bank = dataService.getBankByBankCode(cardInfo.getBankCode());
            if (bank != null) {
                cardInfo.setBankLogo(bank.getBankLogo());
                cardInfo.setBackgroundColor(bank.getBackgroundColor());
                cardInfo.setBankUrl(bank.getBankUrl());
            }
            return new Result(cardInfo);
        }
        return new Result(GlobalApiError.ServerError, "暂不支持此卡，请换张银行卡");
    }

    /**
     * 银⾏卡四要素请求鉴权（下发短信验证码）
     *
     * @param cardNo
     * @param mobile
     * @return
     */
    @POST
    @Path("verifyCard")
    @Override
    public Result verifyCard(@FormParam("cardNo") String cardNo,
                             @FormParam("mobile") String mobile) {
        if (StringUtils.isEmpty(cardNo) || StringUtils.isEmpty(mobile)) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long userId = this.getToken().getUserId();
        // 校验是否实名认证
        UserAuthDO userAuth = userService.getUserAuth(userId);
        if (userAuth == null) {
            return new Result(GlobalApiError.ServerError, "请先实名认证");
        }
        // 判断卡片是否绑定
        if (walletService.isCardNoBind(userId, cardNo)) {
            return new Result(GlobalApiError.ServerError, "该卡片已绑定，请勿重复绑定");
        }
        String idCard = userAuth.getIdCardNo();
        String realName = userAuth.getRealName();

        VerifyRequestDTO requestDTO = cloudAccountService.verifyCard(cardNo, idCard, realName, mobile);
        if (requestDTO.isSuccess()) {
            MutableResult mResult = new MutableResult();
            mResult.put("ref", requestDTO.getRef());
            return mResult.build();
        }
        return new Result(GlobalApiError.ServerError, requestDTO.getMessage());
    }

    /**
     * 银⾏卡四要素确认鉴权（上传短信验证码）
     *
     * @param cardNo
     * @param mobile
     * @param captcha
     * @param ref
     * @return
     */
    @POST
    @Path("verifyConfirm")
    @Override
    public Result verifyConfirm(@FormParam("cardNo") String cardNo,
                                @FormParam("mobile") String mobile,
                                @FormParam("captcha") String captcha,
                                @FormParam("ref") String ref) {

        if (StringUtils.isEmpty(cardNo) || StringUtils.isEmpty(mobile) || StringUtils.isEmpty(captcha) || StringUtils.isEmpty(ref)) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long userId = this.getToken().getUserId();
        // 校验是否实名认证
        UserAuthDO userAuth = userService.getUserAuth(userId);
        if (userAuth == null) {
            return new Result(GlobalApiError.ServerError, "请先实名认证");
        }
        String idCard = userAuth.getIdCardNo();
        String realName = userAuth.getRealName();
        VerifyConfirmRequestDTO confirmRequestDTO = cloudAccountService.verifyConfirm(userId, cardNo, idCard, realName, mobile, captcha, ref);
        if (confirmRequestDTO.isSuccess()) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError, confirmRequestDTO.getMessage());
    }

    /**
     * 获取银行卡列表
     *
     * @return
     */
    @GET
    @Path("getBankcardList")
    @Override
    public Result getBankcardList() {
        Long userId = this.getToken().getUserId();
        UserAuthDO userAuth = userService.getUserAuth(userId);
        if (userAuth == null) {
            log.info("未实名认证");
            return new Result();
        }
        List<BankcardVO> list = walletService.getBankcardList(userId);
        if (!CollectionUtils.isEmpty(list)) {
            Set<String> bankCodes = list.stream().map(e -> e.getBankCode()).collect(Collectors.toSet());
            Map<String, DataBankDO> bankCodeToBankMap = dataService.getBanksByBankCodes(bankCodes);
            for (BankcardVO vo : list) {
                vo.setCardNoMask(vo.getCardNo().substring(vo.getCardNo().length() - 4));
                vo.setMobileMask(MaskUtil.phoneMask(vo.getMobile()));
                vo.setCardNo(null);
                vo.setMobile(null);

                String bankCode = vo.getBankCode();

                DataBankDO bank = bankCodeToBankMap.get(bankCode);
                if (bank != null) {
                    vo.setBankLogo(bank.getBankLogo());
                    vo.setBackgroundColor(bank.getBackgroundColor());
                    vo.setBankUrl(bank.getBankUrl());
                }
            }
        }
        MutableResult mResult = new MutableResult();
        mResult.put("list", list);
        mResult.put("realName", null);
        mResult.put("realNameMask", MaskUtil.userNameMask(userAuth.getRealName()));
        mResult.put("idCard", null);
        mResult.put("idCardMask", MaskUtil.idCardMask(userAuth.getIdCardNo()));
        return mResult.build();
    }

    /**
     * 解绑卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("unBindCard")
    @Override
    public Result unBindCard(@FormParam("cardId") String cardId) {
        if (StringUtils.isEmpty(cardId)) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long userId = walletService.getUserIdByCardId(cardId);
        Long curUserId = this.getToken().getUserId();
        if (!curUserId.equals(userId)) {
            return new Result(GlobalApiError.ServerError, "您不能解绑此卡号");
        }
        if (!walletService.isCardIdBind(cardId)) {
            return new Result(GlobalApiError.ServerError, "您未绑定此卡号");
        }
        if (walletService.unBindCard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 转星钻
     *
     * @param walletType
     * @param amount
     * @return
     */
    @POST
    @Path("transDiamond")
    @Override
    public Result transDiamond(@FormParam("walletType") int walletType,
                               @FormParam("amount") Long amount) {
        WalletTypeEnum walletTypeEnum = WalletTypeEnum.toEnum(walletType);
        if (walletTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "暂不支持该类型");
        }
        if (amount == null || amount < 1) {
            return new Result(GlobalApiError.BadRequest, "金额必须大于0");
        }

        // 实名认证
        if (!userService.isAuthExist(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "请先完成实名认证");
        }

        Long balance;
        switch (walletTypeEnum) {
            case STAR_DUST: {
                // 星尘转星钻 粉钻-不可提现
                if (amount == null || amount % 100 > 0) {
                    return new Result(GlobalApiError.OperationFailed, "转出数量必须是100的整数倍");
                }
                balance = walletService.transDustToDiamond(this.getToken().getUserId(), amount, this.getHeader().getChannelId());
                break;
            }
            case PINK_DIAMOND: {
                // 粉钻-可提现
                if (amount == null || amount % 100 > 0) {
                    return new Result(GlobalApiError.OperationFailed, "转出数量必须是100的整数倍");
                }
                balance = walletService.transPinkDiamondToDiamond(this.getToken().getUserId(), amount, this.getHeader().getChannelId());
                break;
            }
            case GAME_COIN: {
                // 游戏币转星钻
                balance = walletService.transGameCoinToDiamond(this.getToken().getUserId(), amount, this.getHeader().getChannelId());
                break;
            }
            default:
                return new Result(GlobalApiError.OperationFailed, "暂不支持该类型");
        }

        if (balance == null) {
            return new Result(GlobalApiError.OperationFailed, "兑换失败");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", balance);
        return mResult.build();
    }

    /**
     * 兑换钻石（粉钻-不可提现、粉钻-可提现 兑换 星钻）
     *
     * @param total 兑换总额
     * @return
     */
    @POST
    @Path("transDustPinkToDiamond")
    @Override
    public Result transDustPinkToDiamond(@FormParam("total") Long total) {
        // 实名认证
        if (!userService.isAuthExist(this.getToken().getUserId())) {
            return new Result(GlobalApiError.OperationFailed, "请先完成实名认证");
        }

        if (total == null || total < 1) {
            return new Result(GlobalApiError.BadRequest, "金额必须大于0");
        }
        if (total == null || total % 100 > 0) {
            return new Result(GlobalApiError.OperationFailed, "转出数量必须是100的整数倍");
        }
        TransDustPinkToDiamondVO vo = walletService.transDustPinkToDiamond(getToken().getUserId(), total, getHeader().getChannelId());
        if (vo.getErrMsg() != null) {
            return new Result(GlobalApiError.OperationFailed, vo.getErrMsg());
        }

        return new Result(vo);
    }

    /**
     * 转游戏币
     *
     * @param walletType
     * @param amount
     * @return
     */
    @POST
    @Path("transGameCoin")
    @Override
    public Result transGameCoin(@FormParam("walletType") int walletType,
                                @FormParam("amount") Long amount) {
        if (amount == null || amount < 1) {
            return new Result(GlobalApiError.BadRequest, "金额必须大于0");
        }
        WalletTypeEnum walletTypeEnum = WalletTypeEnum.toEnum(walletType);
        if (walletTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "暂不支持该类型");
        }

        Long balance;
        switch (walletTypeEnum) {
            case STAR_DIAMOND: {
                if (!GameUtil.verifyDiamond(amount)) {
                    return new Result(GlobalApiError.BadRequest, "金额必须是100的整数倍");
                }
                // 星钻 转 游戏币
                balance = walletService.transDiamondToGameCoin(this.getToken().getUserId(), amount, this.getHeader().getChannelId());
                break;
            }
            default:
                return new Result(GlobalApiError.OperationFailed, "暂不支持该类型");
        }

        if (balance == null) {
            return new Result(GlobalApiError.OperationFailed, "兑换失败");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", balance);
        return mResult.build();
    }

    /**
     * 查询转换记录
     *
     * @param lastId
     * @param fromWalletType
     * @param destWalletType
     * @param yearMonth
     * @return
     */
    @GET
    @Path("getTransferHisList")
    @Override
    public Result getTransferHisList(@QueryParam("lastId") Long lastId,
                                     @QueryParam("fromWalletType") int fromWalletType,
                                     @QueryParam("destWalletType") int destWalletType,
                                     @QueryParam("yearMonth") Integer yearMonth) {
        Long userId = this.getToken().getUserId();
        return new Result(walletService.getWalletTransferHisList(lastId, GlobalConfig.PAGE_SIZE, userId, fromWalletType, destWalletType, yearMonth));
    }

    /**
     * 获取游戏余额
     *
     * @param platform 平台代码
     * @return
     */
    @GET
    @Path("getGameBalance")
    @Override
    public Result getGameBalance(@QueryParam("platform") String platform) {
        if (org.apache.commons.lang3.StringUtils.isBlank(platform)) {
            return new Result(GlobalApiError.BadRequest, "平台代码不能为空");
        }
        TokenBO token = getToken();
        String userId = token.getUserId() + "";
        QueryBalanceRequest request = new QueryBalanceRequest(platform, userId, userId);
        QueryBalanceResponse res = gameConfig.client().execute(request);
        if (res.getCode() == 0) {
            Result result = new Result(res.getData());
            result.setMsg(res.getMsg());
            return result;
        }
        return new Result(GlobalApiError.OperationFailed, res.getMsg());
    }

    /**
     * 获取游戏余额-所有平台
     *
     * @return
     */
    @GET
    @Path("getGameBalanceAll")
    @Override
    public Result getGameBalanceAll() {
        TokenBO token = getToken();
        String userId = token.getUserId() + "";
        Long total = walletService.getBalance(token.getUserId(), WalletTypeEnum.GAME_COIN);
        Long recycleAble = 0L;
        List<BatQueryBalanceItem> balanceList = getBalanceListAll(userId);
        for (BatQueryBalanceItem item : balanceList) {
            if (NumberUtil.isNumber(item.getBalance())) {
                recycleAble += new BigDecimal(item.getBalance()).longValue();
            }
        }

        BatQueryBalanceVo vo = new BatQueryBalanceVo();
        vo.setTotal(total);
        vo.setRecycleAble(recycleAble);
        vo.setList(balanceList);

        return new Result(vo);
    }

    /**
     * 一键回收所有游戏币
     *
     * @return
     */
    @POST
    @Path("recyclingAllGameCoin")
    @Override
    public Result recyclingAllGameCoin() {
        TokenBO token = getToken();
        String userId = token.getUserId() + "";

        // game 平台所有游戏币
        Long gameCoinTotal = 0L;
        BatQueryBalanceRequest request = new BatQueryBalanceRequest(userId, userId);
        BatQueryBalanceResponse res;
        try {
            res = gameConfig.client().execute(request);
        } catch (cn.hutool.http.HttpException e) {
            log.warn("获取会员余额-所有厅请求超时");
            return new Result(GlobalApiError.OperationFailed, "网络繁忙请重试");
        }
        if (res.getCode() != 0) {
            return new Result(GlobalApiError.OperationFailed, res.getMsg());
        }

        List<BatQueryBalanceData> balanceList = res.getData() == null ? new ArrayList<>() : res.getData();
        for (BatQueryBalanceData data : balanceList) {
            gameCoinTotal += new BigDecimal(data.getBalance()).longValue();
        }

        // gamedev 平台所有游戏币
        Long gameCoinTotalDev = 0L;
        GetBalanceParam param = GetBalanceParam.builder()
                .userid(userId)
                .build();
        GetBalanceRequest requestDev = new GetBalanceRequest();
        requestDev.setParam(param);
        GetBalanceResponse resDev = gameDevConfig.client().execute(requestDev);
        if (isSuccessDev(resDev.getCode())) {
            String data = resDev.getData();
            if (NumberUtil.isLong(data)) {
                gameCoinTotalDev = Long.parseLong(data);
            }
        }

        if (gameCoinTotal < 1 && gameCoinTotalDev < 1) {
            return new Result(GlobalApiError.BadRequest, "没有可回收的余额");
        }

        // game 平台全部转出
        for (BatQueryBalanceData data : balanceList) {
            try {
                String platform = data.getPlatform();
                Long amount = new BigDecimal(data.getBalance()).longValue();
                // 转出时存储
                Long orderId = gameService.addGameCoinOrder(getToken().getUserId(), platform,
                        GameCoinOrderOperateEnum.OUT, GameCoinOrderApiPlatformEnum.API_SAAS, amount);

                TransferRequest transferRequest = new TransferRequest(platform, userId, userId,
                        GameCoinOrderOperateEnum.OUT.getTranType(), amount, orderId + "", notifyUrl);
                TransferResponse transferResponse = gameConfig.client().execute(transferRequest);
                if (transferResponse.getCode() != 0) {
                    log.warn("取款请求失败 orderId={}", orderId);
                }
            } catch (NumberFormatException e) {
                log.warn("余额格式错误 balance={} platform={} userId={}", data.getBalance(),
                        data.getPlatform(), userId);
            }
        }

        // gamedev 平台全部转出
        gameDevProvider.addGameCoinOrder(GameCoinOrderOperateEnum.OUT, gameCoinTotalDev, token.getUserId());

        return new Result("请求提交成功");
    }

    /**
     * 游戏币兑换星钻
     *
     * @param coin
     * @return
     */
    @POST
    @Path("transGameCoinToDiamond")
    @Override
    public Result transGameCoinToDiamond(@FormParam("coin") Long coin) {
        if (coin == null) {
            return new Result(GlobalApiError.BadRequest, "请输入兑换金额");
        }
        if (coin < 1) {
            return new Result(GlobalApiError.BadRequest, "游戏币必须大于0");
        }
        TokenBO token = getToken();
        Long balance = walletService.transGameCoinToDiamond(token.getUserId(), coin, getHeader().getChannelId());
        if (balance == null || balance < 1) {
            return new Result(GlobalApiError.OperationFailed, "兑换失败");
        }

        return new Result(balance);
    }

    /**
     * 获取游戏余额-Dev接口平台
     *
     * @return
     */
    @GET
    @Path("getGameDevBalance")
    @Override
    public Result getGameDevBalance() {
        TokenBO token = getToken();
        String userId = token.getUserId() + "";

        GameDevBalanceVo vo = new GameDevBalanceVo();
        Long total = walletService.getBalance(token.getUserId(), WalletTypeEnum.GAME_COIN);
        vo.setTotal(total);

        GetBalanceParam param = GetBalanceParam.builder()
                .userid(userId)
                .build();
        GetBalanceRequest request = new GetBalanceRequest();
        request.setParam(param);
        GetBalanceResponse res = gameDevConfig.client().execute(request);
        if ("0000".equals(res.getCode())) {
            vo.setBalance(NumberUtil.parseLong(res.getData()));
            Result result = new Result(vo);
            return result;
        }
        return new Result(GlobalApiError.OperationFailed, res.getMessage());
    }

    /**
     * 获粉钻提现金额选项
     *
     * @return
     */
    @GET
    @Path("getWithdrawPinkList")
    @Override
    public Result getWithdrawPinkList() {
        return new Result(withdrawPinkOptions);
    }

    /**
     * 获得Game所有平台余额列表
     *
     * @param userId
     * @return
     */
    private List<BatQueryBalanceItem> getBalanceListByGame(String userId) {
        BatQueryBalanceRequest request = new BatQueryBalanceRequest(userId, userId);
        BatQueryBalanceResponse res = gameConfig.client().execute(request);
        List<BatQueryBalanceItem> itemList = new ArrayList<>();
        if (res.getCode() == 0) {
            List<BatQueryBalanceData> dataList = res.getData();

            for (BatQueryBalanceData data : dataList) {
                BatQueryBalanceItem item = new BatQueryBalanceItem();
                BeanUtils.copyProperties(data, item);
                item.setApiType(10);
                itemList.add(item);
            }
        }
        return itemList;
    }

    /**
     * 获得（Gamedev）自主游戏的余额
     *
     * @param userId
     * @return
     */
    private BatQueryBalanceItem getBalanceByGameDev(String userId) {
        GetBalanceParam param = GetBalanceParam.builder()
                .userid(userId)
                .build();
        GetBalanceRequest request = new GetBalanceRequest();
        request.setParam(param);
        GetBalanceResponse res = gameDevConfig.client().execute(request);
        String balance = "0";
        if ("0000".equals(res.getCode())) {
            balance = res.getData();
        }

        BatQueryBalanceItem item = new BatQueryBalanceItem();
        item.setPlatform("自主游戏");
        item.setApiType(20);
        item.setBalance(balance);
        return item;
    }

    /**
     * 获得所有平台的钱包余额
     *
     * @param userId
     * @return
     */
    private List<BatQueryBalanceItem> getBalanceListAll(String userId) {
        List<BatQueryBalanceItem> list = new ArrayList<>();
        list.add(getBalanceByGameDev(userId));
        list.addAll(getBalanceListByGame(userId));
        return list;
    }

    /**
     * 请求是否成功
     *
     * @param code
     * @return true 成功
     */
    private boolean isSuccessDev(String code) {
        return "0000".equals(code);
    }
}