package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.google.common.math.LongMath;
import com.starbuds.server.api.app.inf.PermitGuest;
import com.starbuds.server.api.app.pojo.*;
import com.starbuds.server.api.app.v1.facade.WatchFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.BroadcastTypeEnum;
import com.starbuds.server.common.constant.data.ConfigBusinessDictEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.user.SexEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.media.LiveRankItem;
import com.starbuds.server.common.pojo.rtc.RtcRankService;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.data.BroadcastHisService;
import com.starbuds.server.common.service.data.ConfigBusinessDictService;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.rtc.RtcRoomAuctionService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.service.user.UserWatchRelationService;
import com.starbuds.server.common.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.junit.platform.commons.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 守护接口实现
 *
 * @author lujiao
 * @date 2021/7/23 14:58
 */
@Path("/v1/watch")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class WatchFacadeImpl extends BaseFacade implements WatchFacade {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    UserWatchRelationService userWatchRelationService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcRoomAuctionService rtcRoomAuctionService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    protected RtcRankService rtcRankService;

    @Reference(version = "1.0.0")
    BroadcastHisService broadcastHisService;

    @Reference(version = "1.0.0")
    ConfigBusinessDictService configBusinessDictService;

    private static final long WEEK_OF_MILLION = 7 * 24 * 60 * 60 * 1000;

    /**
     * 获取配置列表
     *
     * @return
     */
    @POST
    @Path("getWatchConfigList")
    @Override
    public Result getWatchConfigList() {
        List<WatchConfigDO> watchConfigDOList = userWatchRelationService.getWatchConfigList(BoolIntVal.False);
        List<WatchConfigVo> res = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(watchConfigDOList)) {
            watchConfigDOList.forEach(i -> {
                WatchConfigVo tmp = new WatchConfigVo();
                BeanUtils.copyProperties(i, tmp);
                res.add(tmp);
            });
        }
        return new Result(res);
    }

    /**
     * 获取守护列表 顺便返回之前是否开通
     *
     * @param targetId
     * @return
     */
    @POST
    @Path("getWatchConfigWithRelation")
    @Override
    public Result getWatchConfigWithRelation(@FormParam("targetId") Long targetId) {
        List<WatchConfigDO> watchConfigDOList = userWatchRelationService.getWatchConfigList(BoolIntVal.False);
        Long userId = this.getToken().getUserId();
        Set<Long> existConfigSet = userWatchRelationService.getExistConfig(userId, targetId);
        List<WatchConfigVo> res = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(watchConfigDOList)) {
            watchConfigDOList.forEach(i -> {
                WatchConfigVo tmp = new WatchConfigVo();
                BeanUtils.copyProperties(i, tmp);
                tmp.setOpened(existConfigSet.contains(i.getWatchConfigId()) ? 1 : 0);
                res.add(tmp);
            });
        }
        return new Result(res);
    }

    /**
     * 开通守护
     *
     * @return
     */
    @POST
    @Path("newWatchRelationByDiamond")
    @Override
    public Result newWatchRelationByDiamond(@FormParam("targetId") Long targetId, @FormParam("watchConfigId") Long watchConfigId) {
        Long userId = this.getToken().getUserId();
        if (targetId.equals(userId)) {
            return new Result(GlobalApiError.OperationFailed, "无法给自己开通守护");
        }
        UserBasicInfo userBasicInfo = userService.getBasicInfo(targetId);
        if (userBasicInfo == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该用户");
        }
        WatchConfigDO watchConfigDO = userWatchRelationService.getWatchConfig(watchConfigId);
        if (watchConfigDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该类型守护");
        }
        Long balance = walletService.getBalance(userId, WalletTypeEnum.STAR_DIAMOND);
        if (watchConfigDO.getPrice() > balance) {
            return new Result(GlobalApiError.OperationFailed, "账户余额不足");
        }
        return userWatchRelationService.newWatchRelationByDiamond(watchConfigId, userId, targetId);
    }

    /**
     * 获取即将过期的用户
     *
     * @return
     */
    @POST
    @Path("getLimitWatch")
    @Override
    public Result getLimitWatch() {
        Long userId = this.getToken().getUserId();
        List<WatchNotifyHisDO> expireUserList = userWatchRelationService.getLimitWatch(userId);
        if (CollectionUtil.isNotEmpty(expireUserList)) {
            Set<Long> userIds = new HashSet<>(expireUserList.size());
            expireUserList.forEach(i -> userIds.add(i.getTargetId()));
            Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
            List<UserWatchExpireVo> res = new ArrayList<>(userIds.size());
            expireUserList.forEach(i -> {
                UserWatchExpireVo tmp = new UserWatchExpireVo();
                UserBasicInfo userBasicInfo = userBasicInfoMap.get(i.getTargetId());
                if (userBasicInfo != null) {
                    tmp.setUserId(userBasicInfo.getUserId());
                    tmp.setWatchConfigId(i.getWatchId());
                    tmp.setName(i.getWatchName());
                    tmp.setUserName(userBasicInfo.getUserName());
                    tmp.setUserAvatar(userBasicInfo.getUserAvatar());
                    tmp.setUserSex(Optional.ofNullable(userBasicInfo.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
                    tmp.setUserAge(Optional.ofNullable(userBasicInfo.getUserAge()).orElse(18));
                    tmp.setExpireTime(i.getExpireTime());
                    res.add(tmp);
                }
            });
            userWatchRelationService.setWatchNotifyRead(userId);
            return new Result(res);
        } else {
            return new Result(new ArrayList<>());
        }
    }

    /**
     * 获取守护列表
     *
     * @return
     */
    @POST
    @Path("getUserWatchRelation")
    @Override
    public Result getUserWatchRelation(@FormParam("targetUserId") Long targetUserId, @FormParam("pageIndex") Integer pageIndex, @FormParam("pageSize") Integer pageSize) {
        PageList<UserWatchRelationDO> userWatchRelationDOList = userWatchRelationService.getUserWatchRelation(targetUserId, pageIndex, pageSize);
        List<UserWatchRelationDO> watchRelationDOList = userWatchRelationDOList.getList();
        if (CollectionUtil.isNotEmpty(watchRelationDOList)) {
            List<UserWatchExpireVo> tmpList = new ArrayList<>(watchRelationDOList.size());
            Set<Long> userIds = new HashSet<>(watchRelationDOList.size());
            Set<Long> watchConfigIds = new HashSet<>();
            watchRelationDOList.forEach(i -> {
                userIds.add(i.getUserId());
                watchConfigIds.add(i.getWatchId());
            });
            Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
            Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(userIds));
            Map<Long, WatchConfigDO> watchConfigMap = userWatchRelationService.getWatchConfigMap(watchConfigIds);
            watchRelationDOList.forEach(i -> {
                UserWatchExpireVo tmp = new UserWatchExpireVo();
                UserBasicInfo userBasicInfo = userBasicInfoMap.get(i.getUserId());
                if (userBasicInfo != null) {
                    tmp.setUserId(userBasicInfo.getUserId());
                    tmp.setWatchConfigId(i.getWatchId());
                    tmp.setName(i.getWatchName());
                    tmp.setUserName(userBasicInfo.getUserName());
                    tmp.setWatchValue(i.getWatchValue());
                    tmp.setUserAvatar(userBasicInfo.getUserAvatar());
                    tmp.setUserSex(Optional.ofNullable(userBasicInfo.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
                    tmp.setUserAge(Optional.ofNullable(userBasicInfo.getUserAge()).orElse(18));
                    tmp.setExpireTime(i.getExpireTime());
                    HeadwearVO headwearVO = headwearMap.get(i.getUserId());
                    tmp.setHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
                    WatchConfigDO watchConfigDO = watchConfigMap.get(i.getWatchId());
                    tmp.setBgImage(watchConfigDO == null ? "" : watchConfigDO.getBgImage());
                    tmp.setTagImage(watchConfigDO == null ? "" : watchConfigDO.getTagImage());
                    tmpList.add(tmp);
                }
            });
            PageList<UserWatchExpireVo> res = new PageList<>(pageIndex, pageSize);
            res.setTotal(userWatchRelationDOList.getTotal());
            res.setList(tmpList);
            return new Result(res);
        } else {
            return new Result(userWatchRelationDOList);
        }
    }


    /**
     * 获取陪伴榜单
     *
     * @return
     */
    @POST
    @Path("getIntimacyRank")
    @Override
    public Result getIntimacyRank(@FormParam("targetUserId") Long targetUserId) {
        Long userId = this.getToken().getUserId();
        PageList<UserWatchRelationDO> userWatchRelationDOList = userWatchRelationService.getUserWatchRelation(targetUserId, 1, 5);
        Set<Long> userIds = new HashSet<>();
        Set<Long> watchConfigIds = new HashSet<>();
        if (CollectionUtil.isNotEmpty(userWatchRelationDOList.getList())) {
            userWatchRelationDOList.getList().forEach(i -> {
                userIds.add(i.getUserId());
                watchConfigIds.add(i.getWatchId());
            });
        }
        List<UserIntimacyRelationDO> userIntimacyRelationDOList = userWatchRelationService.getUserIntimacyRelation(targetUserId);
        if (CollectionUtil.isNotEmpty(userIntimacyRelationDOList)) {
            userIntimacyRelationDOList.forEach(i -> userIds.add(i.getUserId()));
        }
        if (!userIds.contains(userId)) {
            userIds.add(userId);
        }
        Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
        Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(userIds));
        Map<Long, WatchConfigDO> watchConfigMap = userWatchRelationService.getWatchConfigMap(watchConfigIds);
        Map<Long, Long> userRtcRoomMap = rtcUserService.getUserRoomIdMap(userIds);
        UserIntimacyRankVo res = new UserIntimacyRankVo();
        if (userId.equals(targetUserId)) {
            // 如果用户ID等于需要查找的ID, 返回拍卖房间
            res.setRoomId(rtcRoomAuctionService.getHotAuctionRoomId());
        }
        res.setUserWatchCount(userWatchRelationDOList.getTotal());
        List<UserWatchExpireVo> watchExpireVoList = new ArrayList<>(userWatchRelationDOList.getList().size());
        // 构建守护关系
        if (CollectionUtil.isNotEmpty(userWatchRelationDOList.getList())) {
            userWatchRelationDOList.getList().forEach(i -> {
                UserWatchExpireVo tmp = new UserWatchExpireVo();
                UserBasicInfo userBasicInfo = userBasicInfoMap.get(i.getUserId());
                if (userBasicInfo != null) {
                    tmp.setUserId(userBasicInfo.getUserId());
                    tmp.setName(i.getWatchName());
                    tmp.setUserName(userBasicInfo.getUserName());
                    tmp.setUserAvatar(userBasicInfo.getUserAvatar());
                    tmp.setUserSex(Optional.ofNullable(userBasicInfo.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
                    tmp.setUserAge(Optional.ofNullable(userBasicInfo.getUserAge()).orElse(18));
                    WatchConfigDO watchConfigDO = watchConfigMap.get(i.getWatchId());
                    tmp.setBgImage(watchConfigDO == null ? "" : watchConfigDO.getBgImage());
                    tmp.setTagImage(watchConfigDO == null ? "" : watchConfigDO.getTagImage());
                    watchExpireVoList.add(tmp);
                }
            });
        }
        res.setUserWatchList(watchExpireVoList);
        List<UserIntimacyVo> userIntimacyVoList = new ArrayList<>();
        Boolean hasUserSelf = false;
        if (CollectionUtil.isNotEmpty(userIntimacyRelationDOList)) {
            Integer rank = 1;
            for (UserIntimacyRelationDO item : userIntimacyRelationDOList) {
                UserIntimacyVo tmp = new UserIntimacyVo();
                UserBasicInfo userBasicInfo = userBasicInfoMap.get(item.getUserId());
                if (userBasicInfo != null) {
                    if (userBasicInfo.getUserId().equals(userId)) {
                        hasUserSelf = true;
                    }
                    tmp.setUserId(userBasicInfo.getUserId());
                    int tmpIntimacy = item.getIntimacyValue().intValue();
                    if (tmpIntimacy == 0) {
                        tmpIntimacy = 1;
                    }
                    tmp.setIntimacyValue(tmpIntimacy);
                    tmp.setUserName(userBasicInfo.getUserName());
                    tmp.setUserAvatar(userBasicInfo.getUserAvatar());
                    tmp.setUserSex(Optional.ofNullable(userBasicInfo.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
                    tmp.setUserAge(Optional.ofNullable(userBasicInfo.getUserAge()).orElse(18));
                    tmp.setAccompanyMinutes(0);
                    tmp.setRank(rank);
                    HeadwearVO headwearVO = headwearMap.get(item.getUserId());
                    tmp.setHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
                    userIntimacyVoList.add(tmp);
                    tmp.setRoomId(userRtcRoomMap.get(item.getUserId()));
                    rank++;
                }
            }
        }
        if (!hasUserSelf) {
            UserIntimacyRelationDO userIntimacyRelationDO = userWatchRelationService.getUserIntimacyRelationByUser(userId, targetUserId);
            UserIntimacyVo tmp = new UserIntimacyVo();
            UserBasicInfo userBasicInfo = userBasicInfoMap.get(userId);
            tmp.setUserId(userBasicInfo.getUserId());
            if (userIntimacyRelationDO == null) {
                tmp.setIntimacyValue(0);
            } else {
                tmp.setIntimacyValue(userIntimacyRelationDO.getIntimacyValue().intValue());
            }
            tmp.setUserName(userBasicInfo.getUserName());
            tmp.setUserAvatar(userBasicInfo.getUserAvatar());
            tmp.setUserSex(Optional.ofNullable(userBasicInfo.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
            tmp.setUserAge(Optional.ofNullable(userBasicInfo.getUserAge()).orElse(18));
            tmp.setAccompanyMinutes(0);
            tmp.setRank(-1);
            HeadwearVO headwearVO = headwearMap.get(userId);
            tmp.setHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
            userIntimacyVoList.add(tmp);
        }
        res.setUserIntimacyList(userIntimacyVoList);
        return new Result(res);
    }

    /**
     * 获取历史胜利者
     *
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @POST
    @Path("getHistoryWinner")
    @Override
    public Result getHistoryWinner(@FormParam("pageIndex") int pageIndex, @FormParam("pageSize") int pageSize) {
        PageList<WatchScoreRankHisDO> rankHisList = userWatchRelationService.getHistoryWinner(pageIndex, pageSize);
        PageList<WatchScoreRankHisVo> res = new PageList<>(pageIndex, pageSize);
        res.setTotal(rankHisList.getTotal());
        List<WatchScoreRankHisVo> tmpList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(rankHisList.getList())) {
            Set<Long> userIds = new HashSet<>();
            Set<Long> watchConfigIds = new HashSet<>();
            rankHisList.getList().forEach(i -> {
                userIds.add(i.getUserId());
                userIds.add(i.getTargetUserId());
                watchConfigIds.add(i.getWatchId());
            });
            Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
            Map<Long, WatchConfigDO> watchConfigDOMap = userWatchRelationService.getWatchConfigMap(watchConfigIds);
            rankHisList.getList().forEach(i -> {
                WatchScoreRankHisVo tmp = new WatchScoreRankHisVo();
                tmp.setWatchId(i.getWatchId());
                tmp.setWatchName(i.getWatchName());
                UserBasicInfo self = userBasicInfoMap.get(i.getUserId());
                UserBasicInfo target = userBasicInfoMap.get(i.getTargetUserId());
                tmp.setUserAvatar(self == null ? "" : self.getUserAvatar());
                tmp.setTargetUserAvatar(target == null ? "" : target.getUserAvatar());
                tmp.setWatchId(i.getWatchId());
                tmp.setUserId(i.getUserId());
                tmp.setTargetUserId(i.getTargetUserId());
                tmp.setWatchName(i.getWatchName());
                tmp.setIntimacyValue(i.getIntimacyValue());
                tmp.setSortStartTime(getSortDate(i.getSortStartTime()));
                tmp.setSortEndTime(getSortDate(i.getSortEntTime()));
                WatchConfigDO watchConfigDO = watchConfigDOMap.get(i.getWatchId());
                tmp.setTagImage(watchConfigDO == null ? "" : watchConfigDO.getTagImage());
                tmp.setBgImage(watchConfigDO == null ? "" : watchConfigDO.getBgImage());
                tmpList.add(tmp);
            });
        }
        res.setList(tmpList);
        return new Result(res);
    }

    private static String getSortDate(Long input) {
        SimpleDateFormat sdf = new SimpleDateFormat(DateUtil.SIMPLE_FORMAT);
        return sdf.format(new Date(input));
    }

    /**
     * 获取上周排行
     *
     * @return
     */
    @POST
    @Path("getLastWeekTopWatchIntimacy")
    @Override
    public Result getLastWeekTopWatchIntimacy() {
        List<WatchScoreRankHisDO> watchScoreRankHisDOList = userWatchRelationService.getLastWeekTopWatchIntimacy();
        List<WatchScoreRankHisVo> res = translateWatchScoreRank(watchScoreRankHisDOList);
        MutableResult mResult = new MutableResult();
        mResult.put("list", res);
        Long weekStart = DateUtil.getWeekStart() - WEEK_OF_MILLION;
        Long weekEnd = DateUtil.getWeekEnd() - WEEK_OF_MILLION;
        mResult.put("weekStart", getSortDate(weekStart));
        mResult.put("weekEnd", getSortDate(weekEnd));
        return mResult.build();
    }

    /**
     * 获取上周获胜者
     *
     * @return
     */
    @POST
    @Path("getLastWeekWinner")
    @Override
    public Result getLastWeekWinner() {
        List<WatchScoreRankHisDO> watchScoreRankHisDOList = userWatchRelationService.getLastWeekTopWatchIntimacy();
        List<WatchScoreRankHisVo> res = translateWatchScoreRank(watchScoreRankHisDOList);
        if (CollectionUtil.isNotEmpty(res)) {
            return new Result(res.get(0));
        }
        return new Result();
    }

    private List<WatchScoreRankHisVo> translateWatchScoreRank(List<WatchScoreRankHisDO> watchScoreRankHisDOList) {
        List<WatchScoreRankHisVo> res = new ArrayList<>(20);
        if (CollectionUtil.isNotEmpty(watchScoreRankHisDOList)) {
            Set<Long> userIds = new HashSet<>();
            Set<Long> watchConfigIds = new HashSet<>();
            watchScoreRankHisDOList.forEach(i -> {
                userIds.add(i.getUserId());
                userIds.add(i.getTargetUserId());
                watchConfigIds.add(i.getWatchId());
            });
            Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
            Map<Long, WatchConfigDO> watchConfigDOMap = userWatchRelationService.getWatchConfigMap(watchConfigIds);
            watchScoreRankHisDOList.forEach(i -> {
                WatchScoreRankHisVo tmp = new WatchScoreRankHisVo();
                tmp.setWatchId(i.getWatchId());
                tmp.setWatchName(i.getWatchName());
                UserBasicInfo self = userBasicInfoMap.get(i.getUserId());
                UserBasicInfo target = userBasicInfoMap.get(i.getTargetUserId());
                tmp.setUserAvatar(self == null ? "" : self.getUserAvatar());
                tmp.setTargetUserAvatar(target == null ? "" : target.getUserAvatar());
                tmp.setWatchId(i.getWatchId());
                tmp.setUserId(i.getUserId());
                tmp.setTargetUserId(i.getTargetUserId());
                tmp.setWatchName(i.getWatchName());
                tmp.setIntimacyValue(i.getIntimacyValue());
                tmp.setSort(i.getSort());
                tmp.setUserName(self.getUserName());
                tmp.setTargetUserName(target.getUserName());
                WatchConfigDO watchConfigDO = watchConfigDOMap.get(i.getWatchId());
                tmp.setTagImage(watchConfigDO == null ? "" : watchConfigDO.getTagImage());
                tmp.setBgImage(watchConfigDO == null ? "" : watchConfigDO.getBgImage());
                res.add(tmp);
            });
        }
        return res;
    }

    /**
     * 获取本周排行
     *
     * @return
     */
    @POST
    @Path("getWeekTopWatchIntimacy")
    @Override
    public Result getWeekTopWatchIntimacy() {
        List<WatchScoreRankHisDO> watchScoreRankHisDOList = userWatchRelationService.getWeekTopWatchIntimacy();
        List<WatchScoreRankHisVo> res = translateWatchScoreRank(watchScoreRankHisDOList);
        return new Result(res);
    }

    /**
     * 关系墙记录
     *
     * @return
     */
    @Override
    @GET
    @Path("getWatchWallHis")
    @PermitGuest
    public Result getWatchWallHis(@QueryParam("pageIndex") Integer pageIndex, @QueryParam("pageSize") Integer pageSize) {
        // 获取上周排行
        if (pageIndex == null) {
            pageIndex = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        WatchScoreRankHisDO lastWinner = null;

        // 获取配置
        String config = configBusinessDictService.getConfigDictValue(ConfigBusinessDictEnum.RELATION_WALL_THRESHOLD);
        Long limit = 0L;
        if (StringUtils.isNotBlank(config)) {
            limit = Long.valueOf(config);
        }
        List<UserWatchRelationDO> hisDOList = userWatchRelationService.getWatchWallHis(limit, pageIndex, pageSize);
        if (pageIndex == 1) {
            lastWinner = userWatchRelationService.getLastWeekWinner();
            UserWatchRelationDO head = new UserWatchRelationDO();
            if (lastWinner != null) {
                BeanUtils.copyProperties(lastWinner, head);
                head.setWatchValue(lastWinner.getIntimacyValue());
            } else {
                head.setWatchValue(0);
                head.setUserId(0L);
                head.setTargetUserId(0L);
            }
            hisDOList.add(0, head);
        }

        List<WatchRelationWallHisVO> res = translateRelationHis(hisDOList);
        for (int i = 0; i < res.size(); i++) {
            WatchRelationWallHisVO item = res.get(i);
            if (i == 0 && pageIndex == 1) {
                item.setIsRankWinner(BoolIntVal.True);
            } else {
                if (item.getCostValue() != null && item.getCostValue() > 0) {
                    item.setCostValue((long) Math.ceil(item.getCostValue() * 1.0 / 100));
                }
                res.get(i).setIsRankWinner(BoolIntVal.False);
            }
        }
        return Result.ofList(res);
    }

    private List<WatchRelationWallHisVO> translateRelationHis(List<UserWatchRelationDO> hisDOList) {
        if (CollectionUtil.isEmpty(hisDOList)) {
            return new ArrayList<>();
        }
        Set<Long> userIds = new HashSet(hisDOList.size() * 2);

        Set<Long> auctionIds = new HashSet<>(hisDOList.size());
        hisDOList.forEach(i -> {
            userIds.add(i.getUserId());
            userIds.add(i.getTargetUserId());
            if (i.getCreateBy() != null && i.getCreateBy() == 1 && i.getOrderId() != null) {
                auctionIds.add(Long.valueOf(i.getOrderId()));
            }
        });
        Map<Long, Long> auctionIdRoomIds = userWatchRelationService.getRoomIdsByAuctionId(auctionIds);
        Set<Long> roomIds = new HashSet<>();
        for (Long value : auctionIdRoomIds.values()) {
            roomIds.add(value);
        }
        Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
        Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(userIds));
        Map<Long, RtcRoom> roomMap = rtcRoomService.getRoomMap(roomIds);
        List<WatchRelationWallHisVO> res = new ArrayList<>(hisDOList.size());
        hisDOList.forEach(i -> {
            WatchRelationWallHisVO tmp = new WatchRelationWallHisVO();
            BeanUtils.copyProperties(i, tmp);
            UserBasicInfo userInfo = userBasicInfoMap.get(i.getUserId());
            if (userInfo != null) {
                tmp.setUserAvatar(userInfo.getUserAvatar());
                tmp.setUserName(userInfo.getUserName());
                HeadwearVO headwearVO = headwearMap.get(i.getUserId());
                tmp.setUserHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
            }
            UserBasicInfo targetUserInfo = userBasicInfoMap.get(i.getTargetUserId());
            if (targetUserInfo != null) {
                tmp.setTargetUserAvatar(targetUserInfo.getUserAvatar());
                tmp.setTargetUserName(targetUserInfo.getUserName());
                HeadwearVO headwearVO = headwearMap.get(i.getTargetUserId());
                tmp.setTargetUserHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
            }

            tmp.setCostValue(Long.valueOf(i.getWatchValue()));
            if (i.getCreateBy() != null) {
                if (i.getCreateBy() == 1) {
                    tmp.setCreateBy(20000);
                } else {
                    tmp.setCreateBy(10000);
                }
                if (i.getCreateBy() == 1) {
                    Long auction = Long.valueOf(i.getOrderId());
                    Long roomId = auctionIdRoomIds.get(auction);
                    if (roomId != null) {
                        RtcRoom rtcRoom = roomMap.get(roomId);
                        if (rtcRoom != null) {
                            tmp.setRoomId(roomId);
                            tmp.setRoomType(rtcRoom.getRoomType());
                        }
                    }
                }
            }
            res.add(tmp);
        });
        return res;
    }

    /**
     * 获取最新一条守护记录
     *
     * @return
     */
    @Override
    @GET
    @Path("getLastWatchWallRecord")
    @PermitGuest
    public Result getLastWatchWallRecord() {
        String config = configBusinessDictService.getConfigDictValue(ConfigBusinessDictEnum.RELATION_WALL_THRESHOLD);
        Long limit = 0L;
        if (StringUtils.isNotBlank(config)) {
            limit = Long.valueOf(config);
        }
        List<UserWatchRelationDO> hisDOList = userWatchRelationService.getWatchWallHis(limit, 1, 1);
        if (CollectionUtil.isNotEmpty(hisDOList)) {
            List<WatchRelationWallHisVO> res = translateRelationHis(hisDOList);
            return new Result(res.get(0));
        }
        return new Result();
    }

    /**
     * 广播墙
     *
     * @return
     */
    @Override
    @GET
    @Path("getBroadWallHis")
    @PermitGuest
    public Result getBroadWallHis(@QueryParam("pageIndex") Integer pageIndex, @QueryParam("pageSize") Integer pageSize) {
        // 获取富豪榜
        // 获取魅力榜
        if (pageIndex == null) {
            pageIndex = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        LiveRankItem wealth = null;
        LiveRankItem charm = null;
        if (pageIndex == 1) {
            List<LiveRankItem> wealthList = rtcRankService.getDiamondRankDay(0L, 0, 1, true);
            if (CollectionUtil.isNotEmpty(wealthList)) {
                wealth = wealthList.get(0);
            }
            List<LiveRankItem> charmList = rtcRankService.getDustRankDay(0L, 0, 1, true);
            if (CollectionUtil.isNotEmpty(charmList)) {
                charm = charmList.get(0);
            }
        }
        List<BroadcastHisDO> hisDOList = broadcastHisService.getBroadcastHisList(pageIndex, pageSize);
        Set<Long> userIds = new HashSet<>();
        if (charm != null) {
            userIds.add(charm.getUserId());
        }
        if (wealth != null) {
            userIds.add(wealth.getUserId());
        }
        List<BroadcastItemVO> res = new ArrayList<>();
        hisDOList.forEach(i -> {
            userIds.add(i.getUserId());
            if (i.getTargetId() != null) {
                userIds.add(i.getTargetId());
            }
        });
        Map<Long, UserBasicInfo> userBasicInfoMap = userService.getBasicInfoMap(userIds);
        Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(userIds));
        hisDOList.forEach(i -> {
            BroadcastItemVO tmp = new BroadcastItemVO();
            BeanUtils.copyProperties(i, tmp);
            UserBasicInfo userBasicInfo = userBasicInfoMap.get(i.getUserId());
            if (userBasicInfo != null) {
                tmp.setUserAvatar(userBasicInfo.getUserAvatar());
                tmp.setUserName(userBasicInfo.getUserName());
                HeadwearVO headwearVO = headwearMap.get(i.getUserId());
                tmp.setUserHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
            }
            if (i.getTargetId() != null) {
                UserBasicInfo targetUser = userBasicInfoMap.get(i.getTargetId());
                if (targetUser != null) {
                    tmp.setTargetUserName(targetUser.getUserName());
                }
            }
            res.add(tmp);
        });
        if (pageIndex == 1) {
            // 前排填充
            BroadcastItemVO wealthVo = new BroadcastItemVO();
            wealthVo.setShowType(BroadcastTypeEnum.WEALTH.getCode());
            if (wealth != null) {
                UserBasicInfo userInfo = userBasicInfoMap.get(wealth.getUserId());
                if (userInfo != null) {
                    wealthVo.setUserId(userInfo.getUserId());
                    wealthVo.setUserName(userInfo.getUserName());
                    wealthVo.setUserAvatar(userInfo.getUserAvatar());
                    HeadwearVO headwearVO = headwearMap.get(userInfo.getUserId());
                    wealthVo.setUserHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
                }
            }
            res.add(0, wealthVo);

            BroadcastItemVO charmVo = new BroadcastItemVO();
            charmVo.setShowType(BroadcastTypeEnum.CHARM.getCode());
            if (charm != null) {
                UserBasicInfo userInfo = userBasicInfoMap.get(charm.getUserId());
                if (userInfo != null) {
                    charmVo.setUserId(userInfo.getUserId());
                    charmVo.setUserName(userInfo.getUserName());
                    charmVo.setUserAvatar(userInfo.getUserAvatar());
                    HeadwearVO headwearVO = headwearMap.get(userInfo.getUserId());
                    charmVo.setUserHeadWare(headwearVO == null ? "" : headwearVO.getAnimationFile());
                }
            }
            res.add(1, charmVo);
        }
        return Result.ofList(res);
    }
}
