package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.v1.facade.WishFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.rtc.WishStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.util.PageQO;
import com.starbuds.server.common.service.rtc.WishService;
import com.starbuds.server.common.service.rtc.WishUserTaskService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import java.util.*;

/**
 * 许愿接口实现
 *
 * @author
 * @date
 */
@Path("/v1/rtc/wish")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class WishFacadeImpl extends BaseFacade implements WishFacade {

    @Reference(version = "1.0.0")
    WishService wishService;

    @Reference(version = "1.0.0")
    WishUserTaskService wishUserTaskService;

    /**
     * 获取个人许愿列表
     *
     * @return
     */
    @GET
    @Path("getWishList")
    @Override
    public Result getWishList() {
        Long userId = this.getToken().getUserId();

        // 添加当天不存在的任务，并返回任务列表
        List<WishUserTaskDO> list = wishUserTaskService.addCurrentDayTaskIfNotExist(userId);

        return Result.ofList(list);
    }

    /**
     * 获取个人开启的许愿任务
     *
     * @return
     */
    @GET
    @Path("getWishOpenedDetail")
    @Override
    public Result getWishOpenedDetail(@QueryParam("userId") Long userId) {
        userId = userId == null ? this.getToken().getUserId() : userId;
        WishUserTaskDO task = wishUserTaskService.getOpenedWishTask(userId);

        return new Result(task);
    }

    /**
     * 设置心愿任务开关
     *
     * @return
     */
    @POST
    @Path("setWishTaskSwitch")
    @Override
    public Result setWishTaskSwitch(@FormParam("wishTaskId") Long wishTaskId,
                                    @FormParam("status") Integer status) {

        Long userId = this.getToken().getUserId();

        if (wishTaskId == null) {
            return new Result(GlobalApiError.OperationFailed, "参数缺失:wishTaskId");
        }

        if (status == null) {
            return new Result(GlobalApiError.OperationFailed, "参数缺失:status");
        }

        // 开启时，默认将之前的关闭
        if (status.equals(BoolIntVal.True)) {
            WishUserTaskDO openedWishTask = wishUserTaskService.getOpenedWishTask(userId);
            if (openedWishTask != null) {
                if (!wishUserTaskService.updateWishTaskStatus(openedWishTask.getWishTaskId(), userId, WishStatusEnum.CLOSE.getValue())) {
                    return new Result(GlobalApiError.OperationFailed, "关闭已开启的任务时出现错误");
                }
            }
        }

        wishUserTaskService.updateWishTaskStatus(wishTaskId, userId, status);
        return new Result();
    }

    /**
     * 获取个人许愿完成的列表
     *
     * @return
     */
    @GET
    @Path("getWishCompleteList")
    @Override
    public Result getWishCompleteList(@BeanParam PageQO qo) {
        Long userId = this.getToken().getUserId();

        PageList<WishUserTaskDO> list = wishUserTaskService.getWishCompleteList(qo, userId);

        return new Result(list);
    }

}
