package com.findbug;

import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.test.utils.FileTools;
import lombok.extern.slf4j.Slf4j;
import org.testng.annotations.Test;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 查找在线人数的bug
 *
 * @author aoe
 * @date 2021/9/30
 */
@Slf4j
public class FindOnlineUserCount {

    private final String TAG = "rong-userJoin-";

    @Test
    public void findOnlineUserCountBug() {
        String serviceMessageLog = "/Users/aoe/Downloads/service-message-console.log";
//        List<String> messageSendMsgLines = messageSendMsgLines(serviceMessageLog);
        List<String> appLines = appLines("/Users/aoe/Downloads/api-app-console.log");
        List<String> rtcLines = rtcLines("/Users/aoe/Downloads/service-rtc-console.log");
        List<String> messageReceiveLines = messageReceiveLines(serviceMessageLog);
//        List<String> notifyLines = notifyLines("/Users/aoe/Downloads/api-notify-console.log");

        log.info(LogUtil.devLog("app={} rtc={} messageReceiveLines={} notifyLines={}"),
                appLines.size(), rtcLines.size(), messageReceiveLines.size());
    }

    private List<String> rtcLines(String filePath) {
        String contains = TAG;
        return containsLines(filePath, contains);
    }

    private List<String> messageReceiveLines(String filePath) {
        String contains = TAG;
        return containsLines(filePath, contains);
    }

    private List<String> messageSendMsgLines(String filePath) {
        String contains = TAG;
        return containsLines(filePath, contains, "sendMsg");
    }

    private List<String> appLines(String filePath) {
        String contains = TAG;
        return containsLines(filePath, contains, "sendMsg");
    }

    private List<String> notifyLines(String filePath) {
        String contains = "com.starbuds.server.api.notify.v1.impl.ImFacadeImpl 91 roomStatus";
        return containsLines(filePath, contains, "\"type\":1", "432272486432775");
    }

    private List<String> containsLines(String filePath, String... contains) {
        return FileTools.readList(filePath).stream()
                .filter(line -> {
                    int matchCount = 0;
                    for (int i = 0; i < contains.length; i++) {
                        String contain = contains[i];
                        if (line.contains(contain)) {
                            matchCount++;
                        }
                    }
                    return matchCount - contains.length == 0;
                })
                .collect(Collectors.toList());
    }
}
