package com.starbuds.apichannel;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.starbuds.login.ChannelLoginTest;
import com.starbuds.login.ManageLoginTest;
import com.starbuds.server.common.constant.feed.FeedTypeEnum;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static com.starbuds.test.utils.XyHttpTools.*;
import static com.starbuds.test.utils.XyResultTools.checkOperationFailed;
import static com.starbuds.test.utils.XyResultTools.checkSuccess;

/**
 * @author aoe
 * @date 2020/6/4
 */
public class ChannelFacadeTest {

    private static String HOST = HOST_CHANNEL_DEV;

    static {
        HOST = HOST_CHANNEL_TEST_ALY;

        XyHttpTools.pkgType = PKG_TYPE_ADMIN;
        try {
            ChannelLoginTest.saveLoginToken(HOST);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // 编辑渠道 App
    @Test
    public void editChannelAppTest() throws IOException {
        String action = "api-channel/v1/channel/editChannelApp";

        JSONArray iconArr = new JSONArray();
        for (int i = 1; i < 3; i++) {
            int size = i * 100;
            JSONObject icon = new JSONObject();
            icon.put("x" + size + "_" + size, "http:xx.xx/" + size + ".png");
            iconArr.add(icon);
        }

        Map<String, String> params = new HashMap<>();
        params.put("appName", DateUtil.now() + "应用名称 测试");
        params.put("appIcon", iconArr.toJSONString());
        params.put("apiAuthByPhone", "apiAuthByPhone");
        params.put("apiNoticeUserRegistered", "apiNoticeUserRegistered");
        params.put("apiNoticeUserUpdate", "apiNoticeUserUpdate");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 获得渠道App信息
    @Test
    public void getChannelAppTest() throws IOException {
        String action = "api-channel/v1/channel/getChannelApp";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 生成渠道APP秘钥
    @Test
    public void createSecretKeyTest() throws IOException {
        String action = "api-channel/v1/channel/createSecretKey";
        Map<String, String> params = new HashMap<>();
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 变更渠道APP秘钥
    @Test
    public void changeSecretKeyTest() throws IOException {
        String action = "api-channel/v1/channel/changeSecretKey";
        Map<String, String> params = new HashMap<>();
        params.put("secretId", "1");
        String res = XyHttpTools.post(HOST, action, params);
        checkOperationFailed(res);

        params.put("secretId", "135844916625408");
        res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 删除渠道APP秘钥
    @Test
    public void deleteSecretKeyTest() throws IOException {
        String action = "api-channel/v1/channel/deleteSecretKey";
        Map<String, String> params = new HashMap<>();
        params.put("secretId", "1");
        String res = XyHttpTools.post(HOST, action, params);
        checkOperationFailed(res);

        params.put("secretId", "135844916625408");
        res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 启用/禁用渠道APP秘钥
    @Test
    public void activateSecretKeyTest() throws IOException {
        String action = "api-channel/v1/channel/activateSecretKey";
        Map<String, String> params = new HashMap<>();
        params.put("secretId", "1");
        params.put("isActivate", "1"); // 1:启用; 0:禁用
        String res = XyHttpTools.post(HOST, action, params);
        checkOperationFailed(res);

        params.put("secretId", "135846602735616"); // 135844916625408 135846602735616
        res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 查询（自己渠道下）所有未删除的秘钥
    @Test
    public void getSecretKeyListTest() throws IOException {
        String action = "api-channel/v1/channel/getSecretKeyList";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }
}
