package com.starbuds.apichannel.util;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.util.RsaPair;
import com.starbuds.server.common.utils.RsaUtil;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * 渠道开放平台验签工具
 *
 * @author aoe
 * @date 2020/8/17
 */
public class ChannelOpenApiSingUtil {

    public static final String SIGN = "sign";

    /**
     * RSA签名
     *
     * @param jsonObject
     * @param privStr
     * @return
     * @throws InvalidKeySpecException
     * @throws NoSuchAlgorithmException
     * @throws UnsupportedEncodingException
     * @throws InvalidKeyException
     * @throws SignatureException
     */
    public static String rsaSign(JSONObject jsonObject, String privStr) throws InvalidKeySpecException,
            NoSuchAlgorithmException, UnsupportedEncodingException, InvalidKeyException, SignatureException {
        return rsaSign(toSign(jsonObject), privStr);
    }

    public static String toSign(JSONObject jsonObject) {
        Map<String, Object> params = new HashMap<>();
        for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
            params.put(entry.getKey(), entry.getValue());
        }
        return getToSign(params);
    }

    private static String rsaSign(String plaintext, String privStr) throws NoSuchAlgorithmException,
            SignatureException, InvalidKeySpecException, InvalidKeyException, UnsupportedEncodingException {
        return RsaUtil.sign(privStr, plaintext);
    }

    /**
     * 构建签名原⽂ 按字⺟顺序
     *
     * @param params 参数与值
     * @return
     */
    private static String getToSign(Map<String, Object> params) {
        if (params == null || params.size() == 0) {
            return "";
        }

        StringBuilder sbToSign = new StringBuilder();
        Map<String, Object> toSign = new TreeMap<>(params);
        for (Map.Entry<String, Object> entry : toSign.entrySet()) {
            // 忽略签名字段
            if (SIGN.equals(entry.getKey())) {
                continue;
            }
            sbToSign.append("&");
            sbToSign.append(entry.getKey());
            sbToSign.append("=");
            sbToSign.append(Convert.toStr(entry.getValue(), ""));
        }

        return sbToSign.substring(1);
    }
}
