package com.starbuds.login;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.test.utils.FileTools;
import com.starbuds.test.utils.XyHttpTools;
import lombok.extern.slf4j.Slf4j;
import org.junit.platform.commons.util.StringUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.LongStream;

import static com.google.common.base.Preconditions.checkNotNull;

/**
 * 登录测试
 *
 * @author aoe
 * @date 2020/5/15
 */
@Slf4j
public class AppLoginTest {

    private static String HOST = XyHttpTools.HOST_TEST_ALY_APP;

    @Test
    public void loginTest() throws IOException {
        // 渠道设置 headerPackage.setChannelId(135120971366400L);
        // 15757876459 18972267513 18158183582
        // sw 13958095497
        // zs 17516053225
        // 测试电话号码 18158183600 - 18158183656
        String phone = "13685756236";
//        phone = "18158183606";
        String token = login(HOST, phone);

        log.info("TOKEN = " + token);
        FileTools.writeToFile(FileTools.getTestResourcesFilePath("Login-TOKEN.txt"), Arrays.asList(token));
    }

    // 第三方用户登陆测试
    @Test
    public void loginThirdPartyUserTest() throws IOException {
        // 15757876459 18972267513
        String phone = "19000000000";
        String token = login(HOST, phone);

        log.info("TOKEN = " + token);
    }

    // 批量登录
    @Test
    public void batchLoginTest() throws IOException {
        List<String> tokenList = new ArrayList<>();
        List<String> pkgList = new ArrayList<>();
        for (long i = 19100002000L; i <= 19100004000L; i++) {
            String token = login(HOST, "" + i);
            tokenList.add(StrUtil.format("testUserList.add(new TestUser(\"{}\", \"{}\"));", i, token));
            pkgList.add(XyHttpTools.getAppPkg(token));
        }

        System.out.println(LogUtil.obj2JsonObj(pkgList));
    }

    @Test
    public void batchGetTokens() {
        // hp 18158185000L 18158185100L
        // hp 很多 18158186000L 18158188000L
        // zc 19000000000L 19000000100L
        // zc 很多 19100000000L 19100002000L
        // zc1 很多 19100002000L 19100004000L
        List<String> pkgList = LongStream.range(19100003500L, 19100004000L)
                .parallel()
                .mapToObj(phone -> {
                    try {
                        String token = login(HOST, "" + phone);
                        return token;
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    return null;
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        String filePath = FileTools.getTestResourcesFilePath("many-tokens.txt");
        FileTools.writeToFile(filePath, pkgList);
        log.info("共 {}", pkgList.size());
    }

    @Test
    public void batchPkgListFormTokens() {
        String suffix = "zc1";
        String tokenFilePath = FileTools.getTestResourcesFilePath(String.format("many-tokens-%s.txt", suffix));
        List<String> tokens = FileTools.readList(tokenFilePath);
        List<String> pkgList = tokens.stream()
                .map(XyHttpTools::getAppPkg)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        String filePath = FileTools.getTestResourcesFilePath(String.format("many-pkg-list-%s.txt", suffix));
        FileTools.writeToFile(filePath, pkgList);
    }

    // 批量登录
    @Test
    public void swBatchLoginTest() throws IOException {
        // hp 很多 18158186000L 18158188000L
        List<String> tokenList = new ArrayList<>();
        for (long i = 18158186000L; i < 18158188000L; i++) {
            String token = login(HOST, "" + i);
            tokenList.add(StrUtil.format("swTestUserList.add(new TestUser(\"{}\", \"{}\"));", i, token));
        }

        for (String s : tokenList) {
            System.out.println(s);
        }
    }

    // 生产环境批量登录
    @Test
    public void prodBatchLoginTest() throws IOException {
        String[] phoneArr = new String[]{
                "18158183582",
                "17516053225",
                "18677896136",
                "13958095497",
                "13486047805",
                "18867727198",
                "15502453302",
                "13612849419",
                "18688676996",
                "15618561127",
                "15867389137",
                "19000000000",
        };

        List<String> tokenList = new ArrayList<>();
        for (String phone : phoneArr) {
            if (StringUtils.isBlank(phone)) {
                continue;
            }
            String token = login(HOST, phone);
            tokenList.add(StrUtil.format("swTestUserList.add(new TestUser(\"{}\", \"{}\"));", phone, token));
        }

        for (String s : tokenList) {
            System.out.println(s);
        }
    }

    @Test
    public void loginLoopTest() {
        int count = 10000, threads = 5;
        AtomicInteger phone = new AtomicInteger(count);
        ExecutorService fixedThreadPool = Executors.newFixedThreadPool(threads);
        for (int i = 0; i < count; i++) {
            fixedThreadPool.execute(() -> {
                String code = null;
                try {
                    code = sendSmsCode(HOST, "" + phone.addAndGet(-1));
                } catch (IOException e) {
                    e.printStackTrace();
                }
                Assert.assertEquals(isNumber(code), true);
            });
        }
    }

    public static void saveLoginToken(String host, String phone) throws IOException {
        String token = login(host, phone);

        log.info("TOKEN = " + token);
        FileTools.writeToFile(FileTools.getTestResourcesFilePath("Login-TOKEN.txt"), Arrays.asList(token));
    }

    public static String getAppLoginToken() {
        List<String> tokenList = FileTools.readList(FileTools.getTestResourcesFilePath("Login-TOKEN.txt"));
        return tokenList.get(0);
    }

    public static String sendSmsCode(String host, String phone) throws IOException {
        Map<String, String> params = new HashMap<>();
        params.put("phone", phone);
        params.put("codeType", "101");
        params.put("countryId", "86");
//        String res = XyHttpTools.post(getApiCommonHost(host), "api-common/v1/login/sendSmsCode", params);
        String res = XyHttpTools.post(getApiCommonHost(host), "api-common/v1/login/sendSmsCodeOnlyTest", params);

        String errMsg = String.format("%s 获取 Code 失败 res=%s", phone, res);
        checkNotNull(res, errMsg);
        checkNotNull(JSONObject.parseObject(res).getJSONObject("data"), errMsg);
        checkNotNull(JSONObject.parseObject(res).getJSONObject("data").getString("code"), errMsg);

        return JSONObject.parseObject(res).getJSONObject("data").getString("code");
    }

    public static String loginBySmsCode(String host, String phone, String smsCode) throws IOException {
        Map<String, String> params = new HashMap<>();
        params.put("phone", phone);
        params.put("code", smsCode);
        params.put("countryId", "86");
        String res = XyHttpTools.post(getApiCommonHost(host), "api-common/v1/login/bySmsCode", params);

        String errMsg = String.format("%s 获取 Token 失败 res=%s", phone, res);
        checkNotNull(res, errMsg);
        checkNotNull(JSONObject.parseObject(res).getJSONObject("data"), errMsg);
        checkNotNull(JSONObject.parseObject(res).getJSONObject("data").getString("token"), errMsg);

        return JSONObject.parseObject(res).getJSONObject("data").getString("token");
    }

    public static String login(String host, String phone) throws IOException {
        getInitConfig();
        String smsCode = sendSmsCode(host, phone);
        String token = loginBySmsCode(host, phone, smsCode);
        return token;
    }

    private static String getApiCommonHost(String host) {
        return host.replace("api-app", "");
    }

    /**
     * 判断是否纯数字
     *
     * @param str
     * @return
     */
    public static boolean isNumber(String str) {
        Pattern pattern = Pattern.compile("[0-9]*");
        return pattern.matcher(str).matches();
    }

    /**
     * 获取初始化配置
     *
     * @throws IOException
     */
    private static void getInitConfig() throws IOException {
        String res = XyHttpTools.get(HOST, "api-app/v1/config/getInitConfig");
        log.info("getInitConfig结果\n{}", res);
    }
}
