package com.starbuds.login;

import com.starbuds.test.utils.FileTools;
import org.apache.commons.lang3.StringUtils;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 找出文件中的所有ip
 *
 * @author aoe
 * @date 2021/9/18
 */
public class FindIpByRequestTest {

    public static void main(String[] args) {
        List<String> lines = lines();
        lines.stream()
                .map(line ->
                        Arrays.stream(StringUtils.split(line, "&"))
                                .collect(Collectors.toList())
                )
                .map(list -> {
                            Map<String, String> request = new HashMap<>();
                            list.stream()
                                    .forEach(kv -> {
                                        String[] split = StringUtils.split(kv, "=");
                                        if (split.length == 2) {
                                            request.put(split[0], split[1]);
                                        }
                                    });
                            return request;
                        }
                )
                .map(request -> getIpAddress(request))
                .forEach(ip -> {
                    if (ip.length() == 9) {
                        System.out.println("IP " + ip);
                    }
                });

        System.out.println("总数量 " + lines.size());
    }

    /**
     * 所有请求头开始的行
     *
     * @return
     */
    private static List<String> lines() {
        String filePath = "/Users/aoe/Downloads/api-app-log/console.log";
        return FileTools.readList(filePath).stream()
                .filter(line -> line.startsWith("请求头"))
                .collect(Collectors.toList());
    }

    private static String getIpAddress(Map<String, String> request) {
        String ipAddress = request.get("x-forwarded-for");
        // K8S 中使用 x-original-forwarded-for
        if (StringUtils.isBlank(ipAddress)) {
            ipAddress = request.get("x-original-forwarded-for");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.get("Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.get("WL-Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            // request.getRemoteAddr()
            ipAddress = request.get("remoteip");
            if (ipAddress.equals("127.0.0.1") || ipAddress.equals("0:0:0:0:0:0:0:1")) {
                // 根据网卡取本机配置的IP
                InetAddress address = null;
                try {
                    address = InetAddress.getLocalHost();
                } catch (UnknownHostException e) {
                    e.printStackTrace();
                }
                ipAddress = address.getHostAddress();
            }
        }
        // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        if (ipAddress != null && ipAddress.length() > 15) {
            // "***.***.***.***".length() = 15
            if (ipAddress.indexOf(",") > 0) {
                ipAddress = ipAddress.substring(0, ipAddress.indexOf(","));
            }
        }
        return ipAddress;
    }
}
