package com.starbuds.server.api.app.util;

import com.starbuds.server.api.app.v1.i18n.I18nInitPropertiesTest;
import com.starbuds.test.utils.FileTools;
import com.starbuds.test.BaseTest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.List;

@Slf4j
public class I18nUtilTest extends BaseTest {

    @Autowired
    private I18nUtil i18nUtil;

    @Test
    public void testGetMsg() {
        String messagesEnPath = FileTools.getTestResourcesFilePath(I18nInitPropertiesTest.I18N_EN_PROPERTIES);
        List<String> msgList = FileTools.readList(messagesEnPath);
        Assert.assertEquals(msgList.isEmpty(), false);

        int i18nCount = 0;
        for (String msg : msgList) {
            int index = msg.indexOf("=");
            String key = msg.substring(0, index);
            String enMsg = msg.substring(index + 1);
            Assert.assertEquals(StringUtils.isBlank(key), false);
            checkMsg(key, enMsg);
            log.info("{}={}", key, enMsg);
            i18nCount++;
        }

        log.info("国际化共 {} 条", msgList.size());
        Assert.assertEquals(i18nCount, msgList.size());
    }

    @Test
    public void getKeyTest() {
        Assert.assertEquals(i18nUtil.getKey(""), "KeyIsBlank");
        Assert.assertEquals(i18nUtil.getKey(null), "KeyIsBlank");
        Assert.assertEquals(i18nUtil.getKey(" "), "KeyIsBlank");
        Assert.assertEquals(i18nUtil.getKey("调皮了~ 主播不能被关入小黑屋哦"), "调皮了~主播不能被关入小黑屋哦");
        Assert.assertEquals(i18nUtil.getKey("调皮了~ 主播不能 被关入小黑      屋哦"), "调皮了~主播不能被关入小黑屋哦");
        Assert.assertEquals(i18nUtil.getKey("只能向主播发起PK邀请哦"), "只能向主播发起PK邀请哦");
        Assert.assertEquals(i18nUtil.getKey("%s(ID:%d)正在直播%s，快来围观！"), "%s(ID1XEscapeX1%d)正在直播%s，快来围观！");
    }

    private void checkMsg(String key, String enMsg) {
        String[] langArr = new String[]{null, "null", "zh_CN", "ja", "cn", "en"};
        for (String lang : langArr) {
            String expectedMsg = "en".equals(lang) ? enMsg : key;
            Assert.assertEquals(i18nUtil.getMsg(key, lang), expectedMsg);
        }
    }
}