package com.starbuds.server.api.app.util;

import com.alibaba.fastjson.JSONObject;
import com.starbuds.login.AppLoginTest;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.finance.UserWalletVO;
import com.starbuds.server.common.pojo.user.UserProfile;
import com.starbuds.test.utils.XyHttpTools;
import org.junit.Test;

import java.io.IOException;

import static com.starbuds.test.utils.XyResultTools.*;

/**
 * 用户服务测试类
 *
 * @author aoe
 * @date 2020/11/20
 */
public class UserServiceTest {

    private final String HOST;

    public UserServiceTest(String HOST) {
        this.HOST = HOST;
    }

    /**
     * 用户登录
     *
     * @param phone
     */
    public void login(String phone) {
        try {
            AppLoginTest.saveLoginToken(HOST, phone);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public Long getUserId() throws IOException {
        AppLoginTest.getAppLoginToken();
        return getProfile().getUserId();
    }

    /**
     * 查询用户钱包
     *
     * @return
     * @throws IOException
     */
    public UserWalletVO getUserWallet() throws IOException {
        String action = "api-app/v1/wallet/getUserWallet";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
        UserWalletVO vo = JSONObject.parseObject(getDataStr(res), UserWalletVO.class);
        return vo;
    }

    /**
     * 获得钱包余额
     *
     * @param walletVO
     * @param typeEnum
     * @return
     */
    public Long getBalance(UserWalletVO walletVO, WalletTypeEnum typeEnum) {
        for (UserWalletVO.Item item : walletVO.getWalletItems()) {
            if (typeEnum.getValue() - item.getWalletType() == 0) {
                return item.getBalance();
            }
        }
        return null;
    }

    /**
     * 获得钱包余额
     *
     * @param typeEnum
     * @return
     */
    public Long getBalance(WalletTypeEnum typeEnum) throws IOException {
        UserWalletVO walletVO = getUserWallet();
        for (UserWalletVO.Item item : walletVO.getWalletItems()) {
            if (typeEnum.getValue() - item.getWalletType() == 0) {
                return item.getBalance();
            }
        }
        return null;
    }

    private UserProfile getProfile() throws IOException {
        String action = "api-app/v1/user/getProfile";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
        UserProfile profile = new UserProfile();
        JSONObject json = getData(res);
        profile.setUserId(Long.parseLong(json.getString("userId")));
        return profile;
    }
}
