package com.starbuds.server.api.app.v1.i18n;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import com.starbuds.test.utils.FileTools;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author aoe
 * @date 2020/5/14
 */
public class I18nDBTest {

    private static String host = "http://api.panygo.com/";
    private static String host_api_common = "http://api.panygo.com/";

    static {
//        host = "http://localhost:7152/";
//        host_api_common = "http://localhost:7151/";
    }

    /**
     * 获得所有需要国际化的数据
     *
     * @throws IOException
     */
    @Test
    public void initAllData() throws IOException {
        Set<String> set = new LinkedHashSet<>();
//        set.addAll(getGift(I18nTypeEnum.ZH_CN));
//        set.addAll(getGifts(I18nTypeEnum.ZH_CN));
//        set.addAll(getMedalList(I18nTypeEnum.ZH_CN));
//        set.addAll(getMountsList(I18nTypeEnum.ZH_CN));
//        set.addAll(getGiftList(I18nTypeEnum.ZH_CN));
//        set.addAll(getPropsList(I18nTypeEnum.ZH_CN));
//        set.addAll(getGoodsList(I18nTypeEnum.ZH_CN));
//        set.addAll(getGoodsCategories(I18nTypeEnum.ZH_CN));
//        set.addAll(getUserTagsTest(I18nTypeEnum.ZH_CN));
//        set.addAll(getSingleChatEvaluateTags(I18nTypeEnum.ZH_CN));
//        set.addAll(getInitConfig(I18nTypeEnum.ZH_CN));
//        set.addAll(getHobbyTags(I18nTypeEnum.ZH_CN));
//        set.addAll(getReportReasons(I18nTypeEnum.ZH_CN));
//        set.addAll(getCoinTypeNameMap(I18nTypeEnum.ZH_CN));
//        set.addAll(getCoinRechargeItems(I18nTypeEnum.ZH_CN));
        set.addAll(getSuggestTypeList(I18nTypeEnum.ZH_CN));

        FileTools.writeToFile(FileTools.getTestResourcesFilePath("i18n/db-zh_CN.txt"), new ArrayList<>(set));
    }

    @Test
    public void allEnTest() throws IOException {
        List<String> result = new ArrayList<>();
//        result.addAll(getGift(I18nTypeEnum.EN));
//        result.addAll(getGifts(I18nTypeEnum.EN));
//        result.addAll(getMedalList(I18nTypeEnum.EN));
//        result.addAll(getMountsList(I18nTypeEnum.EN));
//        result.addAll(getGiftList(I18nTypeEnum.EN));
//        result.addAll(getPropsList(I18nTypeEnum.EN));
//        result.addAll(getGoodsList(I18nTypeEnum.EN));
//        result.addAll(getGoodsCategories(I18nTypeEnum.EN));
//        result.addAll(getUserTagsTest(I18nTypeEnum.EN));
//        result.addAll(getSingleChatEvaluateTags(I18nTypeEnum.EN));
//        result.addAll(getInitConfig(I18nTypeEnum.EN));
//        result.addAll(getHobbyTags(I18nTypeEnum.EN));
//        result.addAll(getReportReasons(I18nTypeEnum.EN));
//        result.addAll(getCoinTypeNameMap(I18nTypeEnum.EN));
//        result.addAll(getCoinRechargeItems(I18nTypeEnum.EN));
        result.addAll(getSuggestTypeList(I18nTypeEnum.EN));

        for (String msg : result) {
            Assert.assertEquals(isContainChinese(msg), false);
        }
    }

    /**
     * 字符串是否包含中文
     *
     * @param str 待校验字符串
     * @return true 包含中文字符 false 不包含中文字符
     */
    public static boolean isContainChinese(String str) {
        Pattern p = Pattern.compile("[\u4E00-\u9FA5|\\！|\\，|\\。|\\（|\\）|\\《|\\》|\\“|\\”|\\？|\\：|\\；|\\【|\\】]");
        Matcher m = p.matcher(str);
        if (m.find()) {
            return true;
        }
        return false;
    }

    // 获取礼物
    private List<String> getGift(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/gift/getGift?giftId=35951954886761", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        List<String> list = new ArrayList<>();
        list.add(data.getString("giftPriceTypeName"));
        return list;
    }

    // 获取礼物列表
    private List<String> getGifts(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/gift/getGifts", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");
        List<String> list = getChineseList(listArr, "giftName");
        list.addAll(getChineseList(listArr, "giftPriceTypeName"));
        return list;
    }

    // 获取背包勋章列表
    private List<String> getMedalList(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/bag/getMedalList", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");

        JSONArray itemArr = new JSONArray();
        for (int i = 0; i < listArr.size(); i++) {
            JSONObject item = listArr.getJSONObject(i).getJSONObject("item");
            itemArr.add(item);
        }

        List<String> list = getChineseList(itemArr, "medalName");
        return list;
    }

    // 获取背包坐骑列表
    private List<String> getMountsList(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/bag/getMountsList", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");

        JSONArray itemArr = new JSONArray();
        for (int i = 0; i < listArr.size(); i++) {
            JSONObject item = listArr.getJSONObject(i).getJSONObject("item");
            itemArr.add(item);
        }
        List<String> list = getChineseList(itemArr, "mountsName");
        return list;
    }

    // 获取背包礼物列表
    private List<String> getGiftList(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/bag/getGiftList", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");

        JSONArray itemArr = new JSONArray();
        for (int i = 0; i < listArr.size(); i++) {
            JSONObject item = listArr.getJSONObject(i).getJSONObject("item");
            itemArr.add(item);
        }

        List<String> list = getChineseList(itemArr, "giftName");
        return list;
    }

    // 获取背包道具列表
    private List<String> getPropsList(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/bag/getPropsList", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");

        JSONArray itemArr = new JSONArray();
        for (int i = 0; i < listArr.size(); i++) {
            JSONObject item = listArr.getJSONObject(i).getJSONObject("item");
            itemArr.add(item);
        }

        List<String> list = getChineseList(itemArr, "propsName");
        list.addAll(getChineseList(itemArr, "propsIntro"));
        return list;
    }

    // 金币商城接口>获取商品列表
    private List<String> getGoodsList(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/coinShop/getGoodsList", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");
        List<String> list = getChineseList(listArr, "categoryName");
        list.addAll(getChineseList(listArr, "goodsName"));
        list.addAll(getChineseList(listArr, "goodsPriceTypeName"));
        return list;
    }

    // 金币商城接口>获取商品类目
    private List<String> getGoodsCategories(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/coinShop/getGoodsCategories", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");
        List<String> list = getChineseList(listArr, "categoryName");
        return list;
    }

    // 获取用户标签
    private List<String> getUserTagsTest(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/config/getUserTags", langEnum);
        List<String> list = getChineseList(res, "tagName");
        return list;
    }

    // 获取单聊评价标签
    private List<String> getSingleChatEvaluateTags(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/config/getSingleChatEvaluateTags", langEnum);
        List<String> list = getChineseList(res, "tagName");
        return list;
    }

    // 客户端接口>配置接口>获取初始化配置
    private List<String> getInitConfig(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/config/getInitConfig", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray forbiddenWordOptions = data.getJSONArray("forbiddenWordOptions");

        JSONObject channelConfig = data.getJSONObject("channelConfig");
        JSONArray payWays = channelConfig.getJSONArray("payWays");
        JSONArray oauths = channelConfig.getJSONArray("oauths");
        JSONArray liveCategories = channelConfig.getJSONArray("liveCategories");
        JSONArray singleChatCategories = channelConfig.getJSONArray("singleChatCategories");

        List<String> list = new ArrayList<>();
        list.addAll(getChineseList(forbiddenWordOptions, "optionName"));

        // channelConfig
        list.addAll(getChineseList(payWays, "payName"));
        list.addAll(getChineseList(oauths, "oauthName"));
        list.add(channelConfig.getString("coinName"));
        list.addAll(getChineseList(liveCategories, "categoryName"));
        list.addAll(getChineseList(singleChatCategories, "categoryName"));
        list.add(channelConfig.getString("liveTips"));

        return list;
    }

    // 获取兴趣标签
    private List<String> getHobbyTags(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/config/getHobbyTags", langEnum);
        List<String> list = getChineseList(res, "tagName");
        return list;
    }

    // 获取举报原因
    private List<String> getReportReasons(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/config/getReportReasons", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        JSONArray listArr = data.getJSONArray("list");
        List<String> list = getChineseList(listArr, "reasonValue");
        return list;
    }

    // 配置接口>获取金币类型名称
    private List<String> getCoinTypeNameMap(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/config/getCoinTypeNameMap", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        JSONObject data = json.getJSONObject("data");
        List<String> result = new ArrayList<>();
        for (Map.Entry<String, Object> entry : data.entrySet()) {
            result.add((String) entry.getValue());
        }
        return result;
    }

    // 获取金币充值项目
    private List<String> getCoinRechargeItems(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host, "api-app/v1/recharge/getCoinRechargeItems", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        List<String> list = getChineseList(res, "payWayProvider");
        return list;
    }

    // 获取意见反馈类型
    private List<String> getSuggestTypeList(I18nTypeEnum langEnum) throws IOException {
        String res = XyHttpTools.get(host_api_common, "api-common/v1/data/getSuggestTypeList", langEnum);
        JSONObject json = JSONObject.parseObject(res);
        List<String> list = getChineseList(res, "typeName");
        return list;
    }

    private List<String> getChineseList(JSONArray array, String field) {
        List<String> list = new ArrayList<>(array.size());
        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.getJSONObject(i);
            list.add(obj.getString(field));
        }

        Assert.assertEquals(list.size(), array.size());

        return list;
    }

    private List<String> getChineseList(String res, String field) {
        JSONObject json = JSONObject.parseObject(res);
        JSONArray array = json.getJSONObject("data").getJSONArray("list");

        List<String> list = new ArrayList<>(array.size());
        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.getJSONObject(i);
            list.add(obj.getString(field));
        }

        Assert.assertEquals(list.size(), array.size());

        return list;
    }
}
