package com.starbuds.server.api.app.v1.i18n;

import com.starbuds.server.api.app.util.I18nUtil;
import com.starbuds.test.utils.FileTools;
import com.starbuds.test.BaseTest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

/**
 * 国际化文件初始化测试
 * 查找中文正则  [\u4e00-\u9fa5]
 *
 * @author aoe
 * @date 2020/5/12
 */
@Slf4j
public class I18nInitPropertiesTest extends BaseTest {

    @Autowired
    private I18nUtil i18nUtil;

    /**
     * 国际化翻译后的原始文件-英文（使用本文件正式生成messages_en.properties文件）
     */
    public final static String I18N_EN_PROPERTIES = "i18n/i18n-en.properties";

    /**
     * 原始消息转csv文件
     */
    @Test
    public void text2csv() {
        List<String> msgList = FileTools.readList(FileTools.getTestResourcesFilePath("i18n/init.txt"));
        FileTools.writeToFile(FileTools.getTestResourcesFilePath("i18n/i18n.csv"), msgList);
    }

    /**
     * 翻译后的 csv 文件转正式 i18n-en.properties
     */
    @Test
    public void enCsvToProperties() {
        List<String> msgList = FileTools.readList(FileTools.getTestResourcesFilePath("i18n/i18n-英语.csv"));
        List<String> resultList = new ArrayList<>(msgList.size());
        List<String> errList = new ArrayList<>();
        for (String msg : msgList) {
            int index = msg.indexOf(",");
            String key = msg.substring(0, index);
            String value = msg.substring(index + 1);
            if (StringUtils.isBlank(value)) {
                errList.add(key);
            }
            msg = i18nUtil.getKey(key) + "=" + value;
            resultList.add(msg);
        }
        FileTools.writeToFile(FileTools.getTestResourcesFilePath(I18N_EN_PROPERTIES), resultList);

        for (String err : errList) {
            log.warn("{} 没有对应的值", err);
        }
        Assert.assertEquals(errList.isEmpty(), true);
    }


}
