package com.starbuds.server.api.app.v1.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import com.starbuds.test.utils.OkHttpTools;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.SignType;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class ActivityFacadeImplTest {

    @Test
    public void testGetLotteryI18n() throws IOException {
        testGetLotteryI18n("http://api.panygo.com");
    }

    @Test
    public void testGetLotteryI18nDev() throws IOException {
        testGetLotteryI18n("http://localhost:7152");
    }

    private void testGetLotteryI18n(String host) throws IOException {
        host += "/api-app/v1/activity/getLottery?lotteryId=1";
        String res = OkHttpTools.get(host);
        Assert.assertEquals(getResMsg(res), "没有权限");

        Map<String, String> headers = new HashMap<>();
        headers.put("lang", "en");
        res = OkHttpTools.get(host, headers);
        Assert.assertEquals(getResMsg(res), "Not authorized");
        headers.remove("lang");

        res = XyHttpTools.get(host, "/api-app/v1/activity/getLottery?lotteryId=1", I18nTypeEnum.ZH_CN);
        Assert.assertEquals(getResMsg(res), "未找到该抽奖活动");

        headers.put("lang", "en");
        res = OkHttpTools.get(host, headers);
        Assert.assertEquals(getResMsg(res), "Can’t find the lucky draw");
    }

    private String getResMsg(String jsonStr) {
        JSONObject json = JSONObject.parseObject(jsonStr);
        return json.getString("msg");
    }
}