package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.login.AppLoginTest;
import com.starbuds.server.api.app.pojo.BatQueryBalanceItem;
import com.starbuds.server.api.app.pojo.BatQueryBalanceVo;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.finance.UserWalletVO;
import com.starbuds.test.utils.XyHttpTools;
import lombok.extern.slf4j.Slf4j;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static com.starbuds.test.utils.XyHttpTools.HOST_DEV;
import static com.starbuds.test.utils.XyHttpTools.HOST_TEST_ALY_APP;
import static com.starbuds.test.utils.XyResultTools.*;

/**
 * 游戏钱包相关测试
 *
 * @author aoe
 * @date 2020/11/13
 */
@Slf4j
public class GameAndWalletTest {

    private static String HOST = HOST_DEV;

    static {
        HOST = HOST_TEST_ALY_APP;

        try {
            AppLoginTest.saveLoginToken(HOST, "15757876459");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // 游戏钱包详情
    @Test
    public void gameWalletDetailTest() throws IOException {
        // 用户钱包列表
        List<UserWalletVO.Item> walletItems = getUserWallet();
        // 星钻
        Long diamond = getBalance(walletItems, WalletTypeEnum.STAR_DIAMOND);
        // 游戏币
        Long gameCoin = getBalance(walletItems, WalletTypeEnum.GAME_COIN);

        // 获取游戏余额-所有平台
        BatQueryBalanceVo gameBalanceAllVO = getGameBalanceAll();

        String split = "------------------------------------------";
        String splitBig = "==========================================";
        log.info("\n{}\n星钻 {} \n游戏币 {} \n",
                split,
                diamond,
                gameCoin
        );

        System.out.println(splitBig);
        System.out.println("游戏钱包总金额 " + gameBalanceAllVO.getTotal());
        System.out.println("待回收金额 " + gameBalanceAllVO.getRecycleAble());
        System.out.println(split("游戏钱包明细"));
        for (BatQueryBalanceItem item : gameBalanceAllVO.getList()) {
            String type = 10 == item.getApiType() ? "game" : "gamedev";
            System.out.println(item.getPlatform() + " " + item.getBalance() + "  | " + type);
        }
    }

    // 查询用户钱包
    private List<UserWalletVO.Item> getUserWallet() throws IOException {
        String action = "api-app/v1/wallet/getUserWallet";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);

        JSONArray array = JSON.parseObject(res).getJSONObject("data").getJSONArray("walletItems");
        List<UserWalletVO.Item> items = new ArrayList<>();
        for (int i = 0; i < array.size(); i++) {
            JSONObject json = array.getJSONObject(i);
            UserWalletVO.Item item = new UserWalletVO.Item();
            item.setBalance(Long.parseLong(json.getString("balance")));
            item.setWalletType(json.getInteger("walletType"));
            items.add(item);
        }
        return items;
    }

    /**
     * 获得用户钱包余额
     *
     * @param items
     * @param type
     * @return
     */
    private Long getBalance(List<UserWalletVO.Item> items, WalletTypeEnum type) {
        for (UserWalletVO.Item item : items) {
            if (WalletTypeEnum.toEnum(item.getWalletType()) == type) {
                return item.getBalance();
            }
        }
        return -1L;
    }

    // 获取游戏余额-所有平台
    private BatQueryBalanceVo getGameBalanceAll() throws IOException {
        String action = "api-app/v1/wallet/getGameBalanceAll";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);

        BatQueryBalanceVo vo = JSON.toJavaObject(getData(res), BatQueryBalanceVo.class);
        return vo;
    }

    private String split(String title) {
        return "--------------------title----------------------".replace("title", title);
    }
}
