package com.starbuds.server.api.app.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.login.AppLoginTest;
import com.starbuds.server.api.app.pojo.AddGameDevOrderDetailForm;
import com.starbuds.server.game.sdkdev.response.GetGameplayResponse;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.starbuds.test.utils.XyHttpTools.HOST_DEV;
import static com.starbuds.test.utils.XyHttpTools.HOST_TEST_ALY_APP;
import static com.starbuds.test.utils.XyResultTools.checkSuccess;

public class GameDevFacadeImplTest {

    private static String HOST = HOST_DEV;

    static {
        HOST = HOST_TEST_ALY_APP;

        try {
            AppLoginTest.saveLoginToken(HOST, "15757876459");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // 初始化用户
    @Test
    public void initUserTest() throws IOException {
        String action = "api-app/v1/gamedev/initUser";
        String res = XyHttpTools.post(HOST, action, new HashMap<>());
        checkSuccess(res);
    }

    // 获取游戏列表
    @Test
    public void getGameListTest() throws IOException {
        String action = "api-app/v1/gamedev/getGameList";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取游戏
    @Test
    public void getGameTest() throws IOException {
        String action = "api-app/v1/gamedev/getGame?gameId=JSFFC";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取指定游戏期号信息
    @Test
    public void getGamePlayTest() throws IOException {
        String action = "api-app/v1/gamedev/getGamePlay";
        action += "?gameId=JSFFC";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取开奖历史分页数据
    @Test
    public void getWinNosPageListTest() throws IOException {
        String action = "api-app/v1/gamedev/getWinNosPageList";
        action += "?gameId=JSFFC&pageIndex=1&pageSize=10";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 开奖
    @Test
    public void getNewWinNoTest() throws IOException {
        String action = "api-app/v1/gamedev/getNewWinNo";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取指定游戏玩法
    @Test
    public void getGamePlayInfoTest() throws IOException {
        String action = "api-app/v1/gamedev/getGamePlayInfo";
        action += "?gameId=JSFFC";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 进入下一期
    @Test
    public void getNextPeriodTest() throws IOException {
        String action = "api-app/v1/gamedev/getNextPeriod";
        action += "?gameId=JSFFC";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取玩家余额
    @Test
    public void getBalanceTest() throws IOException {
        String action = "api-app/v1/gamedev/getBalance";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 添加游戏币订单(划转资产，转入/转出)
    @Test
    public void addGameCoinOrderTest() throws IOException {
        long[] coins = new long[]{
                10
        };
        for (long coin : coins) {
            // operate 操作（10:转入;20:转出）
            addGameCoinOrder(10, coin);
        }
    }

    // 游戏Dev下单
    @Test
    public void addOrderTest() throws IOException {
        String action = "api-app/v1/gamedev/addOrder";
        String gameId = "JSFFC";
        Map<String, String> params = new HashMap<>();
        params.put("gameId", gameId);
        // 下单期号
        params.put("curPeriod", getCurPeriod(gameId));
        // 倍数
        params.put("bet", "1");
        // 明细列表
        List<AddGameDevOrderDetailForm> details = new ArrayList<>();
        AddGameDevOrderDetailForm detail = new AddGameDevOrderDetailForm();
        detail.setPlayId("5S_DXDS");
        detail.setDetailOrder(1);
        detail.setSeries("0.97");
        detail.setPrice(1);
        detail.setPlayName("五星大小单双");
        details.add(detail);
        params.put("details", JSON.toJSONString(details));
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 获取订单列表
    @Test
    public void getOrderPageListTest() throws IOException {
        String action = "api-app/v1/gamedev/getOrderPageList";
        action += "?gameId=JSFFC";
        action += "&status=0";
        action += "&pageIndex=1";
        action += "&pageSize=10";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 游戏订单详情
    @Test
    public void getOrderDetailTest() throws IOException {
        String action = "api-app/v1/gamedev/getOrderDetail";
        action += "?orderNo=JS200925089770317A";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 订单统计数据（自主游戏、第三方游戏总和）
    @Test
    public void getOrderStatisticTest() throws IOException {
        String action = "api-app/v1/gamedev/getOrderStatistic";
        action += "?startTime=" + (System.currentTimeMillis() - 60 * 86400000L);
        action += "&endTime=" + System.currentTimeMillis();
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 设置直播游戏
    @Test
    public void setLiveGameTest() throws IOException {
        String action = "api-app/v1/gamedev/setLiveGame";
        Map<String, String> params = new HashMap<>();
        params.put("gameName", "1分时时彩");
        params.put("groupCode", "SSC");
        params.put("logo", "http://154.211.22.54:8299/JSFFC.png");
        params.put("gameId", "JSFFC");
        params.put("series", "0.97");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    /**
     * 获取当前游戏期号
     *
     * @param gameId
     * @return
     * @throws IOException
     */
    private String getCurPeriod(String gameId) throws IOException {
        String action = "api-app/v1/gamedev/getGamePlay";
        action += "?gameId=" + gameId;
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
        GetGameplayResponse resObj = JSONObject.parseObject(res, GetGameplayResponse.class);
        return resObj.getData().getPeriodInfo().getRtAwdCurPeriod();
    }

    /**
     * 添加游戏币订单
     *
     * @param operate 操作（10:转入;20:转出）
     * @param coin
     * @throws IOException
     */
    private void addGameCoinOrder(int operate, long coin) throws IOException {
        String action = "api-app/v1/gamedev/addGameCoinOrder";
        Map<String, String> params = new HashMap<>();
        // 操作（10:转入;20:转出）
        params.put("operate", "" + operate);
        params.put("coin", "" + coin);
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }
}

// @安卓-关文举 @安卓-姚信 @IOS-翟玉磊 @龚圆