package com.starbuds.server.api.app.v1.impl;

import com.starbuds.login.AppLoginTest;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static com.starbuds.test.utils.XyHttpTools.HOST_DEV;
import static com.starbuds.test.utils.XyHttpTools.HOST_TEST_ALY_APP;
import static com.starbuds.test.utils.XyResultTools.checkSuccess;


public class GameFacadeImplTest {

    private static String HOST = HOST_DEV;

    static {
        HOST = HOST_TEST_ALY_APP;

        try {
            AppLoginTest.saveLoginToken(HOST, "15757876459");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // 获取游戏房间列表
    @Test
    public void testGetRoomList() throws IOException {
        String action = "api-app/v1/game/getRoomList";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取热门游戏列表
    @Test
    public void testGetHotGameList() throws IOException {
        String action = "api-app/v1/game/getHotGameList";
        action += "?deviceType=mobile";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取游戏列表
    @Test
    public void testGetGameList() throws IOException {
        String action = "api-app/v1/game/getGameList";
        action += "?roomCode=BBINDZ&deviceType=mobile";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 进入游戏
    @Test
    public void enterGameTest() throws IOException {
        String action = "api-app/v1/game/enterGame";
        Map<String, String> params = new HashMap<>();
        params.put("gameCode", "BBINDZ-HXZJ");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 添加游戏币订单
    @Test
    public void addGameCoinOrderTest() throws IOException {
        long[] coins = new long[]{
                10
        };
        for (long coin : coins) {
            // operate 操作（10:转入;20:转出）
            addGameCoinOrder(10, coin);
        }
    }

    // 获得游戏币订单状态 10:处理中 20:成功 30:失败
    @Test
    public void getOrderStatusTest() throws IOException {
        String action = "api-app/v1/game/getOrderStatus";
        action += "?orderId=209898080043010";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 拉取所有游戏
    @Test
    public void pullAllGamesTest() throws IOException {
        String action = "api-app/v1/game/pullAllGames";
        Map<String, String> params = new HashMap<>();
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    /**
     * 添加游戏币订单
     *
     * @param operate 操作（10:转入;20:转出）
     * @param coin
     * @throws IOException
     */
    private void addGameCoinOrder(int operate, long coin) throws IOException {
        String action = "api-app/v1/game/addGameCoinOrder";
        Map<String, String> params = new HashMap<>();
        params.put("platform", "BBIN");
        // 操作（10:转入;20:转出）
        params.put("operate", "" + operate);
        params.put("coin", "" + coin);
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }
}

// @安卓-关文举 @安卓-姚信 @IOS-翟玉磊 @龚圆
// cn.hutool.http.HttpException

// 请求: {"param":{"amount":"3","notify_url":"http://api.panygo.com/api-notify/v1/game/receiveOrderStatus","order_no":"211448588402688","platform":"BBIN","player_name":"18077106307101","player_password":"18077106307101","tran_type":"OUT"},"responseClass":"com.starbuds.server.game.sdk.response.TransferResponse","serviceName":"/api/transfer.php"}

// 请求参数: https://game-api.cp28.top/api/transfer.php {"order_no":"211448588402688","player_password":"18077106307101","amount":"3","tran_type":"OUT","appid":"8da954c1-e43e-4c00-b7f7-882e8caafe5b","sign":"bdde0f106846bd43149ebbef8734d158","player_name":"18077106307101","notify_url":"http://api.panygo.com/api-notify/v1/game/receiveOrderStatus","platform":"BBIN","ts":"2020-09-17 17:50:36"}

// 收到游戏订单处理结果通知 GameOrderStatusNotify(appid=8da954c1-e43e-4c00-b7f7-882e8caafe5b, platform=BBIN, playerName=tes18077106307101, playerPassword=null, ts=2020-09-17 17:48:17, sign=D2C096A073A7BB08E2F06B8D382B6681, tranType=2, amount=3.0, order_no=null, result=null)

//