package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.util.UserServiceTest;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.test.utils.XyHttpTools;
import org.apache.commons.lang3.StringUtils;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.IntStream;

import static com.starbuds.test.utils.XyHttpTools.HOST_DEV;
import static com.starbuds.test.utils.XyHttpTools.HOST_TEST_ALY_APP;
import static com.starbuds.test.utils.XyResultTools.checkSuccess;

public class RtcRoomAuctionFacadeImplTest {

    private static UserServiceTest userService;
    private static String HOST = HOST_DEV;

    static {
        HOST = HOST_TEST_ALY_APP;
        userService = new UserServiceTest(HOST);
    }

    private final String ROOM_ID = "440426599284736";

    // 获取拍卖房起拍设置的相关数据
    @Test
    public void getStartAuctionSettingData() throws IOException {
        String action = "api-app/v1/rtc/getStartAuctionSettingData";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 设置拍卖信息(设置成功后竞拍开始)
    @Test
    public void setAuction() throws IOException {
        String action = "api-app/v1/rtc/setAuction";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        params.put("watchConfigId", "1");
        params.put("watchDuration", "1");
        params.put("startGiftId", "402594572795904");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 申请上麦-拍卖房
    @Test
    public void sitSeatInAuction() throws IOException {
        String action = "api-app/v1/rtc/sitSeatInAuction";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        // type 麦位类型：1:竞拍麦位；2:普通麦位
        params.put("type", "1");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    @Test
    public void test() {
        System.out.println(RedisRtcKeys.getRtcAuctionRoomSetLock(9678L));
        System.out.println(RedisRtcKeys.getRtcAuctionRoomUser(9678L));
        System.out.println(StringUtils.rightPad("hehe", 10, "W"));
    }


    // 移除麦位, 只能是管理员操作
    @Test
    public void removeQueueUserInAuctionTest() throws IOException {
        String action = "api-app/v1/rtc/removeQueueUserInAuction";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        // type 麦位类型：1:竞拍麦位；2:普通麦位
        params.put("type", "1");
        params.put("userId", "");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    @Test
    public void setUserToSeatInAuctionTest() throws IOException {
        String action = "api-app/v1/rtc/setUserToSeatInAuction";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        // type 麦位类型：1:竞拍麦位；2:普通麦位
        params.put("type", "1");
        params.put("userId", "418663664451584");
        params.put("seatNo", "1");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }


    @Test
    public void leaveQueueInAuctionTest() throws IOException {
        String action = "api-app/v1/rtc/leaveQueueInAuction";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        // type 麦位类型：1:竞拍麦位；2:普通麦位
        params.put("type", "1");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    @Test
    public void rankTest() {
        BigDecimal one = new BigDecimal("0.001");
        BigDecimal timeGap = new BigDecimal("36000");
        BigDecimal result = one.divide(timeGap, 10, RoundingMode.DOWN);
        System.out.println(result.toPlainString());

        BigDecimal manyTimes = result.multiply(new BigDecimal("10000"));
        System.out.println(manyTimes.toPlainString());

        BigDecimal total = IntStream.rangeClosed(1, 86400)
                .boxed()
                .map(gap -> one.divide(new BigDecimal("" + gap), 15, RoundingMode.DOWN))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        System.out.println(total.toPlainString());
    }
}