package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.util.UserServiceTest;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.finance.UserWalletVO;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static com.starbuds.test.utils.XyHttpTools.HOST_DEV;
import static com.starbuds.test.utils.XyHttpTools.HOST_TEST_ALY_APP;
import static com.starbuds.test.utils.XyResultTools.checkSuccess;
import static org.testng.Assert.assertEquals;

/**
 * 语音房测试
 *
 * @author aoe
 * @date 2020/11/20
 */
public class RtcTest {

    private static UserServiceTest userService;
    private static String HOST = HOST_DEV;

    static {
        HOST = HOST_TEST_ALY_APP;
        userService = new UserServiceTest(HOST);
    }

    private final static String ROOM_ID = "170039449747456";

    // 演员上麦
    @Test
    public void sitSeatTest() throws IOException {
        // 演员上麦
        loginActor();
        sitSeat();
    }

    // 演员下麦
    @Test
    public void leaveSeatTest() throws IOException {
        // 演员下麦
        loginActor();
        leaveSeat();
    }

    // 查看钱包
    @Test
    public void checkTest() throws IOException {
        loginActor();
//        loginRichMan();
        UserWalletVO wallet = userService.getUserWallet();
        Long dustAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_DUST);
        Long pinkAmount = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND);
        Long pinkTotal = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND_TOTAL);
        Long diamondAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_DIAMOND);
        Long startYuanAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_YUAN);
        System.out.println("粉钻-不可提现（原星尘） " + dustAmount);
        System.out.println("粉钻-可提现 " + pinkAmount);
        System.out.println("粉钻总量 " + pinkTotal);
        System.out.println("星钻 " + diamondAmount);
        System.out.println("星元 " + startYuanAmount);
    }

    // 昨天的星尘转粉钻
    @Test
    public void transYtdStarDustToPinkDiamondTest() throws IOException {
        // 初始钱包余额
        UserWalletVO wallet = userService.getUserWallet();
        Long dustAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_DUST);
        Long pinkAmount = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND);
        Long diamondAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_DIAMOND);

        String action = "api-app/v1/test/transYtdStarDustToPinkDiamond";
        Map<String, String> params = new HashMap<>();
        params.put("today", "2020-11-21");
        params.put("ytd", "2020-11-21");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);

        // 转移后钱包余额
        wallet = userService.getUserWallet();
        assertEquals(userService.getBalance(wallet, WalletTypeEnum.STAR_DUST).longValue(), 0L);
        assertEquals(userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND).longValue(),
                pinkAmount + dustAmount.longValue());
    }

    // 粉钻转星钻
    @Test
    public void transPinkToDiamond() throws IOException {
        String action = "api-app/v1/wallet/transDiamond";
        Map<String, String> params = new HashMap<>();
        params.put("walletType", "" + WalletTypeEnum.PINK_DIAMOND.getValue());
        params.put("amount", "10000");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 申请粉钻转出
    @Test
    public void applyWithdrawByPink() throws IOException {
        String action = "api-app/v1/wallet/applyWithdrawByPink";
        Map<String, String> params = new HashMap<>();
        Long amount = 1 * 10000L;
        params.put("amount", amount.toString());
        params.put("cardId", "38dbfec51a1d684739584de8a5186ec5");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 赠送礼物 -> 星尘转粉钻 -> 粉钻转星钻
    @Test
    public void rtcTest() throws IOException {
        // 演员上麦
//        loginActor();
//        sitSeat();
//        Long actorUserId = getUserId();
        // 关羽
        Long actorUserId = Long.parseLong("168550899646464");
        // 修改昵称通知测试
        actorUserId = Long.parseLong("141075591200768");

        // 土豪送礼物
        loginRichMan();
        // 热气球 67834646364160 200星钻
        sendRtcGift("" + actorUserId, Long.parseLong("67834646364160"), 1);

        // 演员下麦
//        loginActor();
//        leaveSeat();
    }

    // 申请上麦
    private void sitSeat() throws IOException {
        String action = "api-app/v1/rtc/sitSeat";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        // 麦位 0 主持麦 1-8 普通麦
        params.put("seatNo", "5");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 下麦
    private void leaveSeat() throws IOException {
        String action = "api-app/v1/rtc/leaveSeat";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 送礼物
    private void sendRtcGift(String userIds, Long giftId, int quantity) throws IOException {
        String action = "api-app/v1/gift/sendRtcGift";
        Map<String, String> params = new HashMap<>();
        params.put("roomId", ROOM_ID);
        params.put("userIds", userIds);
        params.put("giftId", giftId.toString());
        params.put("quantity", quantity + "");
        params.put("isFromBag", "false");
        String res = XyHttpTools.post(HOST, action, params);
        checkSuccess(res);
    }

    // 演员登录
    private void loginActor() {
        // 关羽
        String phone = "18158185000";
        // 仙二爷 可申请提现
        phone = "19222222222";
        userService.login(phone);
    }

    // 土豪登录
    private void loginRichMan() {
        userService.login("18158183656");
    }


}
