package com.starbuds.server.api.app.v1.impl;

import com.starbuds.server.api.app.util.UserServiceTest;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.finance.UserWalletVO;
import com.starbuds.test.utils.XyHttpTools;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static com.starbuds.test.utils.XyHttpTools.HOST_DEV;
import static com.starbuds.test.utils.XyHttpTools.HOST_TEST_ALY_APP;
import static com.starbuds.test.utils.XyResultTools.checkSuccess;

public class WalletFacadeImplTest {

    private static UserServiceTest userService;
    private static String HOST = HOST_DEV;

    static {
        HOST = HOST_TEST_ALY_APP;
        userService = new UserServiceTest(HOST);
    }

    // 获取游戏余额
    @Test
    public void getGameBalanceTest() throws IOException {
        String action = "api-app/v1/wallet/getGameBalance";
        action += "?platform=BBIN";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获取游戏余额-所有平台
    @Test
    public void getGameBalanceAllTest() throws IOException {
        String action = "api-app/v1/wallet/getGameBalanceAll";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 一键回收所有游戏币
    @Test
    public void recyclingAllGameCoinTest() throws IOException {
        String action = "api-app/v1/wallet/recyclingAllGameCoin";
        String res = XyHttpTools.post(HOST, action, new HashMap<>());
        checkSuccess(res);
    }

    // 兑换游戏币
    @Test
    public void transGameCoinTest() throws IOException {
        String action = "api-app/v1/wallet/transGameCoin";
        Map<String, String> param = new HashMap<>();
        // 星钻
        param.put("walletType", "11");
        param.put("amount", "2000");
        String res = XyHttpTools.post(HOST, action, param);
        checkSuccess(res);
    }

    // 游戏币兑换星钻
    @Test
    public void transGameCoinToDiamondTest() throws IOException {
        String action = "api-app/v1/wallet/transGameCoinToDiamond";
        Map<String, String> param = new HashMap<>();
        param.put("coin", "201");
        String res = XyHttpTools.post(HOST, action, param);
        checkSuccess(res);
    }

    // 获取游戏余额-Dev接口平台
    @Test
    public void getGameDevBalanceTest() throws IOException {
        String action = "api-app/v1/wallet/getGameDevBalance";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 获粉钻提现金额选项
    @Test
    public void getWithdrawPinkListTest() throws IOException {
        String action = "api-app/v1/wallet/getWithdrawPinkList";
        String res = XyHttpTools.get(HOST, action);
        checkSuccess(res);
    }

    // 兑换钻石（粉钻-不可提现、粉钻-可提现 兑换 星钻）
    @Test
    public void transDustPinkToDiamondTest() throws IOException {
        UserWalletVO wallet = userService.getUserWallet();
        Long dustAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_DUST);
        Long pinkAmount = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND);
        Long pinkTotal = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND_TOTAL);
        Long diamondAmount = userService.getBalance(wallet, WalletTypeEnum.STAR_DIAMOND);

        // 调用接口
        String action = "api-app/v1/wallet/transDustPinkToDiamond";
        Map<String, String> param = new HashMap<>();
        int total = 100;
        param.put("total", "" + total);
        String res = XyHttpTools.post(HOST, action, param);
        checkSuccess(res);

        wallet = userService.getUserWallet();
        Long dustAmountNew = userService.getBalance(wallet, WalletTypeEnum.STAR_DUST);
        Long pinkAmountNew = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND);
        Long pinkTotalNew = userService.getBalance(wallet, WalletTypeEnum.PINK_DIAMOND_TOTAL);
        Long diamondAmountNew = userService.getBalance(wallet, WalletTypeEnum.STAR_DIAMOND);

        System.out.println("总共兑换 " + total);
        System.out.println("-------------------------------------------------");
        System.out.println("粉钻-不可提现（原星尘） " + dustAmount);
        System.out.println("粉钻-可提现 " + pinkAmount);
        System.out.println("粉钻总量 " + pinkTotal);
        System.out.println("星钻 " + diamondAmount);
        System.out.println("-------------------------------------------------");
        System.out.println("粉钻-不可提现（原星尘） " + dustAmountNew);
        System.out.println("粉钻-可提现 " + pinkAmountNew);
        System.out.println("粉钻总量 " + pinkTotalNew);
        System.out.println("星钻 " + diamondAmountNew);
    }
}