package com.starbuds.test.utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 文件处理工具
 *
 * @author aoe
 * @date 2020/5/13
 */
@Slf4j
public class FileTools {

    public static List<String> readList(String... filePathList) {
        List<String> msgList = new ArrayList<>();
        Set<String> wordSet = new HashSet<>();

        for (String filePath : filePathList) {
            File readFile = new File(filePath);
            log.info("读取文件 {}", readFile.getAbsoluteFile().getPath());
            BufferedReader reader = null;
            try {
                reader = new BufferedReader(new FileReader(readFile));
                String line = "";
                while ((line = reader.readLine()) != null) {
                    if (StringUtils.isEmpty(line) || !wordSet.add(line) || line.startsWith("#")) {
                        continue;
                    }
                    msgList.add(line);
                }
            } catch (FileNotFoundException e) {
                log.error("没有找到指定文件", e);
            } catch (IOException e) {
                log.error("文件读写出错", e);
            } finally {
                try {
                    if (reader != null) {
                        reader.close();
                    }
                } catch (IOException e) {
                    log.error("关闭文件错误", e);
                }
            }
        }

        return msgList;
    }

    public static void writeToFile(String filePath, List<String> msgList) {
        File writeFile = new File(filePath);
        if (writeFile.exists()) {
            writeFile.delete();
        }
        if (!writeFile.getParentFile().exists()) {
            writeFile.getParentFile().mkdirs();
        }
        try {
            writeFile.createNewFile();
        } catch (IOException e) {
            e.printStackTrace();
        }
        log.info("初始化文件写入位置 {}", writeFile.getAbsoluteFile().getPath());

        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(writeFile), "UTF-8"));
            for (String msg : msgList) {
                //新增一行数据
                bw.write(msg);
                bw.newLine();
            }
            log.info("导入数据 {} 条", msgList.size());
        } catch (FileNotFoundException e) {
            log.error("没有找到指定文件", e);
        } catch (IOException e) {
            log.error("文件读写出错", e);
        } finally {
            try {
                if (bw != null) {
                    bw.flush();
                    bw.close();
                }
            } catch (IOException e) {
                log.error("关闭文件错误", e);
            }
        }
    }

    /**
     * 获得src/test/resources/目录下的文件路径
     *
     * @param filePath
     * @return
     */
    public static String getTestResourcesFilePath(String filePath) {
        return getRootPath() + "/src/test/resources/" + filePath;
    }

    /**
     * 获得src/main/resources/目录下的文件路径
     *
     * @param filePath
     * @return
     */
    public static String getMainResourcesFilePath(String filePath) {
        return getRootPath() + "/src/main/resources/" + filePath;
    }

    /**
     * 获得项目根目录
     *
     * @return
     */
    private static String getRootPath() {
        File file = new File(FileTools.class.getResource("/").getPath());
        String rootPath = file.getParentFile().getParentFile().getPath();
        return rootPath;
    }
}
