package com.starbuds.test.utils;

import cn.hutool.core.codec.Base64;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.login.AppLoginTest;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.SignType;
import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Http请求工具
 *
 * @author aoe
 * @date 2020/5/13
 */
@Slf4j
public class XyHttpTools {

    public static final String HOST_DEV = "http://127.0.0.1:7152/";
    public static final String HOST_API_COMMON_DEV = "http://127.0.0.1:7151/";
    public static final String HOST_TEST_ALY_APP = "http://api.panygo.com/";

    // 渠道
    public static final String HOST_CHANNEL_DEV = "http://127.0.0.1:7157/";
    public static final String HOST_CHANNEL_TEST_ALY = HOST_TEST_ALY_APP;

    // 包类型 AppPkg AdminPkg
    public final static String PKG_TYPE_APP = "AppPkg";
    public final static String PKG_TYPE_ADMIN = "AdminPkg";
    public static String pkgType = PKG_TYPE_APP;

    public static String get(String host, String action, I18nTypeEnum langEnum) throws IOException {
        Map<String, String> headers = new HashMap<>();
        headers.put("pkg", getPkg(action));
        headers.put("Type", "application%2Fx-www-form-urlencoded%3B%20charset%3Dutf-8");
        headers.put("lang", langEnum.lang());

        String url = host + action;
        String res = OkHttpTools.get(url, headers);
        log.info("\n\n GET {} \n RESULT \n\n {} \n", url, res);
        return res;
    }

    public static String get(String host, String action) throws IOException {
        String pkg = getPkg(action);
        return get(host, action, pkg);
    }

    public static String get(String host, String action, String pkg) throws IOException {
        log.info("pkgpkgpkgpkg = \n{}", pkg);

        Map<String, String> headers = new HashMap<>();
        headers.put("pkg", pkg);
        headers.put("Type", "application%2Fx-www-form-urlencoded%3B%20charset%3Dutf-8");

        String url = host + action;
        String res = OkHttpTools.get(url, headers);
        JSONObject resJson = JSON.parseObject(res);
        log.info("\n\n GET {} \n RESULT \n\n {} \n", url, JSON.toJSONString(resJson, true));
        return res;
    }

    public static String post(String host, String action, Map<String, String> params) throws IOException {
        return post(host, action, params, getPkg(action));
    }

    public static String post(String host, String action, Map<String, String> params, String pkg) throws IOException {
        Map<String, String> headers = new HashMap<>();
        headers.put("pkg", pkg);
        headers.put("Type", "application%2Fx-www-form-urlencoded%3B%20charset%3Dutf-8");

        String url = host + action;
        String res = OkHttpTools.post(url, params, headers);
        JSONObject resJson = JSON.parseObject(res);
        log.info("\n\n POST {} \n RESULT \n\n {} \n", url, JSON.toJSONString(resJson, true));
        return res;
    }

    public static String getAppPkg(String token) {
        // App head pkg
        HeaderPackage headerPackage = new HeaderPackage();
        headerPackage.setClientType(ClientType.Android);
        headerPackage.setAppType(AppType.App);
        headerPackage.setPackageName("com.starbuds.app");
        headerPackage.setTimestamp(System.currentTimeMillis());
        headerPackage.setDeviceId("awerdas2221fyc");
        headerPackage.setToken(token);
        headerPackage.setAppVersion("1.0.0");
//        headerPackage.setChannelId(135120971366400L); // 六神花露水
        headerPackage.setChannelId(152552196800512L); // 挖芝麻
        headerPackage.setChannelId(3L); // 安卓官方
        headerPackage.setSignType(SignType.MD5);
        String toSign = headerPackage.toQueryString();
        toSign = String.format("%s%s.%s.security", toSign, headerPackage.getPackageName(), headerPackage.getClientType().getName());
        headerPackage.setSign(SecureUtil.md5(toSign));
        String headerJson = JSON.toJSONString(headerPackage);
        return Base64.encode(headerJson);
    }

    public static String getAdminPkg(String token) {
        // Admin head pkg
        HeaderPackage headerPackage = new HeaderPackage();
        headerPackage.setClientType(ClientType.Web);
        headerPackage.setAppType(AppType.Admin);
        headerPackage.setPackageName("com.starbuds.manage");
        headerPackage.setTimestamp(System.currentTimeMillis());
        headerPackage.setDeviceId("deviceId");
        headerPackage.setToken(token);
        headerPackage.setAppVersion("1.0.0");
        headerPackage.setSignType(SignType.MD5);
        String toSign = headerPackage.toQueryString();
        headerPackage.setSign(SecureUtil.md5(toSign + "pq" + SignType.MD5));
        String headerJson = JSON.toJSONString(headerPackage);
        return Base64.encode(headerJson);
    }

    private static String getPkg(String action) {
        String token = getLoginToken(action);
        return PKG_TYPE_ADMIN.equals(pkgType) ? getAdminPkg(token) : getAppPkg(token);
    }

    private static String getLoginToken(String action) {
        if ("api-app/v1/config/getInitConfig".equals(action)) {
            return "";
        }
        List<String> tokenList = FileTools.readList(FileTools.getTestResourcesFilePath("Login-TOKEN.txt"));
        return tokenList.get(0);
    }
}
